#ifndef random_systems_quintic_system_hpp
#define random_systems_quintic_system_hpp

#include <utility>
#include <iterator>
#include <math.h>

namespace flopoco
{
namespace random
{
namespace systems
 {

template<class T,class TACC=T>
class QuinticSystem{
private:
  T ss[31];
  T tmom[7];
  template<class TT> TT square(const TT &x) const { return x*x; }
public:
  typedef T real_t;

  int arity() const { return 6; };

  QuinticSystem(  const T *_ss ,   const T *_tmom )
  {
    std::copy(_ss, _ss+31, ss);
    std::copy(_tmom, _tmom+7, tmom);
  }

  void DefaultGuess(T *params) const{
    params[0]=0; // c0
    params[1]=1; // c1
    params[2]=0; // c2
    params[3]=0; // c3
    params[4]=0; // c4
    params[5]=0; // c5
  }

T metric(const T *parameters) const {
  T c0p1 = parameters[0];
  T c1p1 = parameters[1];
  T c2p1 = parameters[2];
  T c3p1 = parameters[3];
  T c4p1 = parameters[4];
  T c5p1 = parameters[5];
  TACC root_0 = convert<TACC>(c1p1*ss[1]);
  root_0 += c2p1*ss[2];
  root_0 += c3p1*ss[3];
  root_0 += c4p1*ss[4];
  root_0 += c5p1*ss[5];
  root_0 += c0p1;
  root_0 += -tmom[1];
  T c0p2 = c0p1*c0p1;
  T c1p2 = c1p1*c1p1;
  T c2p2 = c2p1*c2p1;
  T c3p2 = c3p1*c3p1;
  T c4p2 = c4p1*c4p1;
  T c5p2 = c5p1*c5p1;
  TACC root_1 = convert<TACC>(2*c0p1*c1p1*ss[1]);
  root_1 += 2*c0p1*c2p1*ss[2];
  root_1 += 2*c0p1*c3p1*ss[3];
  root_1 += 2*c0p1*c4p1*ss[4];
  root_1 += 2*c0p1*c5p1*ss[5];
  root_1 += 2*c1p1*c2p1*ss[3];
  root_1 += 2*c1p1*c3p1*ss[4];
  root_1 += 2*c1p1*c4p1*ss[5];
  root_1 += 2*c1p1*c5p1*ss[6];
  root_1 += 2*c2p1*c3p1*ss[5];
  root_1 += 2*c2p1*c4p1*ss[6];
  root_1 += 2*c2p1*c5p1*ss[7];
  root_1 += 2*c3p1*c4p1*ss[7];
  root_1 += 2*c3p1*c5p1*ss[8];
  root_1 += 2*c4p1*c5p1*ss[9];
  root_1 += c1p2*ss[2];
  root_1 += c2p2*ss[4];
  root_1 += c3p2*ss[6];
  root_1 += c4p2*ss[8];
  root_1 += c5p2*ss[10];
  root_1 += c0p2;
  root_1 += -tmom[2];
  T c0p3 = c0p2*c0p1;
  T c1p3 = c1p2*c1p1;
  T c2p3 = c2p2*c2p1;
  T c3p3 = c3p2*c3p1;
  T c4p3 = c4p2*c4p1;
  T c5p3 = c5p2*c5p1;
  TACC root_2 = convert<TACC>(6*c0p1*c1p1*c2p1*ss[3]);
  root_2 += 6*c0p1*c1p1*c3p1*ss[4];
  root_2 += 6*c0p1*c1p1*c4p1*ss[5];
  root_2 += 6*c0p1*c1p1*c5p1*ss[6];
  root_2 += 6*c0p1*c2p1*c3p1*ss[5];
  root_2 += 6*c0p1*c2p1*c4p1*ss[6];
  root_2 += 6*c0p1*c2p1*c5p1*ss[7];
  root_2 += 6*c0p1*c3p1*c4p1*ss[7];
  root_2 += 6*c0p1*c3p1*c5p1*ss[8];
  root_2 += 6*c0p1*c4p1*c5p1*ss[9];
  root_2 += 6*c1p1*c2p1*c3p1*ss[6];
  root_2 += 6*c1p1*c2p1*c4p1*ss[7];
  root_2 += 6*c1p1*c2p1*c5p1*ss[8];
  root_2 += 6*c1p1*c3p1*c4p1*ss[8];
  root_2 += 6*c1p1*c3p1*c5p1*ss[9];
  root_2 += 6*c1p1*c4p1*c5p1*ss[10];
  root_2 += 6*c2p1*c3p1*c4p1*ss[9];
  root_2 += 6*c2p1*c3p1*c5p1*ss[10];
  root_2 += 6*c2p1*c4p1*c5p1*ss[11];
  root_2 += 6*c3p1*c4p1*c5p1*ss[12];
  root_2 += 3*c0p1*c1p2*ss[2];
  root_2 += 3*c0p1*c2p2*ss[4];
  root_2 += 3*c0p1*c3p2*ss[6];
  root_2 += 3*c0p1*c4p2*ss[8];
  root_2 += 3*c0p1*c5p2*ss[10];
  root_2 += 3*c0p2*c1p1*ss[1];
  root_2 += 3*c0p2*c2p1*ss[2];
  root_2 += 3*c0p2*c3p1*ss[3];
  root_2 += 3*c0p2*c4p1*ss[4];
  root_2 += 3*c0p2*c5p1*ss[5];
  root_2 += 3*c1p1*c2p2*ss[5];
  root_2 += 3*c1p1*c3p2*ss[7];
  root_2 += 3*c1p1*c4p2*ss[9];
  root_2 += 3*c1p1*c5p2*ss[11];
  root_2 += 3*c1p2*c2p1*ss[4];
  root_2 += 3*c1p2*c3p1*ss[5];
  root_2 += 3*c1p2*c4p1*ss[6];
  root_2 += 3*c1p2*c5p1*ss[7];
  root_2 += 3*c2p1*c3p2*ss[8];
  root_2 += 3*c2p1*c4p2*ss[10];
  root_2 += 3*c2p1*c5p2*ss[12];
  root_2 += 3*c2p2*c3p1*ss[7];
  root_2 += 3*c2p2*c4p1*ss[8];
  root_2 += 3*c2p2*c5p1*ss[9];
  root_2 += 3*c3p1*c4p2*ss[11];
  root_2 += 3*c3p1*c5p2*ss[13];
  root_2 += 3*c3p2*c4p1*ss[10];
  root_2 += 3*c3p2*c5p1*ss[11];
  root_2 += 3*c4p1*c5p2*ss[14];
  root_2 += 3*c4p2*c5p1*ss[13];
  root_2 += c1p3*ss[3];
  root_2 += c2p3*ss[6];
  root_2 += c3p3*ss[9];
  root_2 += c4p3*ss[12];
  root_2 += c5p3*ss[15];
  root_2 += c0p3;
  root_2 += -tmom[3];
  T c0p4 = c0p3*c0p1;
  T c1p4 = c1p3*c1p1;
  T c2p4 = c2p3*c2p1;
  T c3p4 = c3p3*c3p1;
  T c4p4 = c4p3*c4p1;
  T c5p4 = c5p3*c5p1;
  TACC root_3 = convert<TACC>(24*c0p1*c1p1*c2p1*c3p1*ss[6]);
  root_3 += 24*c0p1*c1p1*c2p1*c4p1*ss[7];
  root_3 += 24*c0p1*c1p1*c2p1*c5p1*ss[8];
  root_3 += 24*c0p1*c1p1*c3p1*c4p1*ss[8];
  root_3 += 24*c0p1*c1p1*c3p1*c5p1*ss[9];
  root_3 += 24*c0p1*c1p1*c4p1*c5p1*ss[10];
  root_3 += 24*c0p1*c2p1*c3p1*c4p1*ss[9];
  root_3 += 24*c0p1*c2p1*c3p1*c5p1*ss[10];
  root_3 += 24*c0p1*c2p1*c4p1*c5p1*ss[11];
  root_3 += 24*c0p1*c3p1*c4p1*c5p1*ss[12];
  root_3 += 24*c1p1*c2p1*c3p1*c4p1*ss[10];
  root_3 += 24*c1p1*c2p1*c3p1*c5p1*ss[11];
  root_3 += 24*c1p1*c2p1*c4p1*c5p1*ss[12];
  root_3 += 24*c1p1*c3p1*c4p1*c5p1*ss[13];
  root_3 += 24*c2p1*c3p1*c4p1*c5p1*ss[14];
  root_3 += 12*c0p1*c1p1*c2p2*ss[5];
  root_3 += 12*c0p1*c1p1*c3p2*ss[7];
  root_3 += 12*c0p1*c1p1*c4p2*ss[9];
  root_3 += 12*c0p1*c1p1*c5p2*ss[11];
  root_3 += 12*c0p1*c1p2*c2p1*ss[4];
  root_3 += 12*c0p1*c1p2*c3p1*ss[5];
  root_3 += 12*c0p1*c1p2*c4p1*ss[6];
  root_3 += 12*c0p1*c1p2*c5p1*ss[7];
  root_3 += 12*c0p1*c2p1*c3p2*ss[8];
  root_3 += 12*c0p1*c2p1*c4p2*ss[10];
  root_3 += 12*c0p1*c2p1*c5p2*ss[12];
  root_3 += 12*c0p1*c2p2*c3p1*ss[7];
  root_3 += 12*c0p1*c2p2*c4p1*ss[8];
  root_3 += 12*c0p1*c2p2*c5p1*ss[9];
  root_3 += 12*c0p1*c3p1*c4p2*ss[11];
  root_3 += 12*c0p1*c3p1*c5p2*ss[13];
  root_3 += 12*c0p1*c3p2*c4p1*ss[10];
  root_3 += 12*c0p1*c3p2*c5p1*ss[11];
  root_3 += 12*c0p1*c4p1*c5p2*ss[14];
  root_3 += 12*c0p1*c4p2*c5p1*ss[13];
  root_3 += 12*c0p2*c1p1*c2p1*ss[3];
  root_3 += 12*c0p2*c1p1*c3p1*ss[4];
  root_3 += 12*c0p2*c1p1*c4p1*ss[5];
  root_3 += 12*c0p2*c1p1*c5p1*ss[6];
  root_3 += 12*c0p2*c2p1*c3p1*ss[5];
  root_3 += 12*c0p2*c2p1*c4p1*ss[6];
  root_3 += 12*c0p2*c2p1*c5p1*ss[7];
  root_3 += 12*c0p2*c3p1*c4p1*ss[7];
  root_3 += 12*c0p2*c3p1*c5p1*ss[8];
  root_3 += 12*c0p2*c4p1*c5p1*ss[9];
  root_3 += 12*c1p1*c2p1*c3p2*ss[9];
  root_3 += 12*c1p1*c2p1*c4p2*ss[11];
  root_3 += 12*c1p1*c2p1*c5p2*ss[13];
  root_3 += 12*c1p1*c2p2*c3p1*ss[8];
  root_3 += 12*c1p1*c2p2*c4p1*ss[9];
  root_3 += 12*c1p1*c2p2*c5p1*ss[10];
  root_3 += 12*c1p1*c3p1*c4p2*ss[12];
  root_3 += 12*c1p1*c3p1*c5p2*ss[14];
  root_3 += 12*c1p1*c3p2*c4p1*ss[11];
  root_3 += 12*c1p1*c3p2*c5p1*ss[12];
  root_3 += 12*c1p1*c4p1*c5p2*ss[15];
  root_3 += 12*c1p1*c4p2*c5p1*ss[14];
  root_3 += 12*c1p2*c2p1*c3p1*ss[7];
  root_3 += 12*c1p2*c2p1*c4p1*ss[8];
  root_3 += 12*c1p2*c2p1*c5p1*ss[9];
  root_3 += 12*c1p2*c3p1*c4p1*ss[9];
  root_3 += 12*c1p2*c3p1*c5p1*ss[10];
  root_3 += 12*c1p2*c4p1*c5p1*ss[11];
  root_3 += 12*c2p1*c3p1*c4p2*ss[13];
  root_3 += 12*c2p1*c3p1*c5p2*ss[15];
  root_3 += 12*c2p1*c3p2*c4p1*ss[12];
  root_3 += 12*c2p1*c3p2*c5p1*ss[13];
  root_3 += 12*c2p1*c4p1*c5p2*ss[16];
  root_3 += 12*c2p1*c4p2*c5p1*ss[15];
  root_3 += 12*c2p2*c3p1*c4p1*ss[11];
  root_3 += 12*c2p2*c3p1*c5p1*ss[12];
  root_3 += 12*c2p2*c4p1*c5p1*ss[13];
  root_3 += 12*c3p1*c4p1*c5p2*ss[17];
  root_3 += 12*c3p1*c4p2*c5p1*ss[16];
  root_3 += 12*c3p2*c4p1*c5p1*ss[15];
  root_3 += 4*c0p1*c1p3*ss[3];
  root_3 += 4*c0p1*c2p3*ss[6];
  root_3 += 4*c0p1*c3p3*ss[9];
  root_3 += 4*c0p1*c4p3*ss[12];
  root_3 += 4*c0p1*c5p3*ss[15];
  root_3 += 6*c0p2*c1p2*ss[2];
  root_3 += 6*c0p2*c2p2*ss[4];
  root_3 += 6*c0p2*c3p2*ss[6];
  root_3 += 6*c0p2*c4p2*ss[8];
  root_3 += 6*c0p2*c5p2*ss[10];
  root_3 += 4*c0p3*c1p1*ss[1];
  root_3 += 4*c0p3*c2p1*ss[2];
  root_3 += 4*c0p3*c3p1*ss[3];
  root_3 += 4*c0p3*c4p1*ss[4];
  root_3 += 4*c0p3*c5p1*ss[5];
  root_3 += 4*c1p1*c2p3*ss[7];
  root_3 += 4*c1p1*c3p3*ss[10];
  root_3 += 4*c1p1*c4p3*ss[13];
  root_3 += 4*c1p1*c5p3*ss[16];
  root_3 += 6*c1p2*c2p2*ss[6];
  root_3 += 6*c1p2*c3p2*ss[8];
  root_3 += 6*c1p2*c4p2*ss[10];
  root_3 += 6*c1p2*c5p2*ss[12];
  root_3 += 4*c1p3*c2p1*ss[5];
  root_3 += 4*c1p3*c3p1*ss[6];
  root_3 += 4*c1p3*c4p1*ss[7];
  root_3 += 4*c1p3*c5p1*ss[8];
  root_3 += 4*c2p1*c3p3*ss[11];
  root_3 += 4*c2p1*c4p3*ss[14];
  root_3 += 4*c2p1*c5p3*ss[17];
  root_3 += 6*c2p2*c3p2*ss[10];
  root_3 += 6*c2p2*c4p2*ss[12];
  root_3 += 6*c2p2*c5p2*ss[14];
  root_3 += 4*c2p3*c3p1*ss[9];
  root_3 += 4*c2p3*c4p1*ss[10];
  root_3 += 4*c2p3*c5p1*ss[11];
  root_3 += 4*c3p1*c4p3*ss[15];
  root_3 += 4*c3p1*c5p3*ss[18];
  root_3 += 6*c3p2*c4p2*ss[14];
  root_3 += 6*c3p2*c5p2*ss[16];
  root_3 += 4*c3p3*c4p1*ss[13];
  root_3 += 4*c3p3*c5p1*ss[14];
  root_3 += 4*c4p1*c5p3*ss[19];
  root_3 += 6*c4p2*c5p2*ss[18];
  root_3 += 4*c4p3*c5p1*ss[17];
  root_3 += c1p4*ss[4];
  root_3 += c2p4*ss[8];
  root_3 += c3p4*ss[12];
  root_3 += c4p4*ss[16];
  root_3 += c5p4*ss[20];
  root_3 += c0p4;
  root_3 += -tmom[4];
  T c0p5 = c0p4*c0p1;
  T c1p5 = c1p4*c1p1;
  T c2p5 = c2p4*c2p1;
  T c3p5 = c3p4*c3p1;
  T c4p5 = c4p4*c4p1;
  T c5p5 = c5p4*c5p1;
  TACC root_4 = convert<TACC>(120*c0p1*c1p1*c2p1*c3p1*c4p1*ss[10]);
  root_4 += 120*c0p1*c1p1*c2p1*c3p1*c5p1*ss[11];
  root_4 += 120*c0p1*c1p1*c2p1*c4p1*c5p1*ss[12];
  root_4 += 120*c0p1*c1p1*c3p1*c4p1*c5p1*ss[13];
  root_4 += 120*c0p1*c2p1*c3p1*c4p1*c5p1*ss[14];
  root_4 += 120*c1p1*c2p1*c3p1*c4p1*c5p1*ss[15];
  root_4 += 60*c0p1*c1p1*c2p1*c3p2*ss[9];
  root_4 += 60*c0p1*c1p1*c2p1*c4p2*ss[11];
  root_4 += 60*c0p1*c1p1*c2p1*c5p2*ss[13];
  root_4 += 60*c0p1*c1p1*c2p2*c3p1*ss[8];
  root_4 += 60*c0p1*c1p1*c2p2*c4p1*ss[9];
  root_4 += 60*c0p1*c1p1*c2p2*c5p1*ss[10];
  root_4 += 60*c0p1*c1p1*c3p1*c4p2*ss[12];
  root_4 += 60*c0p1*c1p1*c3p1*c5p2*ss[14];
  root_4 += 60*c0p1*c1p1*c3p2*c4p1*ss[11];
  root_4 += 60*c0p1*c1p1*c3p2*c5p1*ss[12];
  root_4 += 60*c0p1*c1p1*c4p1*c5p2*ss[15];
  root_4 += 60*c0p1*c1p1*c4p2*c5p1*ss[14];
  root_4 += 60*c0p1*c1p2*c2p1*c3p1*ss[7];
  root_4 += 60*c0p1*c1p2*c2p1*c4p1*ss[8];
  root_4 += 60*c0p1*c1p2*c2p1*c5p1*ss[9];
  root_4 += 60*c0p1*c1p2*c3p1*c4p1*ss[9];
  root_4 += 60*c0p1*c1p2*c3p1*c5p1*ss[10];
  root_4 += 60*c0p1*c1p2*c4p1*c5p1*ss[11];
  root_4 += 60*c0p1*c2p1*c3p1*c4p2*ss[13];
  root_4 += 60*c0p1*c2p1*c3p1*c5p2*ss[15];
  root_4 += 60*c0p1*c2p1*c3p2*c4p1*ss[12];
  root_4 += 60*c0p1*c2p1*c3p2*c5p1*ss[13];
  root_4 += 60*c0p1*c2p1*c4p1*c5p2*ss[16];
  root_4 += 60*c0p1*c2p1*c4p2*c5p1*ss[15];
  root_4 += 60*c0p1*c2p2*c3p1*c4p1*ss[11];
  root_4 += 60*c0p1*c2p2*c3p1*c5p1*ss[12];
  root_4 += 60*c0p1*c2p2*c4p1*c5p1*ss[13];
  root_4 += 60*c0p1*c3p1*c4p1*c5p2*ss[17];
  root_4 += 60*c0p1*c3p1*c4p2*c5p1*ss[16];
  root_4 += 60*c0p1*c3p2*c4p1*c5p1*ss[15];
  root_4 += 60*c0p2*c1p1*c2p1*c3p1*ss[6];
  root_4 += 60*c0p2*c1p1*c2p1*c4p1*ss[7];
  root_4 += 60*c0p2*c1p1*c2p1*c5p1*ss[8];
  root_4 += 60*c0p2*c1p1*c3p1*c4p1*ss[8];
  root_4 += 60*c0p2*c1p1*c3p1*c5p1*ss[9];
  root_4 += 60*c0p2*c1p1*c4p1*c5p1*ss[10];
  root_4 += 60*c0p2*c2p1*c3p1*c4p1*ss[9];
  root_4 += 60*c0p2*c2p1*c3p1*c5p1*ss[10];
  root_4 += 60*c0p2*c2p1*c4p1*c5p1*ss[11];
  root_4 += 60*c0p2*c3p1*c4p1*c5p1*ss[12];
  root_4 += 60*c1p1*c2p1*c3p1*c4p2*ss[14];
  root_4 += 60*c1p1*c2p1*c3p1*c5p2*ss[16];
  root_4 += 60*c1p1*c2p1*c3p2*c4p1*ss[13];
  root_4 += 60*c1p1*c2p1*c3p2*c5p1*ss[14];
  root_4 += 60*c1p1*c2p1*c4p1*c5p2*ss[17];
  root_4 += 60*c1p1*c2p1*c4p2*c5p1*ss[16];
  root_4 += 60*c1p1*c2p2*c3p1*c4p1*ss[12];
  root_4 += 60*c1p1*c2p2*c3p1*c5p1*ss[13];
  root_4 += 60*c1p1*c2p2*c4p1*c5p1*ss[14];
  root_4 += 60*c1p1*c3p1*c4p1*c5p2*ss[18];
  root_4 += 60*c1p1*c3p1*c4p2*c5p1*ss[17];
  root_4 += 60*c1p1*c3p2*c4p1*c5p1*ss[16];
  root_4 += 60*c1p2*c2p1*c3p1*c4p1*ss[11];
  root_4 += 60*c1p2*c2p1*c3p1*c5p1*ss[12];
  root_4 += 60*c1p2*c2p1*c4p1*c5p1*ss[13];
  root_4 += 60*c1p2*c3p1*c4p1*c5p1*ss[14];
  root_4 += 60*c2p1*c3p1*c4p1*c5p2*ss[19];
  root_4 += 60*c2p1*c3p1*c4p2*c5p1*ss[18];
  root_4 += 60*c2p1*c3p2*c4p1*c5p1*ss[17];
  root_4 += 60*c2p2*c3p1*c4p1*c5p1*ss[16];
  root_4 += 20*c0p1*c1p1*c2p3*ss[7];
  root_4 += 20*c0p1*c1p1*c3p3*ss[10];
  root_4 += 20*c0p1*c1p1*c4p3*ss[13];
  root_4 += 20*c0p1*c1p1*c5p3*ss[16];
  root_4 += 30*c0p1*c1p2*c2p2*ss[6];
  root_4 += 30*c0p1*c1p2*c3p2*ss[8];
  root_4 += 30*c0p1*c1p2*c4p2*ss[10];
  root_4 += 30*c0p1*c1p2*c5p2*ss[12];
  root_4 += 20*c0p1*c1p3*c2p1*ss[5];
  root_4 += 20*c0p1*c1p3*c3p1*ss[6];
  root_4 += 20*c0p1*c1p3*c4p1*ss[7];
  root_4 += 20*c0p1*c1p3*c5p1*ss[8];
  root_4 += 20*c0p1*c2p1*c3p3*ss[11];
  root_4 += 20*c0p1*c2p1*c4p3*ss[14];
  root_4 += 20*c0p1*c2p1*c5p3*ss[17];
  root_4 += 30*c0p1*c2p2*c3p2*ss[10];
  root_4 += 30*c0p1*c2p2*c4p2*ss[12];
  root_4 += 30*c0p1*c2p2*c5p2*ss[14];
  root_4 += 20*c0p1*c2p3*c3p1*ss[9];
  root_4 += 20*c0p1*c2p3*c4p1*ss[10];
  root_4 += 20*c0p1*c2p3*c5p1*ss[11];
  root_4 += 20*c0p1*c3p1*c4p3*ss[15];
  root_4 += 20*c0p1*c3p1*c5p3*ss[18];
  root_4 += 30*c0p1*c3p2*c4p2*ss[14];
  root_4 += 30*c0p1*c3p2*c5p2*ss[16];
  root_4 += 20*c0p1*c3p3*c4p1*ss[13];
  root_4 += 20*c0p1*c3p3*c5p1*ss[14];
  root_4 += 20*c0p1*c4p1*c5p3*ss[19];
  root_4 += 30*c0p1*c4p2*c5p2*ss[18];
  root_4 += 20*c0p1*c4p3*c5p1*ss[17];
  root_4 += 30*c0p2*c1p1*c2p2*ss[5];
  root_4 += 30*c0p2*c1p1*c3p2*ss[7];
  root_4 += 30*c0p2*c1p1*c4p2*ss[9];
  root_4 += 30*c0p2*c1p1*c5p2*ss[11];
  root_4 += 30*c0p2*c1p2*c2p1*ss[4];
  root_4 += 30*c0p2*c1p2*c3p1*ss[5];
  root_4 += 30*c0p2*c1p2*c4p1*ss[6];
  root_4 += 30*c0p2*c1p2*c5p1*ss[7];
  root_4 += 30*c0p2*c2p1*c3p2*ss[8];
  root_4 += 30*c0p2*c2p1*c4p2*ss[10];
  root_4 += 30*c0p2*c2p1*c5p2*ss[12];
  root_4 += 30*c0p2*c2p2*c3p1*ss[7];
  root_4 += 30*c0p2*c2p2*c4p1*ss[8];
  root_4 += 30*c0p2*c2p2*c5p1*ss[9];
  root_4 += 30*c0p2*c3p1*c4p2*ss[11];
  root_4 += 30*c0p2*c3p1*c5p2*ss[13];
  root_4 += 30*c0p2*c3p2*c4p1*ss[10];
  root_4 += 30*c0p2*c3p2*c5p1*ss[11];
  root_4 += 30*c0p2*c4p1*c5p2*ss[14];
  root_4 += 30*c0p2*c4p2*c5p1*ss[13];
  root_4 += 20*c0p3*c1p1*c2p1*ss[3];
  root_4 += 20*c0p3*c1p1*c3p1*ss[4];
  root_4 += 20*c0p3*c1p1*c4p1*ss[5];
  root_4 += 20*c0p3*c1p1*c5p1*ss[6];
  root_4 += 20*c0p3*c2p1*c3p1*ss[5];
  root_4 += 20*c0p3*c2p1*c4p1*ss[6];
  root_4 += 20*c0p3*c2p1*c5p1*ss[7];
  root_4 += 20*c0p3*c3p1*c4p1*ss[7];
  root_4 += 20*c0p3*c3p1*c5p1*ss[8];
  root_4 += 20*c0p3*c4p1*c5p1*ss[9];
  root_4 += 20*c1p1*c2p1*c3p3*ss[12];
  root_4 += 20*c1p1*c2p1*c4p3*ss[15];
  root_4 += 20*c1p1*c2p1*c5p3*ss[18];
  root_4 += 30*c1p1*c2p2*c3p2*ss[11];
  root_4 += 30*c1p1*c2p2*c4p2*ss[13];
  root_4 += 30*c1p1*c2p2*c5p2*ss[15];
  root_4 += 20*c1p1*c2p3*c3p1*ss[10];
  root_4 += 20*c1p1*c2p3*c4p1*ss[11];
  root_4 += 20*c1p1*c2p3*c5p1*ss[12];
  root_4 += 20*c1p1*c3p1*c4p3*ss[16];
  root_4 += 20*c1p1*c3p1*c5p3*ss[19];
  root_4 += 30*c1p1*c3p2*c4p2*ss[15];
  root_4 += 30*c1p1*c3p2*c5p2*ss[17];
  root_4 += 20*c1p1*c3p3*c4p1*ss[14];
  root_4 += 20*c1p1*c3p3*c5p1*ss[15];
  root_4 += 20*c1p1*c4p1*c5p3*ss[20];
  root_4 += 30*c1p1*c4p2*c5p2*ss[19];
  root_4 += 20*c1p1*c4p3*c5p1*ss[18];
  root_4 += 30*c1p2*c2p1*c3p2*ss[10];
  root_4 += 30*c1p2*c2p1*c4p2*ss[12];
  root_4 += 30*c1p2*c2p1*c5p2*ss[14];
  root_4 += 30*c1p2*c2p2*c3p1*ss[9];
  root_4 += 30*c1p2*c2p2*c4p1*ss[10];
  root_4 += 30*c1p2*c2p2*c5p1*ss[11];
  root_4 += 30*c1p2*c3p1*c4p2*ss[13];
  root_4 += 30*c1p2*c3p1*c5p2*ss[15];
  root_4 += 30*c1p2*c3p2*c4p1*ss[12];
  root_4 += 30*c1p2*c3p2*c5p1*ss[13];
  root_4 += 30*c1p2*c4p1*c5p2*ss[16];
  root_4 += 30*c1p2*c4p2*c5p1*ss[15];
  root_4 += 20*c1p3*c2p1*c3p1*ss[8];
  root_4 += 20*c1p3*c2p1*c4p1*ss[9];
  root_4 += 20*c1p3*c2p1*c5p1*ss[10];
  root_4 += 20*c1p3*c3p1*c4p1*ss[10];
  root_4 += 20*c1p3*c3p1*c5p1*ss[11];
  root_4 += 20*c1p3*c4p1*c5p1*ss[12];
  root_4 += 20*c2p1*c3p1*c4p3*ss[17];
  root_4 += 20*c2p1*c3p1*c5p3*ss[20];
  root_4 += 30*c2p1*c3p2*c4p2*ss[16];
  root_4 += 30*c2p1*c3p2*c5p2*ss[18];
  root_4 += 20*c2p1*c3p3*c4p1*ss[15];
  root_4 += 20*c2p1*c3p3*c5p1*ss[16];
  root_4 += 20*c2p1*c4p1*c5p3*ss[21];
  root_4 += 30*c2p1*c4p2*c5p2*ss[20];
  root_4 += 20*c2p1*c4p3*c5p1*ss[19];
  root_4 += 30*c2p2*c3p1*c4p2*ss[15];
  root_4 += 30*c2p2*c3p1*c5p2*ss[17];
  root_4 += 30*c2p2*c3p2*c4p1*ss[14];
  root_4 += 30*c2p2*c3p2*c5p1*ss[15];
  root_4 += 30*c2p2*c4p1*c5p2*ss[18];
  root_4 += 30*c2p2*c4p2*c5p1*ss[17];
  root_4 += 20*c2p3*c3p1*c4p1*ss[13];
  root_4 += 20*c2p3*c3p1*c5p1*ss[14];
  root_4 += 20*c2p3*c4p1*c5p1*ss[15];
  root_4 += 20*c3p1*c4p1*c5p3*ss[22];
  root_4 += 30*c3p1*c4p2*c5p2*ss[21];
  root_4 += 20*c3p1*c4p3*c5p1*ss[20];
  root_4 += 30*c3p2*c4p1*c5p2*ss[20];
  root_4 += 30*c3p2*c4p2*c5p1*ss[19];
  root_4 += 20*c3p3*c4p1*c5p1*ss[18];
  root_4 += 5*c0p1*c1p4*ss[4];
  root_4 += 5*c0p1*c2p4*ss[8];
  root_4 += 5*c0p1*c3p4*ss[12];
  root_4 += 5*c0p1*c4p4*ss[16];
  root_4 += 5*c0p1*c5p4*ss[20];
  root_4 += 10*c0p2*c1p3*ss[3];
  root_4 += 10*c0p2*c2p3*ss[6];
  root_4 += 10*c0p2*c3p3*ss[9];
  root_4 += 10*c0p2*c4p3*ss[12];
  root_4 += 10*c0p2*c5p3*ss[15];
  root_4 += 10*c0p3*c1p2*ss[2];
  root_4 += 10*c0p3*c2p2*ss[4];
  root_4 += 10*c0p3*c3p2*ss[6];
  root_4 += 10*c0p3*c4p2*ss[8];
  root_4 += 10*c0p3*c5p2*ss[10];
  root_4 += 5*c0p4*c1p1*ss[1];
  root_4 += 5*c0p4*c2p1*ss[2];
  root_4 += 5*c0p4*c3p1*ss[3];
  root_4 += 5*c0p4*c4p1*ss[4];
  root_4 += 5*c0p4*c5p1*ss[5];
  root_4 += 5*c1p1*c2p4*ss[9];
  root_4 += 5*c1p1*c3p4*ss[13];
  root_4 += 5*c1p1*c4p4*ss[17];
  root_4 += 5*c1p1*c5p4*ss[21];
  root_4 += 10*c1p2*c2p3*ss[8];
  root_4 += 10*c1p2*c3p3*ss[11];
  root_4 += 10*c1p2*c4p3*ss[14];
  root_4 += 10*c1p2*c5p3*ss[17];
  root_4 += 10*c1p3*c2p2*ss[7];
  root_4 += 10*c1p3*c3p2*ss[9];
  root_4 += 10*c1p3*c4p2*ss[11];
  root_4 += 10*c1p3*c5p2*ss[13];
  root_4 += 5*c1p4*c2p1*ss[6];
  root_4 += 5*c1p4*c3p1*ss[7];
  root_4 += 5*c1p4*c4p1*ss[8];
  root_4 += 5*c1p4*c5p1*ss[9];
  root_4 += 5*c2p1*c3p4*ss[14];
  root_4 += 5*c2p1*c4p4*ss[18];
  root_4 += 5*c2p1*c5p4*ss[22];
  root_4 += 10*c2p2*c3p3*ss[13];
  root_4 += 10*c2p2*c4p3*ss[16];
  root_4 += 10*c2p2*c5p3*ss[19];
  root_4 += 10*c2p3*c3p2*ss[12];
  root_4 += 10*c2p3*c4p2*ss[14];
  root_4 += 10*c2p3*c5p2*ss[16];
  root_4 += 5*c2p4*c3p1*ss[11];
  root_4 += 5*c2p4*c4p1*ss[12];
  root_4 += 5*c2p4*c5p1*ss[13];
  root_4 += 5*c3p1*c4p4*ss[19];
  root_4 += 5*c3p1*c5p4*ss[23];
  root_4 += 10*c3p2*c4p3*ss[18];
  root_4 += 10*c3p2*c5p3*ss[21];
  root_4 += 10*c3p3*c4p2*ss[17];
  root_4 += 10*c3p3*c5p2*ss[19];
  root_4 += 5*c3p4*c4p1*ss[16];
  root_4 += 5*c3p4*c5p1*ss[17];
  root_4 += 5*c4p1*c5p4*ss[24];
  root_4 += 10*c4p2*c5p3*ss[23];
  root_4 += 10*c4p3*c5p2*ss[22];
  root_4 += 5*c4p4*c5p1*ss[21];
  root_4 += c1p5*ss[5];
  root_4 += c2p5*ss[10];
  root_4 += c3p5*ss[15];
  root_4 += c4p5*ss[20];
  root_4 += c5p5*ss[25];
  root_4 += c0p5;
  root_4 += -tmom[5];
  T c0p6 = c0p5*c0p1;
  T c1p6 = c1p5*c1p1;
  T c2p6 = c2p5*c2p1;
  T c3p6 = c3p5*c3p1;
  T c4p6 = c4p5*c4p1;
  T c5p6 = c5p5*c5p1;
  TACC root_5 = convert<TACC>(720*c0p1*c1p1*c2p1*c3p1*c4p1*c5p1*ss[15]);
  root_5 += 360*c0p1*c1p1*c2p1*c3p1*c4p2*ss[14];
  root_5 += 360*c0p1*c1p1*c2p1*c3p1*c5p2*ss[16];
  root_5 += 360*c0p1*c1p1*c2p1*c3p2*c4p1*ss[13];
  root_5 += 360*c0p1*c1p1*c2p1*c3p2*c5p1*ss[14];
  root_5 += 360*c0p1*c1p1*c2p1*c4p1*c5p2*ss[17];
  root_5 += 360*c0p1*c1p1*c2p1*c4p2*c5p1*ss[16];
  root_5 += 360*c0p1*c1p1*c2p2*c3p1*c4p1*ss[12];
  root_5 += 360*c0p1*c1p1*c2p2*c3p1*c5p1*ss[13];
  root_5 += 360*c0p1*c1p1*c2p2*c4p1*c5p1*ss[14];
  root_5 += 360*c0p1*c1p1*c3p1*c4p1*c5p2*ss[18];
  root_5 += 360*c0p1*c1p1*c3p1*c4p2*c5p1*ss[17];
  root_5 += 360*c0p1*c1p1*c3p2*c4p1*c5p1*ss[16];
  root_5 += 360*c0p1*c1p2*c2p1*c3p1*c4p1*ss[11];
  root_5 += 360*c0p1*c1p2*c2p1*c3p1*c5p1*ss[12];
  root_5 += 360*c0p1*c1p2*c2p1*c4p1*c5p1*ss[13];
  root_5 += 360*c0p1*c1p2*c3p1*c4p1*c5p1*ss[14];
  root_5 += 360*c0p1*c2p1*c3p1*c4p1*c5p2*ss[19];
  root_5 += 360*c0p1*c2p1*c3p1*c4p2*c5p1*ss[18];
  root_5 += 360*c0p1*c2p1*c3p2*c4p1*c5p1*ss[17];
  root_5 += 360*c0p1*c2p2*c3p1*c4p1*c5p1*ss[16];
  root_5 += 360*c0p2*c1p1*c2p1*c3p1*c4p1*ss[10];
  root_5 += 360*c0p2*c1p1*c2p1*c3p1*c5p1*ss[11];
  root_5 += 360*c0p2*c1p1*c2p1*c4p1*c5p1*ss[12];
  root_5 += 360*c0p2*c1p1*c3p1*c4p1*c5p1*ss[13];
  root_5 += 360*c0p2*c2p1*c3p1*c4p1*c5p1*ss[14];
  root_5 += 360*c1p1*c2p1*c3p1*c4p1*c5p2*ss[20];
  root_5 += 360*c1p1*c2p1*c3p1*c4p2*c5p1*ss[19];
  root_5 += 360*c1p1*c2p1*c3p2*c4p1*c5p1*ss[18];
  root_5 += 360*c1p1*c2p2*c3p1*c4p1*c5p1*ss[17];
  root_5 += 360*c1p2*c2p1*c3p1*c4p1*c5p1*ss[16];
  root_5 += 120*c0p1*c1p1*c2p1*c3p3*ss[12];
  root_5 += 120*c0p1*c1p1*c2p1*c4p3*ss[15];
  root_5 += 120*c0p1*c1p1*c2p1*c5p3*ss[18];
  root_5 += 180*c0p1*c1p1*c2p2*c3p2*ss[11];
  root_5 += 180*c0p1*c1p1*c2p2*c4p2*ss[13];
  root_5 += 180*c0p1*c1p1*c2p2*c5p2*ss[15];
  root_5 += 120*c0p1*c1p1*c2p3*c3p1*ss[10];
  root_5 += 120*c0p1*c1p1*c2p3*c4p1*ss[11];
  root_5 += 120*c0p1*c1p1*c2p3*c5p1*ss[12];
  root_5 += 120*c0p1*c1p1*c3p1*c4p3*ss[16];
  root_5 += 120*c0p1*c1p1*c3p1*c5p3*ss[19];
  root_5 += 180*c0p1*c1p1*c3p2*c4p2*ss[15];
  root_5 += 180*c0p1*c1p1*c3p2*c5p2*ss[17];
  root_5 += 120*c0p1*c1p1*c3p3*c4p1*ss[14];
  root_5 += 120*c0p1*c1p1*c3p3*c5p1*ss[15];
  root_5 += 120*c0p1*c1p1*c4p1*c5p3*ss[20];
  root_5 += 180*c0p1*c1p1*c4p2*c5p2*ss[19];
  root_5 += 120*c0p1*c1p1*c4p3*c5p1*ss[18];
  root_5 += 180*c0p1*c1p2*c2p1*c3p2*ss[10];
  root_5 += 180*c0p1*c1p2*c2p1*c4p2*ss[12];
  root_5 += 180*c0p1*c1p2*c2p1*c5p2*ss[14];
  root_5 += 180*c0p1*c1p2*c2p2*c3p1*ss[9];
  root_5 += 180*c0p1*c1p2*c2p2*c4p1*ss[10];
  root_5 += 180*c0p1*c1p2*c2p2*c5p1*ss[11];
  root_5 += 180*c0p1*c1p2*c3p1*c4p2*ss[13];
  root_5 += 180*c0p1*c1p2*c3p1*c5p2*ss[15];
  root_5 += 180*c0p1*c1p2*c3p2*c4p1*ss[12];
  root_5 += 180*c0p1*c1p2*c3p2*c5p1*ss[13];
  root_5 += 180*c0p1*c1p2*c4p1*c5p2*ss[16];
  root_5 += 180*c0p1*c1p2*c4p2*c5p1*ss[15];
  root_5 += 120*c0p1*c1p3*c2p1*c3p1*ss[8];
  root_5 += 120*c0p1*c1p3*c2p1*c4p1*ss[9];
  root_5 += 120*c0p1*c1p3*c2p1*c5p1*ss[10];
  root_5 += 120*c0p1*c1p3*c3p1*c4p1*ss[10];
  root_5 += 120*c0p1*c1p3*c3p1*c5p1*ss[11];
  root_5 += 120*c0p1*c1p3*c4p1*c5p1*ss[12];
  root_5 += 120*c0p1*c2p1*c3p1*c4p3*ss[17];
  root_5 += 120*c0p1*c2p1*c3p1*c5p3*ss[20];
  root_5 += 180*c0p1*c2p1*c3p2*c4p2*ss[16];
  root_5 += 180*c0p1*c2p1*c3p2*c5p2*ss[18];
  root_5 += 120*c0p1*c2p1*c3p3*c4p1*ss[15];
  root_5 += 120*c0p1*c2p1*c3p3*c5p1*ss[16];
  root_5 += 120*c0p1*c2p1*c4p1*c5p3*ss[21];
  root_5 += 180*c0p1*c2p1*c4p2*c5p2*ss[20];
  root_5 += 120*c0p1*c2p1*c4p3*c5p1*ss[19];
  root_5 += 180*c0p1*c2p2*c3p1*c4p2*ss[15];
  root_5 += 180*c0p1*c2p2*c3p1*c5p2*ss[17];
  root_5 += 180*c0p1*c2p2*c3p2*c4p1*ss[14];
  root_5 += 180*c0p1*c2p2*c3p2*c5p1*ss[15];
  root_5 += 180*c0p1*c2p2*c4p1*c5p2*ss[18];
  root_5 += 180*c0p1*c2p2*c4p2*c5p1*ss[17];
  root_5 += 120*c0p1*c2p3*c3p1*c4p1*ss[13];
  root_5 += 120*c0p1*c2p3*c3p1*c5p1*ss[14];
  root_5 += 120*c0p1*c2p3*c4p1*c5p1*ss[15];
  root_5 += 120*c0p1*c3p1*c4p1*c5p3*ss[22];
  root_5 += 180*c0p1*c3p1*c4p2*c5p2*ss[21];
  root_5 += 120*c0p1*c3p1*c4p3*c5p1*ss[20];
  root_5 += 180*c0p1*c3p2*c4p1*c5p2*ss[20];
  root_5 += 180*c0p1*c3p2*c4p2*c5p1*ss[19];
  root_5 += 120*c0p1*c3p3*c4p1*c5p1*ss[18];
  root_5 += 180*c0p2*c1p1*c2p1*c3p2*ss[9];
  root_5 += 180*c0p2*c1p1*c2p1*c4p2*ss[11];
  root_5 += 180*c0p2*c1p1*c2p1*c5p2*ss[13];
  root_5 += 180*c0p2*c1p1*c2p2*c3p1*ss[8];
  root_5 += 180*c0p2*c1p1*c2p2*c4p1*ss[9];
  root_5 += 180*c0p2*c1p1*c2p2*c5p1*ss[10];
  root_5 += 180*c0p2*c1p1*c3p1*c4p2*ss[12];
  root_5 += 180*c0p2*c1p1*c3p1*c5p2*ss[14];
  root_5 += 180*c0p2*c1p1*c3p2*c4p1*ss[11];
  root_5 += 180*c0p2*c1p1*c3p2*c5p1*ss[12];
  root_5 += 180*c0p2*c1p1*c4p1*c5p2*ss[15];
  root_5 += 180*c0p2*c1p1*c4p2*c5p1*ss[14];
  root_5 += 180*c0p2*c1p2*c2p1*c3p1*ss[7];
  root_5 += 180*c0p2*c1p2*c2p1*c4p1*ss[8];
  root_5 += 180*c0p2*c1p2*c2p1*c5p1*ss[9];
  root_5 += 180*c0p2*c1p2*c3p1*c4p1*ss[9];
  root_5 += 180*c0p2*c1p2*c3p1*c5p1*ss[10];
  root_5 += 180*c0p2*c1p2*c4p1*c5p1*ss[11];
  root_5 += 180*c0p2*c2p1*c3p1*c4p2*ss[13];
  root_5 += 180*c0p2*c2p1*c3p1*c5p2*ss[15];
  root_5 += 180*c0p2*c2p1*c3p2*c4p1*ss[12];
  root_5 += 180*c0p2*c2p1*c3p2*c5p1*ss[13];
  root_5 += 180*c0p2*c2p1*c4p1*c5p2*ss[16];
  root_5 += 180*c0p2*c2p1*c4p2*c5p1*ss[15];
  root_5 += 180*c0p2*c2p2*c3p1*c4p1*ss[11];
  root_5 += 180*c0p2*c2p2*c3p1*c5p1*ss[12];
  root_5 += 180*c0p2*c2p2*c4p1*c5p1*ss[13];
  root_5 += 180*c0p2*c3p1*c4p1*c5p2*ss[17];
  root_5 += 180*c0p2*c3p1*c4p2*c5p1*ss[16];
  root_5 += 180*c0p2*c3p2*c4p1*c5p1*ss[15];
  root_5 += 120*c0p3*c1p1*c2p1*c3p1*ss[6];
  root_5 += 120*c0p3*c1p1*c2p1*c4p1*ss[7];
  root_5 += 120*c0p3*c1p1*c2p1*c5p1*ss[8];
  root_5 += 120*c0p3*c1p1*c3p1*c4p1*ss[8];
  root_5 += 120*c0p3*c1p1*c3p1*c5p1*ss[9];
  root_5 += 120*c0p3*c1p1*c4p1*c5p1*ss[10];
  root_5 += 120*c0p3*c2p1*c3p1*c4p1*ss[9];
  root_5 += 120*c0p3*c2p1*c3p1*c5p1*ss[10];
  root_5 += 120*c0p3*c2p1*c4p1*c5p1*ss[11];
  root_5 += 120*c0p3*c3p1*c4p1*c5p1*ss[12];
  root_5 += 120*c1p1*c2p1*c3p1*c4p3*ss[18];
  root_5 += 120*c1p1*c2p1*c3p1*c5p3*ss[21];
  root_5 += 180*c1p1*c2p1*c3p2*c4p2*ss[17];
  root_5 += 180*c1p1*c2p1*c3p2*c5p2*ss[19];
  root_5 += 120*c1p1*c2p1*c3p3*c4p1*ss[16];
  root_5 += 120*c1p1*c2p1*c3p3*c5p1*ss[17];
  root_5 += 120*c1p1*c2p1*c4p1*c5p3*ss[22];
  root_5 += 180*c1p1*c2p1*c4p2*c5p2*ss[21];
  root_5 += 120*c1p1*c2p1*c4p3*c5p1*ss[20];
  root_5 += 180*c1p1*c2p2*c3p1*c4p2*ss[16];
  root_5 += 180*c1p1*c2p2*c3p1*c5p2*ss[18];
  root_5 += 180*c1p1*c2p2*c3p2*c4p1*ss[15];
  root_5 += 180*c1p1*c2p2*c3p2*c5p1*ss[16];
  root_5 += 180*c1p1*c2p2*c4p1*c5p2*ss[19];
  root_5 += 180*c1p1*c2p2*c4p2*c5p1*ss[18];
  root_5 += 120*c1p1*c2p3*c3p1*c4p1*ss[14];
  root_5 += 120*c1p1*c2p3*c3p1*c5p1*ss[15];
  root_5 += 120*c1p1*c2p3*c4p1*c5p1*ss[16];
  root_5 += 120*c1p1*c3p1*c4p1*c5p3*ss[23];
  root_5 += 180*c1p1*c3p1*c4p2*c5p2*ss[22];
  root_5 += 120*c1p1*c3p1*c4p3*c5p1*ss[21];
  root_5 += 180*c1p1*c3p2*c4p1*c5p2*ss[21];
  root_5 += 180*c1p1*c3p2*c4p2*c5p1*ss[20];
  root_5 += 120*c1p1*c3p3*c4p1*c5p1*ss[19];
  root_5 += 180*c1p2*c2p1*c3p1*c4p2*ss[15];
  root_5 += 180*c1p2*c2p1*c3p1*c5p2*ss[17];
  root_5 += 180*c1p2*c2p1*c3p2*c4p1*ss[14];
  root_5 += 180*c1p2*c2p1*c3p2*c5p1*ss[15];
  root_5 += 180*c1p2*c2p1*c4p1*c5p2*ss[18];
  root_5 += 180*c1p2*c2p1*c4p2*c5p1*ss[17];
  root_5 += 180*c1p2*c2p2*c3p1*c4p1*ss[13];
  root_5 += 180*c1p2*c2p2*c3p1*c5p1*ss[14];
  root_5 += 180*c1p2*c2p2*c4p1*c5p1*ss[15];
  root_5 += 180*c1p2*c3p1*c4p1*c5p2*ss[19];
  root_5 += 180*c1p2*c3p1*c4p2*c5p1*ss[18];
  root_5 += 180*c1p2*c3p2*c4p1*c5p1*ss[17];
  root_5 += 120*c1p3*c2p1*c3p1*c4p1*ss[12];
  root_5 += 120*c1p3*c2p1*c3p1*c5p1*ss[13];
  root_5 += 120*c1p3*c2p1*c4p1*c5p1*ss[14];
  root_5 += 120*c1p3*c3p1*c4p1*c5p1*ss[15];
  root_5 += 120*c2p1*c3p1*c4p1*c5p3*ss[24];
  root_5 += 180*c2p1*c3p1*c4p2*c5p2*ss[23];
  root_5 += 120*c2p1*c3p1*c4p3*c5p1*ss[22];
  root_5 += 180*c2p1*c3p2*c4p1*c5p2*ss[22];
  root_5 += 180*c2p1*c3p2*c4p2*c5p1*ss[21];
  root_5 += 120*c2p1*c3p3*c4p1*c5p1*ss[20];
  root_5 += 180*c2p2*c3p1*c4p1*c5p2*ss[21];
  root_5 += 180*c2p2*c3p1*c4p2*c5p1*ss[20];
  root_5 += 180*c2p2*c3p2*c4p1*c5p1*ss[19];
  root_5 += 120*c2p3*c3p1*c4p1*c5p1*ss[18];
  root_5 += 30*c0p1*c1p1*c2p4*ss[9];
  root_5 += 30*c0p1*c1p1*c3p4*ss[13];
  root_5 += 30*c0p1*c1p1*c4p4*ss[17];
  root_5 += 30*c0p1*c1p1*c5p4*ss[21];
  root_5 += 60*c0p1*c1p2*c2p3*ss[8];
  root_5 += 60*c0p1*c1p2*c3p3*ss[11];
  root_5 += 60*c0p1*c1p2*c4p3*ss[14];
  root_5 += 60*c0p1*c1p2*c5p3*ss[17];
  root_5 += 60*c0p1*c1p3*c2p2*ss[7];
  root_5 += 60*c0p1*c1p3*c3p2*ss[9];
  root_5 += 60*c0p1*c1p3*c4p2*ss[11];
  root_5 += 60*c0p1*c1p3*c5p2*ss[13];
  root_5 += 30*c0p1*c1p4*c2p1*ss[6];
  root_5 += 30*c0p1*c1p4*c3p1*ss[7];
  root_5 += 30*c0p1*c1p4*c4p1*ss[8];
  root_5 += 30*c0p1*c1p4*c5p1*ss[9];
  root_5 += 30*c0p1*c2p1*c3p4*ss[14];
  root_5 += 30*c0p1*c2p1*c4p4*ss[18];
  root_5 += 30*c0p1*c2p1*c5p4*ss[22];
  root_5 += 60*c0p1*c2p2*c3p3*ss[13];
  root_5 += 60*c0p1*c2p2*c4p3*ss[16];
  root_5 += 60*c0p1*c2p2*c5p3*ss[19];
  root_5 += 60*c0p1*c2p3*c3p2*ss[12];
  root_5 += 60*c0p1*c2p3*c4p2*ss[14];
  root_5 += 60*c0p1*c2p3*c5p2*ss[16];
  root_5 += 30*c0p1*c2p4*c3p1*ss[11];
  root_5 += 30*c0p1*c2p4*c4p1*ss[12];
  root_5 += 30*c0p1*c2p4*c5p1*ss[13];
  root_5 += 30*c0p1*c3p1*c4p4*ss[19];
  root_5 += 30*c0p1*c3p1*c5p4*ss[23];
  root_5 += 60*c0p1*c3p2*c4p3*ss[18];
  root_5 += 60*c0p1*c3p2*c5p3*ss[21];
  root_5 += 60*c0p1*c3p3*c4p2*ss[17];
  root_5 += 60*c0p1*c3p3*c5p2*ss[19];
  root_5 += 30*c0p1*c3p4*c4p1*ss[16];
  root_5 += 30*c0p1*c3p4*c5p1*ss[17];
  root_5 += 30*c0p1*c4p1*c5p4*ss[24];
  root_5 += 60*c0p1*c4p2*c5p3*ss[23];
  root_5 += 60*c0p1*c4p3*c5p2*ss[22];
  root_5 += 30*c0p1*c4p4*c5p1*ss[21];
  root_5 += 60*c0p2*c1p1*c2p3*ss[7];
  root_5 += 60*c0p2*c1p1*c3p3*ss[10];
  root_5 += 60*c0p2*c1p1*c4p3*ss[13];
  root_5 += 60*c0p2*c1p1*c5p3*ss[16];
  root_5 += 90*c0p2*c1p2*c2p2*ss[6];
  root_5 += 90*c0p2*c1p2*c3p2*ss[8];
  root_5 += 90*c0p2*c1p2*c4p2*ss[10];
  root_5 += 90*c0p2*c1p2*c5p2*ss[12];
  root_5 += 60*c0p2*c1p3*c2p1*ss[5];
  root_5 += 60*c0p2*c1p3*c3p1*ss[6];
  root_5 += 60*c0p2*c1p3*c4p1*ss[7];
  root_5 += 60*c0p2*c1p3*c5p1*ss[8];
  root_5 += 60*c0p2*c2p1*c3p3*ss[11];
  root_5 += 60*c0p2*c2p1*c4p3*ss[14];
  root_5 += 60*c0p2*c2p1*c5p3*ss[17];
  root_5 += 90*c0p2*c2p2*c3p2*ss[10];
  root_5 += 90*c0p2*c2p2*c4p2*ss[12];
  root_5 += 90*c0p2*c2p2*c5p2*ss[14];
  root_5 += 60*c0p2*c2p3*c3p1*ss[9];
  root_5 += 60*c0p2*c2p3*c4p1*ss[10];
  root_5 += 60*c0p2*c2p3*c5p1*ss[11];
  root_5 += 60*c0p2*c3p1*c4p3*ss[15];
  root_5 += 60*c0p2*c3p1*c5p3*ss[18];
  root_5 += 90*c0p2*c3p2*c4p2*ss[14];
  root_5 += 90*c0p2*c3p2*c5p2*ss[16];
  root_5 += 60*c0p2*c3p3*c4p1*ss[13];
  root_5 += 60*c0p2*c3p3*c5p1*ss[14];
  root_5 += 60*c0p2*c4p1*c5p3*ss[19];
  root_5 += 90*c0p2*c4p2*c5p2*ss[18];
  root_5 += 60*c0p2*c4p3*c5p1*ss[17];
  root_5 += 60*c0p3*c1p1*c2p2*ss[5];
  root_5 += 60*c0p3*c1p1*c3p2*ss[7];
  root_5 += 60*c0p3*c1p1*c4p2*ss[9];
  root_5 += 60*c0p3*c1p1*c5p2*ss[11];
  root_5 += 60*c0p3*c1p2*c2p1*ss[4];
  root_5 += 60*c0p3*c1p2*c3p1*ss[5];
  root_5 += 60*c0p3*c1p2*c4p1*ss[6];
  root_5 += 60*c0p3*c1p2*c5p1*ss[7];
  root_5 += 60*c0p3*c2p1*c3p2*ss[8];
  root_5 += 60*c0p3*c2p1*c4p2*ss[10];
  root_5 += 60*c0p3*c2p1*c5p2*ss[12];
  root_5 += 60*c0p3*c2p2*c3p1*ss[7];
  root_5 += 60*c0p3*c2p2*c4p1*ss[8];
  root_5 += 60*c0p3*c2p2*c5p1*ss[9];
  root_5 += 60*c0p3*c3p1*c4p2*ss[11];
  root_5 += 60*c0p3*c3p1*c5p2*ss[13];
  root_5 += 60*c0p3*c3p2*c4p1*ss[10];
  root_5 += 60*c0p3*c3p2*c5p1*ss[11];
  root_5 += 60*c0p3*c4p1*c5p2*ss[14];
  root_5 += 60*c0p3*c4p2*c5p1*ss[13];
  root_5 += 30*c0p4*c1p1*c2p1*ss[3];
  root_5 += 30*c0p4*c1p1*c3p1*ss[4];
  root_5 += 30*c0p4*c1p1*c4p1*ss[5];
  root_5 += 30*c0p4*c1p1*c5p1*ss[6];
  root_5 += 30*c0p4*c2p1*c3p1*ss[5];
  root_5 += 30*c0p4*c2p1*c4p1*ss[6];
  root_5 += 30*c0p4*c2p1*c5p1*ss[7];
  root_5 += 30*c0p4*c3p1*c4p1*ss[7];
  root_5 += 30*c0p4*c3p1*c5p1*ss[8];
  root_5 += 30*c0p4*c4p1*c5p1*ss[9];
  root_5 += 30*c1p1*c2p1*c3p4*ss[15];
  root_5 += 30*c1p1*c2p1*c4p4*ss[19];
  root_5 += 30*c1p1*c2p1*c5p4*ss[23];
  root_5 += 60*c1p1*c2p2*c3p3*ss[14];
  root_5 += 60*c1p1*c2p2*c4p3*ss[17];
  root_5 += 60*c1p1*c2p2*c5p3*ss[20];
  root_5 += 60*c1p1*c2p3*c3p2*ss[13];
  root_5 += 60*c1p1*c2p3*c4p2*ss[15];
  root_5 += 60*c1p1*c2p3*c5p2*ss[17];
  root_5 += 30*c1p1*c2p4*c3p1*ss[12];
  root_5 += 30*c1p1*c2p4*c4p1*ss[13];
  root_5 += 30*c1p1*c2p4*c5p1*ss[14];
  root_5 += 30*c1p1*c3p1*c4p4*ss[20];
  root_5 += 30*c1p1*c3p1*c5p4*ss[24];
  root_5 += 60*c1p1*c3p2*c4p3*ss[19];
  root_5 += 60*c1p1*c3p2*c5p3*ss[22];
  root_5 += 60*c1p1*c3p3*c4p2*ss[18];
  root_5 += 60*c1p1*c3p3*c5p2*ss[20];
  root_5 += 30*c1p1*c3p4*c4p1*ss[17];
  root_5 += 30*c1p1*c3p4*c5p1*ss[18];
  root_5 += 30*c1p1*c4p1*c5p4*ss[25];
  root_5 += 60*c1p1*c4p2*c5p3*ss[24];
  root_5 += 60*c1p1*c4p3*c5p2*ss[23];
  root_5 += 30*c1p1*c4p4*c5p1*ss[22];
  root_5 += 60*c1p2*c2p1*c3p3*ss[13];
  root_5 += 60*c1p2*c2p1*c4p3*ss[16];
  root_5 += 60*c1p2*c2p1*c5p3*ss[19];
  root_5 += 90*c1p2*c2p2*c3p2*ss[12];
  root_5 += 90*c1p2*c2p2*c4p2*ss[14];
  root_5 += 90*c1p2*c2p2*c5p2*ss[16];
  root_5 += 60*c1p2*c2p3*c3p1*ss[11];
  root_5 += 60*c1p2*c2p3*c4p1*ss[12];
  root_5 += 60*c1p2*c2p3*c5p1*ss[13];
  root_5 += 60*c1p2*c3p1*c4p3*ss[17];
  root_5 += 60*c1p2*c3p1*c5p3*ss[20];
  root_5 += 90*c1p2*c3p2*c4p2*ss[16];
  root_5 += 90*c1p2*c3p2*c5p2*ss[18];
  root_5 += 60*c1p2*c3p3*c4p1*ss[15];
  root_5 += 60*c1p2*c3p3*c5p1*ss[16];
  root_5 += 60*c1p2*c4p1*c5p3*ss[21];
  root_5 += 90*c1p2*c4p2*c5p2*ss[20];
  root_5 += 60*c1p2*c4p3*c5p1*ss[19];
  root_5 += 60*c1p3*c2p1*c3p2*ss[11];
  root_5 += 60*c1p3*c2p1*c4p2*ss[13];
  root_5 += 60*c1p3*c2p1*c5p2*ss[15];
  root_5 += 60*c1p3*c2p2*c3p1*ss[10];
  root_5 += 60*c1p3*c2p2*c4p1*ss[11];
  root_5 += 60*c1p3*c2p2*c5p1*ss[12];
  root_5 += 60*c1p3*c3p1*c4p2*ss[14];
  root_5 += 60*c1p3*c3p1*c5p2*ss[16];
  root_5 += 60*c1p3*c3p2*c4p1*ss[13];
  root_5 += 60*c1p3*c3p2*c5p1*ss[14];
  root_5 += 60*c1p3*c4p1*c5p2*ss[17];
  root_5 += 60*c1p3*c4p2*c5p1*ss[16];
  root_5 += 30*c1p4*c2p1*c3p1*ss[9];
  root_5 += 30*c1p4*c2p1*c4p1*ss[10];
  root_5 += 30*c1p4*c2p1*c5p1*ss[11];
  root_5 += 30*c1p4*c3p1*c4p1*ss[11];
  root_5 += 30*c1p4*c3p1*c5p1*ss[12];
  root_5 += 30*c1p4*c4p1*c5p1*ss[13];
  root_5 += 30*c2p1*c3p1*c4p4*ss[21];
  root_5 += 30*c2p1*c3p1*c5p4*ss[25];
  root_5 += 60*c2p1*c3p2*c4p3*ss[20];
  root_5 += 60*c2p1*c3p2*c5p3*ss[23];
  root_5 += 60*c2p1*c3p3*c4p2*ss[19];
  root_5 += 60*c2p1*c3p3*c5p2*ss[21];
  root_5 += 30*c2p1*c3p4*c4p1*ss[18];
  root_5 += 30*c2p1*c3p4*c5p1*ss[19];
  root_5 += 30*c2p1*c4p1*c5p4*ss[26];
  root_5 += 60*c2p1*c4p2*c5p3*ss[25];
  root_5 += 60*c2p1*c4p3*c5p2*ss[24];
  root_5 += 30*c2p1*c4p4*c5p1*ss[23];
  root_5 += 60*c2p2*c3p1*c4p3*ss[19];
  root_5 += 60*c2p2*c3p1*c5p3*ss[22];
  root_5 += 90*c2p2*c3p2*c4p2*ss[18];
  root_5 += 90*c2p2*c3p2*c5p2*ss[20];
  root_5 += 60*c2p2*c3p3*c4p1*ss[17];
  root_5 += 60*c2p2*c3p3*c5p1*ss[18];
  root_5 += 60*c2p2*c4p1*c5p3*ss[23];
  root_5 += 90*c2p2*c4p2*c5p2*ss[22];
  root_5 += 60*c2p2*c4p3*c5p1*ss[21];
  root_5 += 60*c2p3*c3p1*c4p2*ss[17];
  root_5 += 60*c2p3*c3p1*c5p2*ss[19];
  root_5 += 60*c2p3*c3p2*c4p1*ss[16];
  root_5 += 60*c2p3*c3p2*c5p1*ss[17];
  root_5 += 60*c2p3*c4p1*c5p2*ss[20];
  root_5 += 60*c2p3*c4p2*c5p1*ss[19];
  root_5 += 30*c2p4*c3p1*c4p1*ss[15];
  root_5 += 30*c2p4*c3p1*c5p1*ss[16];
  root_5 += 30*c2p4*c4p1*c5p1*ss[17];
  root_5 += 30*c3p1*c4p1*c5p4*ss[27];
  root_5 += 60*c3p1*c4p2*c5p3*ss[26];
  root_5 += 60*c3p1*c4p3*c5p2*ss[25];
  root_5 += 30*c3p1*c4p4*c5p1*ss[24];
  root_5 += 60*c3p2*c4p1*c5p3*ss[25];
  root_5 += 90*c3p2*c4p2*c5p2*ss[24];
  root_5 += 60*c3p2*c4p3*c5p1*ss[23];
  root_5 += 60*c3p3*c4p1*c5p2*ss[23];
  root_5 += 60*c3p3*c4p2*c5p1*ss[22];
  root_5 += 30*c3p4*c4p1*c5p1*ss[21];
  root_5 += 6*c0p1*c1p5*ss[5];
  root_5 += 6*c0p1*c2p5*ss[10];
  root_5 += 6*c0p1*c3p5*ss[15];
  root_5 += 6*c0p1*c4p5*ss[20];
  root_5 += 6*c0p1*c5p5*ss[25];
  root_5 += 15*c0p2*c1p4*ss[4];
  root_5 += 15*c0p2*c2p4*ss[8];
  root_5 += 15*c0p2*c3p4*ss[12];
  root_5 += 15*c0p2*c4p4*ss[16];
  root_5 += 15*c0p2*c5p4*ss[20];
  root_5 += 20*c0p3*c1p3*ss[3];
  root_5 += 20*c0p3*c2p3*ss[6];
  root_5 += 20*c0p3*c3p3*ss[9];
  root_5 += 20*c0p3*c4p3*ss[12];
  root_5 += 20*c0p3*c5p3*ss[15];
  root_5 += 15*c0p4*c1p2*ss[2];
  root_5 += 15*c0p4*c2p2*ss[4];
  root_5 += 15*c0p4*c3p2*ss[6];
  root_5 += 15*c0p4*c4p2*ss[8];
  root_5 += 15*c0p4*c5p2*ss[10];
  root_5 += 6*c0p5*c1p1*ss[1];
  root_5 += 6*c0p5*c2p1*ss[2];
  root_5 += 6*c0p5*c3p1*ss[3];
  root_5 += 6*c0p5*c4p1*ss[4];
  root_5 += 6*c0p5*c5p1*ss[5];
  root_5 += 6*c1p1*c2p5*ss[11];
  root_5 += 6*c1p1*c3p5*ss[16];
  root_5 += 6*c1p1*c4p5*ss[21];
  root_5 += 6*c1p1*c5p5*ss[26];
  root_5 += 15*c1p2*c2p4*ss[10];
  root_5 += 15*c1p2*c3p4*ss[14];
  root_5 += 15*c1p2*c4p4*ss[18];
  root_5 += 15*c1p2*c5p4*ss[22];
  root_5 += 20*c1p3*c2p3*ss[9];
  root_5 += 20*c1p3*c3p3*ss[12];
  root_5 += 20*c1p3*c4p3*ss[15];
  root_5 += 20*c1p3*c5p3*ss[18];
  root_5 += 15*c1p4*c2p2*ss[8];
  root_5 += 15*c1p4*c3p2*ss[10];
  root_5 += 15*c1p4*c4p2*ss[12];
  root_5 += 15*c1p4*c5p2*ss[14];
  root_5 += 6*c1p5*c2p1*ss[7];
  root_5 += 6*c1p5*c3p1*ss[8];
  root_5 += 6*c1p5*c4p1*ss[9];
  root_5 += 6*c1p5*c5p1*ss[10];
  root_5 += 6*c2p1*c3p5*ss[17];
  root_5 += 6*c2p1*c4p5*ss[22];
  root_5 += 6*c2p1*c5p5*ss[27];
  root_5 += 15*c2p2*c3p4*ss[16];
  root_5 += 15*c2p2*c4p4*ss[20];
  root_5 += 15*c2p2*c5p4*ss[24];
  root_5 += 20*c2p3*c3p3*ss[15];
  root_5 += 20*c2p3*c4p3*ss[18];
  root_5 += 20*c2p3*c5p3*ss[21];
  root_5 += 15*c2p4*c3p2*ss[14];
  root_5 += 15*c2p4*c4p2*ss[16];
  root_5 += 15*c2p4*c5p2*ss[18];
  root_5 += 6*c2p5*c3p1*ss[13];
  root_5 += 6*c2p5*c4p1*ss[14];
  root_5 += 6*c2p5*c5p1*ss[15];
  root_5 += 6*c3p1*c4p5*ss[23];
  root_5 += 6*c3p1*c5p5*ss[28];
  root_5 += 15*c3p2*c4p4*ss[22];
  root_5 += 15*c3p2*c5p4*ss[26];
  root_5 += 20*c3p3*c4p3*ss[21];
  root_5 += 20*c3p3*c5p3*ss[24];
  root_5 += 15*c3p4*c4p2*ss[20];
  root_5 += 15*c3p4*c5p2*ss[22];
  root_5 += 6*c3p5*c4p1*ss[19];
  root_5 += 6*c3p5*c5p1*ss[20];
  root_5 += 6*c4p1*c5p5*ss[29];
  root_5 += 15*c4p2*c5p4*ss[28];
  root_5 += 20*c4p3*c5p3*ss[27];
  root_5 += 15*c4p4*c5p2*ss[26];
  root_5 += 6*c4p5*c5p1*ss[25];
  root_5 += c1p6*ss[6];
  root_5 += c2p6*ss[12];
  root_5 += c3p6*ss[18];
  root_5 += c4p6*ss[24];
  root_5 += c5p6*ss[30];
  root_5 += c0p6;
  root_5 += -tmom[6];
  return convert<T>(sqrt(root_0*root_0+root_1*root_1+root_2*root_2+root_3*root_3+root_4*root_4+root_5*root_5));
}
T metric_and_gradient(const T *parameters, T *gradient) const {
  T c0p1 = parameters[0];
  T c1p1 = parameters[1];
  T c2p1 = parameters[2];
  T c3p1 = parameters[3];
  T c4p1 = parameters[4];
  T c5p1 = parameters[5];
  TACC sys_part_0 = convert<TACC>(c1p1*ss[1]);
  sys_part_0 += c2p1*ss[2];
  sys_part_0 += c3p1*ss[3];
  sys_part_0 += c4p1*ss[4];
  sys_part_0 += c5p1*ss[5];
  sys_part_0 += c0p1;
  sys_part_0 += -tmom[1];
  T c0p2 = c0p1*c0p1;
  T c1p2 = c1p1*c1p1;
  T c2p2 = c2p1*c2p1;
  T c3p2 = c3p1*c3p1;
  T c4p2 = c4p1*c4p1;
  T c5p2 = c5p1*c5p1;
  TACC sys_part_1 = convert<TACC>(2*c0p1*c1p1*ss[1]);
  sys_part_1 += 2*c0p1*c2p1*ss[2];
  sys_part_1 += 2*c0p1*c3p1*ss[3];
  sys_part_1 += 2*c0p1*c4p1*ss[4];
  sys_part_1 += 2*c0p1*c5p1*ss[5];
  sys_part_1 += 2*c1p1*c2p1*ss[3];
  sys_part_1 += 2*c1p1*c3p1*ss[4];
  sys_part_1 += 2*c1p1*c4p1*ss[5];
  sys_part_1 += 2*c1p1*c5p1*ss[6];
  sys_part_1 += 2*c2p1*c3p1*ss[5];
  sys_part_1 += 2*c2p1*c4p1*ss[6];
  sys_part_1 += 2*c2p1*c5p1*ss[7];
  sys_part_1 += 2*c3p1*c4p1*ss[7];
  sys_part_1 += 2*c3p1*c5p1*ss[8];
  sys_part_1 += 2*c4p1*c5p1*ss[9];
  sys_part_1 += c1p2*ss[2];
  sys_part_1 += c2p2*ss[4];
  sys_part_1 += c3p2*ss[6];
  sys_part_1 += c4p2*ss[8];
  sys_part_1 += c5p2*ss[10];
  sys_part_1 += c0p2;
  sys_part_1 += -tmom[2];
  T c0p3 = c0p2*c0p1;
  T c1p3 = c1p2*c1p1;
  T c2p3 = c2p2*c2p1;
  T c3p3 = c3p2*c3p1;
  T c4p3 = c4p2*c4p1;
  T c5p3 = c5p2*c5p1;
  TACC sys_part_2 = convert<TACC>(6*c0p1*c1p1*c2p1*ss[3]);
  sys_part_2 += 6*c0p1*c1p1*c3p1*ss[4];
  sys_part_2 += 6*c0p1*c1p1*c4p1*ss[5];
  sys_part_2 += 6*c0p1*c1p1*c5p1*ss[6];
  sys_part_2 += 6*c0p1*c2p1*c3p1*ss[5];
  sys_part_2 += 6*c0p1*c2p1*c4p1*ss[6];
  sys_part_2 += 6*c0p1*c2p1*c5p1*ss[7];
  sys_part_2 += 6*c0p1*c3p1*c4p1*ss[7];
  sys_part_2 += 6*c0p1*c3p1*c5p1*ss[8];
  sys_part_2 += 6*c0p1*c4p1*c5p1*ss[9];
  sys_part_2 += 6*c1p1*c2p1*c3p1*ss[6];
  sys_part_2 += 6*c1p1*c2p1*c4p1*ss[7];
  sys_part_2 += 6*c1p1*c2p1*c5p1*ss[8];
  sys_part_2 += 6*c1p1*c3p1*c4p1*ss[8];
  sys_part_2 += 6*c1p1*c3p1*c5p1*ss[9];
  sys_part_2 += 6*c1p1*c4p1*c5p1*ss[10];
  sys_part_2 += 6*c2p1*c3p1*c4p1*ss[9];
  sys_part_2 += 6*c2p1*c3p1*c5p1*ss[10];
  sys_part_2 += 6*c2p1*c4p1*c5p1*ss[11];
  sys_part_2 += 6*c3p1*c4p1*c5p1*ss[12];
  sys_part_2 += 3*c0p1*c1p2*ss[2];
  sys_part_2 += 3*c0p1*c2p2*ss[4];
  sys_part_2 += 3*c0p1*c3p2*ss[6];
  sys_part_2 += 3*c0p1*c4p2*ss[8];
  sys_part_2 += 3*c0p1*c5p2*ss[10];
  sys_part_2 += 3*c0p2*c1p1*ss[1];
  sys_part_2 += 3*c0p2*c2p1*ss[2];
  sys_part_2 += 3*c0p2*c3p1*ss[3];
  sys_part_2 += 3*c0p2*c4p1*ss[4];
  sys_part_2 += 3*c0p2*c5p1*ss[5];
  sys_part_2 += 3*c1p1*c2p2*ss[5];
  sys_part_2 += 3*c1p1*c3p2*ss[7];
  sys_part_2 += 3*c1p1*c4p2*ss[9];
  sys_part_2 += 3*c1p1*c5p2*ss[11];
  sys_part_2 += 3*c1p2*c2p1*ss[4];
  sys_part_2 += 3*c1p2*c3p1*ss[5];
  sys_part_2 += 3*c1p2*c4p1*ss[6];
  sys_part_2 += 3*c1p2*c5p1*ss[7];
  sys_part_2 += 3*c2p1*c3p2*ss[8];
  sys_part_2 += 3*c2p1*c4p2*ss[10];
  sys_part_2 += 3*c2p1*c5p2*ss[12];
  sys_part_2 += 3*c2p2*c3p1*ss[7];
  sys_part_2 += 3*c2p2*c4p1*ss[8];
  sys_part_2 += 3*c2p2*c5p1*ss[9];
  sys_part_2 += 3*c3p1*c4p2*ss[11];
  sys_part_2 += 3*c3p1*c5p2*ss[13];
  sys_part_2 += 3*c3p2*c4p1*ss[10];
  sys_part_2 += 3*c3p2*c5p1*ss[11];
  sys_part_2 += 3*c4p1*c5p2*ss[14];
  sys_part_2 += 3*c4p2*c5p1*ss[13];
  sys_part_2 += c1p3*ss[3];
  sys_part_2 += c2p3*ss[6];
  sys_part_2 += c3p3*ss[9];
  sys_part_2 += c4p3*ss[12];
  sys_part_2 += c5p3*ss[15];
  sys_part_2 += c0p3;
  sys_part_2 += -tmom[3];
  T c0p4 = c0p3*c0p1;
  T c1p4 = c1p3*c1p1;
  T c2p4 = c2p3*c2p1;
  T c3p4 = c3p3*c3p1;
  T c4p4 = c4p3*c4p1;
  T c5p4 = c5p3*c5p1;
  TACC sys_part_3 = convert<TACC>(24*c0p1*c1p1*c2p1*c3p1*ss[6]);
  sys_part_3 += 24*c0p1*c1p1*c2p1*c4p1*ss[7];
  sys_part_3 += 24*c0p1*c1p1*c2p1*c5p1*ss[8];
  sys_part_3 += 24*c0p1*c1p1*c3p1*c4p1*ss[8];
  sys_part_3 += 24*c0p1*c1p1*c3p1*c5p1*ss[9];
  sys_part_3 += 24*c0p1*c1p1*c4p1*c5p1*ss[10];
  sys_part_3 += 24*c0p1*c2p1*c3p1*c4p1*ss[9];
  sys_part_3 += 24*c0p1*c2p1*c3p1*c5p1*ss[10];
  sys_part_3 += 24*c0p1*c2p1*c4p1*c5p1*ss[11];
  sys_part_3 += 24*c0p1*c3p1*c4p1*c5p1*ss[12];
  sys_part_3 += 24*c1p1*c2p1*c3p1*c4p1*ss[10];
  sys_part_3 += 24*c1p1*c2p1*c3p1*c5p1*ss[11];
  sys_part_3 += 24*c1p1*c2p1*c4p1*c5p1*ss[12];
  sys_part_3 += 24*c1p1*c3p1*c4p1*c5p1*ss[13];
  sys_part_3 += 24*c2p1*c3p1*c4p1*c5p1*ss[14];
  sys_part_3 += 12*c0p1*c1p1*c2p2*ss[5];
  sys_part_3 += 12*c0p1*c1p1*c3p2*ss[7];
  sys_part_3 += 12*c0p1*c1p1*c4p2*ss[9];
  sys_part_3 += 12*c0p1*c1p1*c5p2*ss[11];
  sys_part_3 += 12*c0p1*c1p2*c2p1*ss[4];
  sys_part_3 += 12*c0p1*c1p2*c3p1*ss[5];
  sys_part_3 += 12*c0p1*c1p2*c4p1*ss[6];
  sys_part_3 += 12*c0p1*c1p2*c5p1*ss[7];
  sys_part_3 += 12*c0p1*c2p1*c3p2*ss[8];
  sys_part_3 += 12*c0p1*c2p1*c4p2*ss[10];
  sys_part_3 += 12*c0p1*c2p1*c5p2*ss[12];
  sys_part_3 += 12*c0p1*c2p2*c3p1*ss[7];
  sys_part_3 += 12*c0p1*c2p2*c4p1*ss[8];
  sys_part_3 += 12*c0p1*c2p2*c5p1*ss[9];
  sys_part_3 += 12*c0p1*c3p1*c4p2*ss[11];
  sys_part_3 += 12*c0p1*c3p1*c5p2*ss[13];
  sys_part_3 += 12*c0p1*c3p2*c4p1*ss[10];
  sys_part_3 += 12*c0p1*c3p2*c5p1*ss[11];
  sys_part_3 += 12*c0p1*c4p1*c5p2*ss[14];
  sys_part_3 += 12*c0p1*c4p2*c5p1*ss[13];
  sys_part_3 += 12*c0p2*c1p1*c2p1*ss[3];
  sys_part_3 += 12*c0p2*c1p1*c3p1*ss[4];
  sys_part_3 += 12*c0p2*c1p1*c4p1*ss[5];
  sys_part_3 += 12*c0p2*c1p1*c5p1*ss[6];
  sys_part_3 += 12*c0p2*c2p1*c3p1*ss[5];
  sys_part_3 += 12*c0p2*c2p1*c4p1*ss[6];
  sys_part_3 += 12*c0p2*c2p1*c5p1*ss[7];
  sys_part_3 += 12*c0p2*c3p1*c4p1*ss[7];
  sys_part_3 += 12*c0p2*c3p1*c5p1*ss[8];
  sys_part_3 += 12*c0p2*c4p1*c5p1*ss[9];
  sys_part_3 += 12*c1p1*c2p1*c3p2*ss[9];
  sys_part_3 += 12*c1p1*c2p1*c4p2*ss[11];
  sys_part_3 += 12*c1p1*c2p1*c5p2*ss[13];
  sys_part_3 += 12*c1p1*c2p2*c3p1*ss[8];
  sys_part_3 += 12*c1p1*c2p2*c4p1*ss[9];
  sys_part_3 += 12*c1p1*c2p2*c5p1*ss[10];
  sys_part_3 += 12*c1p1*c3p1*c4p2*ss[12];
  sys_part_3 += 12*c1p1*c3p1*c5p2*ss[14];
  sys_part_3 += 12*c1p1*c3p2*c4p1*ss[11];
  sys_part_3 += 12*c1p1*c3p2*c5p1*ss[12];
  sys_part_3 += 12*c1p1*c4p1*c5p2*ss[15];
  sys_part_3 += 12*c1p1*c4p2*c5p1*ss[14];
  sys_part_3 += 12*c1p2*c2p1*c3p1*ss[7];
  sys_part_3 += 12*c1p2*c2p1*c4p1*ss[8];
  sys_part_3 += 12*c1p2*c2p1*c5p1*ss[9];
  sys_part_3 += 12*c1p2*c3p1*c4p1*ss[9];
  sys_part_3 += 12*c1p2*c3p1*c5p1*ss[10];
  sys_part_3 += 12*c1p2*c4p1*c5p1*ss[11];
  sys_part_3 += 12*c2p1*c3p1*c4p2*ss[13];
  sys_part_3 += 12*c2p1*c3p1*c5p2*ss[15];
  sys_part_3 += 12*c2p1*c3p2*c4p1*ss[12];
  sys_part_3 += 12*c2p1*c3p2*c5p1*ss[13];
  sys_part_3 += 12*c2p1*c4p1*c5p2*ss[16];
  sys_part_3 += 12*c2p1*c4p2*c5p1*ss[15];
  sys_part_3 += 12*c2p2*c3p1*c4p1*ss[11];
  sys_part_3 += 12*c2p2*c3p1*c5p1*ss[12];
  sys_part_3 += 12*c2p2*c4p1*c5p1*ss[13];
  sys_part_3 += 12*c3p1*c4p1*c5p2*ss[17];
  sys_part_3 += 12*c3p1*c4p2*c5p1*ss[16];
  sys_part_3 += 12*c3p2*c4p1*c5p1*ss[15];
  sys_part_3 += 4*c0p1*c1p3*ss[3];
  sys_part_3 += 4*c0p1*c2p3*ss[6];
  sys_part_3 += 4*c0p1*c3p3*ss[9];
  sys_part_3 += 4*c0p1*c4p3*ss[12];
  sys_part_3 += 4*c0p1*c5p3*ss[15];
  sys_part_3 += 6*c0p2*c1p2*ss[2];
  sys_part_3 += 6*c0p2*c2p2*ss[4];
  sys_part_3 += 6*c0p2*c3p2*ss[6];
  sys_part_3 += 6*c0p2*c4p2*ss[8];
  sys_part_3 += 6*c0p2*c5p2*ss[10];
  sys_part_3 += 4*c0p3*c1p1*ss[1];
  sys_part_3 += 4*c0p3*c2p1*ss[2];
  sys_part_3 += 4*c0p3*c3p1*ss[3];
  sys_part_3 += 4*c0p3*c4p1*ss[4];
  sys_part_3 += 4*c0p3*c5p1*ss[5];
  sys_part_3 += 4*c1p1*c2p3*ss[7];
  sys_part_3 += 4*c1p1*c3p3*ss[10];
  sys_part_3 += 4*c1p1*c4p3*ss[13];
  sys_part_3 += 4*c1p1*c5p3*ss[16];
  sys_part_3 += 6*c1p2*c2p2*ss[6];
  sys_part_3 += 6*c1p2*c3p2*ss[8];
  sys_part_3 += 6*c1p2*c4p2*ss[10];
  sys_part_3 += 6*c1p2*c5p2*ss[12];
  sys_part_3 += 4*c1p3*c2p1*ss[5];
  sys_part_3 += 4*c1p3*c3p1*ss[6];
  sys_part_3 += 4*c1p3*c4p1*ss[7];
  sys_part_3 += 4*c1p3*c5p1*ss[8];
  sys_part_3 += 4*c2p1*c3p3*ss[11];
  sys_part_3 += 4*c2p1*c4p3*ss[14];
  sys_part_3 += 4*c2p1*c5p3*ss[17];
  sys_part_3 += 6*c2p2*c3p2*ss[10];
  sys_part_3 += 6*c2p2*c4p2*ss[12];
  sys_part_3 += 6*c2p2*c5p2*ss[14];
  sys_part_3 += 4*c2p3*c3p1*ss[9];
  sys_part_3 += 4*c2p3*c4p1*ss[10];
  sys_part_3 += 4*c2p3*c5p1*ss[11];
  sys_part_3 += 4*c3p1*c4p3*ss[15];
  sys_part_3 += 4*c3p1*c5p3*ss[18];
  sys_part_3 += 6*c3p2*c4p2*ss[14];
  sys_part_3 += 6*c3p2*c5p2*ss[16];
  sys_part_3 += 4*c3p3*c4p1*ss[13];
  sys_part_3 += 4*c3p3*c5p1*ss[14];
  sys_part_3 += 4*c4p1*c5p3*ss[19];
  sys_part_3 += 6*c4p2*c5p2*ss[18];
  sys_part_3 += 4*c4p3*c5p1*ss[17];
  sys_part_3 += c1p4*ss[4];
  sys_part_3 += c2p4*ss[8];
  sys_part_3 += c3p4*ss[12];
  sys_part_3 += c4p4*ss[16];
  sys_part_3 += c5p4*ss[20];
  sys_part_3 += c0p4;
  sys_part_3 += -tmom[4];
  T c0p5 = c0p4*c0p1;
  T c1p5 = c1p4*c1p1;
  T c2p5 = c2p4*c2p1;
  T c3p5 = c3p4*c3p1;
  T c4p5 = c4p4*c4p1;
  T c5p5 = c5p4*c5p1;
  TACC sys_part_4 = convert<TACC>(120*c0p1*c1p1*c2p1*c3p1*c4p1*ss[10]);
  sys_part_4 += 120*c0p1*c1p1*c2p1*c3p1*c5p1*ss[11];
  sys_part_4 += 120*c0p1*c1p1*c2p1*c4p1*c5p1*ss[12];
  sys_part_4 += 120*c0p1*c1p1*c3p1*c4p1*c5p1*ss[13];
  sys_part_4 += 120*c0p1*c2p1*c3p1*c4p1*c5p1*ss[14];
  sys_part_4 += 120*c1p1*c2p1*c3p1*c4p1*c5p1*ss[15];
  sys_part_4 += 60*c0p1*c1p1*c2p1*c3p2*ss[9];
  sys_part_4 += 60*c0p1*c1p1*c2p1*c4p2*ss[11];
  sys_part_4 += 60*c0p1*c1p1*c2p1*c5p2*ss[13];
  sys_part_4 += 60*c0p1*c1p1*c2p2*c3p1*ss[8];
  sys_part_4 += 60*c0p1*c1p1*c2p2*c4p1*ss[9];
  sys_part_4 += 60*c0p1*c1p1*c2p2*c5p1*ss[10];
  sys_part_4 += 60*c0p1*c1p1*c3p1*c4p2*ss[12];
  sys_part_4 += 60*c0p1*c1p1*c3p1*c5p2*ss[14];
  sys_part_4 += 60*c0p1*c1p1*c3p2*c4p1*ss[11];
  sys_part_4 += 60*c0p1*c1p1*c3p2*c5p1*ss[12];
  sys_part_4 += 60*c0p1*c1p1*c4p1*c5p2*ss[15];
  sys_part_4 += 60*c0p1*c1p1*c4p2*c5p1*ss[14];
  sys_part_4 += 60*c0p1*c1p2*c2p1*c3p1*ss[7];
  sys_part_4 += 60*c0p1*c1p2*c2p1*c4p1*ss[8];
  sys_part_4 += 60*c0p1*c1p2*c2p1*c5p1*ss[9];
  sys_part_4 += 60*c0p1*c1p2*c3p1*c4p1*ss[9];
  sys_part_4 += 60*c0p1*c1p2*c3p1*c5p1*ss[10];
  sys_part_4 += 60*c0p1*c1p2*c4p1*c5p1*ss[11];
  sys_part_4 += 60*c0p1*c2p1*c3p1*c4p2*ss[13];
  sys_part_4 += 60*c0p1*c2p1*c3p1*c5p2*ss[15];
  sys_part_4 += 60*c0p1*c2p1*c3p2*c4p1*ss[12];
  sys_part_4 += 60*c0p1*c2p1*c3p2*c5p1*ss[13];
  sys_part_4 += 60*c0p1*c2p1*c4p1*c5p2*ss[16];
  sys_part_4 += 60*c0p1*c2p1*c4p2*c5p1*ss[15];
  sys_part_4 += 60*c0p1*c2p2*c3p1*c4p1*ss[11];
  sys_part_4 += 60*c0p1*c2p2*c3p1*c5p1*ss[12];
  sys_part_4 += 60*c0p1*c2p2*c4p1*c5p1*ss[13];
  sys_part_4 += 60*c0p1*c3p1*c4p1*c5p2*ss[17];
  sys_part_4 += 60*c0p1*c3p1*c4p2*c5p1*ss[16];
  sys_part_4 += 60*c0p1*c3p2*c4p1*c5p1*ss[15];
  sys_part_4 += 60*c0p2*c1p1*c2p1*c3p1*ss[6];
  sys_part_4 += 60*c0p2*c1p1*c2p1*c4p1*ss[7];
  sys_part_4 += 60*c0p2*c1p1*c2p1*c5p1*ss[8];
  sys_part_4 += 60*c0p2*c1p1*c3p1*c4p1*ss[8];
  sys_part_4 += 60*c0p2*c1p1*c3p1*c5p1*ss[9];
  sys_part_4 += 60*c0p2*c1p1*c4p1*c5p1*ss[10];
  sys_part_4 += 60*c0p2*c2p1*c3p1*c4p1*ss[9];
  sys_part_4 += 60*c0p2*c2p1*c3p1*c5p1*ss[10];
  sys_part_4 += 60*c0p2*c2p1*c4p1*c5p1*ss[11];
  sys_part_4 += 60*c0p2*c3p1*c4p1*c5p1*ss[12];
  sys_part_4 += 60*c1p1*c2p1*c3p1*c4p2*ss[14];
  sys_part_4 += 60*c1p1*c2p1*c3p1*c5p2*ss[16];
  sys_part_4 += 60*c1p1*c2p1*c3p2*c4p1*ss[13];
  sys_part_4 += 60*c1p1*c2p1*c3p2*c5p1*ss[14];
  sys_part_4 += 60*c1p1*c2p1*c4p1*c5p2*ss[17];
  sys_part_4 += 60*c1p1*c2p1*c4p2*c5p1*ss[16];
  sys_part_4 += 60*c1p1*c2p2*c3p1*c4p1*ss[12];
  sys_part_4 += 60*c1p1*c2p2*c3p1*c5p1*ss[13];
  sys_part_4 += 60*c1p1*c2p2*c4p1*c5p1*ss[14];
  sys_part_4 += 60*c1p1*c3p1*c4p1*c5p2*ss[18];
  sys_part_4 += 60*c1p1*c3p1*c4p2*c5p1*ss[17];
  sys_part_4 += 60*c1p1*c3p2*c4p1*c5p1*ss[16];
  sys_part_4 += 60*c1p2*c2p1*c3p1*c4p1*ss[11];
  sys_part_4 += 60*c1p2*c2p1*c3p1*c5p1*ss[12];
  sys_part_4 += 60*c1p2*c2p1*c4p1*c5p1*ss[13];
  sys_part_4 += 60*c1p2*c3p1*c4p1*c5p1*ss[14];
  sys_part_4 += 60*c2p1*c3p1*c4p1*c5p2*ss[19];
  sys_part_4 += 60*c2p1*c3p1*c4p2*c5p1*ss[18];
  sys_part_4 += 60*c2p1*c3p2*c4p1*c5p1*ss[17];
  sys_part_4 += 60*c2p2*c3p1*c4p1*c5p1*ss[16];
  sys_part_4 += 20*c0p1*c1p1*c2p3*ss[7];
  sys_part_4 += 20*c0p1*c1p1*c3p3*ss[10];
  sys_part_4 += 20*c0p1*c1p1*c4p3*ss[13];
  sys_part_4 += 20*c0p1*c1p1*c5p3*ss[16];
  sys_part_4 += 30*c0p1*c1p2*c2p2*ss[6];
  sys_part_4 += 30*c0p1*c1p2*c3p2*ss[8];
  sys_part_4 += 30*c0p1*c1p2*c4p2*ss[10];
  sys_part_4 += 30*c0p1*c1p2*c5p2*ss[12];
  sys_part_4 += 20*c0p1*c1p3*c2p1*ss[5];
  sys_part_4 += 20*c0p1*c1p3*c3p1*ss[6];
  sys_part_4 += 20*c0p1*c1p3*c4p1*ss[7];
  sys_part_4 += 20*c0p1*c1p3*c5p1*ss[8];
  sys_part_4 += 20*c0p1*c2p1*c3p3*ss[11];
  sys_part_4 += 20*c0p1*c2p1*c4p3*ss[14];
  sys_part_4 += 20*c0p1*c2p1*c5p3*ss[17];
  sys_part_4 += 30*c0p1*c2p2*c3p2*ss[10];
  sys_part_4 += 30*c0p1*c2p2*c4p2*ss[12];
  sys_part_4 += 30*c0p1*c2p2*c5p2*ss[14];
  sys_part_4 += 20*c0p1*c2p3*c3p1*ss[9];
  sys_part_4 += 20*c0p1*c2p3*c4p1*ss[10];
  sys_part_4 += 20*c0p1*c2p3*c5p1*ss[11];
  sys_part_4 += 20*c0p1*c3p1*c4p3*ss[15];
  sys_part_4 += 20*c0p1*c3p1*c5p3*ss[18];
  sys_part_4 += 30*c0p1*c3p2*c4p2*ss[14];
  sys_part_4 += 30*c0p1*c3p2*c5p2*ss[16];
  sys_part_4 += 20*c0p1*c3p3*c4p1*ss[13];
  sys_part_4 += 20*c0p1*c3p3*c5p1*ss[14];
  sys_part_4 += 20*c0p1*c4p1*c5p3*ss[19];
  sys_part_4 += 30*c0p1*c4p2*c5p2*ss[18];
  sys_part_4 += 20*c0p1*c4p3*c5p1*ss[17];
  sys_part_4 += 30*c0p2*c1p1*c2p2*ss[5];
  sys_part_4 += 30*c0p2*c1p1*c3p2*ss[7];
  sys_part_4 += 30*c0p2*c1p1*c4p2*ss[9];
  sys_part_4 += 30*c0p2*c1p1*c5p2*ss[11];
  sys_part_4 += 30*c0p2*c1p2*c2p1*ss[4];
  sys_part_4 += 30*c0p2*c1p2*c3p1*ss[5];
  sys_part_4 += 30*c0p2*c1p2*c4p1*ss[6];
  sys_part_4 += 30*c0p2*c1p2*c5p1*ss[7];
  sys_part_4 += 30*c0p2*c2p1*c3p2*ss[8];
  sys_part_4 += 30*c0p2*c2p1*c4p2*ss[10];
  sys_part_4 += 30*c0p2*c2p1*c5p2*ss[12];
  sys_part_4 += 30*c0p2*c2p2*c3p1*ss[7];
  sys_part_4 += 30*c0p2*c2p2*c4p1*ss[8];
  sys_part_4 += 30*c0p2*c2p2*c5p1*ss[9];
  sys_part_4 += 30*c0p2*c3p1*c4p2*ss[11];
  sys_part_4 += 30*c0p2*c3p1*c5p2*ss[13];
  sys_part_4 += 30*c0p2*c3p2*c4p1*ss[10];
  sys_part_4 += 30*c0p2*c3p2*c5p1*ss[11];
  sys_part_4 += 30*c0p2*c4p1*c5p2*ss[14];
  sys_part_4 += 30*c0p2*c4p2*c5p1*ss[13];
  sys_part_4 += 20*c0p3*c1p1*c2p1*ss[3];
  sys_part_4 += 20*c0p3*c1p1*c3p1*ss[4];
  sys_part_4 += 20*c0p3*c1p1*c4p1*ss[5];
  sys_part_4 += 20*c0p3*c1p1*c5p1*ss[6];
  sys_part_4 += 20*c0p3*c2p1*c3p1*ss[5];
  sys_part_4 += 20*c0p3*c2p1*c4p1*ss[6];
  sys_part_4 += 20*c0p3*c2p1*c5p1*ss[7];
  sys_part_4 += 20*c0p3*c3p1*c4p1*ss[7];
  sys_part_4 += 20*c0p3*c3p1*c5p1*ss[8];
  sys_part_4 += 20*c0p3*c4p1*c5p1*ss[9];
  sys_part_4 += 20*c1p1*c2p1*c3p3*ss[12];
  sys_part_4 += 20*c1p1*c2p1*c4p3*ss[15];
  sys_part_4 += 20*c1p1*c2p1*c5p3*ss[18];
  sys_part_4 += 30*c1p1*c2p2*c3p2*ss[11];
  sys_part_4 += 30*c1p1*c2p2*c4p2*ss[13];
  sys_part_4 += 30*c1p1*c2p2*c5p2*ss[15];
  sys_part_4 += 20*c1p1*c2p3*c3p1*ss[10];
  sys_part_4 += 20*c1p1*c2p3*c4p1*ss[11];
  sys_part_4 += 20*c1p1*c2p3*c5p1*ss[12];
  sys_part_4 += 20*c1p1*c3p1*c4p3*ss[16];
  sys_part_4 += 20*c1p1*c3p1*c5p3*ss[19];
  sys_part_4 += 30*c1p1*c3p2*c4p2*ss[15];
  sys_part_4 += 30*c1p1*c3p2*c5p2*ss[17];
  sys_part_4 += 20*c1p1*c3p3*c4p1*ss[14];
  sys_part_4 += 20*c1p1*c3p3*c5p1*ss[15];
  sys_part_4 += 20*c1p1*c4p1*c5p3*ss[20];
  sys_part_4 += 30*c1p1*c4p2*c5p2*ss[19];
  sys_part_4 += 20*c1p1*c4p3*c5p1*ss[18];
  sys_part_4 += 30*c1p2*c2p1*c3p2*ss[10];
  sys_part_4 += 30*c1p2*c2p1*c4p2*ss[12];
  sys_part_4 += 30*c1p2*c2p1*c5p2*ss[14];
  sys_part_4 += 30*c1p2*c2p2*c3p1*ss[9];
  sys_part_4 += 30*c1p2*c2p2*c4p1*ss[10];
  sys_part_4 += 30*c1p2*c2p2*c5p1*ss[11];
  sys_part_4 += 30*c1p2*c3p1*c4p2*ss[13];
  sys_part_4 += 30*c1p2*c3p1*c5p2*ss[15];
  sys_part_4 += 30*c1p2*c3p2*c4p1*ss[12];
  sys_part_4 += 30*c1p2*c3p2*c5p1*ss[13];
  sys_part_4 += 30*c1p2*c4p1*c5p2*ss[16];
  sys_part_4 += 30*c1p2*c4p2*c5p1*ss[15];
  sys_part_4 += 20*c1p3*c2p1*c3p1*ss[8];
  sys_part_4 += 20*c1p3*c2p1*c4p1*ss[9];
  sys_part_4 += 20*c1p3*c2p1*c5p1*ss[10];
  sys_part_4 += 20*c1p3*c3p1*c4p1*ss[10];
  sys_part_4 += 20*c1p3*c3p1*c5p1*ss[11];
  sys_part_4 += 20*c1p3*c4p1*c5p1*ss[12];
  sys_part_4 += 20*c2p1*c3p1*c4p3*ss[17];
  sys_part_4 += 20*c2p1*c3p1*c5p3*ss[20];
  sys_part_4 += 30*c2p1*c3p2*c4p2*ss[16];
  sys_part_4 += 30*c2p1*c3p2*c5p2*ss[18];
  sys_part_4 += 20*c2p1*c3p3*c4p1*ss[15];
  sys_part_4 += 20*c2p1*c3p3*c5p1*ss[16];
  sys_part_4 += 20*c2p1*c4p1*c5p3*ss[21];
  sys_part_4 += 30*c2p1*c4p2*c5p2*ss[20];
  sys_part_4 += 20*c2p1*c4p3*c5p1*ss[19];
  sys_part_4 += 30*c2p2*c3p1*c4p2*ss[15];
  sys_part_4 += 30*c2p2*c3p1*c5p2*ss[17];
  sys_part_4 += 30*c2p2*c3p2*c4p1*ss[14];
  sys_part_4 += 30*c2p2*c3p2*c5p1*ss[15];
  sys_part_4 += 30*c2p2*c4p1*c5p2*ss[18];
  sys_part_4 += 30*c2p2*c4p2*c5p1*ss[17];
  sys_part_4 += 20*c2p3*c3p1*c4p1*ss[13];
  sys_part_4 += 20*c2p3*c3p1*c5p1*ss[14];
  sys_part_4 += 20*c2p3*c4p1*c5p1*ss[15];
  sys_part_4 += 20*c3p1*c4p1*c5p3*ss[22];
  sys_part_4 += 30*c3p1*c4p2*c5p2*ss[21];
  sys_part_4 += 20*c3p1*c4p3*c5p1*ss[20];
  sys_part_4 += 30*c3p2*c4p1*c5p2*ss[20];
  sys_part_4 += 30*c3p2*c4p2*c5p1*ss[19];
  sys_part_4 += 20*c3p3*c4p1*c5p1*ss[18];
  sys_part_4 += 5*c0p1*c1p4*ss[4];
  sys_part_4 += 5*c0p1*c2p4*ss[8];
  sys_part_4 += 5*c0p1*c3p4*ss[12];
  sys_part_4 += 5*c0p1*c4p4*ss[16];
  sys_part_4 += 5*c0p1*c5p4*ss[20];
  sys_part_4 += 10*c0p2*c1p3*ss[3];
  sys_part_4 += 10*c0p2*c2p3*ss[6];
  sys_part_4 += 10*c0p2*c3p3*ss[9];
  sys_part_4 += 10*c0p2*c4p3*ss[12];
  sys_part_4 += 10*c0p2*c5p3*ss[15];
  sys_part_4 += 10*c0p3*c1p2*ss[2];
  sys_part_4 += 10*c0p3*c2p2*ss[4];
  sys_part_4 += 10*c0p3*c3p2*ss[6];
  sys_part_4 += 10*c0p3*c4p2*ss[8];
  sys_part_4 += 10*c0p3*c5p2*ss[10];
  sys_part_4 += 5*c0p4*c1p1*ss[1];
  sys_part_4 += 5*c0p4*c2p1*ss[2];
  sys_part_4 += 5*c0p4*c3p1*ss[3];
  sys_part_4 += 5*c0p4*c4p1*ss[4];
  sys_part_4 += 5*c0p4*c5p1*ss[5];
  sys_part_4 += 5*c1p1*c2p4*ss[9];
  sys_part_4 += 5*c1p1*c3p4*ss[13];
  sys_part_4 += 5*c1p1*c4p4*ss[17];
  sys_part_4 += 5*c1p1*c5p4*ss[21];
  sys_part_4 += 10*c1p2*c2p3*ss[8];
  sys_part_4 += 10*c1p2*c3p3*ss[11];
  sys_part_4 += 10*c1p2*c4p3*ss[14];
  sys_part_4 += 10*c1p2*c5p3*ss[17];
  sys_part_4 += 10*c1p3*c2p2*ss[7];
  sys_part_4 += 10*c1p3*c3p2*ss[9];
  sys_part_4 += 10*c1p3*c4p2*ss[11];
  sys_part_4 += 10*c1p3*c5p2*ss[13];
  sys_part_4 += 5*c1p4*c2p1*ss[6];
  sys_part_4 += 5*c1p4*c3p1*ss[7];
  sys_part_4 += 5*c1p4*c4p1*ss[8];
  sys_part_4 += 5*c1p4*c5p1*ss[9];
  sys_part_4 += 5*c2p1*c3p4*ss[14];
  sys_part_4 += 5*c2p1*c4p4*ss[18];
  sys_part_4 += 5*c2p1*c5p4*ss[22];
  sys_part_4 += 10*c2p2*c3p3*ss[13];
  sys_part_4 += 10*c2p2*c4p3*ss[16];
  sys_part_4 += 10*c2p2*c5p3*ss[19];
  sys_part_4 += 10*c2p3*c3p2*ss[12];
  sys_part_4 += 10*c2p3*c4p2*ss[14];
  sys_part_4 += 10*c2p3*c5p2*ss[16];
  sys_part_4 += 5*c2p4*c3p1*ss[11];
  sys_part_4 += 5*c2p4*c4p1*ss[12];
  sys_part_4 += 5*c2p4*c5p1*ss[13];
  sys_part_4 += 5*c3p1*c4p4*ss[19];
  sys_part_4 += 5*c3p1*c5p4*ss[23];
  sys_part_4 += 10*c3p2*c4p3*ss[18];
  sys_part_4 += 10*c3p2*c5p3*ss[21];
  sys_part_4 += 10*c3p3*c4p2*ss[17];
  sys_part_4 += 10*c3p3*c5p2*ss[19];
  sys_part_4 += 5*c3p4*c4p1*ss[16];
  sys_part_4 += 5*c3p4*c5p1*ss[17];
  sys_part_4 += 5*c4p1*c5p4*ss[24];
  sys_part_4 += 10*c4p2*c5p3*ss[23];
  sys_part_4 += 10*c4p3*c5p2*ss[22];
  sys_part_4 += 5*c4p4*c5p1*ss[21];
  sys_part_4 += c1p5*ss[5];
  sys_part_4 += c2p5*ss[10];
  sys_part_4 += c3p5*ss[15];
  sys_part_4 += c4p5*ss[20];
  sys_part_4 += c5p5*ss[25];
  sys_part_4 += c0p5;
  sys_part_4 += -tmom[5];
  T c0p6 = c0p5*c0p1;
  T c1p6 = c1p5*c1p1;
  T c2p6 = c2p5*c2p1;
  T c3p6 = c3p5*c3p1;
  T c4p6 = c4p5*c4p1;
  T c5p6 = c5p5*c5p1;
  TACC sys_part_5 = convert<TACC>(720*c0p1*c1p1*c2p1*c3p1*c4p1*c5p1*ss[15]);
  sys_part_5 += 360*c0p1*c1p1*c2p1*c3p1*c4p2*ss[14];
  sys_part_5 += 360*c0p1*c1p1*c2p1*c3p1*c5p2*ss[16];
  sys_part_5 += 360*c0p1*c1p1*c2p1*c3p2*c4p1*ss[13];
  sys_part_5 += 360*c0p1*c1p1*c2p1*c3p2*c5p1*ss[14];
  sys_part_5 += 360*c0p1*c1p1*c2p1*c4p1*c5p2*ss[17];
  sys_part_5 += 360*c0p1*c1p1*c2p1*c4p2*c5p1*ss[16];
  sys_part_5 += 360*c0p1*c1p1*c2p2*c3p1*c4p1*ss[12];
  sys_part_5 += 360*c0p1*c1p1*c2p2*c3p1*c5p1*ss[13];
  sys_part_5 += 360*c0p1*c1p1*c2p2*c4p1*c5p1*ss[14];
  sys_part_5 += 360*c0p1*c1p1*c3p1*c4p1*c5p2*ss[18];
  sys_part_5 += 360*c0p1*c1p1*c3p1*c4p2*c5p1*ss[17];
  sys_part_5 += 360*c0p1*c1p1*c3p2*c4p1*c5p1*ss[16];
  sys_part_5 += 360*c0p1*c1p2*c2p1*c3p1*c4p1*ss[11];
  sys_part_5 += 360*c0p1*c1p2*c2p1*c3p1*c5p1*ss[12];
  sys_part_5 += 360*c0p1*c1p2*c2p1*c4p1*c5p1*ss[13];
  sys_part_5 += 360*c0p1*c1p2*c3p1*c4p1*c5p1*ss[14];
  sys_part_5 += 360*c0p1*c2p1*c3p1*c4p1*c5p2*ss[19];
  sys_part_5 += 360*c0p1*c2p1*c3p1*c4p2*c5p1*ss[18];
  sys_part_5 += 360*c0p1*c2p1*c3p2*c4p1*c5p1*ss[17];
  sys_part_5 += 360*c0p1*c2p2*c3p1*c4p1*c5p1*ss[16];
  sys_part_5 += 360*c0p2*c1p1*c2p1*c3p1*c4p1*ss[10];
  sys_part_5 += 360*c0p2*c1p1*c2p1*c3p1*c5p1*ss[11];
  sys_part_5 += 360*c0p2*c1p1*c2p1*c4p1*c5p1*ss[12];
  sys_part_5 += 360*c0p2*c1p1*c3p1*c4p1*c5p1*ss[13];
  sys_part_5 += 360*c0p2*c2p1*c3p1*c4p1*c5p1*ss[14];
  sys_part_5 += 360*c1p1*c2p1*c3p1*c4p1*c5p2*ss[20];
  sys_part_5 += 360*c1p1*c2p1*c3p1*c4p2*c5p1*ss[19];
  sys_part_5 += 360*c1p1*c2p1*c3p2*c4p1*c5p1*ss[18];
  sys_part_5 += 360*c1p1*c2p2*c3p1*c4p1*c5p1*ss[17];
  sys_part_5 += 360*c1p2*c2p1*c3p1*c4p1*c5p1*ss[16];
  sys_part_5 += 120*c0p1*c1p1*c2p1*c3p3*ss[12];
  sys_part_5 += 120*c0p1*c1p1*c2p1*c4p3*ss[15];
  sys_part_5 += 120*c0p1*c1p1*c2p1*c5p3*ss[18];
  sys_part_5 += 180*c0p1*c1p1*c2p2*c3p2*ss[11];
  sys_part_5 += 180*c0p1*c1p1*c2p2*c4p2*ss[13];
  sys_part_5 += 180*c0p1*c1p1*c2p2*c5p2*ss[15];
  sys_part_5 += 120*c0p1*c1p1*c2p3*c3p1*ss[10];
  sys_part_5 += 120*c0p1*c1p1*c2p3*c4p1*ss[11];
  sys_part_5 += 120*c0p1*c1p1*c2p3*c5p1*ss[12];
  sys_part_5 += 120*c0p1*c1p1*c3p1*c4p3*ss[16];
  sys_part_5 += 120*c0p1*c1p1*c3p1*c5p3*ss[19];
  sys_part_5 += 180*c0p1*c1p1*c3p2*c4p2*ss[15];
  sys_part_5 += 180*c0p1*c1p1*c3p2*c5p2*ss[17];
  sys_part_5 += 120*c0p1*c1p1*c3p3*c4p1*ss[14];
  sys_part_5 += 120*c0p1*c1p1*c3p3*c5p1*ss[15];
  sys_part_5 += 120*c0p1*c1p1*c4p1*c5p3*ss[20];
  sys_part_5 += 180*c0p1*c1p1*c4p2*c5p2*ss[19];
  sys_part_5 += 120*c0p1*c1p1*c4p3*c5p1*ss[18];
  sys_part_5 += 180*c0p1*c1p2*c2p1*c3p2*ss[10];
  sys_part_5 += 180*c0p1*c1p2*c2p1*c4p2*ss[12];
  sys_part_5 += 180*c0p1*c1p2*c2p1*c5p2*ss[14];
  sys_part_5 += 180*c0p1*c1p2*c2p2*c3p1*ss[9];
  sys_part_5 += 180*c0p1*c1p2*c2p2*c4p1*ss[10];
  sys_part_5 += 180*c0p1*c1p2*c2p2*c5p1*ss[11];
  sys_part_5 += 180*c0p1*c1p2*c3p1*c4p2*ss[13];
  sys_part_5 += 180*c0p1*c1p2*c3p1*c5p2*ss[15];
  sys_part_5 += 180*c0p1*c1p2*c3p2*c4p1*ss[12];
  sys_part_5 += 180*c0p1*c1p2*c3p2*c5p1*ss[13];
  sys_part_5 += 180*c0p1*c1p2*c4p1*c5p2*ss[16];
  sys_part_5 += 180*c0p1*c1p2*c4p2*c5p1*ss[15];
  sys_part_5 += 120*c0p1*c1p3*c2p1*c3p1*ss[8];
  sys_part_5 += 120*c0p1*c1p3*c2p1*c4p1*ss[9];
  sys_part_5 += 120*c0p1*c1p3*c2p1*c5p1*ss[10];
  sys_part_5 += 120*c0p1*c1p3*c3p1*c4p1*ss[10];
  sys_part_5 += 120*c0p1*c1p3*c3p1*c5p1*ss[11];
  sys_part_5 += 120*c0p1*c1p3*c4p1*c5p1*ss[12];
  sys_part_5 += 120*c0p1*c2p1*c3p1*c4p3*ss[17];
  sys_part_5 += 120*c0p1*c2p1*c3p1*c5p3*ss[20];
  sys_part_5 += 180*c0p1*c2p1*c3p2*c4p2*ss[16];
  sys_part_5 += 180*c0p1*c2p1*c3p2*c5p2*ss[18];
  sys_part_5 += 120*c0p1*c2p1*c3p3*c4p1*ss[15];
  sys_part_5 += 120*c0p1*c2p1*c3p3*c5p1*ss[16];
  sys_part_5 += 120*c0p1*c2p1*c4p1*c5p3*ss[21];
  sys_part_5 += 180*c0p1*c2p1*c4p2*c5p2*ss[20];
  sys_part_5 += 120*c0p1*c2p1*c4p3*c5p1*ss[19];
  sys_part_5 += 180*c0p1*c2p2*c3p1*c4p2*ss[15];
  sys_part_5 += 180*c0p1*c2p2*c3p1*c5p2*ss[17];
  sys_part_5 += 180*c0p1*c2p2*c3p2*c4p1*ss[14];
  sys_part_5 += 180*c0p1*c2p2*c3p2*c5p1*ss[15];
  sys_part_5 += 180*c0p1*c2p2*c4p1*c5p2*ss[18];
  sys_part_5 += 180*c0p1*c2p2*c4p2*c5p1*ss[17];
  sys_part_5 += 120*c0p1*c2p3*c3p1*c4p1*ss[13];
  sys_part_5 += 120*c0p1*c2p3*c3p1*c5p1*ss[14];
  sys_part_5 += 120*c0p1*c2p3*c4p1*c5p1*ss[15];
  sys_part_5 += 120*c0p1*c3p1*c4p1*c5p3*ss[22];
  sys_part_5 += 180*c0p1*c3p1*c4p2*c5p2*ss[21];
  sys_part_5 += 120*c0p1*c3p1*c4p3*c5p1*ss[20];
  sys_part_5 += 180*c0p1*c3p2*c4p1*c5p2*ss[20];
  sys_part_5 += 180*c0p1*c3p2*c4p2*c5p1*ss[19];
  sys_part_5 += 120*c0p1*c3p3*c4p1*c5p1*ss[18];
  sys_part_5 += 180*c0p2*c1p1*c2p1*c3p2*ss[9];
  sys_part_5 += 180*c0p2*c1p1*c2p1*c4p2*ss[11];
  sys_part_5 += 180*c0p2*c1p1*c2p1*c5p2*ss[13];
  sys_part_5 += 180*c0p2*c1p1*c2p2*c3p1*ss[8];
  sys_part_5 += 180*c0p2*c1p1*c2p2*c4p1*ss[9];
  sys_part_5 += 180*c0p2*c1p1*c2p2*c5p1*ss[10];
  sys_part_5 += 180*c0p2*c1p1*c3p1*c4p2*ss[12];
  sys_part_5 += 180*c0p2*c1p1*c3p1*c5p2*ss[14];
  sys_part_5 += 180*c0p2*c1p1*c3p2*c4p1*ss[11];
  sys_part_5 += 180*c0p2*c1p1*c3p2*c5p1*ss[12];
  sys_part_5 += 180*c0p2*c1p1*c4p1*c5p2*ss[15];
  sys_part_5 += 180*c0p2*c1p1*c4p2*c5p1*ss[14];
  sys_part_5 += 180*c0p2*c1p2*c2p1*c3p1*ss[7];
  sys_part_5 += 180*c0p2*c1p2*c2p1*c4p1*ss[8];
  sys_part_5 += 180*c0p2*c1p2*c2p1*c5p1*ss[9];
  sys_part_5 += 180*c0p2*c1p2*c3p1*c4p1*ss[9];
  sys_part_5 += 180*c0p2*c1p2*c3p1*c5p1*ss[10];
  sys_part_5 += 180*c0p2*c1p2*c4p1*c5p1*ss[11];
  sys_part_5 += 180*c0p2*c2p1*c3p1*c4p2*ss[13];
  sys_part_5 += 180*c0p2*c2p1*c3p1*c5p2*ss[15];
  sys_part_5 += 180*c0p2*c2p1*c3p2*c4p1*ss[12];
  sys_part_5 += 180*c0p2*c2p1*c3p2*c5p1*ss[13];
  sys_part_5 += 180*c0p2*c2p1*c4p1*c5p2*ss[16];
  sys_part_5 += 180*c0p2*c2p1*c4p2*c5p1*ss[15];
  sys_part_5 += 180*c0p2*c2p2*c3p1*c4p1*ss[11];
  sys_part_5 += 180*c0p2*c2p2*c3p1*c5p1*ss[12];
  sys_part_5 += 180*c0p2*c2p2*c4p1*c5p1*ss[13];
  sys_part_5 += 180*c0p2*c3p1*c4p1*c5p2*ss[17];
  sys_part_5 += 180*c0p2*c3p1*c4p2*c5p1*ss[16];
  sys_part_5 += 180*c0p2*c3p2*c4p1*c5p1*ss[15];
  sys_part_5 += 120*c0p3*c1p1*c2p1*c3p1*ss[6];
  sys_part_5 += 120*c0p3*c1p1*c2p1*c4p1*ss[7];
  sys_part_5 += 120*c0p3*c1p1*c2p1*c5p1*ss[8];
  sys_part_5 += 120*c0p3*c1p1*c3p1*c4p1*ss[8];
  sys_part_5 += 120*c0p3*c1p1*c3p1*c5p1*ss[9];
  sys_part_5 += 120*c0p3*c1p1*c4p1*c5p1*ss[10];
  sys_part_5 += 120*c0p3*c2p1*c3p1*c4p1*ss[9];
  sys_part_5 += 120*c0p3*c2p1*c3p1*c5p1*ss[10];
  sys_part_5 += 120*c0p3*c2p1*c4p1*c5p1*ss[11];
  sys_part_5 += 120*c0p3*c3p1*c4p1*c5p1*ss[12];
  sys_part_5 += 120*c1p1*c2p1*c3p1*c4p3*ss[18];
  sys_part_5 += 120*c1p1*c2p1*c3p1*c5p3*ss[21];
  sys_part_5 += 180*c1p1*c2p1*c3p2*c4p2*ss[17];
  sys_part_5 += 180*c1p1*c2p1*c3p2*c5p2*ss[19];
  sys_part_5 += 120*c1p1*c2p1*c3p3*c4p1*ss[16];
  sys_part_5 += 120*c1p1*c2p1*c3p3*c5p1*ss[17];
  sys_part_5 += 120*c1p1*c2p1*c4p1*c5p3*ss[22];
  sys_part_5 += 180*c1p1*c2p1*c4p2*c5p2*ss[21];
  sys_part_5 += 120*c1p1*c2p1*c4p3*c5p1*ss[20];
  sys_part_5 += 180*c1p1*c2p2*c3p1*c4p2*ss[16];
  sys_part_5 += 180*c1p1*c2p2*c3p1*c5p2*ss[18];
  sys_part_5 += 180*c1p1*c2p2*c3p2*c4p1*ss[15];
  sys_part_5 += 180*c1p1*c2p2*c3p2*c5p1*ss[16];
  sys_part_5 += 180*c1p1*c2p2*c4p1*c5p2*ss[19];
  sys_part_5 += 180*c1p1*c2p2*c4p2*c5p1*ss[18];
  sys_part_5 += 120*c1p1*c2p3*c3p1*c4p1*ss[14];
  sys_part_5 += 120*c1p1*c2p3*c3p1*c5p1*ss[15];
  sys_part_5 += 120*c1p1*c2p3*c4p1*c5p1*ss[16];
  sys_part_5 += 120*c1p1*c3p1*c4p1*c5p3*ss[23];
  sys_part_5 += 180*c1p1*c3p1*c4p2*c5p2*ss[22];
  sys_part_5 += 120*c1p1*c3p1*c4p3*c5p1*ss[21];
  sys_part_5 += 180*c1p1*c3p2*c4p1*c5p2*ss[21];
  sys_part_5 += 180*c1p1*c3p2*c4p2*c5p1*ss[20];
  sys_part_5 += 120*c1p1*c3p3*c4p1*c5p1*ss[19];
  sys_part_5 += 180*c1p2*c2p1*c3p1*c4p2*ss[15];
  sys_part_5 += 180*c1p2*c2p1*c3p1*c5p2*ss[17];
  sys_part_5 += 180*c1p2*c2p1*c3p2*c4p1*ss[14];
  sys_part_5 += 180*c1p2*c2p1*c3p2*c5p1*ss[15];
  sys_part_5 += 180*c1p2*c2p1*c4p1*c5p2*ss[18];
  sys_part_5 += 180*c1p2*c2p1*c4p2*c5p1*ss[17];
  sys_part_5 += 180*c1p2*c2p2*c3p1*c4p1*ss[13];
  sys_part_5 += 180*c1p2*c2p2*c3p1*c5p1*ss[14];
  sys_part_5 += 180*c1p2*c2p2*c4p1*c5p1*ss[15];
  sys_part_5 += 180*c1p2*c3p1*c4p1*c5p2*ss[19];
  sys_part_5 += 180*c1p2*c3p1*c4p2*c5p1*ss[18];
  sys_part_5 += 180*c1p2*c3p2*c4p1*c5p1*ss[17];
  sys_part_5 += 120*c1p3*c2p1*c3p1*c4p1*ss[12];
  sys_part_5 += 120*c1p3*c2p1*c3p1*c5p1*ss[13];
  sys_part_5 += 120*c1p3*c2p1*c4p1*c5p1*ss[14];
  sys_part_5 += 120*c1p3*c3p1*c4p1*c5p1*ss[15];
  sys_part_5 += 120*c2p1*c3p1*c4p1*c5p3*ss[24];
  sys_part_5 += 180*c2p1*c3p1*c4p2*c5p2*ss[23];
  sys_part_5 += 120*c2p1*c3p1*c4p3*c5p1*ss[22];
  sys_part_5 += 180*c2p1*c3p2*c4p1*c5p2*ss[22];
  sys_part_5 += 180*c2p1*c3p2*c4p2*c5p1*ss[21];
  sys_part_5 += 120*c2p1*c3p3*c4p1*c5p1*ss[20];
  sys_part_5 += 180*c2p2*c3p1*c4p1*c5p2*ss[21];
  sys_part_5 += 180*c2p2*c3p1*c4p2*c5p1*ss[20];
  sys_part_5 += 180*c2p2*c3p2*c4p1*c5p1*ss[19];
  sys_part_5 += 120*c2p3*c3p1*c4p1*c5p1*ss[18];
  sys_part_5 += 30*c0p1*c1p1*c2p4*ss[9];
  sys_part_5 += 30*c0p1*c1p1*c3p4*ss[13];
  sys_part_5 += 30*c0p1*c1p1*c4p4*ss[17];
  sys_part_5 += 30*c0p1*c1p1*c5p4*ss[21];
  sys_part_5 += 60*c0p1*c1p2*c2p3*ss[8];
  sys_part_5 += 60*c0p1*c1p2*c3p3*ss[11];
  sys_part_5 += 60*c0p1*c1p2*c4p3*ss[14];
  sys_part_5 += 60*c0p1*c1p2*c5p3*ss[17];
  sys_part_5 += 60*c0p1*c1p3*c2p2*ss[7];
  sys_part_5 += 60*c0p1*c1p3*c3p2*ss[9];
  sys_part_5 += 60*c0p1*c1p3*c4p2*ss[11];
  sys_part_5 += 60*c0p1*c1p3*c5p2*ss[13];
  sys_part_5 += 30*c0p1*c1p4*c2p1*ss[6];
  sys_part_5 += 30*c0p1*c1p4*c3p1*ss[7];
  sys_part_5 += 30*c0p1*c1p4*c4p1*ss[8];
  sys_part_5 += 30*c0p1*c1p4*c5p1*ss[9];
  sys_part_5 += 30*c0p1*c2p1*c3p4*ss[14];
  sys_part_5 += 30*c0p1*c2p1*c4p4*ss[18];
  sys_part_5 += 30*c0p1*c2p1*c5p4*ss[22];
  sys_part_5 += 60*c0p1*c2p2*c3p3*ss[13];
  sys_part_5 += 60*c0p1*c2p2*c4p3*ss[16];
  sys_part_5 += 60*c0p1*c2p2*c5p3*ss[19];
  sys_part_5 += 60*c0p1*c2p3*c3p2*ss[12];
  sys_part_5 += 60*c0p1*c2p3*c4p2*ss[14];
  sys_part_5 += 60*c0p1*c2p3*c5p2*ss[16];
  sys_part_5 += 30*c0p1*c2p4*c3p1*ss[11];
  sys_part_5 += 30*c0p1*c2p4*c4p1*ss[12];
  sys_part_5 += 30*c0p1*c2p4*c5p1*ss[13];
  sys_part_5 += 30*c0p1*c3p1*c4p4*ss[19];
  sys_part_5 += 30*c0p1*c3p1*c5p4*ss[23];
  sys_part_5 += 60*c0p1*c3p2*c4p3*ss[18];
  sys_part_5 += 60*c0p1*c3p2*c5p3*ss[21];
  sys_part_5 += 60*c0p1*c3p3*c4p2*ss[17];
  sys_part_5 += 60*c0p1*c3p3*c5p2*ss[19];
  sys_part_5 += 30*c0p1*c3p4*c4p1*ss[16];
  sys_part_5 += 30*c0p1*c3p4*c5p1*ss[17];
  sys_part_5 += 30*c0p1*c4p1*c5p4*ss[24];
  sys_part_5 += 60*c0p1*c4p2*c5p3*ss[23];
  sys_part_5 += 60*c0p1*c4p3*c5p2*ss[22];
  sys_part_5 += 30*c0p1*c4p4*c5p1*ss[21];
  sys_part_5 += 60*c0p2*c1p1*c2p3*ss[7];
  sys_part_5 += 60*c0p2*c1p1*c3p3*ss[10];
  sys_part_5 += 60*c0p2*c1p1*c4p3*ss[13];
  sys_part_5 += 60*c0p2*c1p1*c5p3*ss[16];
  sys_part_5 += 90*c0p2*c1p2*c2p2*ss[6];
  sys_part_5 += 90*c0p2*c1p2*c3p2*ss[8];
  sys_part_5 += 90*c0p2*c1p2*c4p2*ss[10];
  sys_part_5 += 90*c0p2*c1p2*c5p2*ss[12];
  sys_part_5 += 60*c0p2*c1p3*c2p1*ss[5];
  sys_part_5 += 60*c0p2*c1p3*c3p1*ss[6];
  sys_part_5 += 60*c0p2*c1p3*c4p1*ss[7];
  sys_part_5 += 60*c0p2*c1p3*c5p1*ss[8];
  sys_part_5 += 60*c0p2*c2p1*c3p3*ss[11];
  sys_part_5 += 60*c0p2*c2p1*c4p3*ss[14];
  sys_part_5 += 60*c0p2*c2p1*c5p3*ss[17];
  sys_part_5 += 90*c0p2*c2p2*c3p2*ss[10];
  sys_part_5 += 90*c0p2*c2p2*c4p2*ss[12];
  sys_part_5 += 90*c0p2*c2p2*c5p2*ss[14];
  sys_part_5 += 60*c0p2*c2p3*c3p1*ss[9];
  sys_part_5 += 60*c0p2*c2p3*c4p1*ss[10];
  sys_part_5 += 60*c0p2*c2p3*c5p1*ss[11];
  sys_part_5 += 60*c0p2*c3p1*c4p3*ss[15];
  sys_part_5 += 60*c0p2*c3p1*c5p3*ss[18];
  sys_part_5 += 90*c0p2*c3p2*c4p2*ss[14];
  sys_part_5 += 90*c0p2*c3p2*c5p2*ss[16];
  sys_part_5 += 60*c0p2*c3p3*c4p1*ss[13];
  sys_part_5 += 60*c0p2*c3p3*c5p1*ss[14];
  sys_part_5 += 60*c0p2*c4p1*c5p3*ss[19];
  sys_part_5 += 90*c0p2*c4p2*c5p2*ss[18];
  sys_part_5 += 60*c0p2*c4p3*c5p1*ss[17];
  sys_part_5 += 60*c0p3*c1p1*c2p2*ss[5];
  sys_part_5 += 60*c0p3*c1p1*c3p2*ss[7];
  sys_part_5 += 60*c0p3*c1p1*c4p2*ss[9];
  sys_part_5 += 60*c0p3*c1p1*c5p2*ss[11];
  sys_part_5 += 60*c0p3*c1p2*c2p1*ss[4];
  sys_part_5 += 60*c0p3*c1p2*c3p1*ss[5];
  sys_part_5 += 60*c0p3*c1p2*c4p1*ss[6];
  sys_part_5 += 60*c0p3*c1p2*c5p1*ss[7];
  sys_part_5 += 60*c0p3*c2p1*c3p2*ss[8];
  sys_part_5 += 60*c0p3*c2p1*c4p2*ss[10];
  sys_part_5 += 60*c0p3*c2p1*c5p2*ss[12];
  sys_part_5 += 60*c0p3*c2p2*c3p1*ss[7];
  sys_part_5 += 60*c0p3*c2p2*c4p1*ss[8];
  sys_part_5 += 60*c0p3*c2p2*c5p1*ss[9];
  sys_part_5 += 60*c0p3*c3p1*c4p2*ss[11];
  sys_part_5 += 60*c0p3*c3p1*c5p2*ss[13];
  sys_part_5 += 60*c0p3*c3p2*c4p1*ss[10];
  sys_part_5 += 60*c0p3*c3p2*c5p1*ss[11];
  sys_part_5 += 60*c0p3*c4p1*c5p2*ss[14];
  sys_part_5 += 60*c0p3*c4p2*c5p1*ss[13];
  sys_part_5 += 30*c0p4*c1p1*c2p1*ss[3];
  sys_part_5 += 30*c0p4*c1p1*c3p1*ss[4];
  sys_part_5 += 30*c0p4*c1p1*c4p1*ss[5];
  sys_part_5 += 30*c0p4*c1p1*c5p1*ss[6];
  sys_part_5 += 30*c0p4*c2p1*c3p1*ss[5];
  sys_part_5 += 30*c0p4*c2p1*c4p1*ss[6];
  sys_part_5 += 30*c0p4*c2p1*c5p1*ss[7];
  sys_part_5 += 30*c0p4*c3p1*c4p1*ss[7];
  sys_part_5 += 30*c0p4*c3p1*c5p1*ss[8];
  sys_part_5 += 30*c0p4*c4p1*c5p1*ss[9];
  sys_part_5 += 30*c1p1*c2p1*c3p4*ss[15];
  sys_part_5 += 30*c1p1*c2p1*c4p4*ss[19];
  sys_part_5 += 30*c1p1*c2p1*c5p4*ss[23];
  sys_part_5 += 60*c1p1*c2p2*c3p3*ss[14];
  sys_part_5 += 60*c1p1*c2p2*c4p3*ss[17];
  sys_part_5 += 60*c1p1*c2p2*c5p3*ss[20];
  sys_part_5 += 60*c1p1*c2p3*c3p2*ss[13];
  sys_part_5 += 60*c1p1*c2p3*c4p2*ss[15];
  sys_part_5 += 60*c1p1*c2p3*c5p2*ss[17];
  sys_part_5 += 30*c1p1*c2p4*c3p1*ss[12];
  sys_part_5 += 30*c1p1*c2p4*c4p1*ss[13];
  sys_part_5 += 30*c1p1*c2p4*c5p1*ss[14];
  sys_part_5 += 30*c1p1*c3p1*c4p4*ss[20];
  sys_part_5 += 30*c1p1*c3p1*c5p4*ss[24];
  sys_part_5 += 60*c1p1*c3p2*c4p3*ss[19];
  sys_part_5 += 60*c1p1*c3p2*c5p3*ss[22];
  sys_part_5 += 60*c1p1*c3p3*c4p2*ss[18];
  sys_part_5 += 60*c1p1*c3p3*c5p2*ss[20];
  sys_part_5 += 30*c1p1*c3p4*c4p1*ss[17];
  sys_part_5 += 30*c1p1*c3p4*c5p1*ss[18];
  sys_part_5 += 30*c1p1*c4p1*c5p4*ss[25];
  sys_part_5 += 60*c1p1*c4p2*c5p3*ss[24];
  sys_part_5 += 60*c1p1*c4p3*c5p2*ss[23];
  sys_part_5 += 30*c1p1*c4p4*c5p1*ss[22];
  sys_part_5 += 60*c1p2*c2p1*c3p3*ss[13];
  sys_part_5 += 60*c1p2*c2p1*c4p3*ss[16];
  sys_part_5 += 60*c1p2*c2p1*c5p3*ss[19];
  sys_part_5 += 90*c1p2*c2p2*c3p2*ss[12];
  sys_part_5 += 90*c1p2*c2p2*c4p2*ss[14];
  sys_part_5 += 90*c1p2*c2p2*c5p2*ss[16];
  sys_part_5 += 60*c1p2*c2p3*c3p1*ss[11];
  sys_part_5 += 60*c1p2*c2p3*c4p1*ss[12];
  sys_part_5 += 60*c1p2*c2p3*c5p1*ss[13];
  sys_part_5 += 60*c1p2*c3p1*c4p3*ss[17];
  sys_part_5 += 60*c1p2*c3p1*c5p3*ss[20];
  sys_part_5 += 90*c1p2*c3p2*c4p2*ss[16];
  sys_part_5 += 90*c1p2*c3p2*c5p2*ss[18];
  sys_part_5 += 60*c1p2*c3p3*c4p1*ss[15];
  sys_part_5 += 60*c1p2*c3p3*c5p1*ss[16];
  sys_part_5 += 60*c1p2*c4p1*c5p3*ss[21];
  sys_part_5 += 90*c1p2*c4p2*c5p2*ss[20];
  sys_part_5 += 60*c1p2*c4p3*c5p1*ss[19];
  sys_part_5 += 60*c1p3*c2p1*c3p2*ss[11];
  sys_part_5 += 60*c1p3*c2p1*c4p2*ss[13];
  sys_part_5 += 60*c1p3*c2p1*c5p2*ss[15];
  sys_part_5 += 60*c1p3*c2p2*c3p1*ss[10];
  sys_part_5 += 60*c1p3*c2p2*c4p1*ss[11];
  sys_part_5 += 60*c1p3*c2p2*c5p1*ss[12];
  sys_part_5 += 60*c1p3*c3p1*c4p2*ss[14];
  sys_part_5 += 60*c1p3*c3p1*c5p2*ss[16];
  sys_part_5 += 60*c1p3*c3p2*c4p1*ss[13];
  sys_part_5 += 60*c1p3*c3p2*c5p1*ss[14];
  sys_part_5 += 60*c1p3*c4p1*c5p2*ss[17];
  sys_part_5 += 60*c1p3*c4p2*c5p1*ss[16];
  sys_part_5 += 30*c1p4*c2p1*c3p1*ss[9];
  sys_part_5 += 30*c1p4*c2p1*c4p1*ss[10];
  sys_part_5 += 30*c1p4*c2p1*c5p1*ss[11];
  sys_part_5 += 30*c1p4*c3p1*c4p1*ss[11];
  sys_part_5 += 30*c1p4*c3p1*c5p1*ss[12];
  sys_part_5 += 30*c1p4*c4p1*c5p1*ss[13];
  sys_part_5 += 30*c2p1*c3p1*c4p4*ss[21];
  sys_part_5 += 30*c2p1*c3p1*c5p4*ss[25];
  sys_part_5 += 60*c2p1*c3p2*c4p3*ss[20];
  sys_part_5 += 60*c2p1*c3p2*c5p3*ss[23];
  sys_part_5 += 60*c2p1*c3p3*c4p2*ss[19];
  sys_part_5 += 60*c2p1*c3p3*c5p2*ss[21];
  sys_part_5 += 30*c2p1*c3p4*c4p1*ss[18];
  sys_part_5 += 30*c2p1*c3p4*c5p1*ss[19];
  sys_part_5 += 30*c2p1*c4p1*c5p4*ss[26];
  sys_part_5 += 60*c2p1*c4p2*c5p3*ss[25];
  sys_part_5 += 60*c2p1*c4p3*c5p2*ss[24];
  sys_part_5 += 30*c2p1*c4p4*c5p1*ss[23];
  sys_part_5 += 60*c2p2*c3p1*c4p3*ss[19];
  sys_part_5 += 60*c2p2*c3p1*c5p3*ss[22];
  sys_part_5 += 90*c2p2*c3p2*c4p2*ss[18];
  sys_part_5 += 90*c2p2*c3p2*c5p2*ss[20];
  sys_part_5 += 60*c2p2*c3p3*c4p1*ss[17];
  sys_part_5 += 60*c2p2*c3p3*c5p1*ss[18];
  sys_part_5 += 60*c2p2*c4p1*c5p3*ss[23];
  sys_part_5 += 90*c2p2*c4p2*c5p2*ss[22];
  sys_part_5 += 60*c2p2*c4p3*c5p1*ss[21];
  sys_part_5 += 60*c2p3*c3p1*c4p2*ss[17];
  sys_part_5 += 60*c2p3*c3p1*c5p2*ss[19];
  sys_part_5 += 60*c2p3*c3p2*c4p1*ss[16];
  sys_part_5 += 60*c2p3*c3p2*c5p1*ss[17];
  sys_part_5 += 60*c2p3*c4p1*c5p2*ss[20];
  sys_part_5 += 60*c2p3*c4p2*c5p1*ss[19];
  sys_part_5 += 30*c2p4*c3p1*c4p1*ss[15];
  sys_part_5 += 30*c2p4*c3p1*c5p1*ss[16];
  sys_part_5 += 30*c2p4*c4p1*c5p1*ss[17];
  sys_part_5 += 30*c3p1*c4p1*c5p4*ss[27];
  sys_part_5 += 60*c3p1*c4p2*c5p3*ss[26];
  sys_part_5 += 60*c3p1*c4p3*c5p2*ss[25];
  sys_part_5 += 30*c3p1*c4p4*c5p1*ss[24];
  sys_part_5 += 60*c3p2*c4p1*c5p3*ss[25];
  sys_part_5 += 90*c3p2*c4p2*c5p2*ss[24];
  sys_part_5 += 60*c3p2*c4p3*c5p1*ss[23];
  sys_part_5 += 60*c3p3*c4p1*c5p2*ss[23];
  sys_part_5 += 60*c3p3*c4p2*c5p1*ss[22];
  sys_part_5 += 30*c3p4*c4p1*c5p1*ss[21];
  sys_part_5 += 6*c0p1*c1p5*ss[5];
  sys_part_5 += 6*c0p1*c2p5*ss[10];
  sys_part_5 += 6*c0p1*c3p5*ss[15];
  sys_part_5 += 6*c0p1*c4p5*ss[20];
  sys_part_5 += 6*c0p1*c5p5*ss[25];
  sys_part_5 += 15*c0p2*c1p4*ss[4];
  sys_part_5 += 15*c0p2*c2p4*ss[8];
  sys_part_5 += 15*c0p2*c3p4*ss[12];
  sys_part_5 += 15*c0p2*c4p4*ss[16];
  sys_part_5 += 15*c0p2*c5p4*ss[20];
  sys_part_5 += 20*c0p3*c1p3*ss[3];
  sys_part_5 += 20*c0p3*c2p3*ss[6];
  sys_part_5 += 20*c0p3*c3p3*ss[9];
  sys_part_5 += 20*c0p3*c4p3*ss[12];
  sys_part_5 += 20*c0p3*c5p3*ss[15];
  sys_part_5 += 15*c0p4*c1p2*ss[2];
  sys_part_5 += 15*c0p4*c2p2*ss[4];
  sys_part_5 += 15*c0p4*c3p2*ss[6];
  sys_part_5 += 15*c0p4*c4p2*ss[8];
  sys_part_5 += 15*c0p4*c5p2*ss[10];
  sys_part_5 += 6*c0p5*c1p1*ss[1];
  sys_part_5 += 6*c0p5*c2p1*ss[2];
  sys_part_5 += 6*c0p5*c3p1*ss[3];
  sys_part_5 += 6*c0p5*c4p1*ss[4];
  sys_part_5 += 6*c0p5*c5p1*ss[5];
  sys_part_5 += 6*c1p1*c2p5*ss[11];
  sys_part_5 += 6*c1p1*c3p5*ss[16];
  sys_part_5 += 6*c1p1*c4p5*ss[21];
  sys_part_5 += 6*c1p1*c5p5*ss[26];
  sys_part_5 += 15*c1p2*c2p4*ss[10];
  sys_part_5 += 15*c1p2*c3p4*ss[14];
  sys_part_5 += 15*c1p2*c4p4*ss[18];
  sys_part_5 += 15*c1p2*c5p4*ss[22];
  sys_part_5 += 20*c1p3*c2p3*ss[9];
  sys_part_5 += 20*c1p3*c3p3*ss[12];
  sys_part_5 += 20*c1p3*c4p3*ss[15];
  sys_part_5 += 20*c1p3*c5p3*ss[18];
  sys_part_5 += 15*c1p4*c2p2*ss[8];
  sys_part_5 += 15*c1p4*c3p2*ss[10];
  sys_part_5 += 15*c1p4*c4p2*ss[12];
  sys_part_5 += 15*c1p4*c5p2*ss[14];
  sys_part_5 += 6*c1p5*c2p1*ss[7];
  sys_part_5 += 6*c1p5*c3p1*ss[8];
  sys_part_5 += 6*c1p5*c4p1*ss[9];
  sys_part_5 += 6*c1p5*c5p1*ss[10];
  sys_part_5 += 6*c2p1*c3p5*ss[17];
  sys_part_5 += 6*c2p1*c4p5*ss[22];
  sys_part_5 += 6*c2p1*c5p5*ss[27];
  sys_part_5 += 15*c2p2*c3p4*ss[16];
  sys_part_5 += 15*c2p2*c4p4*ss[20];
  sys_part_5 += 15*c2p2*c5p4*ss[24];
  sys_part_5 += 20*c2p3*c3p3*ss[15];
  sys_part_5 += 20*c2p3*c4p3*ss[18];
  sys_part_5 += 20*c2p3*c5p3*ss[21];
  sys_part_5 += 15*c2p4*c3p2*ss[14];
  sys_part_5 += 15*c2p4*c4p2*ss[16];
  sys_part_5 += 15*c2p4*c5p2*ss[18];
  sys_part_5 += 6*c2p5*c3p1*ss[13];
  sys_part_5 += 6*c2p5*c4p1*ss[14];
  sys_part_5 += 6*c2p5*c5p1*ss[15];
  sys_part_5 += 6*c3p1*c4p5*ss[23];
  sys_part_5 += 6*c3p1*c5p5*ss[28];
  sys_part_5 += 15*c3p2*c4p4*ss[22];
  sys_part_5 += 15*c3p2*c5p4*ss[26];
  sys_part_5 += 20*c3p3*c4p3*ss[21];
  sys_part_5 += 20*c3p3*c5p3*ss[24];
  sys_part_5 += 15*c3p4*c4p2*ss[20];
  sys_part_5 += 15*c3p4*c5p2*ss[22];
  sys_part_5 += 6*c3p5*c4p1*ss[19];
  sys_part_5 += 6*c3p5*c5p1*ss[20];
  sys_part_5 += 6*c4p1*c5p5*ss[29];
  sys_part_5 += 15*c4p2*c5p4*ss[28];
  sys_part_5 += 20*c4p3*c5p3*ss[27];
  sys_part_5 += 15*c4p4*c5p2*ss[26];
  sys_part_5 += 6*c4p5*c5p1*ss[25];
  sys_part_5 += c1p6*ss[6];
  sys_part_5 += c2p6*ss[12];
  sys_part_5 += c3p6*ss[18];
  sys_part_5 += c4p6*ss[24];
  sys_part_5 += c5p6*ss[30];
  sys_part_5 += c0p6;
  sys_part_5 += -tmom[6];
  TACC grad_0_num = convert<TACC>(2*(c1p1*ss[1] + c2p1*ss[2] + c3p1*ss[3] + c4p1*ss[4] + c5p1*ss[5] + c0p1)*sys_part_1);
  grad_0_num += 3*(2*c0p1*c1p1*ss[1] + 2*c0p1*c2p1*ss[2] + 2*c0p1*c3p1*ss[3] + 2*c0p1*c4p1*ss[4] + 2*c0p1*c5p1*ss[5] + 2*c1p1*c2p1*ss[3] + 2*c1p1*c3p1*ss[4] + 2*c1p1*c4p1*ss[5] + 2*c1p1*c5p1*ss[6] + 2*c2p1*c3p1*ss[5] + 2*c2p1*c4p1*ss[6] + 2*c2p1*c5p1*ss[7] + 2*c3p1*c4p1*ss[7] + 2*c3p1*c5p1*ss[8] + 2*c4p1*c5p1*ss[9] + c1p2*ss[2] + c2p2*ss[4] + c3p2*ss[6] + c4p2*ss[8] + c5p2*ss[10] + c0p2)*sys_part_2;
  grad_0_num += 4*(6*c0p1*c1p1*c2p1*ss[3] + 6*c0p1*c1p1*c3p1*ss[4] + 6*c0p1*c1p1*c4p1*ss[5] + 6*c0p1*c1p1*c5p1*ss[6] + 6*c0p1*c2p1*c3p1*ss[5] + 6*c0p1*c2p1*c4p1*ss[6] + 6*c0p1*c2p1*c5p1*ss[7] + 6*c0p1*c3p1*c4p1*ss[7] + 6*c0p1*c3p1*c5p1*ss[8] + 6*c0p1*c4p1*c5p1*ss[9] + 6*c1p1*c2p1*c3p1*ss[6] + 6*c1p1*c2p1*c4p1*ss[7] + 6*c1p1*c2p1*c5p1*ss[8] + 6*c1p1*c3p1*c4p1*ss[8] + 6*c1p1*c3p1*c5p1*ss[9] + 6*c1p1*c4p1*c5p1*ss[10] + 6*c2p1*c3p1*c4p1*ss[9] + 6*c2p1*c3p1*c5p1*ss[10] + 6*c2p1*c4p1*c5p1*ss[11] + 6*c3p1*c4p1*c5p1*ss[12] + 3*c0p1*c1p2*ss[2] + 3*c0p1*c2p2*ss[4] + 3*c0p1*c3p2*ss[6] + 3*c0p1*c4p2*ss[8] + 3*c0p1*c5p2*ss[10] + 3*c0p2*c1p1*ss[1] + 3*c0p2*c2p1*ss[2] + 3*c0p2*c3p1*ss[3] + 3*c0p2*c4p1*ss[4] + 3*c0p2*c5p1*ss[5] + 3*c1p1*c2p2*ss[5] + 3*c1p1*c3p2*ss[7] + 3*c1p1*c4p2*ss[9] + 3*c1p1*c5p2*ss[11] + 3*c1p2*c2p1*ss[4] + 3*c1p2*c3p1*ss[5] + 3*c1p2*c4p1*ss[6] + 3*c1p2*c5p1*ss[7] + 3*c2p1*c3p2*ss[8] + 3*c2p1*c4p2*ss[10] + 3*c2p1*c5p2*ss[12] + 3*c2p2*c3p1*ss[7] + 3*c2p2*c4p1*ss[8] + 3*c2p2*c5p1*ss[9] + 3*c3p1*c4p2*ss[11] + 3*c3p1*c5p2*ss[13] + 3*c3p2*c4p1*ss[10] + 3*c3p2*c5p1*ss[11] + 3*c4p1*c5p2*ss[14] + 3*c4p2*c5p1*ss[13] + c1p3*ss[3] + c2p3*ss[6] + c3p3*ss[9] + c4p3*ss[12] + c5p3*ss[15] + c0p3)*sys_part_3;
  grad_0_num += 5*(24*c0p1*c1p1*c2p1*c3p1*ss[6] + 24*c0p1*c1p1*c2p1*c4p1*ss[7] + 24*c0p1*c1p1*c2p1*c5p1*ss[8] + 24*c0p1*c1p1*c3p1*c4p1*ss[8] + 24*c0p1*c1p1*c3p1*c5p1*ss[9] + 24*c0p1*c1p1*c4p1*c5p1*ss[10] + 24*c0p1*c2p1*c3p1*c4p1*ss[9] + 24*c0p1*c2p1*c3p1*c5p1*ss[10] + 24*c0p1*c2p1*c4p1*c5p1*ss[11] + 24*c0p1*c3p1*c4p1*c5p1*ss[12] + 24*c1p1*c2p1*c3p1*c4p1*ss[10] + 24*c1p1*c2p1*c3p1*c5p1*ss[11] + 24*c1p1*c2p1*c4p1*c5p1*ss[12] + 24*c1p1*c3p1*c4p1*c5p1*ss[13] + 24*c2p1*c3p1*c4p1*c5p1*ss[14] + 12*c0p1*c1p1*c2p2*ss[5] + 12*c0p1*c1p1*c3p2*ss[7] + 12*c0p1*c1p1*c4p2*ss[9] + 12*c0p1*c1p1*c5p2*ss[11] + 12*c0p1*c1p2*c2p1*ss[4] + 12*c0p1*c1p2*c3p1*ss[5] + 12*c0p1*c1p2*c4p1*ss[6] + 12*c0p1*c1p2*c5p1*ss[7] + 12*c0p1*c2p1*c3p2*ss[8] + 12*c0p1*c2p1*c4p2*ss[10] + 12*c0p1*c2p1*c5p2*ss[12] + 12*c0p1*c2p2*c3p1*ss[7] + 12*c0p1*c2p2*c4p1*ss[8] + 12*c0p1*c2p2*c5p1*ss[9] + 12*c0p1*c3p1*c4p2*ss[11] + 12*c0p1*c3p1*c5p2*ss[13] + 12*c0p1*c3p2*c4p1*ss[10] + 12*c0p1*c3p2*c5p1*ss[11] + 12*c0p1*c4p1*c5p2*ss[14] + 12*c0p1*c4p2*c5p1*ss[13] + 12*c0p2*c1p1*c2p1*ss[3] + 12*c0p2*c1p1*c3p1*ss[4] + 12*c0p2*c1p1*c4p1*ss[5] + 12*c0p2*c1p1*c5p1*ss[6] + 12*c0p2*c2p1*c3p1*ss[5] + 12*c0p2*c2p1*c4p1*ss[6] + 12*c0p2*c2p1*c5p1*ss[7] + 12*c0p2*c3p1*c4p1*ss[7] + 12*c0p2*c3p1*c5p1*ss[8] + 12*c0p2*c4p1*c5p1*ss[9] + 12*c1p1*c2p1*c3p2*ss[9] + 12*c1p1*c2p1*c4p2*ss[11] + 12*c1p1*c2p1*c5p2*ss[13] + 12*c1p1*c2p2*c3p1*ss[8] + 12*c1p1*c2p2*c4p1*ss[9] + 12*c1p1*c2p2*c5p1*ss[10] + 12*c1p1*c3p1*c4p2*ss[12] + 12*c1p1*c3p1*c5p2*ss[14] + 12*c1p1*c3p2*c4p1*ss[11] + 12*c1p1*c3p2*c5p1*ss[12] + 12*c1p1*c4p1*c5p2*ss[15] + 12*c1p1*c4p2*c5p1*ss[14] + 12*c1p2*c2p1*c3p1*ss[7] + 12*c1p2*c2p1*c4p1*ss[8] + 12*c1p2*c2p1*c5p1*ss[9] + 12*c1p2*c3p1*c4p1*ss[9] + 12*c1p2*c3p1*c5p1*ss[10] + 12*c1p2*c4p1*c5p1*ss[11] + 12*c2p1*c3p1*c4p2*ss[13] + 12*c2p1*c3p1*c5p2*ss[15] + 12*c2p1*c3p2*c4p1*ss[12] + 12*c2p1*c3p2*c5p1*ss[13] + 12*c2p1*c4p1*c5p2*ss[16] + 12*c2p1*c4p2*c5p1*ss[15] + 12*c2p2*c3p1*c4p1*ss[11] + 12*c2p2*c3p1*c5p1*ss[12] + 12*c2p2*c4p1*c5p1*ss[13] + 12*c3p1*c4p1*c5p2*ss[17] + 12*c3p1*c4p2*c5p1*ss[16] + 12*c3p2*c4p1*c5p1*ss[15] + 4*c0p1*c1p3*ss[3] + 4*c0p1*c2p3*ss[6] + 4*c0p1*c3p3*ss[9] + 4*c0p1*c4p3*ss[12] + 4*c0p1*c5p3*ss[15] + 6*c0p2*c1p2*ss[2] + 6*c0p2*c2p2*ss[4] + 6*c0p2*c3p2*ss[6] + 6*c0p2*c4p2*ss[8] + 6*c0p2*c5p2*ss[10] + 4*c0p3*c1p1*ss[1] + 4*c0p3*c2p1*ss[2] + 4*c0p3*c3p1*ss[3] + 4*c0p3*c4p1*ss[4] + 4*c0p3*c5p1*ss[5] + 4*c1p1*c2p3*ss[7] + 4*c1p1*c3p3*ss[10] + 4*c1p1*c4p3*ss[13] + 4*c1p1*c5p3*ss[16] + 6*c1p2*c2p2*ss[6] + 6*c1p2*c3p2*ss[8] + 6*c1p2*c4p2*ss[10] + 6*c1p2*c5p2*ss[12] + 4*c1p3*c2p1*ss[5] + 4*c1p3*c3p1*ss[6] + 4*c1p3*c4p1*ss[7] + 4*c1p3*c5p1*ss[8] + 4*c2p1*c3p3*ss[11] + 4*c2p1*c4p3*ss[14] + 4*c2p1*c5p3*ss[17] + 6*c2p2*c3p2*ss[10] + 6*c2p2*c4p2*ss[12] + 6*c2p2*c5p2*ss[14] + 4*c2p3*c3p1*ss[9] + 4*c2p3*c4p1*ss[10] + 4*c2p3*c5p1*ss[11] + 4*c3p1*c4p3*ss[15] + 4*c3p1*c5p3*ss[18] + 6*c3p2*c4p2*ss[14] + 6*c3p2*c5p2*ss[16] + 4*c3p3*c4p1*ss[13] + 4*c3p3*c5p1*ss[14] + 4*c4p1*c5p3*ss[19] + 6*c4p2*c5p2*ss[18] + 4*c4p3*c5p1*ss[17] + c1p4*ss[4] + c2p4*ss[8] + c3p4*ss[12] + c4p4*ss[16] + c5p4*ss[20] + c0p4)*sys_part_4;
  grad_0_num += 6*(120*c0p1*c1p1*c2p1*c3p1*c4p1*ss[10] + 120*c0p1*c1p1*c2p1*c3p1*c5p1*ss[11] + 120*c0p1*c1p1*c2p1*c4p1*c5p1*ss[12] + 120*c0p1*c1p1*c3p1*c4p1*c5p1*ss[13] + 120*c0p1*c2p1*c3p1*c4p1*c5p1*ss[14] + 120*c1p1*c2p1*c3p1*c4p1*c5p1*ss[15] + 60*c0p1*c1p1*c2p1*c3p2*ss[9] + 60*c0p1*c1p1*c2p1*c4p2*ss[11] + 60*c0p1*c1p1*c2p1*c5p2*ss[13] + 60*c0p1*c1p1*c2p2*c3p1*ss[8] + 60*c0p1*c1p1*c2p2*c4p1*ss[9] + 60*c0p1*c1p1*c2p2*c5p1*ss[10] + 60*c0p1*c1p1*c3p1*c4p2*ss[12] + 60*c0p1*c1p1*c3p1*c5p2*ss[14] + 60*c0p1*c1p1*c3p2*c4p1*ss[11] + 60*c0p1*c1p1*c3p2*c5p1*ss[12] + 60*c0p1*c1p1*c4p1*c5p2*ss[15] + 60*c0p1*c1p1*c4p2*c5p1*ss[14] + 60*c0p1*c1p2*c2p1*c3p1*ss[7] + 60*c0p1*c1p2*c2p1*c4p1*ss[8] + 60*c0p1*c1p2*c2p1*c5p1*ss[9] + 60*c0p1*c1p2*c3p1*c4p1*ss[9] + 60*c0p1*c1p2*c3p1*c5p1*ss[10] + 60*c0p1*c1p2*c4p1*c5p1*ss[11] + 60*c0p1*c2p1*c3p1*c4p2*ss[13] + 60*c0p1*c2p1*c3p1*c5p2*ss[15] + 60*c0p1*c2p1*c3p2*c4p1*ss[12] + 60*c0p1*c2p1*c3p2*c5p1*ss[13] + 60*c0p1*c2p1*c4p1*c5p2*ss[16] + 60*c0p1*c2p1*c4p2*c5p1*ss[15] + 60*c0p1*c2p2*c3p1*c4p1*ss[11] + 60*c0p1*c2p2*c3p1*c5p1*ss[12] + 60*c0p1*c2p2*c4p1*c5p1*ss[13] + 60*c0p1*c3p1*c4p1*c5p2*ss[17] + 60*c0p1*c3p1*c4p2*c5p1*ss[16] + 60*c0p1*c3p2*c4p1*c5p1*ss[15] + 60*c0p2*c1p1*c2p1*c3p1*ss[6] + 60*c0p2*c1p1*c2p1*c4p1*ss[7] + 60*c0p2*c1p1*c2p1*c5p1*ss[8] + 60*c0p2*c1p1*c3p1*c4p1*ss[8] + 60*c0p2*c1p1*c3p1*c5p1*ss[9] + 60*c0p2*c1p1*c4p1*c5p1*ss[10] + 60*c0p2*c2p1*c3p1*c4p1*ss[9] + 60*c0p2*c2p1*c3p1*c5p1*ss[10] + 60*c0p2*c2p1*c4p1*c5p1*ss[11] + 60*c0p2*c3p1*c4p1*c5p1*ss[12] + 60*c1p1*c2p1*c3p1*c4p2*ss[14] + 60*c1p1*c2p1*c3p1*c5p2*ss[16] + 60*c1p1*c2p1*c3p2*c4p1*ss[13] + 60*c1p1*c2p1*c3p2*c5p1*ss[14] + 60*c1p1*c2p1*c4p1*c5p2*ss[17] + 60*c1p1*c2p1*c4p2*c5p1*ss[16] + 60*c1p1*c2p2*c3p1*c4p1*ss[12] + 60*c1p1*c2p2*c3p1*c5p1*ss[13] + 60*c1p1*c2p2*c4p1*c5p1*ss[14] + 60*c1p1*c3p1*c4p1*c5p2*ss[18] + 60*c1p1*c3p1*c4p2*c5p1*ss[17] + 60*c1p1*c3p2*c4p1*c5p1*ss[16] + 60*c1p2*c2p1*c3p1*c4p1*ss[11] + 60*c1p2*c2p1*c3p1*c5p1*ss[12] + 60*c1p2*c2p1*c4p1*c5p1*ss[13] + 60*c1p2*c3p1*c4p1*c5p1*ss[14] + 60*c2p1*c3p1*c4p1*c5p2*ss[19] + 60*c2p1*c3p1*c4p2*c5p1*ss[18] + 60*c2p1*c3p2*c4p1*c5p1*ss[17] + 60*c2p2*c3p1*c4p1*c5p1*ss[16] + 20*c0p1*c1p1*c2p3*ss[7] + 20*c0p1*c1p1*c3p3*ss[10] + 20*c0p1*c1p1*c4p3*ss[13] + 20*c0p1*c1p1*c5p3*ss[16] + 30*c0p1*c1p2*c2p2*ss[6] + 30*c0p1*c1p2*c3p2*ss[8] + 30*c0p1*c1p2*c4p2*ss[10] + 30*c0p1*c1p2*c5p2*ss[12] + 20*c0p1*c1p3*c2p1*ss[5] + 20*c0p1*c1p3*c3p1*ss[6] + 20*c0p1*c1p3*c4p1*ss[7] + 20*c0p1*c1p3*c5p1*ss[8] + 20*c0p1*c2p1*c3p3*ss[11] + 20*c0p1*c2p1*c4p3*ss[14] + 20*c0p1*c2p1*c5p3*ss[17] + 30*c0p1*c2p2*c3p2*ss[10] + 30*c0p1*c2p2*c4p2*ss[12] + 30*c0p1*c2p2*c5p2*ss[14] + 20*c0p1*c2p3*c3p1*ss[9] + 20*c0p1*c2p3*c4p1*ss[10] + 20*c0p1*c2p3*c5p1*ss[11] + 20*c0p1*c3p1*c4p3*ss[15] + 20*c0p1*c3p1*c5p3*ss[18] + 30*c0p1*c3p2*c4p2*ss[14] + 30*c0p1*c3p2*c5p2*ss[16] + 20*c0p1*c3p3*c4p1*ss[13] + 20*c0p1*c3p3*c5p1*ss[14] + 20*c0p1*c4p1*c5p3*ss[19] + 30*c0p1*c4p2*c5p2*ss[18] + 20*c0p1*c4p3*c5p1*ss[17] + 30*c0p2*c1p1*c2p2*ss[5] + 30*c0p2*c1p1*c3p2*ss[7] + 30*c0p2*c1p1*c4p2*ss[9] + 30*c0p2*c1p1*c5p2*ss[11] + 30*c0p2*c1p2*c2p1*ss[4] + 30*c0p2*c1p2*c3p1*ss[5] + 30*c0p2*c1p2*c4p1*ss[6] + 30*c0p2*c1p2*c5p1*ss[7] + 30*c0p2*c2p1*c3p2*ss[8] + 30*c0p2*c2p1*c4p2*ss[10] + 30*c0p2*c2p1*c5p2*ss[12] + 30*c0p2*c2p2*c3p1*ss[7] + 30*c0p2*c2p2*c4p1*ss[8] + 30*c0p2*c2p2*c5p1*ss[9] + 30*c0p2*c3p1*c4p2*ss[11] + 30*c0p2*c3p1*c5p2*ss[13] + 30*c0p2*c3p2*c4p1*ss[10] + 30*c0p2*c3p2*c5p1*ss[11] + 30*c0p2*c4p1*c5p2*ss[14] + 30*c0p2*c4p2*c5p1*ss[13] + 20*c0p3*c1p1*c2p1*ss[3] + 20*c0p3*c1p1*c3p1*ss[4] + 20*c0p3*c1p1*c4p1*ss[5] + 20*c0p3*c1p1*c5p1*ss[6] + 20*c0p3*c2p1*c3p1*ss[5] + 20*c0p3*c2p1*c4p1*ss[6] + 20*c0p3*c2p1*c5p1*ss[7] + 20*c0p3*c3p1*c4p1*ss[7] + 20*c0p3*c3p1*c5p1*ss[8] + 20*c0p3*c4p1*c5p1*ss[9] + 20*c1p1*c2p1*c3p3*ss[12] + 20*c1p1*c2p1*c4p3*ss[15] + 20*c1p1*c2p1*c5p3*ss[18] + 30*c1p1*c2p2*c3p2*ss[11] + 30*c1p1*c2p2*c4p2*ss[13] + 30*c1p1*c2p2*c5p2*ss[15] + 20*c1p1*c2p3*c3p1*ss[10] + 20*c1p1*c2p3*c4p1*ss[11] + 20*c1p1*c2p3*c5p1*ss[12] + 20*c1p1*c3p1*c4p3*ss[16] + 20*c1p1*c3p1*c5p3*ss[19] + 30*c1p1*c3p2*c4p2*ss[15] + 30*c1p1*c3p2*c5p2*ss[17] + 20*c1p1*c3p3*c4p1*ss[14] + 20*c1p1*c3p3*c5p1*ss[15] + 20*c1p1*c4p1*c5p3*ss[20] + 30*c1p1*c4p2*c5p2*ss[19] + 20*c1p1*c4p3*c5p1*ss[18] + 30*c1p2*c2p1*c3p2*ss[10] + 30*c1p2*c2p1*c4p2*ss[12] + 30*c1p2*c2p1*c5p2*ss[14] + 30*c1p2*c2p2*c3p1*ss[9] + 30*c1p2*c2p2*c4p1*ss[10] + 30*c1p2*c2p2*c5p1*ss[11] + 30*c1p2*c3p1*c4p2*ss[13] + 30*c1p2*c3p1*c5p2*ss[15] + 30*c1p2*c3p2*c4p1*ss[12] + 30*c1p2*c3p2*c5p1*ss[13] + 30*c1p2*c4p1*c5p2*ss[16] + 30*c1p2*c4p2*c5p1*ss[15] + 20*c1p3*c2p1*c3p1*ss[8] + 20*c1p3*c2p1*c4p1*ss[9] + 20*c1p3*c2p1*c5p1*ss[10] + 20*c1p3*c3p1*c4p1*ss[10] + 20*c1p3*c3p1*c5p1*ss[11] + 20*c1p3*c4p1*c5p1*ss[12] + 20*c2p1*c3p1*c4p3*ss[17] + 20*c2p1*c3p1*c5p3*ss[20] + 30*c2p1*c3p2*c4p2*ss[16] + 30*c2p1*c3p2*c5p2*ss[18] + 20*c2p1*c3p3*c4p1*ss[15] + 20*c2p1*c3p3*c5p1*ss[16] + 20*c2p1*c4p1*c5p3*ss[21] + 30*c2p1*c4p2*c5p2*ss[20] + 20*c2p1*c4p3*c5p1*ss[19] + 30*c2p2*c3p1*c4p2*ss[15] + 30*c2p2*c3p1*c5p2*ss[17] + 30*c2p2*c3p2*c4p1*ss[14] + 30*c2p2*c3p2*c5p1*ss[15] + 30*c2p2*c4p1*c5p2*ss[18] + 30*c2p2*c4p2*c5p1*ss[17] + 20*c2p3*c3p1*c4p1*ss[13] + 20*c2p3*c3p1*c5p1*ss[14] + 20*c2p3*c4p1*c5p1*ss[15] + 20*c3p1*c4p1*c5p3*ss[22] + 30*c3p1*c4p2*c5p2*ss[21] + 20*c3p1*c4p3*c5p1*ss[20] + 30*c3p2*c4p1*c5p2*ss[20] + 30*c3p2*c4p2*c5p1*ss[19] + 20*c3p3*c4p1*c5p1*ss[18] + 5*c0p1*c1p4*ss[4] + 5*c0p1*c2p4*ss[8] + 5*c0p1*c3p4*ss[12] + 5*c0p1*c4p4*ss[16] + 5*c0p1*c5p4*ss[20] + 10*c0p2*c1p3*ss[3] + 10*c0p2*c2p3*ss[6] + 10*c0p2*c3p3*ss[9] + 10*c0p2*c4p3*ss[12] + 10*c0p2*c5p3*ss[15] + 10*c0p3*c1p2*ss[2] + 10*c0p3*c2p2*ss[4] + 10*c0p3*c3p2*ss[6] + 10*c0p3*c4p2*ss[8] + 10*c0p3*c5p2*ss[10] + 5*c0p4*c1p1*ss[1] + 5*c0p4*c2p1*ss[2] + 5*c0p4*c3p1*ss[3] + 5*c0p4*c4p1*ss[4] + 5*c0p4*c5p1*ss[5] + 5*c1p1*c2p4*ss[9] + 5*c1p1*c3p4*ss[13] + 5*c1p1*c4p4*ss[17] + 5*c1p1*c5p4*ss[21] + 10*c1p2*c2p3*ss[8] + 10*c1p2*c3p3*ss[11] + 10*c1p2*c4p3*ss[14] + 10*c1p2*c5p3*ss[17] + 10*c1p3*c2p2*ss[7] + 10*c1p3*c3p2*ss[9] + 10*c1p3*c4p2*ss[11] + 10*c1p3*c5p2*ss[13] + 5*c1p4*c2p1*ss[6] + 5*c1p4*c3p1*ss[7] + 5*c1p4*c4p1*ss[8] + 5*c1p4*c5p1*ss[9] + 5*c2p1*c3p4*ss[14] + 5*c2p1*c4p4*ss[18] + 5*c2p1*c5p4*ss[22] + 10*c2p2*c3p3*ss[13] + 10*c2p2*c4p3*ss[16] + 10*c2p2*c5p3*ss[19] + 10*c2p3*c3p2*ss[12] + 10*c2p3*c4p2*ss[14] + 10*c2p3*c5p2*ss[16] + 5*c2p4*c3p1*ss[11] + 5*c2p4*c4p1*ss[12] + 5*c2p4*c5p1*ss[13] + 5*c3p1*c4p4*ss[19] + 5*c3p1*c5p4*ss[23] + 10*c3p2*c4p3*ss[18] + 10*c3p2*c5p3*ss[21] + 10*c3p3*c4p2*ss[17] + 10*c3p3*c5p2*ss[19] + 5*c3p4*c4p1*ss[16] + 5*c3p4*c5p1*ss[17] + 5*c4p1*c5p4*ss[24] + 10*c4p2*c5p3*ss[23] + 10*c4p3*c5p2*ss[22] + 5*c4p4*c5p1*ss[21] + c1p5*ss[5] + c2p5*ss[10] + c3p5*ss[15] + c4p5*ss[20] + c5p5*ss[25] + c0p5)*sys_part_5;
  grad_0_num += c1p1*ss[1];
  grad_0_num += c2p1*ss[2];
  grad_0_num += c3p1*ss[3];
  grad_0_num += c4p1*ss[4];
  grad_0_num += c5p1*ss[5];
  grad_0_num += c0p1;
  grad_0_num += -tmom[1];
  TACC grad_0_den = convert<TACC>(sqrt(square(sys_part_0) + square(sys_part_1) + square(sys_part_2) + square(sys_part_3) + square(sys_part_4) + square(sys_part_5)));
  TACC grad_1_num = convert<TACC>(2*(c0p1*ss[1] + c1p1*ss[2] + c2p1*ss[3] + c3p1*ss[4] + c4p1*ss[5] + c5p1*ss[6])*sys_part_1);
  grad_1_num += 3*(2*c0p1*c1p1*ss[2] + 2*c0p1*c2p1*ss[3] + 2*c0p1*c3p1*ss[4] + 2*c0p1*c4p1*ss[5] + 2*c0p1*c5p1*ss[6] + 2*c1p1*c2p1*ss[4] + 2*c1p1*c3p1*ss[5] + 2*c1p1*c4p1*ss[6] + 2*c1p1*c5p1*ss[7] + 2*c2p1*c3p1*ss[6] + 2*c2p1*c4p1*ss[7] + 2*c2p1*c5p1*ss[8] + 2*c3p1*c4p1*ss[8] + 2*c3p1*c5p1*ss[9] + 2*c4p1*c5p1*ss[10] + c0p2*ss[1] + c1p2*ss[3] + c2p2*ss[5] + c3p2*ss[7] + c4p2*ss[9] + c5p2*ss[11])*sys_part_2;
  grad_1_num += 4*(6*c0p1*c1p1*c2p1*ss[4] + 6*c0p1*c1p1*c3p1*ss[5] + 6*c0p1*c1p1*c4p1*ss[6] + 6*c0p1*c1p1*c5p1*ss[7] + 6*c0p1*c2p1*c3p1*ss[6] + 6*c0p1*c2p1*c4p1*ss[7] + 6*c0p1*c2p1*c5p1*ss[8] + 6*c0p1*c3p1*c4p1*ss[8] + 6*c0p1*c3p1*c5p1*ss[9] + 6*c0p1*c4p1*c5p1*ss[10] + 6*c1p1*c2p1*c3p1*ss[7] + 6*c1p1*c2p1*c4p1*ss[8] + 6*c1p1*c2p1*c5p1*ss[9] + 6*c1p1*c3p1*c4p1*ss[9] + 6*c1p1*c3p1*c5p1*ss[10] + 6*c1p1*c4p1*c5p1*ss[11] + 6*c2p1*c3p1*c4p1*ss[10] + 6*c2p1*c3p1*c5p1*ss[11] + 6*c2p1*c4p1*c5p1*ss[12] + 6*c3p1*c4p1*c5p1*ss[13] + 3*c0p1*c1p2*ss[3] + 3*c0p1*c2p2*ss[5] + 3*c0p1*c3p2*ss[7] + 3*c0p1*c4p2*ss[9] + 3*c0p1*c5p2*ss[11] + 3*c0p2*c1p1*ss[2] + 3*c0p2*c2p1*ss[3] + 3*c0p2*c3p1*ss[4] + 3*c0p2*c4p1*ss[5] + 3*c0p2*c5p1*ss[6] + 3*c1p1*c2p2*ss[6] + 3*c1p1*c3p2*ss[8] + 3*c1p1*c4p2*ss[10] + 3*c1p1*c5p2*ss[12] + 3*c1p2*c2p1*ss[5] + 3*c1p2*c3p1*ss[6] + 3*c1p2*c4p1*ss[7] + 3*c1p2*c5p1*ss[8] + 3*c2p1*c3p2*ss[9] + 3*c2p1*c4p2*ss[11] + 3*c2p1*c5p2*ss[13] + 3*c2p2*c3p1*ss[8] + 3*c2p2*c4p1*ss[9] + 3*c2p2*c5p1*ss[10] + 3*c3p1*c4p2*ss[12] + 3*c3p1*c5p2*ss[14] + 3*c3p2*c4p1*ss[11] + 3*c3p2*c5p1*ss[12] + 3*c4p1*c5p2*ss[15] + 3*c4p2*c5p1*ss[14] + c0p3*ss[1] + c1p3*ss[4] + c2p3*ss[7] + c3p3*ss[10] + c4p3*ss[13] + c5p3*ss[16])*sys_part_3;
  grad_1_num += 5*(24*c0p1*c1p1*c2p1*c3p1*ss[7] + 24*c0p1*c1p1*c2p1*c4p1*ss[8] + 24*c0p1*c1p1*c2p1*c5p1*ss[9] + 24*c0p1*c1p1*c3p1*c4p1*ss[9] + 24*c0p1*c1p1*c3p1*c5p1*ss[10] + 24*c0p1*c1p1*c4p1*c5p1*ss[11] + 24*c0p1*c2p1*c3p1*c4p1*ss[10] + 24*c0p1*c2p1*c3p1*c5p1*ss[11] + 24*c0p1*c2p1*c4p1*c5p1*ss[12] + 24*c0p1*c3p1*c4p1*c5p1*ss[13] + 24*c1p1*c2p1*c3p1*c4p1*ss[11] + 24*c1p1*c2p1*c3p1*c5p1*ss[12] + 24*c1p1*c2p1*c4p1*c5p1*ss[13] + 24*c1p1*c3p1*c4p1*c5p1*ss[14] + 24*c2p1*c3p1*c4p1*c5p1*ss[15] + 12*c0p1*c1p1*c2p2*ss[6] + 12*c0p1*c1p1*c3p2*ss[8] + 12*c0p1*c1p1*c4p2*ss[10] + 12*c0p1*c1p1*c5p2*ss[12] + 12*c0p1*c1p2*c2p1*ss[5] + 12*c0p1*c1p2*c3p1*ss[6] + 12*c0p1*c1p2*c4p1*ss[7] + 12*c0p1*c1p2*c5p1*ss[8] + 12*c0p1*c2p1*c3p2*ss[9] + 12*c0p1*c2p1*c4p2*ss[11] + 12*c0p1*c2p1*c5p2*ss[13] + 12*c0p1*c2p2*c3p1*ss[8] + 12*c0p1*c2p2*c4p1*ss[9] + 12*c0p1*c2p2*c5p1*ss[10] + 12*c0p1*c3p1*c4p2*ss[12] + 12*c0p1*c3p1*c5p2*ss[14] + 12*c0p1*c3p2*c4p1*ss[11] + 12*c0p1*c3p2*c5p1*ss[12] + 12*c0p1*c4p1*c5p2*ss[15] + 12*c0p1*c4p2*c5p1*ss[14] + 12*c0p2*c1p1*c2p1*ss[4] + 12*c0p2*c1p1*c3p1*ss[5] + 12*c0p2*c1p1*c4p1*ss[6] + 12*c0p2*c1p1*c5p1*ss[7] + 12*c0p2*c2p1*c3p1*ss[6] + 12*c0p2*c2p1*c4p1*ss[7] + 12*c0p2*c2p1*c5p1*ss[8] + 12*c0p2*c3p1*c4p1*ss[8] + 12*c0p2*c3p1*c5p1*ss[9] + 12*c0p2*c4p1*c5p1*ss[10] + 12*c1p1*c2p1*c3p2*ss[10] + 12*c1p1*c2p1*c4p2*ss[12] + 12*c1p1*c2p1*c5p2*ss[14] + 12*c1p1*c2p2*c3p1*ss[9] + 12*c1p1*c2p2*c4p1*ss[10] + 12*c1p1*c2p2*c5p1*ss[11] + 12*c1p1*c3p1*c4p2*ss[13] + 12*c1p1*c3p1*c5p2*ss[15] + 12*c1p1*c3p2*c4p1*ss[12] + 12*c1p1*c3p2*c5p1*ss[13] + 12*c1p1*c4p1*c5p2*ss[16] + 12*c1p1*c4p2*c5p1*ss[15] + 12*c1p2*c2p1*c3p1*ss[8] + 12*c1p2*c2p1*c4p1*ss[9] + 12*c1p2*c2p1*c5p1*ss[10] + 12*c1p2*c3p1*c4p1*ss[10] + 12*c1p2*c3p1*c5p1*ss[11] + 12*c1p2*c4p1*c5p1*ss[12] + 12*c2p1*c3p1*c4p2*ss[14] + 12*c2p1*c3p1*c5p2*ss[16] + 12*c2p1*c3p2*c4p1*ss[13] + 12*c2p1*c3p2*c5p1*ss[14] + 12*c2p1*c4p1*c5p2*ss[17] + 12*c2p1*c4p2*c5p1*ss[16] + 12*c2p2*c3p1*c4p1*ss[12] + 12*c2p2*c3p1*c5p1*ss[13] + 12*c2p2*c4p1*c5p1*ss[14] + 12*c3p1*c4p1*c5p2*ss[18] + 12*c3p1*c4p2*c5p1*ss[17] + 12*c3p2*c4p1*c5p1*ss[16] + 4*c0p1*c1p3*ss[4] + 4*c0p1*c2p3*ss[7] + 4*c0p1*c3p3*ss[10] + 4*c0p1*c4p3*ss[13] + 4*c0p1*c5p3*ss[16] + 6*c0p2*c1p2*ss[3] + 6*c0p2*c2p2*ss[5] + 6*c0p2*c3p2*ss[7] + 6*c0p2*c4p2*ss[9] + 6*c0p2*c5p2*ss[11] + 4*c0p3*c1p1*ss[2] + 4*c0p3*c2p1*ss[3] + 4*c0p3*c3p1*ss[4] + 4*c0p3*c4p1*ss[5] + 4*c0p3*c5p1*ss[6] + 4*c1p1*c2p3*ss[8] + 4*c1p1*c3p3*ss[11] + 4*c1p1*c4p3*ss[14] + 4*c1p1*c5p3*ss[17] + 6*c1p2*c2p2*ss[7] + 6*c1p2*c3p2*ss[9] + 6*c1p2*c4p2*ss[11] + 6*c1p2*c5p2*ss[13] + 4*c1p3*c2p1*ss[6] + 4*c1p3*c3p1*ss[7] + 4*c1p3*c4p1*ss[8] + 4*c1p3*c5p1*ss[9] + 4*c2p1*c3p3*ss[12] + 4*c2p1*c4p3*ss[15] + 4*c2p1*c5p3*ss[18] + 6*c2p2*c3p2*ss[11] + 6*c2p2*c4p2*ss[13] + 6*c2p2*c5p2*ss[15] + 4*c2p3*c3p1*ss[10] + 4*c2p3*c4p1*ss[11] + 4*c2p3*c5p1*ss[12] + 4*c3p1*c4p3*ss[16] + 4*c3p1*c5p3*ss[19] + 6*c3p2*c4p2*ss[15] + 6*c3p2*c5p2*ss[17] + 4*c3p3*c4p1*ss[14] + 4*c3p3*c5p1*ss[15] + 4*c4p1*c5p3*ss[20] + 6*c4p2*c5p2*ss[19] + 4*c4p3*c5p1*ss[18] + c0p4*ss[1] + c1p4*ss[5] + c2p4*ss[9] + c3p4*ss[13] + c4p4*ss[17] + c5p4*ss[21])*sys_part_4;
  grad_1_num += 6*(120*c0p1*c1p1*c2p1*c3p1*c4p1*ss[11] + 120*c0p1*c1p1*c2p1*c3p1*c5p1*ss[12] + 120*c0p1*c1p1*c2p1*c4p1*c5p1*ss[13] + 120*c0p1*c1p1*c3p1*c4p1*c5p1*ss[14] + 120*c0p1*c2p1*c3p1*c4p1*c5p1*ss[15] + 120*c1p1*c2p1*c3p1*c4p1*c5p1*ss[16] + 60*c0p1*c1p1*c2p1*c3p2*ss[10] + 60*c0p1*c1p1*c2p1*c4p2*ss[12] + 60*c0p1*c1p1*c2p1*c5p2*ss[14] + 60*c0p1*c1p1*c2p2*c3p1*ss[9] + 60*c0p1*c1p1*c2p2*c4p1*ss[10] + 60*c0p1*c1p1*c2p2*c5p1*ss[11] + 60*c0p1*c1p1*c3p1*c4p2*ss[13] + 60*c0p1*c1p1*c3p1*c5p2*ss[15] + 60*c0p1*c1p1*c3p2*c4p1*ss[12] + 60*c0p1*c1p1*c3p2*c5p1*ss[13] + 60*c0p1*c1p1*c4p1*c5p2*ss[16] + 60*c0p1*c1p1*c4p2*c5p1*ss[15] + 60*c0p1*c1p2*c2p1*c3p1*ss[8] + 60*c0p1*c1p2*c2p1*c4p1*ss[9] + 60*c0p1*c1p2*c2p1*c5p1*ss[10] + 60*c0p1*c1p2*c3p1*c4p1*ss[10] + 60*c0p1*c1p2*c3p1*c5p1*ss[11] + 60*c0p1*c1p2*c4p1*c5p1*ss[12] + 60*c0p1*c2p1*c3p1*c4p2*ss[14] + 60*c0p1*c2p1*c3p1*c5p2*ss[16] + 60*c0p1*c2p1*c3p2*c4p1*ss[13] + 60*c0p1*c2p1*c3p2*c5p1*ss[14] + 60*c0p1*c2p1*c4p1*c5p2*ss[17] + 60*c0p1*c2p1*c4p2*c5p1*ss[16] + 60*c0p1*c2p2*c3p1*c4p1*ss[12] + 60*c0p1*c2p2*c3p1*c5p1*ss[13] + 60*c0p1*c2p2*c4p1*c5p1*ss[14] + 60*c0p1*c3p1*c4p1*c5p2*ss[18] + 60*c0p1*c3p1*c4p2*c5p1*ss[17] + 60*c0p1*c3p2*c4p1*c5p1*ss[16] + 60*c0p2*c1p1*c2p1*c3p1*ss[7] + 60*c0p2*c1p1*c2p1*c4p1*ss[8] + 60*c0p2*c1p1*c2p1*c5p1*ss[9] + 60*c0p2*c1p1*c3p1*c4p1*ss[9] + 60*c0p2*c1p1*c3p1*c5p1*ss[10] + 60*c0p2*c1p1*c4p1*c5p1*ss[11] + 60*c0p2*c2p1*c3p1*c4p1*ss[10] + 60*c0p2*c2p1*c3p1*c5p1*ss[11] + 60*c0p2*c2p1*c4p1*c5p1*ss[12] + 60*c0p2*c3p1*c4p1*c5p1*ss[13] + 60*c1p1*c2p1*c3p1*c4p2*ss[15] + 60*c1p1*c2p1*c3p1*c5p2*ss[17] + 60*c1p1*c2p1*c3p2*c4p1*ss[14] + 60*c1p1*c2p1*c3p2*c5p1*ss[15] + 60*c1p1*c2p1*c4p1*c5p2*ss[18] + 60*c1p1*c2p1*c4p2*c5p1*ss[17] + 60*c1p1*c2p2*c3p1*c4p1*ss[13] + 60*c1p1*c2p2*c3p1*c5p1*ss[14] + 60*c1p1*c2p2*c4p1*c5p1*ss[15] + 60*c1p1*c3p1*c4p1*c5p2*ss[19] + 60*c1p1*c3p1*c4p2*c5p1*ss[18] + 60*c1p1*c3p2*c4p1*c5p1*ss[17] + 60*c1p2*c2p1*c3p1*c4p1*ss[12] + 60*c1p2*c2p1*c3p1*c5p1*ss[13] + 60*c1p2*c2p1*c4p1*c5p1*ss[14] + 60*c1p2*c3p1*c4p1*c5p1*ss[15] + 60*c2p1*c3p1*c4p1*c5p2*ss[20] + 60*c2p1*c3p1*c4p2*c5p1*ss[19] + 60*c2p1*c3p2*c4p1*c5p1*ss[18] + 60*c2p2*c3p1*c4p1*c5p1*ss[17] + 20*c0p1*c1p1*c2p3*ss[8] + 20*c0p1*c1p1*c3p3*ss[11] + 20*c0p1*c1p1*c4p3*ss[14] + 20*c0p1*c1p1*c5p3*ss[17] + 30*c0p1*c1p2*c2p2*ss[7] + 30*c0p1*c1p2*c3p2*ss[9] + 30*c0p1*c1p2*c4p2*ss[11] + 30*c0p1*c1p2*c5p2*ss[13] + 20*c0p1*c1p3*c2p1*ss[6] + 20*c0p1*c1p3*c3p1*ss[7] + 20*c0p1*c1p3*c4p1*ss[8] + 20*c0p1*c1p3*c5p1*ss[9] + 20*c0p1*c2p1*c3p3*ss[12] + 20*c0p1*c2p1*c4p3*ss[15] + 20*c0p1*c2p1*c5p3*ss[18] + 30*c0p1*c2p2*c3p2*ss[11] + 30*c0p1*c2p2*c4p2*ss[13] + 30*c0p1*c2p2*c5p2*ss[15] + 20*c0p1*c2p3*c3p1*ss[10] + 20*c0p1*c2p3*c4p1*ss[11] + 20*c0p1*c2p3*c5p1*ss[12] + 20*c0p1*c3p1*c4p3*ss[16] + 20*c0p1*c3p1*c5p3*ss[19] + 30*c0p1*c3p2*c4p2*ss[15] + 30*c0p1*c3p2*c5p2*ss[17] + 20*c0p1*c3p3*c4p1*ss[14] + 20*c0p1*c3p3*c5p1*ss[15] + 20*c0p1*c4p1*c5p3*ss[20] + 30*c0p1*c4p2*c5p2*ss[19] + 20*c0p1*c4p3*c5p1*ss[18] + 30*c0p2*c1p1*c2p2*ss[6] + 30*c0p2*c1p1*c3p2*ss[8] + 30*c0p2*c1p1*c4p2*ss[10] + 30*c0p2*c1p1*c5p2*ss[12] + 30*c0p2*c1p2*c2p1*ss[5] + 30*c0p2*c1p2*c3p1*ss[6] + 30*c0p2*c1p2*c4p1*ss[7] + 30*c0p2*c1p2*c5p1*ss[8] + 30*c0p2*c2p1*c3p2*ss[9] + 30*c0p2*c2p1*c4p2*ss[11] + 30*c0p2*c2p1*c5p2*ss[13] + 30*c0p2*c2p2*c3p1*ss[8] + 30*c0p2*c2p2*c4p1*ss[9] + 30*c0p2*c2p2*c5p1*ss[10] + 30*c0p2*c3p1*c4p2*ss[12] + 30*c0p2*c3p1*c5p2*ss[14] + 30*c0p2*c3p2*c4p1*ss[11] + 30*c0p2*c3p2*c5p1*ss[12] + 30*c0p2*c4p1*c5p2*ss[15] + 30*c0p2*c4p2*c5p1*ss[14] + 20*c0p3*c1p1*c2p1*ss[4] + 20*c0p3*c1p1*c3p1*ss[5] + 20*c0p3*c1p1*c4p1*ss[6] + 20*c0p3*c1p1*c5p1*ss[7] + 20*c0p3*c2p1*c3p1*ss[6] + 20*c0p3*c2p1*c4p1*ss[7] + 20*c0p3*c2p1*c5p1*ss[8] + 20*c0p3*c3p1*c4p1*ss[8] + 20*c0p3*c3p1*c5p1*ss[9] + 20*c0p3*c4p1*c5p1*ss[10] + 20*c1p1*c2p1*c3p3*ss[13] + 20*c1p1*c2p1*c4p3*ss[16] + 20*c1p1*c2p1*c5p3*ss[19] + 30*c1p1*c2p2*c3p2*ss[12] + 30*c1p1*c2p2*c4p2*ss[14] + 30*c1p1*c2p2*c5p2*ss[16] + 20*c1p1*c2p3*c3p1*ss[11] + 20*c1p1*c2p3*c4p1*ss[12] + 20*c1p1*c2p3*c5p1*ss[13] + 20*c1p1*c3p1*c4p3*ss[17] + 20*c1p1*c3p1*c5p3*ss[20] + 30*c1p1*c3p2*c4p2*ss[16] + 30*c1p1*c3p2*c5p2*ss[18] + 20*c1p1*c3p3*c4p1*ss[15] + 20*c1p1*c3p3*c5p1*ss[16] + 20*c1p1*c4p1*c5p3*ss[21] + 30*c1p1*c4p2*c5p2*ss[20] + 20*c1p1*c4p3*c5p1*ss[19] + 30*c1p2*c2p1*c3p2*ss[11] + 30*c1p2*c2p1*c4p2*ss[13] + 30*c1p2*c2p1*c5p2*ss[15] + 30*c1p2*c2p2*c3p1*ss[10] + 30*c1p2*c2p2*c4p1*ss[11] + 30*c1p2*c2p2*c5p1*ss[12] + 30*c1p2*c3p1*c4p2*ss[14] + 30*c1p2*c3p1*c5p2*ss[16] + 30*c1p2*c3p2*c4p1*ss[13] + 30*c1p2*c3p2*c5p1*ss[14] + 30*c1p2*c4p1*c5p2*ss[17] + 30*c1p2*c4p2*c5p1*ss[16] + 20*c1p3*c2p1*c3p1*ss[9] + 20*c1p3*c2p1*c4p1*ss[10] + 20*c1p3*c2p1*c5p1*ss[11] + 20*c1p3*c3p1*c4p1*ss[11] + 20*c1p3*c3p1*c5p1*ss[12] + 20*c1p3*c4p1*c5p1*ss[13] + 20*c2p1*c3p1*c4p3*ss[18] + 20*c2p1*c3p1*c5p3*ss[21] + 30*c2p1*c3p2*c4p2*ss[17] + 30*c2p1*c3p2*c5p2*ss[19] + 20*c2p1*c3p3*c4p1*ss[16] + 20*c2p1*c3p3*c5p1*ss[17] + 20*c2p1*c4p1*c5p3*ss[22] + 30*c2p1*c4p2*c5p2*ss[21] + 20*c2p1*c4p3*c5p1*ss[20] + 30*c2p2*c3p1*c4p2*ss[16] + 30*c2p2*c3p1*c5p2*ss[18] + 30*c2p2*c3p2*c4p1*ss[15] + 30*c2p2*c3p2*c5p1*ss[16] + 30*c2p2*c4p1*c5p2*ss[19] + 30*c2p2*c4p2*c5p1*ss[18] + 20*c2p3*c3p1*c4p1*ss[14] + 20*c2p3*c3p1*c5p1*ss[15] + 20*c2p3*c4p1*c5p1*ss[16] + 20*c3p1*c4p1*c5p3*ss[23] + 30*c3p1*c4p2*c5p2*ss[22] + 20*c3p1*c4p3*c5p1*ss[21] + 30*c3p2*c4p1*c5p2*ss[21] + 30*c3p2*c4p2*c5p1*ss[20] + 20*c3p3*c4p1*c5p1*ss[19] + 5*c0p1*c1p4*ss[5] + 5*c0p1*c2p4*ss[9] + 5*c0p1*c3p4*ss[13] + 5*c0p1*c4p4*ss[17] + 5*c0p1*c5p4*ss[21] + 10*c0p2*c1p3*ss[4] + 10*c0p2*c2p3*ss[7] + 10*c0p2*c3p3*ss[10] + 10*c0p2*c4p3*ss[13] + 10*c0p2*c5p3*ss[16] + 10*c0p3*c1p2*ss[3] + 10*c0p3*c2p2*ss[5] + 10*c0p3*c3p2*ss[7] + 10*c0p3*c4p2*ss[9] + 10*c0p3*c5p2*ss[11] + 5*c0p4*c1p1*ss[2] + 5*c0p4*c2p1*ss[3] + 5*c0p4*c3p1*ss[4] + 5*c0p4*c4p1*ss[5] + 5*c0p4*c5p1*ss[6] + 5*c1p1*c2p4*ss[10] + 5*c1p1*c3p4*ss[14] + 5*c1p1*c4p4*ss[18] + 5*c1p1*c5p4*ss[22] + 10*c1p2*c2p3*ss[9] + 10*c1p2*c3p3*ss[12] + 10*c1p2*c4p3*ss[15] + 10*c1p2*c5p3*ss[18] + 10*c1p3*c2p2*ss[8] + 10*c1p3*c3p2*ss[10] + 10*c1p3*c4p2*ss[12] + 10*c1p3*c5p2*ss[14] + 5*c1p4*c2p1*ss[7] + 5*c1p4*c3p1*ss[8] + 5*c1p4*c4p1*ss[9] + 5*c1p4*c5p1*ss[10] + 5*c2p1*c3p4*ss[15] + 5*c2p1*c4p4*ss[19] + 5*c2p1*c5p4*ss[23] + 10*c2p2*c3p3*ss[14] + 10*c2p2*c4p3*ss[17] + 10*c2p2*c5p3*ss[20] + 10*c2p3*c3p2*ss[13] + 10*c2p3*c4p2*ss[15] + 10*c2p3*c5p2*ss[17] + 5*c2p4*c3p1*ss[12] + 5*c2p4*c4p1*ss[13] + 5*c2p4*c5p1*ss[14] + 5*c3p1*c4p4*ss[20] + 5*c3p1*c5p4*ss[24] + 10*c3p2*c4p3*ss[19] + 10*c3p2*c5p3*ss[22] + 10*c3p3*c4p2*ss[18] + 10*c3p3*c5p2*ss[20] + 5*c3p4*c4p1*ss[17] + 5*c3p4*c5p1*ss[18] + 5*c4p1*c5p4*ss[25] + 10*c4p2*c5p3*ss[24] + 10*c4p3*c5p2*ss[23] + 5*c4p4*c5p1*ss[22] + c0p5*ss[1] + c1p5*ss[6] + c2p5*ss[11] + c3p5*ss[16] + c4p5*ss[21] + c5p5*ss[26])*sys_part_5;
  grad_1_num += ss[1]*sys_part_0;
  TACC grad_1_den = convert<TACC>(sqrt(square(sys_part_0) + square(sys_part_1) + square(sys_part_2) + square(sys_part_3) + square(sys_part_4) + square(sys_part_5)));
  TACC grad_2_num = convert<TACC>(2*(c0p1*ss[2] + c1p1*ss[3] + c2p1*ss[4] + c3p1*ss[5] + c4p1*ss[6] + c5p1*ss[7])*sys_part_1);
  grad_2_num += 3*(2*c0p1*c1p1*ss[3] + 2*c0p1*c2p1*ss[4] + 2*c0p1*c3p1*ss[5] + 2*c0p1*c4p1*ss[6] + 2*c0p1*c5p1*ss[7] + 2*c1p1*c2p1*ss[5] + 2*c1p1*c3p1*ss[6] + 2*c1p1*c4p1*ss[7] + 2*c1p1*c5p1*ss[8] + 2*c2p1*c3p1*ss[7] + 2*c2p1*c4p1*ss[8] + 2*c2p1*c5p1*ss[9] + 2*c3p1*c4p1*ss[9] + 2*c3p1*c5p1*ss[10] + 2*c4p1*c5p1*ss[11] + c0p2*ss[2] + c1p2*ss[4] + c2p2*ss[6] + c3p2*ss[8] + c4p2*ss[10] + c5p2*ss[12])*sys_part_2;
  grad_2_num += 4*(6*c0p1*c1p1*c2p1*ss[5] + 6*c0p1*c1p1*c3p1*ss[6] + 6*c0p1*c1p1*c4p1*ss[7] + 6*c0p1*c1p1*c5p1*ss[8] + 6*c0p1*c2p1*c3p1*ss[7] + 6*c0p1*c2p1*c4p1*ss[8] + 6*c0p1*c2p1*c5p1*ss[9] + 6*c0p1*c3p1*c4p1*ss[9] + 6*c0p1*c3p1*c5p1*ss[10] + 6*c0p1*c4p1*c5p1*ss[11] + 6*c1p1*c2p1*c3p1*ss[8] + 6*c1p1*c2p1*c4p1*ss[9] + 6*c1p1*c2p1*c5p1*ss[10] + 6*c1p1*c3p1*c4p1*ss[10] + 6*c1p1*c3p1*c5p1*ss[11] + 6*c1p1*c4p1*c5p1*ss[12] + 6*c2p1*c3p1*c4p1*ss[11] + 6*c2p1*c3p1*c5p1*ss[12] + 6*c2p1*c4p1*c5p1*ss[13] + 6*c3p1*c4p1*c5p1*ss[14] + 3*c0p1*c1p2*ss[4] + 3*c0p1*c2p2*ss[6] + 3*c0p1*c3p2*ss[8] + 3*c0p1*c4p2*ss[10] + 3*c0p1*c5p2*ss[12] + 3*c0p2*c1p1*ss[3] + 3*c0p2*c2p1*ss[4] + 3*c0p2*c3p1*ss[5] + 3*c0p2*c4p1*ss[6] + 3*c0p2*c5p1*ss[7] + 3*c1p1*c2p2*ss[7] + 3*c1p1*c3p2*ss[9] + 3*c1p1*c4p2*ss[11] + 3*c1p1*c5p2*ss[13] + 3*c1p2*c2p1*ss[6] + 3*c1p2*c3p1*ss[7] + 3*c1p2*c4p1*ss[8] + 3*c1p2*c5p1*ss[9] + 3*c2p1*c3p2*ss[10] + 3*c2p1*c4p2*ss[12] + 3*c2p1*c5p2*ss[14] + 3*c2p2*c3p1*ss[9] + 3*c2p2*c4p1*ss[10] + 3*c2p2*c5p1*ss[11] + 3*c3p1*c4p2*ss[13] + 3*c3p1*c5p2*ss[15] + 3*c3p2*c4p1*ss[12] + 3*c3p2*c5p1*ss[13] + 3*c4p1*c5p2*ss[16] + 3*c4p2*c5p1*ss[15] + c0p3*ss[2] + c1p3*ss[5] + c2p3*ss[8] + c3p3*ss[11] + c4p3*ss[14] + c5p3*ss[17])*sys_part_3;
  grad_2_num += 5*(24*c0p1*c1p1*c2p1*c3p1*ss[8] + 24*c0p1*c1p1*c2p1*c4p1*ss[9] + 24*c0p1*c1p1*c2p1*c5p1*ss[10] + 24*c0p1*c1p1*c3p1*c4p1*ss[10] + 24*c0p1*c1p1*c3p1*c5p1*ss[11] + 24*c0p1*c1p1*c4p1*c5p1*ss[12] + 24*c0p1*c2p1*c3p1*c4p1*ss[11] + 24*c0p1*c2p1*c3p1*c5p1*ss[12] + 24*c0p1*c2p1*c4p1*c5p1*ss[13] + 24*c0p1*c3p1*c4p1*c5p1*ss[14] + 24*c1p1*c2p1*c3p1*c4p1*ss[12] + 24*c1p1*c2p1*c3p1*c5p1*ss[13] + 24*c1p1*c2p1*c4p1*c5p1*ss[14] + 24*c1p1*c3p1*c4p1*c5p1*ss[15] + 24*c2p1*c3p1*c4p1*c5p1*ss[16] + 12*c0p1*c1p1*c2p2*ss[7] + 12*c0p1*c1p1*c3p2*ss[9] + 12*c0p1*c1p1*c4p2*ss[11] + 12*c0p1*c1p1*c5p2*ss[13] + 12*c0p1*c1p2*c2p1*ss[6] + 12*c0p1*c1p2*c3p1*ss[7] + 12*c0p1*c1p2*c4p1*ss[8] + 12*c0p1*c1p2*c5p1*ss[9] + 12*c0p1*c2p1*c3p2*ss[10] + 12*c0p1*c2p1*c4p2*ss[12] + 12*c0p1*c2p1*c5p2*ss[14] + 12*c0p1*c2p2*c3p1*ss[9] + 12*c0p1*c2p2*c4p1*ss[10] + 12*c0p1*c2p2*c5p1*ss[11] + 12*c0p1*c3p1*c4p2*ss[13] + 12*c0p1*c3p1*c5p2*ss[15] + 12*c0p1*c3p2*c4p1*ss[12] + 12*c0p1*c3p2*c5p1*ss[13] + 12*c0p1*c4p1*c5p2*ss[16] + 12*c0p1*c4p2*c5p1*ss[15] + 12*c0p2*c1p1*c2p1*ss[5] + 12*c0p2*c1p1*c3p1*ss[6] + 12*c0p2*c1p1*c4p1*ss[7] + 12*c0p2*c1p1*c5p1*ss[8] + 12*c0p2*c2p1*c3p1*ss[7] + 12*c0p2*c2p1*c4p1*ss[8] + 12*c0p2*c2p1*c5p1*ss[9] + 12*c0p2*c3p1*c4p1*ss[9] + 12*c0p2*c3p1*c5p1*ss[10] + 12*c0p2*c4p1*c5p1*ss[11] + 12*c1p1*c2p1*c3p2*ss[11] + 12*c1p1*c2p1*c4p2*ss[13] + 12*c1p1*c2p1*c5p2*ss[15] + 12*c1p1*c2p2*c3p1*ss[10] + 12*c1p1*c2p2*c4p1*ss[11] + 12*c1p1*c2p2*c5p1*ss[12] + 12*c1p1*c3p1*c4p2*ss[14] + 12*c1p1*c3p1*c5p2*ss[16] + 12*c1p1*c3p2*c4p1*ss[13] + 12*c1p1*c3p2*c5p1*ss[14] + 12*c1p1*c4p1*c5p2*ss[17] + 12*c1p1*c4p2*c5p1*ss[16] + 12*c1p2*c2p1*c3p1*ss[9] + 12*c1p2*c2p1*c4p1*ss[10] + 12*c1p2*c2p1*c5p1*ss[11] + 12*c1p2*c3p1*c4p1*ss[11] + 12*c1p2*c3p1*c5p1*ss[12] + 12*c1p2*c4p1*c5p1*ss[13] + 12*c2p1*c3p1*c4p2*ss[15] + 12*c2p1*c3p1*c5p2*ss[17] + 12*c2p1*c3p2*c4p1*ss[14] + 12*c2p1*c3p2*c5p1*ss[15] + 12*c2p1*c4p1*c5p2*ss[18] + 12*c2p1*c4p2*c5p1*ss[17] + 12*c2p2*c3p1*c4p1*ss[13] + 12*c2p2*c3p1*c5p1*ss[14] + 12*c2p2*c4p1*c5p1*ss[15] + 12*c3p1*c4p1*c5p2*ss[19] + 12*c3p1*c4p2*c5p1*ss[18] + 12*c3p2*c4p1*c5p1*ss[17] + 4*c0p1*c1p3*ss[5] + 4*c0p1*c2p3*ss[8] + 4*c0p1*c3p3*ss[11] + 4*c0p1*c4p3*ss[14] + 4*c0p1*c5p3*ss[17] + 6*c0p2*c1p2*ss[4] + 6*c0p2*c2p2*ss[6] + 6*c0p2*c3p2*ss[8] + 6*c0p2*c4p2*ss[10] + 6*c0p2*c5p2*ss[12] + 4*c0p3*c1p1*ss[3] + 4*c0p3*c2p1*ss[4] + 4*c0p3*c3p1*ss[5] + 4*c0p3*c4p1*ss[6] + 4*c0p3*c5p1*ss[7] + 4*c1p1*c2p3*ss[9] + 4*c1p1*c3p3*ss[12] + 4*c1p1*c4p3*ss[15] + 4*c1p1*c5p3*ss[18] + 6*c1p2*c2p2*ss[8] + 6*c1p2*c3p2*ss[10] + 6*c1p2*c4p2*ss[12] + 6*c1p2*c5p2*ss[14] + 4*c1p3*c2p1*ss[7] + 4*c1p3*c3p1*ss[8] + 4*c1p3*c4p1*ss[9] + 4*c1p3*c5p1*ss[10] + 4*c2p1*c3p3*ss[13] + 4*c2p1*c4p3*ss[16] + 4*c2p1*c5p3*ss[19] + 6*c2p2*c3p2*ss[12] + 6*c2p2*c4p2*ss[14] + 6*c2p2*c5p2*ss[16] + 4*c2p3*c3p1*ss[11] + 4*c2p3*c4p1*ss[12] + 4*c2p3*c5p1*ss[13] + 4*c3p1*c4p3*ss[17] + 4*c3p1*c5p3*ss[20] + 6*c3p2*c4p2*ss[16] + 6*c3p2*c5p2*ss[18] + 4*c3p3*c4p1*ss[15] + 4*c3p3*c5p1*ss[16] + 4*c4p1*c5p3*ss[21] + 6*c4p2*c5p2*ss[20] + 4*c4p3*c5p1*ss[19] + c0p4*ss[2] + c1p4*ss[6] + c2p4*ss[10] + c3p4*ss[14] + c4p4*ss[18] + c5p4*ss[22])*sys_part_4;
  grad_2_num += 6*(120*c0p1*c1p1*c2p1*c3p1*c4p1*ss[12] + 120*c0p1*c1p1*c2p1*c3p1*c5p1*ss[13] + 120*c0p1*c1p1*c2p1*c4p1*c5p1*ss[14] + 120*c0p1*c1p1*c3p1*c4p1*c5p1*ss[15] + 120*c0p1*c2p1*c3p1*c4p1*c5p1*ss[16] + 120*c1p1*c2p1*c3p1*c4p1*c5p1*ss[17] + 60*c0p1*c1p1*c2p1*c3p2*ss[11] + 60*c0p1*c1p1*c2p1*c4p2*ss[13] + 60*c0p1*c1p1*c2p1*c5p2*ss[15] + 60*c0p1*c1p1*c2p2*c3p1*ss[10] + 60*c0p1*c1p1*c2p2*c4p1*ss[11] + 60*c0p1*c1p1*c2p2*c5p1*ss[12] + 60*c0p1*c1p1*c3p1*c4p2*ss[14] + 60*c0p1*c1p1*c3p1*c5p2*ss[16] + 60*c0p1*c1p1*c3p2*c4p1*ss[13] + 60*c0p1*c1p1*c3p2*c5p1*ss[14] + 60*c0p1*c1p1*c4p1*c5p2*ss[17] + 60*c0p1*c1p1*c4p2*c5p1*ss[16] + 60*c0p1*c1p2*c2p1*c3p1*ss[9] + 60*c0p1*c1p2*c2p1*c4p1*ss[10] + 60*c0p1*c1p2*c2p1*c5p1*ss[11] + 60*c0p1*c1p2*c3p1*c4p1*ss[11] + 60*c0p1*c1p2*c3p1*c5p1*ss[12] + 60*c0p1*c1p2*c4p1*c5p1*ss[13] + 60*c0p1*c2p1*c3p1*c4p2*ss[15] + 60*c0p1*c2p1*c3p1*c5p2*ss[17] + 60*c0p1*c2p1*c3p2*c4p1*ss[14] + 60*c0p1*c2p1*c3p2*c5p1*ss[15] + 60*c0p1*c2p1*c4p1*c5p2*ss[18] + 60*c0p1*c2p1*c4p2*c5p1*ss[17] + 60*c0p1*c2p2*c3p1*c4p1*ss[13] + 60*c0p1*c2p2*c3p1*c5p1*ss[14] + 60*c0p1*c2p2*c4p1*c5p1*ss[15] + 60*c0p1*c3p1*c4p1*c5p2*ss[19] + 60*c0p1*c3p1*c4p2*c5p1*ss[18] + 60*c0p1*c3p2*c4p1*c5p1*ss[17] + 60*c0p2*c1p1*c2p1*c3p1*ss[8] + 60*c0p2*c1p1*c2p1*c4p1*ss[9] + 60*c0p2*c1p1*c2p1*c5p1*ss[10] + 60*c0p2*c1p1*c3p1*c4p1*ss[10] + 60*c0p2*c1p1*c3p1*c5p1*ss[11] + 60*c0p2*c1p1*c4p1*c5p1*ss[12] + 60*c0p2*c2p1*c3p1*c4p1*ss[11] + 60*c0p2*c2p1*c3p1*c5p1*ss[12] + 60*c0p2*c2p1*c4p1*c5p1*ss[13] + 60*c0p2*c3p1*c4p1*c5p1*ss[14] + 60*c1p1*c2p1*c3p1*c4p2*ss[16] + 60*c1p1*c2p1*c3p1*c5p2*ss[18] + 60*c1p1*c2p1*c3p2*c4p1*ss[15] + 60*c1p1*c2p1*c3p2*c5p1*ss[16] + 60*c1p1*c2p1*c4p1*c5p2*ss[19] + 60*c1p1*c2p1*c4p2*c5p1*ss[18] + 60*c1p1*c2p2*c3p1*c4p1*ss[14] + 60*c1p1*c2p2*c3p1*c5p1*ss[15] + 60*c1p1*c2p2*c4p1*c5p1*ss[16] + 60*c1p1*c3p1*c4p1*c5p2*ss[20] + 60*c1p1*c3p1*c4p2*c5p1*ss[19] + 60*c1p1*c3p2*c4p1*c5p1*ss[18] + 60*c1p2*c2p1*c3p1*c4p1*ss[13] + 60*c1p2*c2p1*c3p1*c5p1*ss[14] + 60*c1p2*c2p1*c4p1*c5p1*ss[15] + 60*c1p2*c3p1*c4p1*c5p1*ss[16] + 60*c2p1*c3p1*c4p1*c5p2*ss[21] + 60*c2p1*c3p1*c4p2*c5p1*ss[20] + 60*c2p1*c3p2*c4p1*c5p1*ss[19] + 60*c2p2*c3p1*c4p1*c5p1*ss[18] + 20*c0p1*c1p1*c2p3*ss[9] + 20*c0p1*c1p1*c3p3*ss[12] + 20*c0p1*c1p1*c4p3*ss[15] + 20*c0p1*c1p1*c5p3*ss[18] + 30*c0p1*c1p2*c2p2*ss[8] + 30*c0p1*c1p2*c3p2*ss[10] + 30*c0p1*c1p2*c4p2*ss[12] + 30*c0p1*c1p2*c5p2*ss[14] + 20*c0p1*c1p3*c2p1*ss[7] + 20*c0p1*c1p3*c3p1*ss[8] + 20*c0p1*c1p3*c4p1*ss[9] + 20*c0p1*c1p3*c5p1*ss[10] + 20*c0p1*c2p1*c3p3*ss[13] + 20*c0p1*c2p1*c4p3*ss[16] + 20*c0p1*c2p1*c5p3*ss[19] + 30*c0p1*c2p2*c3p2*ss[12] + 30*c0p1*c2p2*c4p2*ss[14] + 30*c0p1*c2p2*c5p2*ss[16] + 20*c0p1*c2p3*c3p1*ss[11] + 20*c0p1*c2p3*c4p1*ss[12] + 20*c0p1*c2p3*c5p1*ss[13] + 20*c0p1*c3p1*c4p3*ss[17] + 20*c0p1*c3p1*c5p3*ss[20] + 30*c0p1*c3p2*c4p2*ss[16] + 30*c0p1*c3p2*c5p2*ss[18] + 20*c0p1*c3p3*c4p1*ss[15] + 20*c0p1*c3p3*c5p1*ss[16] + 20*c0p1*c4p1*c5p3*ss[21] + 30*c0p1*c4p2*c5p2*ss[20] + 20*c0p1*c4p3*c5p1*ss[19] + 30*c0p2*c1p1*c2p2*ss[7] + 30*c0p2*c1p1*c3p2*ss[9] + 30*c0p2*c1p1*c4p2*ss[11] + 30*c0p2*c1p1*c5p2*ss[13] + 30*c0p2*c1p2*c2p1*ss[6] + 30*c0p2*c1p2*c3p1*ss[7] + 30*c0p2*c1p2*c4p1*ss[8] + 30*c0p2*c1p2*c5p1*ss[9] + 30*c0p2*c2p1*c3p2*ss[10] + 30*c0p2*c2p1*c4p2*ss[12] + 30*c0p2*c2p1*c5p2*ss[14] + 30*c0p2*c2p2*c3p1*ss[9] + 30*c0p2*c2p2*c4p1*ss[10] + 30*c0p2*c2p2*c5p1*ss[11] + 30*c0p2*c3p1*c4p2*ss[13] + 30*c0p2*c3p1*c5p2*ss[15] + 30*c0p2*c3p2*c4p1*ss[12] + 30*c0p2*c3p2*c5p1*ss[13] + 30*c0p2*c4p1*c5p2*ss[16] + 30*c0p2*c4p2*c5p1*ss[15] + 20*c0p3*c1p1*c2p1*ss[5] + 20*c0p3*c1p1*c3p1*ss[6] + 20*c0p3*c1p1*c4p1*ss[7] + 20*c0p3*c1p1*c5p1*ss[8] + 20*c0p3*c2p1*c3p1*ss[7] + 20*c0p3*c2p1*c4p1*ss[8] + 20*c0p3*c2p1*c5p1*ss[9] + 20*c0p3*c3p1*c4p1*ss[9] + 20*c0p3*c3p1*c5p1*ss[10] + 20*c0p3*c4p1*c5p1*ss[11] + 20*c1p1*c2p1*c3p3*ss[14] + 20*c1p1*c2p1*c4p3*ss[17] + 20*c1p1*c2p1*c5p3*ss[20] + 30*c1p1*c2p2*c3p2*ss[13] + 30*c1p1*c2p2*c4p2*ss[15] + 30*c1p1*c2p2*c5p2*ss[17] + 20*c1p1*c2p3*c3p1*ss[12] + 20*c1p1*c2p3*c4p1*ss[13] + 20*c1p1*c2p3*c5p1*ss[14] + 20*c1p1*c3p1*c4p3*ss[18] + 20*c1p1*c3p1*c5p3*ss[21] + 30*c1p1*c3p2*c4p2*ss[17] + 30*c1p1*c3p2*c5p2*ss[19] + 20*c1p1*c3p3*c4p1*ss[16] + 20*c1p1*c3p3*c5p1*ss[17] + 20*c1p1*c4p1*c5p3*ss[22] + 30*c1p1*c4p2*c5p2*ss[21] + 20*c1p1*c4p3*c5p1*ss[20] + 30*c1p2*c2p1*c3p2*ss[12] + 30*c1p2*c2p1*c4p2*ss[14] + 30*c1p2*c2p1*c5p2*ss[16] + 30*c1p2*c2p2*c3p1*ss[11] + 30*c1p2*c2p2*c4p1*ss[12] + 30*c1p2*c2p2*c5p1*ss[13] + 30*c1p2*c3p1*c4p2*ss[15] + 30*c1p2*c3p1*c5p2*ss[17] + 30*c1p2*c3p2*c4p1*ss[14] + 30*c1p2*c3p2*c5p1*ss[15] + 30*c1p2*c4p1*c5p2*ss[18] + 30*c1p2*c4p2*c5p1*ss[17] + 20*c1p3*c2p1*c3p1*ss[10] + 20*c1p3*c2p1*c4p1*ss[11] + 20*c1p3*c2p1*c5p1*ss[12] + 20*c1p3*c3p1*c4p1*ss[12] + 20*c1p3*c3p1*c5p1*ss[13] + 20*c1p3*c4p1*c5p1*ss[14] + 20*c2p1*c3p1*c4p3*ss[19] + 20*c2p1*c3p1*c5p3*ss[22] + 30*c2p1*c3p2*c4p2*ss[18] + 30*c2p1*c3p2*c5p2*ss[20] + 20*c2p1*c3p3*c4p1*ss[17] + 20*c2p1*c3p3*c5p1*ss[18] + 20*c2p1*c4p1*c5p3*ss[23] + 30*c2p1*c4p2*c5p2*ss[22] + 20*c2p1*c4p3*c5p1*ss[21] + 30*c2p2*c3p1*c4p2*ss[17] + 30*c2p2*c3p1*c5p2*ss[19] + 30*c2p2*c3p2*c4p1*ss[16] + 30*c2p2*c3p2*c5p1*ss[17] + 30*c2p2*c4p1*c5p2*ss[20] + 30*c2p2*c4p2*c5p1*ss[19] + 20*c2p3*c3p1*c4p1*ss[15] + 20*c2p3*c3p1*c5p1*ss[16] + 20*c2p3*c4p1*c5p1*ss[17] + 20*c3p1*c4p1*c5p3*ss[24] + 30*c3p1*c4p2*c5p2*ss[23] + 20*c3p1*c4p3*c5p1*ss[22] + 30*c3p2*c4p1*c5p2*ss[22] + 30*c3p2*c4p2*c5p1*ss[21] + 20*c3p3*c4p1*c5p1*ss[20] + 5*c0p1*c1p4*ss[6] + 5*c0p1*c2p4*ss[10] + 5*c0p1*c3p4*ss[14] + 5*c0p1*c4p4*ss[18] + 5*c0p1*c5p4*ss[22] + 10*c0p2*c1p3*ss[5] + 10*c0p2*c2p3*ss[8] + 10*c0p2*c3p3*ss[11] + 10*c0p2*c4p3*ss[14] + 10*c0p2*c5p3*ss[17] + 10*c0p3*c1p2*ss[4] + 10*c0p3*c2p2*ss[6] + 10*c0p3*c3p2*ss[8] + 10*c0p3*c4p2*ss[10] + 10*c0p3*c5p2*ss[12] + 5*c0p4*c1p1*ss[3] + 5*c0p4*c2p1*ss[4] + 5*c0p4*c3p1*ss[5] + 5*c0p4*c4p1*ss[6] + 5*c0p4*c5p1*ss[7] + 5*c1p1*c2p4*ss[11] + 5*c1p1*c3p4*ss[15] + 5*c1p1*c4p4*ss[19] + 5*c1p1*c5p4*ss[23] + 10*c1p2*c2p3*ss[10] + 10*c1p2*c3p3*ss[13] + 10*c1p2*c4p3*ss[16] + 10*c1p2*c5p3*ss[19] + 10*c1p3*c2p2*ss[9] + 10*c1p3*c3p2*ss[11] + 10*c1p3*c4p2*ss[13] + 10*c1p3*c5p2*ss[15] + 5*c1p4*c2p1*ss[8] + 5*c1p4*c3p1*ss[9] + 5*c1p4*c4p1*ss[10] + 5*c1p4*c5p1*ss[11] + 5*c2p1*c3p4*ss[16] + 5*c2p1*c4p4*ss[20] + 5*c2p1*c5p4*ss[24] + 10*c2p2*c3p3*ss[15] + 10*c2p2*c4p3*ss[18] + 10*c2p2*c5p3*ss[21] + 10*c2p3*c3p2*ss[14] + 10*c2p3*c4p2*ss[16] + 10*c2p3*c5p2*ss[18] + 5*c2p4*c3p1*ss[13] + 5*c2p4*c4p1*ss[14] + 5*c2p4*c5p1*ss[15] + 5*c3p1*c4p4*ss[21] + 5*c3p1*c5p4*ss[25] + 10*c3p2*c4p3*ss[20] + 10*c3p2*c5p3*ss[23] + 10*c3p3*c4p2*ss[19] + 10*c3p3*c5p2*ss[21] + 5*c3p4*c4p1*ss[18] + 5*c3p4*c5p1*ss[19] + 5*c4p1*c5p4*ss[26] + 10*c4p2*c5p3*ss[25] + 10*c4p3*c5p2*ss[24] + 5*c4p4*c5p1*ss[23] + c0p5*ss[2] + c1p5*ss[7] + c2p5*ss[12] + c3p5*ss[17] + c4p5*ss[22] + c5p5*ss[27])*sys_part_5;
  grad_2_num += ss[2]*sys_part_0;
  TACC grad_2_den = convert<TACC>(sqrt(square(sys_part_0) + square(sys_part_1) + square(sys_part_2) + square(sys_part_3) + square(sys_part_4) + square(sys_part_5)));
  TACC grad_3_num = convert<TACC>(2*(c0p1*ss[3] + c1p1*ss[4] + c2p1*ss[5] + c3p1*ss[6] + c4p1*ss[7] + c5p1*ss[8])*sys_part_1);
  grad_3_num += 3*(2*c0p1*c1p1*ss[4] + 2*c0p1*c2p1*ss[5] + 2*c0p1*c3p1*ss[6] + 2*c0p1*c4p1*ss[7] + 2*c0p1*c5p1*ss[8] + 2*c1p1*c2p1*ss[6] + 2*c1p1*c3p1*ss[7] + 2*c1p1*c4p1*ss[8] + 2*c1p1*c5p1*ss[9] + 2*c2p1*c3p1*ss[8] + 2*c2p1*c4p1*ss[9] + 2*c2p1*c5p1*ss[10] + 2*c3p1*c4p1*ss[10] + 2*c3p1*c5p1*ss[11] + 2*c4p1*c5p1*ss[12] + c0p2*ss[3] + c1p2*ss[5] + c2p2*ss[7] + c3p2*ss[9] + c4p2*ss[11] + c5p2*ss[13])*sys_part_2;
  grad_3_num += 4*(6*c0p1*c1p1*c2p1*ss[6] + 6*c0p1*c1p1*c3p1*ss[7] + 6*c0p1*c1p1*c4p1*ss[8] + 6*c0p1*c1p1*c5p1*ss[9] + 6*c0p1*c2p1*c3p1*ss[8] + 6*c0p1*c2p1*c4p1*ss[9] + 6*c0p1*c2p1*c5p1*ss[10] + 6*c0p1*c3p1*c4p1*ss[10] + 6*c0p1*c3p1*c5p1*ss[11] + 6*c0p1*c4p1*c5p1*ss[12] + 6*c1p1*c2p1*c3p1*ss[9] + 6*c1p1*c2p1*c4p1*ss[10] + 6*c1p1*c2p1*c5p1*ss[11] + 6*c1p1*c3p1*c4p1*ss[11] + 6*c1p1*c3p1*c5p1*ss[12] + 6*c1p1*c4p1*c5p1*ss[13] + 6*c2p1*c3p1*c4p1*ss[12] + 6*c2p1*c3p1*c5p1*ss[13] + 6*c2p1*c4p1*c5p1*ss[14] + 6*c3p1*c4p1*c5p1*ss[15] + 3*c0p1*c1p2*ss[5] + 3*c0p1*c2p2*ss[7] + 3*c0p1*c3p2*ss[9] + 3*c0p1*c4p2*ss[11] + 3*c0p1*c5p2*ss[13] + 3*c0p2*c1p1*ss[4] + 3*c0p2*c2p1*ss[5] + 3*c0p2*c3p1*ss[6] + 3*c0p2*c4p1*ss[7] + 3*c0p2*c5p1*ss[8] + 3*c1p1*c2p2*ss[8] + 3*c1p1*c3p2*ss[10] + 3*c1p1*c4p2*ss[12] + 3*c1p1*c5p2*ss[14] + 3*c1p2*c2p1*ss[7] + 3*c1p2*c3p1*ss[8] + 3*c1p2*c4p1*ss[9] + 3*c1p2*c5p1*ss[10] + 3*c2p1*c3p2*ss[11] + 3*c2p1*c4p2*ss[13] + 3*c2p1*c5p2*ss[15] + 3*c2p2*c3p1*ss[10] + 3*c2p2*c4p1*ss[11] + 3*c2p2*c5p1*ss[12] + 3*c3p1*c4p2*ss[14] + 3*c3p1*c5p2*ss[16] + 3*c3p2*c4p1*ss[13] + 3*c3p2*c5p1*ss[14] + 3*c4p1*c5p2*ss[17] + 3*c4p2*c5p1*ss[16] + c0p3*ss[3] + c1p3*ss[6] + c2p3*ss[9] + c3p3*ss[12] + c4p3*ss[15] + c5p3*ss[18])*sys_part_3;
  grad_3_num += 5*(24*c0p1*c1p1*c2p1*c3p1*ss[9] + 24*c0p1*c1p1*c2p1*c4p1*ss[10] + 24*c0p1*c1p1*c2p1*c5p1*ss[11] + 24*c0p1*c1p1*c3p1*c4p1*ss[11] + 24*c0p1*c1p1*c3p1*c5p1*ss[12] + 24*c0p1*c1p1*c4p1*c5p1*ss[13] + 24*c0p1*c2p1*c3p1*c4p1*ss[12] + 24*c0p1*c2p1*c3p1*c5p1*ss[13] + 24*c0p1*c2p1*c4p1*c5p1*ss[14] + 24*c0p1*c3p1*c4p1*c5p1*ss[15] + 24*c1p1*c2p1*c3p1*c4p1*ss[13] + 24*c1p1*c2p1*c3p1*c5p1*ss[14] + 24*c1p1*c2p1*c4p1*c5p1*ss[15] + 24*c1p1*c3p1*c4p1*c5p1*ss[16] + 24*c2p1*c3p1*c4p1*c5p1*ss[17] + 12*c0p1*c1p1*c2p2*ss[8] + 12*c0p1*c1p1*c3p2*ss[10] + 12*c0p1*c1p1*c4p2*ss[12] + 12*c0p1*c1p1*c5p2*ss[14] + 12*c0p1*c1p2*c2p1*ss[7] + 12*c0p1*c1p2*c3p1*ss[8] + 12*c0p1*c1p2*c4p1*ss[9] + 12*c0p1*c1p2*c5p1*ss[10] + 12*c0p1*c2p1*c3p2*ss[11] + 12*c0p1*c2p1*c4p2*ss[13] + 12*c0p1*c2p1*c5p2*ss[15] + 12*c0p1*c2p2*c3p1*ss[10] + 12*c0p1*c2p2*c4p1*ss[11] + 12*c0p1*c2p2*c5p1*ss[12] + 12*c0p1*c3p1*c4p2*ss[14] + 12*c0p1*c3p1*c5p2*ss[16] + 12*c0p1*c3p2*c4p1*ss[13] + 12*c0p1*c3p2*c5p1*ss[14] + 12*c0p1*c4p1*c5p2*ss[17] + 12*c0p1*c4p2*c5p1*ss[16] + 12*c0p2*c1p1*c2p1*ss[6] + 12*c0p2*c1p1*c3p1*ss[7] + 12*c0p2*c1p1*c4p1*ss[8] + 12*c0p2*c1p1*c5p1*ss[9] + 12*c0p2*c2p1*c3p1*ss[8] + 12*c0p2*c2p1*c4p1*ss[9] + 12*c0p2*c2p1*c5p1*ss[10] + 12*c0p2*c3p1*c4p1*ss[10] + 12*c0p2*c3p1*c5p1*ss[11] + 12*c0p2*c4p1*c5p1*ss[12] + 12*c1p1*c2p1*c3p2*ss[12] + 12*c1p1*c2p1*c4p2*ss[14] + 12*c1p1*c2p1*c5p2*ss[16] + 12*c1p1*c2p2*c3p1*ss[11] + 12*c1p1*c2p2*c4p1*ss[12] + 12*c1p1*c2p2*c5p1*ss[13] + 12*c1p1*c3p1*c4p2*ss[15] + 12*c1p1*c3p1*c5p2*ss[17] + 12*c1p1*c3p2*c4p1*ss[14] + 12*c1p1*c3p2*c5p1*ss[15] + 12*c1p1*c4p1*c5p2*ss[18] + 12*c1p1*c4p2*c5p1*ss[17] + 12*c1p2*c2p1*c3p1*ss[10] + 12*c1p2*c2p1*c4p1*ss[11] + 12*c1p2*c2p1*c5p1*ss[12] + 12*c1p2*c3p1*c4p1*ss[12] + 12*c1p2*c3p1*c5p1*ss[13] + 12*c1p2*c4p1*c5p1*ss[14] + 12*c2p1*c3p1*c4p2*ss[16] + 12*c2p1*c3p1*c5p2*ss[18] + 12*c2p1*c3p2*c4p1*ss[15] + 12*c2p1*c3p2*c5p1*ss[16] + 12*c2p1*c4p1*c5p2*ss[19] + 12*c2p1*c4p2*c5p1*ss[18] + 12*c2p2*c3p1*c4p1*ss[14] + 12*c2p2*c3p1*c5p1*ss[15] + 12*c2p2*c4p1*c5p1*ss[16] + 12*c3p1*c4p1*c5p2*ss[20] + 12*c3p1*c4p2*c5p1*ss[19] + 12*c3p2*c4p1*c5p1*ss[18] + 4*c0p1*c1p3*ss[6] + 4*c0p1*c2p3*ss[9] + 4*c0p1*c3p3*ss[12] + 4*c0p1*c4p3*ss[15] + 4*c0p1*c5p3*ss[18] + 6*c0p2*c1p2*ss[5] + 6*c0p2*c2p2*ss[7] + 6*c0p2*c3p2*ss[9] + 6*c0p2*c4p2*ss[11] + 6*c0p2*c5p2*ss[13] + 4*c0p3*c1p1*ss[4] + 4*c0p3*c2p1*ss[5] + 4*c0p3*c3p1*ss[6] + 4*c0p3*c4p1*ss[7] + 4*c0p3*c5p1*ss[8] + 4*c1p1*c2p3*ss[10] + 4*c1p1*c3p3*ss[13] + 4*c1p1*c4p3*ss[16] + 4*c1p1*c5p3*ss[19] + 6*c1p2*c2p2*ss[9] + 6*c1p2*c3p2*ss[11] + 6*c1p2*c4p2*ss[13] + 6*c1p2*c5p2*ss[15] + 4*c1p3*c2p1*ss[8] + 4*c1p3*c3p1*ss[9] + 4*c1p3*c4p1*ss[10] + 4*c1p3*c5p1*ss[11] + 4*c2p1*c3p3*ss[14] + 4*c2p1*c4p3*ss[17] + 4*c2p1*c5p3*ss[20] + 6*c2p2*c3p2*ss[13] + 6*c2p2*c4p2*ss[15] + 6*c2p2*c5p2*ss[17] + 4*c2p3*c3p1*ss[12] + 4*c2p3*c4p1*ss[13] + 4*c2p3*c5p1*ss[14] + 4*c3p1*c4p3*ss[18] + 4*c3p1*c5p3*ss[21] + 6*c3p2*c4p2*ss[17] + 6*c3p2*c5p2*ss[19] + 4*c3p3*c4p1*ss[16] + 4*c3p3*c5p1*ss[17] + 4*c4p1*c5p3*ss[22] + 6*c4p2*c5p2*ss[21] + 4*c4p3*c5p1*ss[20] + c0p4*ss[3] + c1p4*ss[7] + c2p4*ss[11] + c3p4*ss[15] + c4p4*ss[19] + c5p4*ss[23])*sys_part_4;
  grad_3_num += 6*(120*c0p1*c1p1*c2p1*c3p1*c4p1*ss[13] + 120*c0p1*c1p1*c2p1*c3p1*c5p1*ss[14] + 120*c0p1*c1p1*c2p1*c4p1*c5p1*ss[15] + 120*c0p1*c1p1*c3p1*c4p1*c5p1*ss[16] + 120*c0p1*c2p1*c3p1*c4p1*c5p1*ss[17] + 120*c1p1*c2p1*c3p1*c4p1*c5p1*ss[18] + 60*c0p1*c1p1*c2p1*c3p2*ss[12] + 60*c0p1*c1p1*c2p1*c4p2*ss[14] + 60*c0p1*c1p1*c2p1*c5p2*ss[16] + 60*c0p1*c1p1*c2p2*c3p1*ss[11] + 60*c0p1*c1p1*c2p2*c4p1*ss[12] + 60*c0p1*c1p1*c2p2*c5p1*ss[13] + 60*c0p1*c1p1*c3p1*c4p2*ss[15] + 60*c0p1*c1p1*c3p1*c5p2*ss[17] + 60*c0p1*c1p1*c3p2*c4p1*ss[14] + 60*c0p1*c1p1*c3p2*c5p1*ss[15] + 60*c0p1*c1p1*c4p1*c5p2*ss[18] + 60*c0p1*c1p1*c4p2*c5p1*ss[17] + 60*c0p1*c1p2*c2p1*c3p1*ss[10] + 60*c0p1*c1p2*c2p1*c4p1*ss[11] + 60*c0p1*c1p2*c2p1*c5p1*ss[12] + 60*c0p1*c1p2*c3p1*c4p1*ss[12] + 60*c0p1*c1p2*c3p1*c5p1*ss[13] + 60*c0p1*c1p2*c4p1*c5p1*ss[14] + 60*c0p1*c2p1*c3p1*c4p2*ss[16] + 60*c0p1*c2p1*c3p1*c5p2*ss[18] + 60*c0p1*c2p1*c3p2*c4p1*ss[15] + 60*c0p1*c2p1*c3p2*c5p1*ss[16] + 60*c0p1*c2p1*c4p1*c5p2*ss[19] + 60*c0p1*c2p1*c4p2*c5p1*ss[18] + 60*c0p1*c2p2*c3p1*c4p1*ss[14] + 60*c0p1*c2p2*c3p1*c5p1*ss[15] + 60*c0p1*c2p2*c4p1*c5p1*ss[16] + 60*c0p1*c3p1*c4p1*c5p2*ss[20] + 60*c0p1*c3p1*c4p2*c5p1*ss[19] + 60*c0p1*c3p2*c4p1*c5p1*ss[18] + 60*c0p2*c1p1*c2p1*c3p1*ss[9] + 60*c0p2*c1p1*c2p1*c4p1*ss[10] + 60*c0p2*c1p1*c2p1*c5p1*ss[11] + 60*c0p2*c1p1*c3p1*c4p1*ss[11] + 60*c0p2*c1p1*c3p1*c5p1*ss[12] + 60*c0p2*c1p1*c4p1*c5p1*ss[13] + 60*c0p2*c2p1*c3p1*c4p1*ss[12] + 60*c0p2*c2p1*c3p1*c5p1*ss[13] + 60*c0p2*c2p1*c4p1*c5p1*ss[14] + 60*c0p2*c3p1*c4p1*c5p1*ss[15] + 60*c1p1*c2p1*c3p1*c4p2*ss[17] + 60*c1p1*c2p1*c3p1*c5p2*ss[19] + 60*c1p1*c2p1*c3p2*c4p1*ss[16] + 60*c1p1*c2p1*c3p2*c5p1*ss[17] + 60*c1p1*c2p1*c4p1*c5p2*ss[20] + 60*c1p1*c2p1*c4p2*c5p1*ss[19] + 60*c1p1*c2p2*c3p1*c4p1*ss[15] + 60*c1p1*c2p2*c3p1*c5p1*ss[16] + 60*c1p1*c2p2*c4p1*c5p1*ss[17] + 60*c1p1*c3p1*c4p1*c5p2*ss[21] + 60*c1p1*c3p1*c4p2*c5p1*ss[20] + 60*c1p1*c3p2*c4p1*c5p1*ss[19] + 60*c1p2*c2p1*c3p1*c4p1*ss[14] + 60*c1p2*c2p1*c3p1*c5p1*ss[15] + 60*c1p2*c2p1*c4p1*c5p1*ss[16] + 60*c1p2*c3p1*c4p1*c5p1*ss[17] + 60*c2p1*c3p1*c4p1*c5p2*ss[22] + 60*c2p1*c3p1*c4p2*c5p1*ss[21] + 60*c2p1*c3p2*c4p1*c5p1*ss[20] + 60*c2p2*c3p1*c4p1*c5p1*ss[19] + 20*c0p1*c1p1*c2p3*ss[10] + 20*c0p1*c1p1*c3p3*ss[13] + 20*c0p1*c1p1*c4p3*ss[16] + 20*c0p1*c1p1*c5p3*ss[19] + 30*c0p1*c1p2*c2p2*ss[9] + 30*c0p1*c1p2*c3p2*ss[11] + 30*c0p1*c1p2*c4p2*ss[13] + 30*c0p1*c1p2*c5p2*ss[15] + 20*c0p1*c1p3*c2p1*ss[8] + 20*c0p1*c1p3*c3p1*ss[9] + 20*c0p1*c1p3*c4p1*ss[10] + 20*c0p1*c1p3*c5p1*ss[11] + 20*c0p1*c2p1*c3p3*ss[14] + 20*c0p1*c2p1*c4p3*ss[17] + 20*c0p1*c2p1*c5p3*ss[20] + 30*c0p1*c2p2*c3p2*ss[13] + 30*c0p1*c2p2*c4p2*ss[15] + 30*c0p1*c2p2*c5p2*ss[17] + 20*c0p1*c2p3*c3p1*ss[12] + 20*c0p1*c2p3*c4p1*ss[13] + 20*c0p1*c2p3*c5p1*ss[14] + 20*c0p1*c3p1*c4p3*ss[18] + 20*c0p1*c3p1*c5p3*ss[21] + 30*c0p1*c3p2*c4p2*ss[17] + 30*c0p1*c3p2*c5p2*ss[19] + 20*c0p1*c3p3*c4p1*ss[16] + 20*c0p1*c3p3*c5p1*ss[17] + 20*c0p1*c4p1*c5p3*ss[22] + 30*c0p1*c4p2*c5p2*ss[21] + 20*c0p1*c4p3*c5p1*ss[20] + 30*c0p2*c1p1*c2p2*ss[8] + 30*c0p2*c1p1*c3p2*ss[10] + 30*c0p2*c1p1*c4p2*ss[12] + 30*c0p2*c1p1*c5p2*ss[14] + 30*c0p2*c1p2*c2p1*ss[7] + 30*c0p2*c1p2*c3p1*ss[8] + 30*c0p2*c1p2*c4p1*ss[9] + 30*c0p2*c1p2*c5p1*ss[10] + 30*c0p2*c2p1*c3p2*ss[11] + 30*c0p2*c2p1*c4p2*ss[13] + 30*c0p2*c2p1*c5p2*ss[15] + 30*c0p2*c2p2*c3p1*ss[10] + 30*c0p2*c2p2*c4p1*ss[11] + 30*c0p2*c2p2*c5p1*ss[12] + 30*c0p2*c3p1*c4p2*ss[14] + 30*c0p2*c3p1*c5p2*ss[16] + 30*c0p2*c3p2*c4p1*ss[13] + 30*c0p2*c3p2*c5p1*ss[14] + 30*c0p2*c4p1*c5p2*ss[17] + 30*c0p2*c4p2*c5p1*ss[16] + 20*c0p3*c1p1*c2p1*ss[6] + 20*c0p3*c1p1*c3p1*ss[7] + 20*c0p3*c1p1*c4p1*ss[8] + 20*c0p3*c1p1*c5p1*ss[9] + 20*c0p3*c2p1*c3p1*ss[8] + 20*c0p3*c2p1*c4p1*ss[9] + 20*c0p3*c2p1*c5p1*ss[10] + 20*c0p3*c3p1*c4p1*ss[10] + 20*c0p3*c3p1*c5p1*ss[11] + 20*c0p3*c4p1*c5p1*ss[12] + 20*c1p1*c2p1*c3p3*ss[15] + 20*c1p1*c2p1*c4p3*ss[18] + 20*c1p1*c2p1*c5p3*ss[21] + 30*c1p1*c2p2*c3p2*ss[14] + 30*c1p1*c2p2*c4p2*ss[16] + 30*c1p1*c2p2*c5p2*ss[18] + 20*c1p1*c2p3*c3p1*ss[13] + 20*c1p1*c2p3*c4p1*ss[14] + 20*c1p1*c2p3*c5p1*ss[15] + 20*c1p1*c3p1*c4p3*ss[19] + 20*c1p1*c3p1*c5p3*ss[22] + 30*c1p1*c3p2*c4p2*ss[18] + 30*c1p1*c3p2*c5p2*ss[20] + 20*c1p1*c3p3*c4p1*ss[17] + 20*c1p1*c3p3*c5p1*ss[18] + 20*c1p1*c4p1*c5p3*ss[23] + 30*c1p1*c4p2*c5p2*ss[22] + 20*c1p1*c4p3*c5p1*ss[21] + 30*c1p2*c2p1*c3p2*ss[13] + 30*c1p2*c2p1*c4p2*ss[15] + 30*c1p2*c2p1*c5p2*ss[17] + 30*c1p2*c2p2*c3p1*ss[12] + 30*c1p2*c2p2*c4p1*ss[13] + 30*c1p2*c2p2*c5p1*ss[14] + 30*c1p2*c3p1*c4p2*ss[16] + 30*c1p2*c3p1*c5p2*ss[18] + 30*c1p2*c3p2*c4p1*ss[15] + 30*c1p2*c3p2*c5p1*ss[16] + 30*c1p2*c4p1*c5p2*ss[19] + 30*c1p2*c4p2*c5p1*ss[18] + 20*c1p3*c2p1*c3p1*ss[11] + 20*c1p3*c2p1*c4p1*ss[12] + 20*c1p3*c2p1*c5p1*ss[13] + 20*c1p3*c3p1*c4p1*ss[13] + 20*c1p3*c3p1*c5p1*ss[14] + 20*c1p3*c4p1*c5p1*ss[15] + 20*c2p1*c3p1*c4p3*ss[20] + 20*c2p1*c3p1*c5p3*ss[23] + 30*c2p1*c3p2*c4p2*ss[19] + 30*c2p1*c3p2*c5p2*ss[21] + 20*c2p1*c3p3*c4p1*ss[18] + 20*c2p1*c3p3*c5p1*ss[19] + 20*c2p1*c4p1*c5p3*ss[24] + 30*c2p1*c4p2*c5p2*ss[23] + 20*c2p1*c4p3*c5p1*ss[22] + 30*c2p2*c3p1*c4p2*ss[18] + 30*c2p2*c3p1*c5p2*ss[20] + 30*c2p2*c3p2*c4p1*ss[17] + 30*c2p2*c3p2*c5p1*ss[18] + 30*c2p2*c4p1*c5p2*ss[21] + 30*c2p2*c4p2*c5p1*ss[20] + 20*c2p3*c3p1*c4p1*ss[16] + 20*c2p3*c3p1*c5p1*ss[17] + 20*c2p3*c4p1*c5p1*ss[18] + 20*c3p1*c4p1*c5p3*ss[25] + 30*c3p1*c4p2*c5p2*ss[24] + 20*c3p1*c4p3*c5p1*ss[23] + 30*c3p2*c4p1*c5p2*ss[23] + 30*c3p2*c4p2*c5p1*ss[22] + 20*c3p3*c4p1*c5p1*ss[21] + 5*c0p1*c1p4*ss[7] + 5*c0p1*c2p4*ss[11] + 5*c0p1*c3p4*ss[15] + 5*c0p1*c4p4*ss[19] + 5*c0p1*c5p4*ss[23] + 10*c0p2*c1p3*ss[6] + 10*c0p2*c2p3*ss[9] + 10*c0p2*c3p3*ss[12] + 10*c0p2*c4p3*ss[15] + 10*c0p2*c5p3*ss[18] + 10*c0p3*c1p2*ss[5] + 10*c0p3*c2p2*ss[7] + 10*c0p3*c3p2*ss[9] + 10*c0p3*c4p2*ss[11] + 10*c0p3*c5p2*ss[13] + 5*c0p4*c1p1*ss[4] + 5*c0p4*c2p1*ss[5] + 5*c0p4*c3p1*ss[6] + 5*c0p4*c4p1*ss[7] + 5*c0p4*c5p1*ss[8] + 5*c1p1*c2p4*ss[12] + 5*c1p1*c3p4*ss[16] + 5*c1p1*c4p4*ss[20] + 5*c1p1*c5p4*ss[24] + 10*c1p2*c2p3*ss[11] + 10*c1p2*c3p3*ss[14] + 10*c1p2*c4p3*ss[17] + 10*c1p2*c5p3*ss[20] + 10*c1p3*c2p2*ss[10] + 10*c1p3*c3p2*ss[12] + 10*c1p3*c4p2*ss[14] + 10*c1p3*c5p2*ss[16] + 5*c1p4*c2p1*ss[9] + 5*c1p4*c3p1*ss[10] + 5*c1p4*c4p1*ss[11] + 5*c1p4*c5p1*ss[12] + 5*c2p1*c3p4*ss[17] + 5*c2p1*c4p4*ss[21] + 5*c2p1*c5p4*ss[25] + 10*c2p2*c3p3*ss[16] + 10*c2p2*c4p3*ss[19] + 10*c2p2*c5p3*ss[22] + 10*c2p3*c3p2*ss[15] + 10*c2p3*c4p2*ss[17] + 10*c2p3*c5p2*ss[19] + 5*c2p4*c3p1*ss[14] + 5*c2p4*c4p1*ss[15] + 5*c2p4*c5p1*ss[16] + 5*c3p1*c4p4*ss[22] + 5*c3p1*c5p4*ss[26] + 10*c3p2*c4p3*ss[21] + 10*c3p2*c5p3*ss[24] + 10*c3p3*c4p2*ss[20] + 10*c3p3*c5p2*ss[22] + 5*c3p4*c4p1*ss[19] + 5*c3p4*c5p1*ss[20] + 5*c4p1*c5p4*ss[27] + 10*c4p2*c5p3*ss[26] + 10*c4p3*c5p2*ss[25] + 5*c4p4*c5p1*ss[24] + c0p5*ss[3] + c1p5*ss[8] + c2p5*ss[13] + c3p5*ss[18] + c4p5*ss[23] + c5p5*ss[28])*sys_part_5;
  grad_3_num += ss[3]*sys_part_0;
  TACC grad_3_den = convert<TACC>(sqrt(square(sys_part_0) + square(sys_part_1) + square(sys_part_2) + square(sys_part_3) + square(sys_part_4) + square(sys_part_5)));
  TACC grad_4_num = convert<TACC>(2*(c0p1*ss[4] + c1p1*ss[5] + c2p1*ss[6] + c3p1*ss[7] + c4p1*ss[8] + c5p1*ss[9])*sys_part_1);
  grad_4_num += 3*(2*c0p1*c1p1*ss[5] + 2*c0p1*c2p1*ss[6] + 2*c0p1*c3p1*ss[7] + 2*c0p1*c4p1*ss[8] + 2*c0p1*c5p1*ss[9] + 2*c1p1*c2p1*ss[7] + 2*c1p1*c3p1*ss[8] + 2*c1p1*c4p1*ss[9] + 2*c1p1*c5p1*ss[10] + 2*c2p1*c3p1*ss[9] + 2*c2p1*c4p1*ss[10] + 2*c2p1*c5p1*ss[11] + 2*c3p1*c4p1*ss[11] + 2*c3p1*c5p1*ss[12] + 2*c4p1*c5p1*ss[13] + c0p2*ss[4] + c1p2*ss[6] + c2p2*ss[8] + c3p2*ss[10] + c4p2*ss[12] + c5p2*ss[14])*sys_part_2;
  grad_4_num += 4*(6*c0p1*c1p1*c2p1*ss[7] + 6*c0p1*c1p1*c3p1*ss[8] + 6*c0p1*c1p1*c4p1*ss[9] + 6*c0p1*c1p1*c5p1*ss[10] + 6*c0p1*c2p1*c3p1*ss[9] + 6*c0p1*c2p1*c4p1*ss[10] + 6*c0p1*c2p1*c5p1*ss[11] + 6*c0p1*c3p1*c4p1*ss[11] + 6*c0p1*c3p1*c5p1*ss[12] + 6*c0p1*c4p1*c5p1*ss[13] + 6*c1p1*c2p1*c3p1*ss[10] + 6*c1p1*c2p1*c4p1*ss[11] + 6*c1p1*c2p1*c5p1*ss[12] + 6*c1p1*c3p1*c4p1*ss[12] + 6*c1p1*c3p1*c5p1*ss[13] + 6*c1p1*c4p1*c5p1*ss[14] + 6*c2p1*c3p1*c4p1*ss[13] + 6*c2p1*c3p1*c5p1*ss[14] + 6*c2p1*c4p1*c5p1*ss[15] + 6*c3p1*c4p1*c5p1*ss[16] + 3*c0p1*c1p2*ss[6] + 3*c0p1*c2p2*ss[8] + 3*c0p1*c3p2*ss[10] + 3*c0p1*c4p2*ss[12] + 3*c0p1*c5p2*ss[14] + 3*c0p2*c1p1*ss[5] + 3*c0p2*c2p1*ss[6] + 3*c0p2*c3p1*ss[7] + 3*c0p2*c4p1*ss[8] + 3*c0p2*c5p1*ss[9] + 3*c1p1*c2p2*ss[9] + 3*c1p1*c3p2*ss[11] + 3*c1p1*c4p2*ss[13] + 3*c1p1*c5p2*ss[15] + 3*c1p2*c2p1*ss[8] + 3*c1p2*c3p1*ss[9] + 3*c1p2*c4p1*ss[10] + 3*c1p2*c5p1*ss[11] + 3*c2p1*c3p2*ss[12] + 3*c2p1*c4p2*ss[14] + 3*c2p1*c5p2*ss[16] + 3*c2p2*c3p1*ss[11] + 3*c2p2*c4p1*ss[12] + 3*c2p2*c5p1*ss[13] + 3*c3p1*c4p2*ss[15] + 3*c3p1*c5p2*ss[17] + 3*c3p2*c4p1*ss[14] + 3*c3p2*c5p1*ss[15] + 3*c4p1*c5p2*ss[18] + 3*c4p2*c5p1*ss[17] + c0p3*ss[4] + c1p3*ss[7] + c2p3*ss[10] + c3p3*ss[13] + c4p3*ss[16] + c5p3*ss[19])*sys_part_3;
  grad_4_num += 5*(24*c0p1*c1p1*c2p1*c3p1*ss[10] + 24*c0p1*c1p1*c2p1*c4p1*ss[11] + 24*c0p1*c1p1*c2p1*c5p1*ss[12] + 24*c0p1*c1p1*c3p1*c4p1*ss[12] + 24*c0p1*c1p1*c3p1*c5p1*ss[13] + 24*c0p1*c1p1*c4p1*c5p1*ss[14] + 24*c0p1*c2p1*c3p1*c4p1*ss[13] + 24*c0p1*c2p1*c3p1*c5p1*ss[14] + 24*c0p1*c2p1*c4p1*c5p1*ss[15] + 24*c0p1*c3p1*c4p1*c5p1*ss[16] + 24*c1p1*c2p1*c3p1*c4p1*ss[14] + 24*c1p1*c2p1*c3p1*c5p1*ss[15] + 24*c1p1*c2p1*c4p1*c5p1*ss[16] + 24*c1p1*c3p1*c4p1*c5p1*ss[17] + 24*c2p1*c3p1*c4p1*c5p1*ss[18] + 12*c0p1*c1p1*c2p2*ss[9] + 12*c0p1*c1p1*c3p2*ss[11] + 12*c0p1*c1p1*c4p2*ss[13] + 12*c0p1*c1p1*c5p2*ss[15] + 12*c0p1*c1p2*c2p1*ss[8] + 12*c0p1*c1p2*c3p1*ss[9] + 12*c0p1*c1p2*c4p1*ss[10] + 12*c0p1*c1p2*c5p1*ss[11] + 12*c0p1*c2p1*c3p2*ss[12] + 12*c0p1*c2p1*c4p2*ss[14] + 12*c0p1*c2p1*c5p2*ss[16] + 12*c0p1*c2p2*c3p1*ss[11] + 12*c0p1*c2p2*c4p1*ss[12] + 12*c0p1*c2p2*c5p1*ss[13] + 12*c0p1*c3p1*c4p2*ss[15] + 12*c0p1*c3p1*c5p2*ss[17] + 12*c0p1*c3p2*c4p1*ss[14] + 12*c0p1*c3p2*c5p1*ss[15] + 12*c0p1*c4p1*c5p2*ss[18] + 12*c0p1*c4p2*c5p1*ss[17] + 12*c0p2*c1p1*c2p1*ss[7] + 12*c0p2*c1p1*c3p1*ss[8] + 12*c0p2*c1p1*c4p1*ss[9] + 12*c0p2*c1p1*c5p1*ss[10] + 12*c0p2*c2p1*c3p1*ss[9] + 12*c0p2*c2p1*c4p1*ss[10] + 12*c0p2*c2p1*c5p1*ss[11] + 12*c0p2*c3p1*c4p1*ss[11] + 12*c0p2*c3p1*c5p1*ss[12] + 12*c0p2*c4p1*c5p1*ss[13] + 12*c1p1*c2p1*c3p2*ss[13] + 12*c1p1*c2p1*c4p2*ss[15] + 12*c1p1*c2p1*c5p2*ss[17] + 12*c1p1*c2p2*c3p1*ss[12] + 12*c1p1*c2p2*c4p1*ss[13] + 12*c1p1*c2p2*c5p1*ss[14] + 12*c1p1*c3p1*c4p2*ss[16] + 12*c1p1*c3p1*c5p2*ss[18] + 12*c1p1*c3p2*c4p1*ss[15] + 12*c1p1*c3p2*c5p1*ss[16] + 12*c1p1*c4p1*c5p2*ss[19] + 12*c1p1*c4p2*c5p1*ss[18] + 12*c1p2*c2p1*c3p1*ss[11] + 12*c1p2*c2p1*c4p1*ss[12] + 12*c1p2*c2p1*c5p1*ss[13] + 12*c1p2*c3p1*c4p1*ss[13] + 12*c1p2*c3p1*c5p1*ss[14] + 12*c1p2*c4p1*c5p1*ss[15] + 12*c2p1*c3p1*c4p2*ss[17] + 12*c2p1*c3p1*c5p2*ss[19] + 12*c2p1*c3p2*c4p1*ss[16] + 12*c2p1*c3p2*c5p1*ss[17] + 12*c2p1*c4p1*c5p2*ss[20] + 12*c2p1*c4p2*c5p1*ss[19] + 12*c2p2*c3p1*c4p1*ss[15] + 12*c2p2*c3p1*c5p1*ss[16] + 12*c2p2*c4p1*c5p1*ss[17] + 12*c3p1*c4p1*c5p2*ss[21] + 12*c3p1*c4p2*c5p1*ss[20] + 12*c3p2*c4p1*c5p1*ss[19] + 4*c0p1*c1p3*ss[7] + 4*c0p1*c2p3*ss[10] + 4*c0p1*c3p3*ss[13] + 4*c0p1*c4p3*ss[16] + 4*c0p1*c5p3*ss[19] + 6*c0p2*c1p2*ss[6] + 6*c0p2*c2p2*ss[8] + 6*c0p2*c3p2*ss[10] + 6*c0p2*c4p2*ss[12] + 6*c0p2*c5p2*ss[14] + 4*c0p3*c1p1*ss[5] + 4*c0p3*c2p1*ss[6] + 4*c0p3*c3p1*ss[7] + 4*c0p3*c4p1*ss[8] + 4*c0p3*c5p1*ss[9] + 4*c1p1*c2p3*ss[11] + 4*c1p1*c3p3*ss[14] + 4*c1p1*c4p3*ss[17] + 4*c1p1*c5p3*ss[20] + 6*c1p2*c2p2*ss[10] + 6*c1p2*c3p2*ss[12] + 6*c1p2*c4p2*ss[14] + 6*c1p2*c5p2*ss[16] + 4*c1p3*c2p1*ss[9] + 4*c1p3*c3p1*ss[10] + 4*c1p3*c4p1*ss[11] + 4*c1p3*c5p1*ss[12] + 4*c2p1*c3p3*ss[15] + 4*c2p1*c4p3*ss[18] + 4*c2p1*c5p3*ss[21] + 6*c2p2*c3p2*ss[14] + 6*c2p2*c4p2*ss[16] + 6*c2p2*c5p2*ss[18] + 4*c2p3*c3p1*ss[13] + 4*c2p3*c4p1*ss[14] + 4*c2p3*c5p1*ss[15] + 4*c3p1*c4p3*ss[19] + 4*c3p1*c5p3*ss[22] + 6*c3p2*c4p2*ss[18] + 6*c3p2*c5p2*ss[20] + 4*c3p3*c4p1*ss[17] + 4*c3p3*c5p1*ss[18] + 4*c4p1*c5p3*ss[23] + 6*c4p2*c5p2*ss[22] + 4*c4p3*c5p1*ss[21] + c0p4*ss[4] + c1p4*ss[8] + c2p4*ss[12] + c3p4*ss[16] + c4p4*ss[20] + c5p4*ss[24])*sys_part_4;
  grad_4_num += 6*(120*c0p1*c1p1*c2p1*c3p1*c4p1*ss[14] + 120*c0p1*c1p1*c2p1*c3p1*c5p1*ss[15] + 120*c0p1*c1p1*c2p1*c4p1*c5p1*ss[16] + 120*c0p1*c1p1*c3p1*c4p1*c5p1*ss[17] + 120*c0p1*c2p1*c3p1*c4p1*c5p1*ss[18] + 120*c1p1*c2p1*c3p1*c4p1*c5p1*ss[19] + 60*c0p1*c1p1*c2p1*c3p2*ss[13] + 60*c0p1*c1p1*c2p1*c4p2*ss[15] + 60*c0p1*c1p1*c2p1*c5p2*ss[17] + 60*c0p1*c1p1*c2p2*c3p1*ss[12] + 60*c0p1*c1p1*c2p2*c4p1*ss[13] + 60*c0p1*c1p1*c2p2*c5p1*ss[14] + 60*c0p1*c1p1*c3p1*c4p2*ss[16] + 60*c0p1*c1p1*c3p1*c5p2*ss[18] + 60*c0p1*c1p1*c3p2*c4p1*ss[15] + 60*c0p1*c1p1*c3p2*c5p1*ss[16] + 60*c0p1*c1p1*c4p1*c5p2*ss[19] + 60*c0p1*c1p1*c4p2*c5p1*ss[18] + 60*c0p1*c1p2*c2p1*c3p1*ss[11] + 60*c0p1*c1p2*c2p1*c4p1*ss[12] + 60*c0p1*c1p2*c2p1*c5p1*ss[13] + 60*c0p1*c1p2*c3p1*c4p1*ss[13] + 60*c0p1*c1p2*c3p1*c5p1*ss[14] + 60*c0p1*c1p2*c4p1*c5p1*ss[15] + 60*c0p1*c2p1*c3p1*c4p2*ss[17] + 60*c0p1*c2p1*c3p1*c5p2*ss[19] + 60*c0p1*c2p1*c3p2*c4p1*ss[16] + 60*c0p1*c2p1*c3p2*c5p1*ss[17] + 60*c0p1*c2p1*c4p1*c5p2*ss[20] + 60*c0p1*c2p1*c4p2*c5p1*ss[19] + 60*c0p1*c2p2*c3p1*c4p1*ss[15] + 60*c0p1*c2p2*c3p1*c5p1*ss[16] + 60*c0p1*c2p2*c4p1*c5p1*ss[17] + 60*c0p1*c3p1*c4p1*c5p2*ss[21] + 60*c0p1*c3p1*c4p2*c5p1*ss[20] + 60*c0p1*c3p2*c4p1*c5p1*ss[19] + 60*c0p2*c1p1*c2p1*c3p1*ss[10] + 60*c0p2*c1p1*c2p1*c4p1*ss[11] + 60*c0p2*c1p1*c2p1*c5p1*ss[12] + 60*c0p2*c1p1*c3p1*c4p1*ss[12] + 60*c0p2*c1p1*c3p1*c5p1*ss[13] + 60*c0p2*c1p1*c4p1*c5p1*ss[14] + 60*c0p2*c2p1*c3p1*c4p1*ss[13] + 60*c0p2*c2p1*c3p1*c5p1*ss[14] + 60*c0p2*c2p1*c4p1*c5p1*ss[15] + 60*c0p2*c3p1*c4p1*c5p1*ss[16] + 60*c1p1*c2p1*c3p1*c4p2*ss[18] + 60*c1p1*c2p1*c3p1*c5p2*ss[20] + 60*c1p1*c2p1*c3p2*c4p1*ss[17] + 60*c1p1*c2p1*c3p2*c5p1*ss[18] + 60*c1p1*c2p1*c4p1*c5p2*ss[21] + 60*c1p1*c2p1*c4p2*c5p1*ss[20] + 60*c1p1*c2p2*c3p1*c4p1*ss[16] + 60*c1p1*c2p2*c3p1*c5p1*ss[17] + 60*c1p1*c2p2*c4p1*c5p1*ss[18] + 60*c1p1*c3p1*c4p1*c5p2*ss[22] + 60*c1p1*c3p1*c4p2*c5p1*ss[21] + 60*c1p1*c3p2*c4p1*c5p1*ss[20] + 60*c1p2*c2p1*c3p1*c4p1*ss[15] + 60*c1p2*c2p1*c3p1*c5p1*ss[16] + 60*c1p2*c2p1*c4p1*c5p1*ss[17] + 60*c1p2*c3p1*c4p1*c5p1*ss[18] + 60*c2p1*c3p1*c4p1*c5p2*ss[23] + 60*c2p1*c3p1*c4p2*c5p1*ss[22] + 60*c2p1*c3p2*c4p1*c5p1*ss[21] + 60*c2p2*c3p1*c4p1*c5p1*ss[20] + 20*c0p1*c1p1*c2p3*ss[11] + 20*c0p1*c1p1*c3p3*ss[14] + 20*c0p1*c1p1*c4p3*ss[17] + 20*c0p1*c1p1*c5p3*ss[20] + 30*c0p1*c1p2*c2p2*ss[10] + 30*c0p1*c1p2*c3p2*ss[12] + 30*c0p1*c1p2*c4p2*ss[14] + 30*c0p1*c1p2*c5p2*ss[16] + 20*c0p1*c1p3*c2p1*ss[9] + 20*c0p1*c1p3*c3p1*ss[10] + 20*c0p1*c1p3*c4p1*ss[11] + 20*c0p1*c1p3*c5p1*ss[12] + 20*c0p1*c2p1*c3p3*ss[15] + 20*c0p1*c2p1*c4p3*ss[18] + 20*c0p1*c2p1*c5p3*ss[21] + 30*c0p1*c2p2*c3p2*ss[14] + 30*c0p1*c2p2*c4p2*ss[16] + 30*c0p1*c2p2*c5p2*ss[18] + 20*c0p1*c2p3*c3p1*ss[13] + 20*c0p1*c2p3*c4p1*ss[14] + 20*c0p1*c2p3*c5p1*ss[15] + 20*c0p1*c3p1*c4p3*ss[19] + 20*c0p1*c3p1*c5p3*ss[22] + 30*c0p1*c3p2*c4p2*ss[18] + 30*c0p1*c3p2*c5p2*ss[20] + 20*c0p1*c3p3*c4p1*ss[17] + 20*c0p1*c3p3*c5p1*ss[18] + 20*c0p1*c4p1*c5p3*ss[23] + 30*c0p1*c4p2*c5p2*ss[22] + 20*c0p1*c4p3*c5p1*ss[21] + 30*c0p2*c1p1*c2p2*ss[9] + 30*c0p2*c1p1*c3p2*ss[11] + 30*c0p2*c1p1*c4p2*ss[13] + 30*c0p2*c1p1*c5p2*ss[15] + 30*c0p2*c1p2*c2p1*ss[8] + 30*c0p2*c1p2*c3p1*ss[9] + 30*c0p2*c1p2*c4p1*ss[10] + 30*c0p2*c1p2*c5p1*ss[11] + 30*c0p2*c2p1*c3p2*ss[12] + 30*c0p2*c2p1*c4p2*ss[14] + 30*c0p2*c2p1*c5p2*ss[16] + 30*c0p2*c2p2*c3p1*ss[11] + 30*c0p2*c2p2*c4p1*ss[12] + 30*c0p2*c2p2*c5p1*ss[13] + 30*c0p2*c3p1*c4p2*ss[15] + 30*c0p2*c3p1*c5p2*ss[17] + 30*c0p2*c3p2*c4p1*ss[14] + 30*c0p2*c3p2*c5p1*ss[15] + 30*c0p2*c4p1*c5p2*ss[18] + 30*c0p2*c4p2*c5p1*ss[17] + 20*c0p3*c1p1*c2p1*ss[7] + 20*c0p3*c1p1*c3p1*ss[8] + 20*c0p3*c1p1*c4p1*ss[9] + 20*c0p3*c1p1*c5p1*ss[10] + 20*c0p3*c2p1*c3p1*ss[9] + 20*c0p3*c2p1*c4p1*ss[10] + 20*c0p3*c2p1*c5p1*ss[11] + 20*c0p3*c3p1*c4p1*ss[11] + 20*c0p3*c3p1*c5p1*ss[12] + 20*c0p3*c4p1*c5p1*ss[13] + 20*c1p1*c2p1*c3p3*ss[16] + 20*c1p1*c2p1*c4p3*ss[19] + 20*c1p1*c2p1*c5p3*ss[22] + 30*c1p1*c2p2*c3p2*ss[15] + 30*c1p1*c2p2*c4p2*ss[17] + 30*c1p1*c2p2*c5p2*ss[19] + 20*c1p1*c2p3*c3p1*ss[14] + 20*c1p1*c2p3*c4p1*ss[15] + 20*c1p1*c2p3*c5p1*ss[16] + 20*c1p1*c3p1*c4p3*ss[20] + 20*c1p1*c3p1*c5p3*ss[23] + 30*c1p1*c3p2*c4p2*ss[19] + 30*c1p1*c3p2*c5p2*ss[21] + 20*c1p1*c3p3*c4p1*ss[18] + 20*c1p1*c3p3*c5p1*ss[19] + 20*c1p1*c4p1*c5p3*ss[24] + 30*c1p1*c4p2*c5p2*ss[23] + 20*c1p1*c4p3*c5p1*ss[22] + 30*c1p2*c2p1*c3p2*ss[14] + 30*c1p2*c2p1*c4p2*ss[16] + 30*c1p2*c2p1*c5p2*ss[18] + 30*c1p2*c2p2*c3p1*ss[13] + 30*c1p2*c2p2*c4p1*ss[14] + 30*c1p2*c2p2*c5p1*ss[15] + 30*c1p2*c3p1*c4p2*ss[17] + 30*c1p2*c3p1*c5p2*ss[19] + 30*c1p2*c3p2*c4p1*ss[16] + 30*c1p2*c3p2*c5p1*ss[17] + 30*c1p2*c4p1*c5p2*ss[20] + 30*c1p2*c4p2*c5p1*ss[19] + 20*c1p3*c2p1*c3p1*ss[12] + 20*c1p3*c2p1*c4p1*ss[13] + 20*c1p3*c2p1*c5p1*ss[14] + 20*c1p3*c3p1*c4p1*ss[14] + 20*c1p3*c3p1*c5p1*ss[15] + 20*c1p3*c4p1*c5p1*ss[16] + 20*c2p1*c3p1*c4p3*ss[21] + 20*c2p1*c3p1*c5p3*ss[24] + 30*c2p1*c3p2*c4p2*ss[20] + 30*c2p1*c3p2*c5p2*ss[22] + 20*c2p1*c3p3*c4p1*ss[19] + 20*c2p1*c3p3*c5p1*ss[20] + 20*c2p1*c4p1*c5p3*ss[25] + 30*c2p1*c4p2*c5p2*ss[24] + 20*c2p1*c4p3*c5p1*ss[23] + 30*c2p2*c3p1*c4p2*ss[19] + 30*c2p2*c3p1*c5p2*ss[21] + 30*c2p2*c3p2*c4p1*ss[18] + 30*c2p2*c3p2*c5p1*ss[19] + 30*c2p2*c4p1*c5p2*ss[22] + 30*c2p2*c4p2*c5p1*ss[21] + 20*c2p3*c3p1*c4p1*ss[17] + 20*c2p3*c3p1*c5p1*ss[18] + 20*c2p3*c4p1*c5p1*ss[19] + 20*c3p1*c4p1*c5p3*ss[26] + 30*c3p1*c4p2*c5p2*ss[25] + 20*c3p1*c4p3*c5p1*ss[24] + 30*c3p2*c4p1*c5p2*ss[24] + 30*c3p2*c4p2*c5p1*ss[23] + 20*c3p3*c4p1*c5p1*ss[22] + 5*c0p1*c1p4*ss[8] + 5*c0p1*c2p4*ss[12] + 5*c0p1*c3p4*ss[16] + 5*c0p1*c4p4*ss[20] + 5*c0p1*c5p4*ss[24] + 10*c0p2*c1p3*ss[7] + 10*c0p2*c2p3*ss[10] + 10*c0p2*c3p3*ss[13] + 10*c0p2*c4p3*ss[16] + 10*c0p2*c5p3*ss[19] + 10*c0p3*c1p2*ss[6] + 10*c0p3*c2p2*ss[8] + 10*c0p3*c3p2*ss[10] + 10*c0p3*c4p2*ss[12] + 10*c0p3*c5p2*ss[14] + 5*c0p4*c1p1*ss[5] + 5*c0p4*c2p1*ss[6] + 5*c0p4*c3p1*ss[7] + 5*c0p4*c4p1*ss[8] + 5*c0p4*c5p1*ss[9] + 5*c1p1*c2p4*ss[13] + 5*c1p1*c3p4*ss[17] + 5*c1p1*c4p4*ss[21] + 5*c1p1*c5p4*ss[25] + 10*c1p2*c2p3*ss[12] + 10*c1p2*c3p3*ss[15] + 10*c1p2*c4p3*ss[18] + 10*c1p2*c5p3*ss[21] + 10*c1p3*c2p2*ss[11] + 10*c1p3*c3p2*ss[13] + 10*c1p3*c4p2*ss[15] + 10*c1p3*c5p2*ss[17] + 5*c1p4*c2p1*ss[10] + 5*c1p4*c3p1*ss[11] + 5*c1p4*c4p1*ss[12] + 5*c1p4*c5p1*ss[13] + 5*c2p1*c3p4*ss[18] + 5*c2p1*c4p4*ss[22] + 5*c2p1*c5p4*ss[26] + 10*c2p2*c3p3*ss[17] + 10*c2p2*c4p3*ss[20] + 10*c2p2*c5p3*ss[23] + 10*c2p3*c3p2*ss[16] + 10*c2p3*c4p2*ss[18] + 10*c2p3*c5p2*ss[20] + 5*c2p4*c3p1*ss[15] + 5*c2p4*c4p1*ss[16] + 5*c2p4*c5p1*ss[17] + 5*c3p1*c4p4*ss[23] + 5*c3p1*c5p4*ss[27] + 10*c3p2*c4p3*ss[22] + 10*c3p2*c5p3*ss[25] + 10*c3p3*c4p2*ss[21] + 10*c3p3*c5p2*ss[23] + 5*c3p4*c4p1*ss[20] + 5*c3p4*c5p1*ss[21] + 5*c4p1*c5p4*ss[28] + 10*c4p2*c5p3*ss[27] + 10*c4p3*c5p2*ss[26] + 5*c4p4*c5p1*ss[25] + c0p5*ss[4] + c1p5*ss[9] + c2p5*ss[14] + c3p5*ss[19] + c4p5*ss[24] + c5p5*ss[29])*sys_part_5;
  grad_4_num += ss[4]*sys_part_0;
  TACC grad_4_den = convert<TACC>(sqrt(square(sys_part_0) + square(sys_part_1) + square(sys_part_2) + square(sys_part_3) + square(sys_part_4) + square(sys_part_5)));
  TACC grad_5_num = convert<TACC>(2*(c0p1*ss[5] + c1p1*ss[6] + c2p1*ss[7] + c3p1*ss[8] + c4p1*ss[9] + c5p1*ss[10])*sys_part_1);
  grad_5_num += 3*(2*c0p1*c1p1*ss[6] + 2*c0p1*c2p1*ss[7] + 2*c0p1*c3p1*ss[8] + 2*c0p1*c4p1*ss[9] + 2*c0p1*c5p1*ss[10] + 2*c1p1*c2p1*ss[8] + 2*c1p1*c3p1*ss[9] + 2*c1p1*c4p1*ss[10] + 2*c1p1*c5p1*ss[11] + 2*c2p1*c3p1*ss[10] + 2*c2p1*c4p1*ss[11] + 2*c2p1*c5p1*ss[12] + 2*c3p1*c4p1*ss[12] + 2*c3p1*c5p1*ss[13] + 2*c4p1*c5p1*ss[14] + c0p2*ss[5] + c1p2*ss[7] + c2p2*ss[9] + c3p2*ss[11] + c4p2*ss[13] + c5p2*ss[15])*sys_part_2;
  grad_5_num += 4*(6*c0p1*c1p1*c2p1*ss[8] + 6*c0p1*c1p1*c3p1*ss[9] + 6*c0p1*c1p1*c4p1*ss[10] + 6*c0p1*c1p1*c5p1*ss[11] + 6*c0p1*c2p1*c3p1*ss[10] + 6*c0p1*c2p1*c4p1*ss[11] + 6*c0p1*c2p1*c5p1*ss[12] + 6*c0p1*c3p1*c4p1*ss[12] + 6*c0p1*c3p1*c5p1*ss[13] + 6*c0p1*c4p1*c5p1*ss[14] + 6*c1p1*c2p1*c3p1*ss[11] + 6*c1p1*c2p1*c4p1*ss[12] + 6*c1p1*c2p1*c5p1*ss[13] + 6*c1p1*c3p1*c4p1*ss[13] + 6*c1p1*c3p1*c5p1*ss[14] + 6*c1p1*c4p1*c5p1*ss[15] + 6*c2p1*c3p1*c4p1*ss[14] + 6*c2p1*c3p1*c5p1*ss[15] + 6*c2p1*c4p1*c5p1*ss[16] + 6*c3p1*c4p1*c5p1*ss[17] + 3*c0p1*c1p2*ss[7] + 3*c0p1*c2p2*ss[9] + 3*c0p1*c3p2*ss[11] + 3*c0p1*c4p2*ss[13] + 3*c0p1*c5p2*ss[15] + 3*c0p2*c1p1*ss[6] + 3*c0p2*c2p1*ss[7] + 3*c0p2*c3p1*ss[8] + 3*c0p2*c4p1*ss[9] + 3*c0p2*c5p1*ss[10] + 3*c1p1*c2p2*ss[10] + 3*c1p1*c3p2*ss[12] + 3*c1p1*c4p2*ss[14] + 3*c1p1*c5p2*ss[16] + 3*c1p2*c2p1*ss[9] + 3*c1p2*c3p1*ss[10] + 3*c1p2*c4p1*ss[11] + 3*c1p2*c5p1*ss[12] + 3*c2p1*c3p2*ss[13] + 3*c2p1*c4p2*ss[15] + 3*c2p1*c5p2*ss[17] + 3*c2p2*c3p1*ss[12] + 3*c2p2*c4p1*ss[13] + 3*c2p2*c5p1*ss[14] + 3*c3p1*c4p2*ss[16] + 3*c3p1*c5p2*ss[18] + 3*c3p2*c4p1*ss[15] + 3*c3p2*c5p1*ss[16] + 3*c4p1*c5p2*ss[19] + 3*c4p2*c5p1*ss[18] + c0p3*ss[5] + c1p3*ss[8] + c2p3*ss[11] + c3p3*ss[14] + c4p3*ss[17] + c5p3*ss[20])*sys_part_3;
  grad_5_num += 5*(24*c0p1*c1p1*c2p1*c3p1*ss[11] + 24*c0p1*c1p1*c2p1*c4p1*ss[12] + 24*c0p1*c1p1*c2p1*c5p1*ss[13] + 24*c0p1*c1p1*c3p1*c4p1*ss[13] + 24*c0p1*c1p1*c3p1*c5p1*ss[14] + 24*c0p1*c1p1*c4p1*c5p1*ss[15] + 24*c0p1*c2p1*c3p1*c4p1*ss[14] + 24*c0p1*c2p1*c3p1*c5p1*ss[15] + 24*c0p1*c2p1*c4p1*c5p1*ss[16] + 24*c0p1*c3p1*c4p1*c5p1*ss[17] + 24*c1p1*c2p1*c3p1*c4p1*ss[15] + 24*c1p1*c2p1*c3p1*c5p1*ss[16] + 24*c1p1*c2p1*c4p1*c5p1*ss[17] + 24*c1p1*c3p1*c4p1*c5p1*ss[18] + 24*c2p1*c3p1*c4p1*c5p1*ss[19] + 12*c0p1*c1p1*c2p2*ss[10] + 12*c0p1*c1p1*c3p2*ss[12] + 12*c0p1*c1p1*c4p2*ss[14] + 12*c0p1*c1p1*c5p2*ss[16] + 12*c0p1*c1p2*c2p1*ss[9] + 12*c0p1*c1p2*c3p1*ss[10] + 12*c0p1*c1p2*c4p1*ss[11] + 12*c0p1*c1p2*c5p1*ss[12] + 12*c0p1*c2p1*c3p2*ss[13] + 12*c0p1*c2p1*c4p2*ss[15] + 12*c0p1*c2p1*c5p2*ss[17] + 12*c0p1*c2p2*c3p1*ss[12] + 12*c0p1*c2p2*c4p1*ss[13] + 12*c0p1*c2p2*c5p1*ss[14] + 12*c0p1*c3p1*c4p2*ss[16] + 12*c0p1*c3p1*c5p2*ss[18] + 12*c0p1*c3p2*c4p1*ss[15] + 12*c0p1*c3p2*c5p1*ss[16] + 12*c0p1*c4p1*c5p2*ss[19] + 12*c0p1*c4p2*c5p1*ss[18] + 12*c0p2*c1p1*c2p1*ss[8] + 12*c0p2*c1p1*c3p1*ss[9] + 12*c0p2*c1p1*c4p1*ss[10] + 12*c0p2*c1p1*c5p1*ss[11] + 12*c0p2*c2p1*c3p1*ss[10] + 12*c0p2*c2p1*c4p1*ss[11] + 12*c0p2*c2p1*c5p1*ss[12] + 12*c0p2*c3p1*c4p1*ss[12] + 12*c0p2*c3p1*c5p1*ss[13] + 12*c0p2*c4p1*c5p1*ss[14] + 12*c1p1*c2p1*c3p2*ss[14] + 12*c1p1*c2p1*c4p2*ss[16] + 12*c1p1*c2p1*c5p2*ss[18] + 12*c1p1*c2p2*c3p1*ss[13] + 12*c1p1*c2p2*c4p1*ss[14] + 12*c1p1*c2p2*c5p1*ss[15] + 12*c1p1*c3p1*c4p2*ss[17] + 12*c1p1*c3p1*c5p2*ss[19] + 12*c1p1*c3p2*c4p1*ss[16] + 12*c1p1*c3p2*c5p1*ss[17] + 12*c1p1*c4p1*c5p2*ss[20] + 12*c1p1*c4p2*c5p1*ss[19] + 12*c1p2*c2p1*c3p1*ss[12] + 12*c1p2*c2p1*c4p1*ss[13] + 12*c1p2*c2p1*c5p1*ss[14] + 12*c1p2*c3p1*c4p1*ss[14] + 12*c1p2*c3p1*c5p1*ss[15] + 12*c1p2*c4p1*c5p1*ss[16] + 12*c2p1*c3p1*c4p2*ss[18] + 12*c2p1*c3p1*c5p2*ss[20] + 12*c2p1*c3p2*c4p1*ss[17] + 12*c2p1*c3p2*c5p1*ss[18] + 12*c2p1*c4p1*c5p2*ss[21] + 12*c2p1*c4p2*c5p1*ss[20] + 12*c2p2*c3p1*c4p1*ss[16] + 12*c2p2*c3p1*c5p1*ss[17] + 12*c2p2*c4p1*c5p1*ss[18] + 12*c3p1*c4p1*c5p2*ss[22] + 12*c3p1*c4p2*c5p1*ss[21] + 12*c3p2*c4p1*c5p1*ss[20] + 4*c0p1*c1p3*ss[8] + 4*c0p1*c2p3*ss[11] + 4*c0p1*c3p3*ss[14] + 4*c0p1*c4p3*ss[17] + 4*c0p1*c5p3*ss[20] + 6*c0p2*c1p2*ss[7] + 6*c0p2*c2p2*ss[9] + 6*c0p2*c3p2*ss[11] + 6*c0p2*c4p2*ss[13] + 6*c0p2*c5p2*ss[15] + 4*c0p3*c1p1*ss[6] + 4*c0p3*c2p1*ss[7] + 4*c0p3*c3p1*ss[8] + 4*c0p3*c4p1*ss[9] + 4*c0p3*c5p1*ss[10] + 4*c1p1*c2p3*ss[12] + 4*c1p1*c3p3*ss[15] + 4*c1p1*c4p3*ss[18] + 4*c1p1*c5p3*ss[21] + 6*c1p2*c2p2*ss[11] + 6*c1p2*c3p2*ss[13] + 6*c1p2*c4p2*ss[15] + 6*c1p2*c5p2*ss[17] + 4*c1p3*c2p1*ss[10] + 4*c1p3*c3p1*ss[11] + 4*c1p3*c4p1*ss[12] + 4*c1p3*c5p1*ss[13] + 4*c2p1*c3p3*ss[16] + 4*c2p1*c4p3*ss[19] + 4*c2p1*c5p3*ss[22] + 6*c2p2*c3p2*ss[15] + 6*c2p2*c4p2*ss[17] + 6*c2p2*c5p2*ss[19] + 4*c2p3*c3p1*ss[14] + 4*c2p3*c4p1*ss[15] + 4*c2p3*c5p1*ss[16] + 4*c3p1*c4p3*ss[20] + 4*c3p1*c5p3*ss[23] + 6*c3p2*c4p2*ss[19] + 6*c3p2*c5p2*ss[21] + 4*c3p3*c4p1*ss[18] + 4*c3p3*c5p1*ss[19] + 4*c4p1*c5p3*ss[24] + 6*c4p2*c5p2*ss[23] + 4*c4p3*c5p1*ss[22] + c0p4*ss[5] + c1p4*ss[9] + c2p4*ss[13] + c3p4*ss[17] + c4p4*ss[21] + c5p4*ss[25])*sys_part_4;
  grad_5_num += 6*(120*c0p1*c1p1*c2p1*c3p1*c4p1*ss[15] + 120*c0p1*c1p1*c2p1*c3p1*c5p1*ss[16] + 120*c0p1*c1p1*c2p1*c4p1*c5p1*ss[17] + 120*c0p1*c1p1*c3p1*c4p1*c5p1*ss[18] + 120*c0p1*c2p1*c3p1*c4p1*c5p1*ss[19] + 120*c1p1*c2p1*c3p1*c4p1*c5p1*ss[20] + 60*c0p1*c1p1*c2p1*c3p2*ss[14] + 60*c0p1*c1p1*c2p1*c4p2*ss[16] + 60*c0p1*c1p1*c2p1*c5p2*ss[18] + 60*c0p1*c1p1*c2p2*c3p1*ss[13] + 60*c0p1*c1p1*c2p2*c4p1*ss[14] + 60*c0p1*c1p1*c2p2*c5p1*ss[15] + 60*c0p1*c1p1*c3p1*c4p2*ss[17] + 60*c0p1*c1p1*c3p1*c5p2*ss[19] + 60*c0p1*c1p1*c3p2*c4p1*ss[16] + 60*c0p1*c1p1*c3p2*c5p1*ss[17] + 60*c0p1*c1p1*c4p1*c5p2*ss[20] + 60*c0p1*c1p1*c4p2*c5p1*ss[19] + 60*c0p1*c1p2*c2p1*c3p1*ss[12] + 60*c0p1*c1p2*c2p1*c4p1*ss[13] + 60*c0p1*c1p2*c2p1*c5p1*ss[14] + 60*c0p1*c1p2*c3p1*c4p1*ss[14] + 60*c0p1*c1p2*c3p1*c5p1*ss[15] + 60*c0p1*c1p2*c4p1*c5p1*ss[16] + 60*c0p1*c2p1*c3p1*c4p2*ss[18] + 60*c0p1*c2p1*c3p1*c5p2*ss[20] + 60*c0p1*c2p1*c3p2*c4p1*ss[17] + 60*c0p1*c2p1*c3p2*c5p1*ss[18] + 60*c0p1*c2p1*c4p1*c5p2*ss[21] + 60*c0p1*c2p1*c4p2*c5p1*ss[20] + 60*c0p1*c2p2*c3p1*c4p1*ss[16] + 60*c0p1*c2p2*c3p1*c5p1*ss[17] + 60*c0p1*c2p2*c4p1*c5p1*ss[18] + 60*c0p1*c3p1*c4p1*c5p2*ss[22] + 60*c0p1*c3p1*c4p2*c5p1*ss[21] + 60*c0p1*c3p2*c4p1*c5p1*ss[20] + 60*c0p2*c1p1*c2p1*c3p1*ss[11] + 60*c0p2*c1p1*c2p1*c4p1*ss[12] + 60*c0p2*c1p1*c2p1*c5p1*ss[13] + 60*c0p2*c1p1*c3p1*c4p1*ss[13] + 60*c0p2*c1p1*c3p1*c5p1*ss[14] + 60*c0p2*c1p1*c4p1*c5p1*ss[15] + 60*c0p2*c2p1*c3p1*c4p1*ss[14] + 60*c0p2*c2p1*c3p1*c5p1*ss[15] + 60*c0p2*c2p1*c4p1*c5p1*ss[16] + 60*c0p2*c3p1*c4p1*c5p1*ss[17] + 60*c1p1*c2p1*c3p1*c4p2*ss[19] + 60*c1p1*c2p1*c3p1*c5p2*ss[21] + 60*c1p1*c2p1*c3p2*c4p1*ss[18] + 60*c1p1*c2p1*c3p2*c5p1*ss[19] + 60*c1p1*c2p1*c4p1*c5p2*ss[22] + 60*c1p1*c2p1*c4p2*c5p1*ss[21] + 60*c1p1*c2p2*c3p1*c4p1*ss[17] + 60*c1p1*c2p2*c3p1*c5p1*ss[18] + 60*c1p1*c2p2*c4p1*c5p1*ss[19] + 60*c1p1*c3p1*c4p1*c5p2*ss[23] + 60*c1p1*c3p1*c4p2*c5p1*ss[22] + 60*c1p1*c3p2*c4p1*c5p1*ss[21] + 60*c1p2*c2p1*c3p1*c4p1*ss[16] + 60*c1p2*c2p1*c3p1*c5p1*ss[17] + 60*c1p2*c2p1*c4p1*c5p1*ss[18] + 60*c1p2*c3p1*c4p1*c5p1*ss[19] + 60*c2p1*c3p1*c4p1*c5p2*ss[24] + 60*c2p1*c3p1*c4p2*c5p1*ss[23] + 60*c2p1*c3p2*c4p1*c5p1*ss[22] + 60*c2p2*c3p1*c4p1*c5p1*ss[21] + 20*c0p1*c1p1*c2p3*ss[12] + 20*c0p1*c1p1*c3p3*ss[15] + 20*c0p1*c1p1*c4p3*ss[18] + 20*c0p1*c1p1*c5p3*ss[21] + 30*c0p1*c1p2*c2p2*ss[11] + 30*c0p1*c1p2*c3p2*ss[13] + 30*c0p1*c1p2*c4p2*ss[15] + 30*c0p1*c1p2*c5p2*ss[17] + 20*c0p1*c1p3*c2p1*ss[10] + 20*c0p1*c1p3*c3p1*ss[11] + 20*c0p1*c1p3*c4p1*ss[12] + 20*c0p1*c1p3*c5p1*ss[13] + 20*c0p1*c2p1*c3p3*ss[16] + 20*c0p1*c2p1*c4p3*ss[19] + 20*c0p1*c2p1*c5p3*ss[22] + 30*c0p1*c2p2*c3p2*ss[15] + 30*c0p1*c2p2*c4p2*ss[17] + 30*c0p1*c2p2*c5p2*ss[19] + 20*c0p1*c2p3*c3p1*ss[14] + 20*c0p1*c2p3*c4p1*ss[15] + 20*c0p1*c2p3*c5p1*ss[16] + 20*c0p1*c3p1*c4p3*ss[20] + 20*c0p1*c3p1*c5p3*ss[23] + 30*c0p1*c3p2*c4p2*ss[19] + 30*c0p1*c3p2*c5p2*ss[21] + 20*c0p1*c3p3*c4p1*ss[18] + 20*c0p1*c3p3*c5p1*ss[19] + 20*c0p1*c4p1*c5p3*ss[24] + 30*c0p1*c4p2*c5p2*ss[23] + 20*c0p1*c4p3*c5p1*ss[22] + 30*c0p2*c1p1*c2p2*ss[10] + 30*c0p2*c1p1*c3p2*ss[12] + 30*c0p2*c1p1*c4p2*ss[14] + 30*c0p2*c1p1*c5p2*ss[16] + 30*c0p2*c1p2*c2p1*ss[9] + 30*c0p2*c1p2*c3p1*ss[10] + 30*c0p2*c1p2*c4p1*ss[11] + 30*c0p2*c1p2*c5p1*ss[12] + 30*c0p2*c2p1*c3p2*ss[13] + 30*c0p2*c2p1*c4p2*ss[15] + 30*c0p2*c2p1*c5p2*ss[17] + 30*c0p2*c2p2*c3p1*ss[12] + 30*c0p2*c2p2*c4p1*ss[13] + 30*c0p2*c2p2*c5p1*ss[14] + 30*c0p2*c3p1*c4p2*ss[16] + 30*c0p2*c3p1*c5p2*ss[18] + 30*c0p2*c3p2*c4p1*ss[15] + 30*c0p2*c3p2*c5p1*ss[16] + 30*c0p2*c4p1*c5p2*ss[19] + 30*c0p2*c4p2*c5p1*ss[18] + 20*c0p3*c1p1*c2p1*ss[8] + 20*c0p3*c1p1*c3p1*ss[9] + 20*c0p3*c1p1*c4p1*ss[10] + 20*c0p3*c1p1*c5p1*ss[11] + 20*c0p3*c2p1*c3p1*ss[10] + 20*c0p3*c2p1*c4p1*ss[11] + 20*c0p3*c2p1*c5p1*ss[12] + 20*c0p3*c3p1*c4p1*ss[12] + 20*c0p3*c3p1*c5p1*ss[13] + 20*c0p3*c4p1*c5p1*ss[14] + 20*c1p1*c2p1*c3p3*ss[17] + 20*c1p1*c2p1*c4p3*ss[20] + 20*c1p1*c2p1*c5p3*ss[23] + 30*c1p1*c2p2*c3p2*ss[16] + 30*c1p1*c2p2*c4p2*ss[18] + 30*c1p1*c2p2*c5p2*ss[20] + 20*c1p1*c2p3*c3p1*ss[15] + 20*c1p1*c2p3*c4p1*ss[16] + 20*c1p1*c2p3*c5p1*ss[17] + 20*c1p1*c3p1*c4p3*ss[21] + 20*c1p1*c3p1*c5p3*ss[24] + 30*c1p1*c3p2*c4p2*ss[20] + 30*c1p1*c3p2*c5p2*ss[22] + 20*c1p1*c3p3*c4p1*ss[19] + 20*c1p1*c3p3*c5p1*ss[20] + 20*c1p1*c4p1*c5p3*ss[25] + 30*c1p1*c4p2*c5p2*ss[24] + 20*c1p1*c4p3*c5p1*ss[23] + 30*c1p2*c2p1*c3p2*ss[15] + 30*c1p2*c2p1*c4p2*ss[17] + 30*c1p2*c2p1*c5p2*ss[19] + 30*c1p2*c2p2*c3p1*ss[14] + 30*c1p2*c2p2*c4p1*ss[15] + 30*c1p2*c2p2*c5p1*ss[16] + 30*c1p2*c3p1*c4p2*ss[18] + 30*c1p2*c3p1*c5p2*ss[20] + 30*c1p2*c3p2*c4p1*ss[17] + 30*c1p2*c3p2*c5p1*ss[18] + 30*c1p2*c4p1*c5p2*ss[21] + 30*c1p2*c4p2*c5p1*ss[20] + 20*c1p3*c2p1*c3p1*ss[13] + 20*c1p3*c2p1*c4p1*ss[14] + 20*c1p3*c2p1*c5p1*ss[15] + 20*c1p3*c3p1*c4p1*ss[15] + 20*c1p3*c3p1*c5p1*ss[16] + 20*c1p3*c4p1*c5p1*ss[17] + 20*c2p1*c3p1*c4p3*ss[22] + 20*c2p1*c3p1*c5p3*ss[25] + 30*c2p1*c3p2*c4p2*ss[21] + 30*c2p1*c3p2*c5p2*ss[23] + 20*c2p1*c3p3*c4p1*ss[20] + 20*c2p1*c3p3*c5p1*ss[21] + 20*c2p1*c4p1*c5p3*ss[26] + 30*c2p1*c4p2*c5p2*ss[25] + 20*c2p1*c4p3*c5p1*ss[24] + 30*c2p2*c3p1*c4p2*ss[20] + 30*c2p2*c3p1*c5p2*ss[22] + 30*c2p2*c3p2*c4p1*ss[19] + 30*c2p2*c3p2*c5p1*ss[20] + 30*c2p2*c4p1*c5p2*ss[23] + 30*c2p2*c4p2*c5p1*ss[22] + 20*c2p3*c3p1*c4p1*ss[18] + 20*c2p3*c3p1*c5p1*ss[19] + 20*c2p3*c4p1*c5p1*ss[20] + 20*c3p1*c4p1*c5p3*ss[27] + 30*c3p1*c4p2*c5p2*ss[26] + 20*c3p1*c4p3*c5p1*ss[25] + 30*c3p2*c4p1*c5p2*ss[25] + 30*c3p2*c4p2*c5p1*ss[24] + 20*c3p3*c4p1*c5p1*ss[23] + 5*c0p1*c1p4*ss[9] + 5*c0p1*c2p4*ss[13] + 5*c0p1*c3p4*ss[17] + 5*c0p1*c4p4*ss[21] + 5*c0p1*c5p4*ss[25] + 10*c0p2*c1p3*ss[8] + 10*c0p2*c2p3*ss[11] + 10*c0p2*c3p3*ss[14] + 10*c0p2*c4p3*ss[17] + 10*c0p2*c5p3*ss[20] + 10*c0p3*c1p2*ss[7] + 10*c0p3*c2p2*ss[9] + 10*c0p3*c3p2*ss[11] + 10*c0p3*c4p2*ss[13] + 10*c0p3*c5p2*ss[15] + 5*c0p4*c1p1*ss[6] + 5*c0p4*c2p1*ss[7] + 5*c0p4*c3p1*ss[8] + 5*c0p4*c4p1*ss[9] + 5*c0p4*c5p1*ss[10] + 5*c1p1*c2p4*ss[14] + 5*c1p1*c3p4*ss[18] + 5*c1p1*c4p4*ss[22] + 5*c1p1*c5p4*ss[26] + 10*c1p2*c2p3*ss[13] + 10*c1p2*c3p3*ss[16] + 10*c1p2*c4p3*ss[19] + 10*c1p2*c5p3*ss[22] + 10*c1p3*c2p2*ss[12] + 10*c1p3*c3p2*ss[14] + 10*c1p3*c4p2*ss[16] + 10*c1p3*c5p2*ss[18] + 5*c1p4*c2p1*ss[11] + 5*c1p4*c3p1*ss[12] + 5*c1p4*c4p1*ss[13] + 5*c1p4*c5p1*ss[14] + 5*c2p1*c3p4*ss[19] + 5*c2p1*c4p4*ss[23] + 5*c2p1*c5p4*ss[27] + 10*c2p2*c3p3*ss[18] + 10*c2p2*c4p3*ss[21] + 10*c2p2*c5p3*ss[24] + 10*c2p3*c3p2*ss[17] + 10*c2p3*c4p2*ss[19] + 10*c2p3*c5p2*ss[21] + 5*c2p4*c3p1*ss[16] + 5*c2p4*c4p1*ss[17] + 5*c2p4*c5p1*ss[18] + 5*c3p1*c4p4*ss[24] + 5*c3p1*c5p4*ss[28] + 10*c3p2*c4p3*ss[23] + 10*c3p2*c5p3*ss[26] + 10*c3p3*c4p2*ss[22] + 10*c3p3*c5p2*ss[24] + 5*c3p4*c4p1*ss[21] + 5*c3p4*c5p1*ss[22] + 5*c4p1*c5p4*ss[29] + 10*c4p2*c5p3*ss[28] + 10*c4p3*c5p2*ss[27] + 5*c4p4*c5p1*ss[26] + c0p5*ss[5] + c1p5*ss[10] + c2p5*ss[15] + c3p5*ss[20] + c4p5*ss[25] + c5p5*ss[30])*sys_part_5;
  grad_5_num += ss[5]*sys_part_0;
  TACC grad_5_den = convert<TACC>(sqrt(square(sys_part_0) + square(sys_part_1) + square(sys_part_2) + square(sys_part_3) + square(sys_part_4) + square(sys_part_5)));
  TACC metric_sqr = convert<TACC>(square(sys_part_0));
  metric_sqr += square(sys_part_1);
  metric_sqr += square(sys_part_2);
  metric_sqr += square(sys_part_3);
  metric_sqr += square(sys_part_4);
  metric_sqr += square(sys_part_5);
  gradient[0]=convert<T>(grad_0_num / grad_0_den);
  gradient[1]=convert<T>(grad_1_num / grad_1_den);
  gradient[2]=convert<T>(grad_2_num / grad_2_den);
  gradient[3]=convert<T>(grad_3_num / grad_3_den);
  gradient[4]=convert<T>(grad_4_num / grad_4_den);
  gradient[5]=convert<T>(grad_5_num / grad_5_den);
  return convert<T>(sqrt(metric_sqr));
}
void roots(const T *parameters, T *roots) const {
  T c0p1 = parameters[0];
  T c1p1 = parameters[1];
  T c2p1 = parameters[2];
  T c3p1 = parameters[3];
  T c4p1 = parameters[4];
  T c5p1 = parameters[5];
  TACC root_0 = convert<TACC>(c1p1*ss[1]);
  root_0 += c2p1*ss[2];
  root_0 += c3p1*ss[3];
  root_0 += c4p1*ss[4];
  root_0 += c5p1*ss[5];
  root_0 += c0p1;
  root_0 += -tmom[1];
  T c0p2 = c0p1*c0p1;
  T c1p2 = c1p1*c1p1;
  T c2p2 = c2p1*c2p1;
  T c3p2 = c3p1*c3p1;
  T c4p2 = c4p1*c4p1;
  T c5p2 = c5p1*c5p1;
  TACC root_1 = convert<TACC>(2*c0p1*c1p1*ss[1]);
  root_1 += 2*c0p1*c2p1*ss[2];
  root_1 += 2*c0p1*c3p1*ss[3];
  root_1 += 2*c0p1*c4p1*ss[4];
  root_1 += 2*c0p1*c5p1*ss[5];
  root_1 += 2*c1p1*c2p1*ss[3];
  root_1 += 2*c1p1*c3p1*ss[4];
  root_1 += 2*c1p1*c4p1*ss[5];
  root_1 += 2*c1p1*c5p1*ss[6];
  root_1 += 2*c2p1*c3p1*ss[5];
  root_1 += 2*c2p1*c4p1*ss[6];
  root_1 += 2*c2p1*c5p1*ss[7];
  root_1 += 2*c3p1*c4p1*ss[7];
  root_1 += 2*c3p1*c5p1*ss[8];
  root_1 += 2*c4p1*c5p1*ss[9];
  root_1 += c1p2*ss[2];
  root_1 += c2p2*ss[4];
  root_1 += c3p2*ss[6];
  root_1 += c4p2*ss[8];
  root_1 += c5p2*ss[10];
  root_1 += c0p2;
  root_1 += -tmom[2];
  T c0p3 = c0p2*c0p1;
  T c1p3 = c1p2*c1p1;
  T c2p3 = c2p2*c2p1;
  T c3p3 = c3p2*c3p1;
  T c4p3 = c4p2*c4p1;
  T c5p3 = c5p2*c5p1;
  TACC root_2 = convert<TACC>(6*c0p1*c1p1*c2p1*ss[3]);
  root_2 += 6*c0p1*c1p1*c3p1*ss[4];
  root_2 += 6*c0p1*c1p1*c4p1*ss[5];
  root_2 += 6*c0p1*c1p1*c5p1*ss[6];
  root_2 += 6*c0p1*c2p1*c3p1*ss[5];
  root_2 += 6*c0p1*c2p1*c4p1*ss[6];
  root_2 += 6*c0p1*c2p1*c5p1*ss[7];
  root_2 += 6*c0p1*c3p1*c4p1*ss[7];
  root_2 += 6*c0p1*c3p1*c5p1*ss[8];
  root_2 += 6*c0p1*c4p1*c5p1*ss[9];
  root_2 += 6*c1p1*c2p1*c3p1*ss[6];
  root_2 += 6*c1p1*c2p1*c4p1*ss[7];
  root_2 += 6*c1p1*c2p1*c5p1*ss[8];
  root_2 += 6*c1p1*c3p1*c4p1*ss[8];
  root_2 += 6*c1p1*c3p1*c5p1*ss[9];
  root_2 += 6*c1p1*c4p1*c5p1*ss[10];
  root_2 += 6*c2p1*c3p1*c4p1*ss[9];
  root_2 += 6*c2p1*c3p1*c5p1*ss[10];
  root_2 += 6*c2p1*c4p1*c5p1*ss[11];
  root_2 += 6*c3p1*c4p1*c5p1*ss[12];
  root_2 += 3*c0p1*c1p2*ss[2];
  root_2 += 3*c0p1*c2p2*ss[4];
  root_2 += 3*c0p1*c3p2*ss[6];
  root_2 += 3*c0p1*c4p2*ss[8];
  root_2 += 3*c0p1*c5p2*ss[10];
  root_2 += 3*c0p2*c1p1*ss[1];
  root_2 += 3*c0p2*c2p1*ss[2];
  root_2 += 3*c0p2*c3p1*ss[3];
  root_2 += 3*c0p2*c4p1*ss[4];
  root_2 += 3*c0p2*c5p1*ss[5];
  root_2 += 3*c1p1*c2p2*ss[5];
  root_2 += 3*c1p1*c3p2*ss[7];
  root_2 += 3*c1p1*c4p2*ss[9];
  root_2 += 3*c1p1*c5p2*ss[11];
  root_2 += 3*c1p2*c2p1*ss[4];
  root_2 += 3*c1p2*c3p1*ss[5];
  root_2 += 3*c1p2*c4p1*ss[6];
  root_2 += 3*c1p2*c5p1*ss[7];
  root_2 += 3*c2p1*c3p2*ss[8];
  root_2 += 3*c2p1*c4p2*ss[10];
  root_2 += 3*c2p1*c5p2*ss[12];
  root_2 += 3*c2p2*c3p1*ss[7];
  root_2 += 3*c2p2*c4p1*ss[8];
  root_2 += 3*c2p2*c5p1*ss[9];
  root_2 += 3*c3p1*c4p2*ss[11];
  root_2 += 3*c3p1*c5p2*ss[13];
  root_2 += 3*c3p2*c4p1*ss[10];
  root_2 += 3*c3p2*c5p1*ss[11];
  root_2 += 3*c4p1*c5p2*ss[14];
  root_2 += 3*c4p2*c5p1*ss[13];
  root_2 += c1p3*ss[3];
  root_2 += c2p3*ss[6];
  root_2 += c3p3*ss[9];
  root_2 += c4p3*ss[12];
  root_2 += c5p3*ss[15];
  root_2 += c0p3;
  root_2 += -tmom[3];
  T c0p4 = c0p3*c0p1;
  T c1p4 = c1p3*c1p1;
  T c2p4 = c2p3*c2p1;
  T c3p4 = c3p3*c3p1;
  T c4p4 = c4p3*c4p1;
  T c5p4 = c5p3*c5p1;
  TACC root_3 = convert<TACC>(24*c0p1*c1p1*c2p1*c3p1*ss[6]);
  root_3 += 24*c0p1*c1p1*c2p1*c4p1*ss[7];
  root_3 += 24*c0p1*c1p1*c2p1*c5p1*ss[8];
  root_3 += 24*c0p1*c1p1*c3p1*c4p1*ss[8];
  root_3 += 24*c0p1*c1p1*c3p1*c5p1*ss[9];
  root_3 += 24*c0p1*c1p1*c4p1*c5p1*ss[10];
  root_3 += 24*c0p1*c2p1*c3p1*c4p1*ss[9];
  root_3 += 24*c0p1*c2p1*c3p1*c5p1*ss[10];
  root_3 += 24*c0p1*c2p1*c4p1*c5p1*ss[11];
  root_3 += 24*c0p1*c3p1*c4p1*c5p1*ss[12];
  root_3 += 24*c1p1*c2p1*c3p1*c4p1*ss[10];
  root_3 += 24*c1p1*c2p1*c3p1*c5p1*ss[11];
  root_3 += 24*c1p1*c2p1*c4p1*c5p1*ss[12];
  root_3 += 24*c1p1*c3p1*c4p1*c5p1*ss[13];
  root_3 += 24*c2p1*c3p1*c4p1*c5p1*ss[14];
  root_3 += 12*c0p1*c1p1*c2p2*ss[5];
  root_3 += 12*c0p1*c1p1*c3p2*ss[7];
  root_3 += 12*c0p1*c1p1*c4p2*ss[9];
  root_3 += 12*c0p1*c1p1*c5p2*ss[11];
  root_3 += 12*c0p1*c1p2*c2p1*ss[4];
  root_3 += 12*c0p1*c1p2*c3p1*ss[5];
  root_3 += 12*c0p1*c1p2*c4p1*ss[6];
  root_3 += 12*c0p1*c1p2*c5p1*ss[7];
  root_3 += 12*c0p1*c2p1*c3p2*ss[8];
  root_3 += 12*c0p1*c2p1*c4p2*ss[10];
  root_3 += 12*c0p1*c2p1*c5p2*ss[12];
  root_3 += 12*c0p1*c2p2*c3p1*ss[7];
  root_3 += 12*c0p1*c2p2*c4p1*ss[8];
  root_3 += 12*c0p1*c2p2*c5p1*ss[9];
  root_3 += 12*c0p1*c3p1*c4p2*ss[11];
  root_3 += 12*c0p1*c3p1*c5p2*ss[13];
  root_3 += 12*c0p1*c3p2*c4p1*ss[10];
  root_3 += 12*c0p1*c3p2*c5p1*ss[11];
  root_3 += 12*c0p1*c4p1*c5p2*ss[14];
  root_3 += 12*c0p1*c4p2*c5p1*ss[13];
  root_3 += 12*c0p2*c1p1*c2p1*ss[3];
  root_3 += 12*c0p2*c1p1*c3p1*ss[4];
  root_3 += 12*c0p2*c1p1*c4p1*ss[5];
  root_3 += 12*c0p2*c1p1*c5p1*ss[6];
  root_3 += 12*c0p2*c2p1*c3p1*ss[5];
  root_3 += 12*c0p2*c2p1*c4p1*ss[6];
  root_3 += 12*c0p2*c2p1*c5p1*ss[7];
  root_3 += 12*c0p2*c3p1*c4p1*ss[7];
  root_3 += 12*c0p2*c3p1*c5p1*ss[8];
  root_3 += 12*c0p2*c4p1*c5p1*ss[9];
  root_3 += 12*c1p1*c2p1*c3p2*ss[9];
  root_3 += 12*c1p1*c2p1*c4p2*ss[11];
  root_3 += 12*c1p1*c2p1*c5p2*ss[13];
  root_3 += 12*c1p1*c2p2*c3p1*ss[8];
  root_3 += 12*c1p1*c2p2*c4p1*ss[9];
  root_3 += 12*c1p1*c2p2*c5p1*ss[10];
  root_3 += 12*c1p1*c3p1*c4p2*ss[12];
  root_3 += 12*c1p1*c3p1*c5p2*ss[14];
  root_3 += 12*c1p1*c3p2*c4p1*ss[11];
  root_3 += 12*c1p1*c3p2*c5p1*ss[12];
  root_3 += 12*c1p1*c4p1*c5p2*ss[15];
  root_3 += 12*c1p1*c4p2*c5p1*ss[14];
  root_3 += 12*c1p2*c2p1*c3p1*ss[7];
  root_3 += 12*c1p2*c2p1*c4p1*ss[8];
  root_3 += 12*c1p2*c2p1*c5p1*ss[9];
  root_3 += 12*c1p2*c3p1*c4p1*ss[9];
  root_3 += 12*c1p2*c3p1*c5p1*ss[10];
  root_3 += 12*c1p2*c4p1*c5p1*ss[11];
  root_3 += 12*c2p1*c3p1*c4p2*ss[13];
  root_3 += 12*c2p1*c3p1*c5p2*ss[15];
  root_3 += 12*c2p1*c3p2*c4p1*ss[12];
  root_3 += 12*c2p1*c3p2*c5p1*ss[13];
  root_3 += 12*c2p1*c4p1*c5p2*ss[16];
  root_3 += 12*c2p1*c4p2*c5p1*ss[15];
  root_3 += 12*c2p2*c3p1*c4p1*ss[11];
  root_3 += 12*c2p2*c3p1*c5p1*ss[12];
  root_3 += 12*c2p2*c4p1*c5p1*ss[13];
  root_3 += 12*c3p1*c4p1*c5p2*ss[17];
  root_3 += 12*c3p1*c4p2*c5p1*ss[16];
  root_3 += 12*c3p2*c4p1*c5p1*ss[15];
  root_3 += 4*c0p1*c1p3*ss[3];
  root_3 += 4*c0p1*c2p3*ss[6];
  root_3 += 4*c0p1*c3p3*ss[9];
  root_3 += 4*c0p1*c4p3*ss[12];
  root_3 += 4*c0p1*c5p3*ss[15];
  root_3 += 6*c0p2*c1p2*ss[2];
  root_3 += 6*c0p2*c2p2*ss[4];
  root_3 += 6*c0p2*c3p2*ss[6];
  root_3 += 6*c0p2*c4p2*ss[8];
  root_3 += 6*c0p2*c5p2*ss[10];
  root_3 += 4*c0p3*c1p1*ss[1];
  root_3 += 4*c0p3*c2p1*ss[2];
  root_3 += 4*c0p3*c3p1*ss[3];
  root_3 += 4*c0p3*c4p1*ss[4];
  root_3 += 4*c0p3*c5p1*ss[5];
  root_3 += 4*c1p1*c2p3*ss[7];
  root_3 += 4*c1p1*c3p3*ss[10];
  root_3 += 4*c1p1*c4p3*ss[13];
  root_3 += 4*c1p1*c5p3*ss[16];
  root_3 += 6*c1p2*c2p2*ss[6];
  root_3 += 6*c1p2*c3p2*ss[8];
  root_3 += 6*c1p2*c4p2*ss[10];
  root_3 += 6*c1p2*c5p2*ss[12];
  root_3 += 4*c1p3*c2p1*ss[5];
  root_3 += 4*c1p3*c3p1*ss[6];
  root_3 += 4*c1p3*c4p1*ss[7];
  root_3 += 4*c1p3*c5p1*ss[8];
  root_3 += 4*c2p1*c3p3*ss[11];
  root_3 += 4*c2p1*c4p3*ss[14];
  root_3 += 4*c2p1*c5p3*ss[17];
  root_3 += 6*c2p2*c3p2*ss[10];
  root_3 += 6*c2p2*c4p2*ss[12];
  root_3 += 6*c2p2*c5p2*ss[14];
  root_3 += 4*c2p3*c3p1*ss[9];
  root_3 += 4*c2p3*c4p1*ss[10];
  root_3 += 4*c2p3*c5p1*ss[11];
  root_3 += 4*c3p1*c4p3*ss[15];
  root_3 += 4*c3p1*c5p3*ss[18];
  root_3 += 6*c3p2*c4p2*ss[14];
  root_3 += 6*c3p2*c5p2*ss[16];
  root_3 += 4*c3p3*c4p1*ss[13];
  root_3 += 4*c3p3*c5p1*ss[14];
  root_3 += 4*c4p1*c5p3*ss[19];
  root_3 += 6*c4p2*c5p2*ss[18];
  root_3 += 4*c4p3*c5p1*ss[17];
  root_3 += c1p4*ss[4];
  root_3 += c2p4*ss[8];
  root_3 += c3p4*ss[12];
  root_3 += c4p4*ss[16];
  root_3 += c5p4*ss[20];
  root_3 += c0p4;
  root_3 += -tmom[4];
  T c0p5 = c0p4*c0p1;
  T c1p5 = c1p4*c1p1;
  T c2p5 = c2p4*c2p1;
  T c3p5 = c3p4*c3p1;
  T c4p5 = c4p4*c4p1;
  T c5p5 = c5p4*c5p1;
  TACC root_4 = convert<TACC>(120*c0p1*c1p1*c2p1*c3p1*c4p1*ss[10]);
  root_4 += 120*c0p1*c1p1*c2p1*c3p1*c5p1*ss[11];
  root_4 += 120*c0p1*c1p1*c2p1*c4p1*c5p1*ss[12];
  root_4 += 120*c0p1*c1p1*c3p1*c4p1*c5p1*ss[13];
  root_4 += 120*c0p1*c2p1*c3p1*c4p1*c5p1*ss[14];
  root_4 += 120*c1p1*c2p1*c3p1*c4p1*c5p1*ss[15];
  root_4 += 60*c0p1*c1p1*c2p1*c3p2*ss[9];
  root_4 += 60*c0p1*c1p1*c2p1*c4p2*ss[11];
  root_4 += 60*c0p1*c1p1*c2p1*c5p2*ss[13];
  root_4 += 60*c0p1*c1p1*c2p2*c3p1*ss[8];
  root_4 += 60*c0p1*c1p1*c2p2*c4p1*ss[9];
  root_4 += 60*c0p1*c1p1*c2p2*c5p1*ss[10];
  root_4 += 60*c0p1*c1p1*c3p1*c4p2*ss[12];
  root_4 += 60*c0p1*c1p1*c3p1*c5p2*ss[14];
  root_4 += 60*c0p1*c1p1*c3p2*c4p1*ss[11];
  root_4 += 60*c0p1*c1p1*c3p2*c5p1*ss[12];
  root_4 += 60*c0p1*c1p1*c4p1*c5p2*ss[15];
  root_4 += 60*c0p1*c1p1*c4p2*c5p1*ss[14];
  root_4 += 60*c0p1*c1p2*c2p1*c3p1*ss[7];
  root_4 += 60*c0p1*c1p2*c2p1*c4p1*ss[8];
  root_4 += 60*c0p1*c1p2*c2p1*c5p1*ss[9];
  root_4 += 60*c0p1*c1p2*c3p1*c4p1*ss[9];
  root_4 += 60*c0p1*c1p2*c3p1*c5p1*ss[10];
  root_4 += 60*c0p1*c1p2*c4p1*c5p1*ss[11];
  root_4 += 60*c0p1*c2p1*c3p1*c4p2*ss[13];
  root_4 += 60*c0p1*c2p1*c3p1*c5p2*ss[15];
  root_4 += 60*c0p1*c2p1*c3p2*c4p1*ss[12];
  root_4 += 60*c0p1*c2p1*c3p2*c5p1*ss[13];
  root_4 += 60*c0p1*c2p1*c4p1*c5p2*ss[16];
  root_4 += 60*c0p1*c2p1*c4p2*c5p1*ss[15];
  root_4 += 60*c0p1*c2p2*c3p1*c4p1*ss[11];
  root_4 += 60*c0p1*c2p2*c3p1*c5p1*ss[12];
  root_4 += 60*c0p1*c2p2*c4p1*c5p1*ss[13];
  root_4 += 60*c0p1*c3p1*c4p1*c5p2*ss[17];
  root_4 += 60*c0p1*c3p1*c4p2*c5p1*ss[16];
  root_4 += 60*c0p1*c3p2*c4p1*c5p1*ss[15];
  root_4 += 60*c0p2*c1p1*c2p1*c3p1*ss[6];
  root_4 += 60*c0p2*c1p1*c2p1*c4p1*ss[7];
  root_4 += 60*c0p2*c1p1*c2p1*c5p1*ss[8];
  root_4 += 60*c0p2*c1p1*c3p1*c4p1*ss[8];
  root_4 += 60*c0p2*c1p1*c3p1*c5p1*ss[9];
  root_4 += 60*c0p2*c1p1*c4p1*c5p1*ss[10];
  root_4 += 60*c0p2*c2p1*c3p1*c4p1*ss[9];
  root_4 += 60*c0p2*c2p1*c3p1*c5p1*ss[10];
  root_4 += 60*c0p2*c2p1*c4p1*c5p1*ss[11];
  root_4 += 60*c0p2*c3p1*c4p1*c5p1*ss[12];
  root_4 += 60*c1p1*c2p1*c3p1*c4p2*ss[14];
  root_4 += 60*c1p1*c2p1*c3p1*c5p2*ss[16];
  root_4 += 60*c1p1*c2p1*c3p2*c4p1*ss[13];
  root_4 += 60*c1p1*c2p1*c3p2*c5p1*ss[14];
  root_4 += 60*c1p1*c2p1*c4p1*c5p2*ss[17];
  root_4 += 60*c1p1*c2p1*c4p2*c5p1*ss[16];
  root_4 += 60*c1p1*c2p2*c3p1*c4p1*ss[12];
  root_4 += 60*c1p1*c2p2*c3p1*c5p1*ss[13];
  root_4 += 60*c1p1*c2p2*c4p1*c5p1*ss[14];
  root_4 += 60*c1p1*c3p1*c4p1*c5p2*ss[18];
  root_4 += 60*c1p1*c3p1*c4p2*c5p1*ss[17];
  root_4 += 60*c1p1*c3p2*c4p1*c5p1*ss[16];
  root_4 += 60*c1p2*c2p1*c3p1*c4p1*ss[11];
  root_4 += 60*c1p2*c2p1*c3p1*c5p1*ss[12];
  root_4 += 60*c1p2*c2p1*c4p1*c5p1*ss[13];
  root_4 += 60*c1p2*c3p1*c4p1*c5p1*ss[14];
  root_4 += 60*c2p1*c3p1*c4p1*c5p2*ss[19];
  root_4 += 60*c2p1*c3p1*c4p2*c5p1*ss[18];
  root_4 += 60*c2p1*c3p2*c4p1*c5p1*ss[17];
  root_4 += 60*c2p2*c3p1*c4p1*c5p1*ss[16];
  root_4 += 20*c0p1*c1p1*c2p3*ss[7];
  root_4 += 20*c0p1*c1p1*c3p3*ss[10];
  root_4 += 20*c0p1*c1p1*c4p3*ss[13];
  root_4 += 20*c0p1*c1p1*c5p3*ss[16];
  root_4 += 30*c0p1*c1p2*c2p2*ss[6];
  root_4 += 30*c0p1*c1p2*c3p2*ss[8];
  root_4 += 30*c0p1*c1p2*c4p2*ss[10];
  root_4 += 30*c0p1*c1p2*c5p2*ss[12];
  root_4 += 20*c0p1*c1p3*c2p1*ss[5];
  root_4 += 20*c0p1*c1p3*c3p1*ss[6];
  root_4 += 20*c0p1*c1p3*c4p1*ss[7];
  root_4 += 20*c0p1*c1p3*c5p1*ss[8];
  root_4 += 20*c0p1*c2p1*c3p3*ss[11];
  root_4 += 20*c0p1*c2p1*c4p3*ss[14];
  root_4 += 20*c0p1*c2p1*c5p3*ss[17];
  root_4 += 30*c0p1*c2p2*c3p2*ss[10];
  root_4 += 30*c0p1*c2p2*c4p2*ss[12];
  root_4 += 30*c0p1*c2p2*c5p2*ss[14];
  root_4 += 20*c0p1*c2p3*c3p1*ss[9];
  root_4 += 20*c0p1*c2p3*c4p1*ss[10];
  root_4 += 20*c0p1*c2p3*c5p1*ss[11];
  root_4 += 20*c0p1*c3p1*c4p3*ss[15];
  root_4 += 20*c0p1*c3p1*c5p3*ss[18];
  root_4 += 30*c0p1*c3p2*c4p2*ss[14];
  root_4 += 30*c0p1*c3p2*c5p2*ss[16];
  root_4 += 20*c0p1*c3p3*c4p1*ss[13];
  root_4 += 20*c0p1*c3p3*c5p1*ss[14];
  root_4 += 20*c0p1*c4p1*c5p3*ss[19];
  root_4 += 30*c0p1*c4p2*c5p2*ss[18];
  root_4 += 20*c0p1*c4p3*c5p1*ss[17];
  root_4 += 30*c0p2*c1p1*c2p2*ss[5];
  root_4 += 30*c0p2*c1p1*c3p2*ss[7];
  root_4 += 30*c0p2*c1p1*c4p2*ss[9];
  root_4 += 30*c0p2*c1p1*c5p2*ss[11];
  root_4 += 30*c0p2*c1p2*c2p1*ss[4];
  root_4 += 30*c0p2*c1p2*c3p1*ss[5];
  root_4 += 30*c0p2*c1p2*c4p1*ss[6];
  root_4 += 30*c0p2*c1p2*c5p1*ss[7];
  root_4 += 30*c0p2*c2p1*c3p2*ss[8];
  root_4 += 30*c0p2*c2p1*c4p2*ss[10];
  root_4 += 30*c0p2*c2p1*c5p2*ss[12];
  root_4 += 30*c0p2*c2p2*c3p1*ss[7];
  root_4 += 30*c0p2*c2p2*c4p1*ss[8];
  root_4 += 30*c0p2*c2p2*c5p1*ss[9];
  root_4 += 30*c0p2*c3p1*c4p2*ss[11];
  root_4 += 30*c0p2*c3p1*c5p2*ss[13];
  root_4 += 30*c0p2*c3p2*c4p1*ss[10];
  root_4 += 30*c0p2*c3p2*c5p1*ss[11];
  root_4 += 30*c0p2*c4p1*c5p2*ss[14];
  root_4 += 30*c0p2*c4p2*c5p1*ss[13];
  root_4 += 20*c0p3*c1p1*c2p1*ss[3];
  root_4 += 20*c0p3*c1p1*c3p1*ss[4];
  root_4 += 20*c0p3*c1p1*c4p1*ss[5];
  root_4 += 20*c0p3*c1p1*c5p1*ss[6];
  root_4 += 20*c0p3*c2p1*c3p1*ss[5];
  root_4 += 20*c0p3*c2p1*c4p1*ss[6];
  root_4 += 20*c0p3*c2p1*c5p1*ss[7];
  root_4 += 20*c0p3*c3p1*c4p1*ss[7];
  root_4 += 20*c0p3*c3p1*c5p1*ss[8];
  root_4 += 20*c0p3*c4p1*c5p1*ss[9];
  root_4 += 20*c1p1*c2p1*c3p3*ss[12];
  root_4 += 20*c1p1*c2p1*c4p3*ss[15];
  root_4 += 20*c1p1*c2p1*c5p3*ss[18];
  root_4 += 30*c1p1*c2p2*c3p2*ss[11];
  root_4 += 30*c1p1*c2p2*c4p2*ss[13];
  root_4 += 30*c1p1*c2p2*c5p2*ss[15];
  root_4 += 20*c1p1*c2p3*c3p1*ss[10];
  root_4 += 20*c1p1*c2p3*c4p1*ss[11];
  root_4 += 20*c1p1*c2p3*c5p1*ss[12];
  root_4 += 20*c1p1*c3p1*c4p3*ss[16];
  root_4 += 20*c1p1*c3p1*c5p3*ss[19];
  root_4 += 30*c1p1*c3p2*c4p2*ss[15];
  root_4 += 30*c1p1*c3p2*c5p2*ss[17];
  root_4 += 20*c1p1*c3p3*c4p1*ss[14];
  root_4 += 20*c1p1*c3p3*c5p1*ss[15];
  root_4 += 20*c1p1*c4p1*c5p3*ss[20];
  root_4 += 30*c1p1*c4p2*c5p2*ss[19];
  root_4 += 20*c1p1*c4p3*c5p1*ss[18];
  root_4 += 30*c1p2*c2p1*c3p2*ss[10];
  root_4 += 30*c1p2*c2p1*c4p2*ss[12];
  root_4 += 30*c1p2*c2p1*c5p2*ss[14];
  root_4 += 30*c1p2*c2p2*c3p1*ss[9];
  root_4 += 30*c1p2*c2p2*c4p1*ss[10];
  root_4 += 30*c1p2*c2p2*c5p1*ss[11];
  root_4 += 30*c1p2*c3p1*c4p2*ss[13];
  root_4 += 30*c1p2*c3p1*c5p2*ss[15];
  root_4 += 30*c1p2*c3p2*c4p1*ss[12];
  root_4 += 30*c1p2*c3p2*c5p1*ss[13];
  root_4 += 30*c1p2*c4p1*c5p2*ss[16];
  root_4 += 30*c1p2*c4p2*c5p1*ss[15];
  root_4 += 20*c1p3*c2p1*c3p1*ss[8];
  root_4 += 20*c1p3*c2p1*c4p1*ss[9];
  root_4 += 20*c1p3*c2p1*c5p1*ss[10];
  root_4 += 20*c1p3*c3p1*c4p1*ss[10];
  root_4 += 20*c1p3*c3p1*c5p1*ss[11];
  root_4 += 20*c1p3*c4p1*c5p1*ss[12];
  root_4 += 20*c2p1*c3p1*c4p3*ss[17];
  root_4 += 20*c2p1*c3p1*c5p3*ss[20];
  root_4 += 30*c2p1*c3p2*c4p2*ss[16];
  root_4 += 30*c2p1*c3p2*c5p2*ss[18];
  root_4 += 20*c2p1*c3p3*c4p1*ss[15];
  root_4 += 20*c2p1*c3p3*c5p1*ss[16];
  root_4 += 20*c2p1*c4p1*c5p3*ss[21];
  root_4 += 30*c2p1*c4p2*c5p2*ss[20];
  root_4 += 20*c2p1*c4p3*c5p1*ss[19];
  root_4 += 30*c2p2*c3p1*c4p2*ss[15];
  root_4 += 30*c2p2*c3p1*c5p2*ss[17];
  root_4 += 30*c2p2*c3p2*c4p1*ss[14];
  root_4 += 30*c2p2*c3p2*c5p1*ss[15];
  root_4 += 30*c2p2*c4p1*c5p2*ss[18];
  root_4 += 30*c2p2*c4p2*c5p1*ss[17];
  root_4 += 20*c2p3*c3p1*c4p1*ss[13];
  root_4 += 20*c2p3*c3p1*c5p1*ss[14];
  root_4 += 20*c2p3*c4p1*c5p1*ss[15];
  root_4 += 20*c3p1*c4p1*c5p3*ss[22];
  root_4 += 30*c3p1*c4p2*c5p2*ss[21];
  root_4 += 20*c3p1*c4p3*c5p1*ss[20];
  root_4 += 30*c3p2*c4p1*c5p2*ss[20];
  root_4 += 30*c3p2*c4p2*c5p1*ss[19];
  root_4 += 20*c3p3*c4p1*c5p1*ss[18];
  root_4 += 5*c0p1*c1p4*ss[4];
  root_4 += 5*c0p1*c2p4*ss[8];
  root_4 += 5*c0p1*c3p4*ss[12];
  root_4 += 5*c0p1*c4p4*ss[16];
  root_4 += 5*c0p1*c5p4*ss[20];
  root_4 += 10*c0p2*c1p3*ss[3];
  root_4 += 10*c0p2*c2p3*ss[6];
  root_4 += 10*c0p2*c3p3*ss[9];
  root_4 += 10*c0p2*c4p3*ss[12];
  root_4 += 10*c0p2*c5p3*ss[15];
  root_4 += 10*c0p3*c1p2*ss[2];
  root_4 += 10*c0p3*c2p2*ss[4];
  root_4 += 10*c0p3*c3p2*ss[6];
  root_4 += 10*c0p3*c4p2*ss[8];
  root_4 += 10*c0p3*c5p2*ss[10];
  root_4 += 5*c0p4*c1p1*ss[1];
  root_4 += 5*c0p4*c2p1*ss[2];
  root_4 += 5*c0p4*c3p1*ss[3];
  root_4 += 5*c0p4*c4p1*ss[4];
  root_4 += 5*c0p4*c5p1*ss[5];
  root_4 += 5*c1p1*c2p4*ss[9];
  root_4 += 5*c1p1*c3p4*ss[13];
  root_4 += 5*c1p1*c4p4*ss[17];
  root_4 += 5*c1p1*c5p4*ss[21];
  root_4 += 10*c1p2*c2p3*ss[8];
  root_4 += 10*c1p2*c3p3*ss[11];
  root_4 += 10*c1p2*c4p3*ss[14];
  root_4 += 10*c1p2*c5p3*ss[17];
  root_4 += 10*c1p3*c2p2*ss[7];
  root_4 += 10*c1p3*c3p2*ss[9];
  root_4 += 10*c1p3*c4p2*ss[11];
  root_4 += 10*c1p3*c5p2*ss[13];
  root_4 += 5*c1p4*c2p1*ss[6];
  root_4 += 5*c1p4*c3p1*ss[7];
  root_4 += 5*c1p4*c4p1*ss[8];
  root_4 += 5*c1p4*c5p1*ss[9];
  root_4 += 5*c2p1*c3p4*ss[14];
  root_4 += 5*c2p1*c4p4*ss[18];
  root_4 += 5*c2p1*c5p4*ss[22];
  root_4 += 10*c2p2*c3p3*ss[13];
  root_4 += 10*c2p2*c4p3*ss[16];
  root_4 += 10*c2p2*c5p3*ss[19];
  root_4 += 10*c2p3*c3p2*ss[12];
  root_4 += 10*c2p3*c4p2*ss[14];
  root_4 += 10*c2p3*c5p2*ss[16];
  root_4 += 5*c2p4*c3p1*ss[11];
  root_4 += 5*c2p4*c4p1*ss[12];
  root_4 += 5*c2p4*c5p1*ss[13];
  root_4 += 5*c3p1*c4p4*ss[19];
  root_4 += 5*c3p1*c5p4*ss[23];
  root_4 += 10*c3p2*c4p3*ss[18];
  root_4 += 10*c3p2*c5p3*ss[21];
  root_4 += 10*c3p3*c4p2*ss[17];
  root_4 += 10*c3p3*c5p2*ss[19];
  root_4 += 5*c3p4*c4p1*ss[16];
  root_4 += 5*c3p4*c5p1*ss[17];
  root_4 += 5*c4p1*c5p4*ss[24];
  root_4 += 10*c4p2*c5p3*ss[23];
  root_4 += 10*c4p3*c5p2*ss[22];
  root_4 += 5*c4p4*c5p1*ss[21];
  root_4 += c1p5*ss[5];
  root_4 += c2p5*ss[10];
  root_4 += c3p5*ss[15];
  root_4 += c4p5*ss[20];
  root_4 += c5p5*ss[25];
  root_4 += c0p5;
  root_4 += -tmom[5];
  T c0p6 = c0p5*c0p1;
  T c1p6 = c1p5*c1p1;
  T c2p6 = c2p5*c2p1;
  T c3p6 = c3p5*c3p1;
  T c4p6 = c4p5*c4p1;
  T c5p6 = c5p5*c5p1;
  TACC root_5 = convert<TACC>(720*c0p1*c1p1*c2p1*c3p1*c4p1*c5p1*ss[15]);
  root_5 += 360*c0p1*c1p1*c2p1*c3p1*c4p2*ss[14];
  root_5 += 360*c0p1*c1p1*c2p1*c3p1*c5p2*ss[16];
  root_5 += 360*c0p1*c1p1*c2p1*c3p2*c4p1*ss[13];
  root_5 += 360*c0p1*c1p1*c2p1*c3p2*c5p1*ss[14];
  root_5 += 360*c0p1*c1p1*c2p1*c4p1*c5p2*ss[17];
  root_5 += 360*c0p1*c1p1*c2p1*c4p2*c5p1*ss[16];
  root_5 += 360*c0p1*c1p1*c2p2*c3p1*c4p1*ss[12];
  root_5 += 360*c0p1*c1p1*c2p2*c3p1*c5p1*ss[13];
  root_5 += 360*c0p1*c1p1*c2p2*c4p1*c5p1*ss[14];
  root_5 += 360*c0p1*c1p1*c3p1*c4p1*c5p2*ss[18];
  root_5 += 360*c0p1*c1p1*c3p1*c4p2*c5p1*ss[17];
  root_5 += 360*c0p1*c1p1*c3p2*c4p1*c5p1*ss[16];
  root_5 += 360*c0p1*c1p2*c2p1*c3p1*c4p1*ss[11];
  root_5 += 360*c0p1*c1p2*c2p1*c3p1*c5p1*ss[12];
  root_5 += 360*c0p1*c1p2*c2p1*c4p1*c5p1*ss[13];
  root_5 += 360*c0p1*c1p2*c3p1*c4p1*c5p1*ss[14];
  root_5 += 360*c0p1*c2p1*c3p1*c4p1*c5p2*ss[19];
  root_5 += 360*c0p1*c2p1*c3p1*c4p2*c5p1*ss[18];
  root_5 += 360*c0p1*c2p1*c3p2*c4p1*c5p1*ss[17];
  root_5 += 360*c0p1*c2p2*c3p1*c4p1*c5p1*ss[16];
  root_5 += 360*c0p2*c1p1*c2p1*c3p1*c4p1*ss[10];
  root_5 += 360*c0p2*c1p1*c2p1*c3p1*c5p1*ss[11];
  root_5 += 360*c0p2*c1p1*c2p1*c4p1*c5p1*ss[12];
  root_5 += 360*c0p2*c1p1*c3p1*c4p1*c5p1*ss[13];
  root_5 += 360*c0p2*c2p1*c3p1*c4p1*c5p1*ss[14];
  root_5 += 360*c1p1*c2p1*c3p1*c4p1*c5p2*ss[20];
  root_5 += 360*c1p1*c2p1*c3p1*c4p2*c5p1*ss[19];
  root_5 += 360*c1p1*c2p1*c3p2*c4p1*c5p1*ss[18];
  root_5 += 360*c1p1*c2p2*c3p1*c4p1*c5p1*ss[17];
  root_5 += 360*c1p2*c2p1*c3p1*c4p1*c5p1*ss[16];
  root_5 += 120*c0p1*c1p1*c2p1*c3p3*ss[12];
  root_5 += 120*c0p1*c1p1*c2p1*c4p3*ss[15];
  root_5 += 120*c0p1*c1p1*c2p1*c5p3*ss[18];
  root_5 += 180*c0p1*c1p1*c2p2*c3p2*ss[11];
  root_5 += 180*c0p1*c1p1*c2p2*c4p2*ss[13];
  root_5 += 180*c0p1*c1p1*c2p2*c5p2*ss[15];
  root_5 += 120*c0p1*c1p1*c2p3*c3p1*ss[10];
  root_5 += 120*c0p1*c1p1*c2p3*c4p1*ss[11];
  root_5 += 120*c0p1*c1p1*c2p3*c5p1*ss[12];
  root_5 += 120*c0p1*c1p1*c3p1*c4p3*ss[16];
  root_5 += 120*c0p1*c1p1*c3p1*c5p3*ss[19];
  root_5 += 180*c0p1*c1p1*c3p2*c4p2*ss[15];
  root_5 += 180*c0p1*c1p1*c3p2*c5p2*ss[17];
  root_5 += 120*c0p1*c1p1*c3p3*c4p1*ss[14];
  root_5 += 120*c0p1*c1p1*c3p3*c5p1*ss[15];
  root_5 += 120*c0p1*c1p1*c4p1*c5p3*ss[20];
  root_5 += 180*c0p1*c1p1*c4p2*c5p2*ss[19];
  root_5 += 120*c0p1*c1p1*c4p3*c5p1*ss[18];
  root_5 += 180*c0p1*c1p2*c2p1*c3p2*ss[10];
  root_5 += 180*c0p1*c1p2*c2p1*c4p2*ss[12];
  root_5 += 180*c0p1*c1p2*c2p1*c5p2*ss[14];
  root_5 += 180*c0p1*c1p2*c2p2*c3p1*ss[9];
  root_5 += 180*c0p1*c1p2*c2p2*c4p1*ss[10];
  root_5 += 180*c0p1*c1p2*c2p2*c5p1*ss[11];
  root_5 += 180*c0p1*c1p2*c3p1*c4p2*ss[13];
  root_5 += 180*c0p1*c1p2*c3p1*c5p2*ss[15];
  root_5 += 180*c0p1*c1p2*c3p2*c4p1*ss[12];
  root_5 += 180*c0p1*c1p2*c3p2*c5p1*ss[13];
  root_5 += 180*c0p1*c1p2*c4p1*c5p2*ss[16];
  root_5 += 180*c0p1*c1p2*c4p2*c5p1*ss[15];
  root_5 += 120*c0p1*c1p3*c2p1*c3p1*ss[8];
  root_5 += 120*c0p1*c1p3*c2p1*c4p1*ss[9];
  root_5 += 120*c0p1*c1p3*c2p1*c5p1*ss[10];
  root_5 += 120*c0p1*c1p3*c3p1*c4p1*ss[10];
  root_5 += 120*c0p1*c1p3*c3p1*c5p1*ss[11];
  root_5 += 120*c0p1*c1p3*c4p1*c5p1*ss[12];
  root_5 += 120*c0p1*c2p1*c3p1*c4p3*ss[17];
  root_5 += 120*c0p1*c2p1*c3p1*c5p3*ss[20];
  root_5 += 180*c0p1*c2p1*c3p2*c4p2*ss[16];
  root_5 += 180*c0p1*c2p1*c3p2*c5p2*ss[18];
  root_5 += 120*c0p1*c2p1*c3p3*c4p1*ss[15];
  root_5 += 120*c0p1*c2p1*c3p3*c5p1*ss[16];
  root_5 += 120*c0p1*c2p1*c4p1*c5p3*ss[21];
  root_5 += 180*c0p1*c2p1*c4p2*c5p2*ss[20];
  root_5 += 120*c0p1*c2p1*c4p3*c5p1*ss[19];
  root_5 += 180*c0p1*c2p2*c3p1*c4p2*ss[15];
  root_5 += 180*c0p1*c2p2*c3p1*c5p2*ss[17];
  root_5 += 180*c0p1*c2p2*c3p2*c4p1*ss[14];
  root_5 += 180*c0p1*c2p2*c3p2*c5p1*ss[15];
  root_5 += 180*c0p1*c2p2*c4p1*c5p2*ss[18];
  root_5 += 180*c0p1*c2p2*c4p2*c5p1*ss[17];
  root_5 += 120*c0p1*c2p3*c3p1*c4p1*ss[13];
  root_5 += 120*c0p1*c2p3*c3p1*c5p1*ss[14];
  root_5 += 120*c0p1*c2p3*c4p1*c5p1*ss[15];
  root_5 += 120*c0p1*c3p1*c4p1*c5p3*ss[22];
  root_5 += 180*c0p1*c3p1*c4p2*c5p2*ss[21];
  root_5 += 120*c0p1*c3p1*c4p3*c5p1*ss[20];
  root_5 += 180*c0p1*c3p2*c4p1*c5p2*ss[20];
  root_5 += 180*c0p1*c3p2*c4p2*c5p1*ss[19];
  root_5 += 120*c0p1*c3p3*c4p1*c5p1*ss[18];
  root_5 += 180*c0p2*c1p1*c2p1*c3p2*ss[9];
  root_5 += 180*c0p2*c1p1*c2p1*c4p2*ss[11];
  root_5 += 180*c0p2*c1p1*c2p1*c5p2*ss[13];
  root_5 += 180*c0p2*c1p1*c2p2*c3p1*ss[8];
  root_5 += 180*c0p2*c1p1*c2p2*c4p1*ss[9];
  root_5 += 180*c0p2*c1p1*c2p2*c5p1*ss[10];
  root_5 += 180*c0p2*c1p1*c3p1*c4p2*ss[12];
  root_5 += 180*c0p2*c1p1*c3p1*c5p2*ss[14];
  root_5 += 180*c0p2*c1p1*c3p2*c4p1*ss[11];
  root_5 += 180*c0p2*c1p1*c3p2*c5p1*ss[12];
  root_5 += 180*c0p2*c1p1*c4p1*c5p2*ss[15];
  root_5 += 180*c0p2*c1p1*c4p2*c5p1*ss[14];
  root_5 += 180*c0p2*c1p2*c2p1*c3p1*ss[7];
  root_5 += 180*c0p2*c1p2*c2p1*c4p1*ss[8];
  root_5 += 180*c0p2*c1p2*c2p1*c5p1*ss[9];
  root_5 += 180*c0p2*c1p2*c3p1*c4p1*ss[9];
  root_5 += 180*c0p2*c1p2*c3p1*c5p1*ss[10];
  root_5 += 180*c0p2*c1p2*c4p1*c5p1*ss[11];
  root_5 += 180*c0p2*c2p1*c3p1*c4p2*ss[13];
  root_5 += 180*c0p2*c2p1*c3p1*c5p2*ss[15];
  root_5 += 180*c0p2*c2p1*c3p2*c4p1*ss[12];
  root_5 += 180*c0p2*c2p1*c3p2*c5p1*ss[13];
  root_5 += 180*c0p2*c2p1*c4p1*c5p2*ss[16];
  root_5 += 180*c0p2*c2p1*c4p2*c5p1*ss[15];
  root_5 += 180*c0p2*c2p2*c3p1*c4p1*ss[11];
  root_5 += 180*c0p2*c2p2*c3p1*c5p1*ss[12];
  root_5 += 180*c0p2*c2p2*c4p1*c5p1*ss[13];
  root_5 += 180*c0p2*c3p1*c4p1*c5p2*ss[17];
  root_5 += 180*c0p2*c3p1*c4p2*c5p1*ss[16];
  root_5 += 180*c0p2*c3p2*c4p1*c5p1*ss[15];
  root_5 += 120*c0p3*c1p1*c2p1*c3p1*ss[6];
  root_5 += 120*c0p3*c1p1*c2p1*c4p1*ss[7];
  root_5 += 120*c0p3*c1p1*c2p1*c5p1*ss[8];
  root_5 += 120*c0p3*c1p1*c3p1*c4p1*ss[8];
  root_5 += 120*c0p3*c1p1*c3p1*c5p1*ss[9];
  root_5 += 120*c0p3*c1p1*c4p1*c5p1*ss[10];
  root_5 += 120*c0p3*c2p1*c3p1*c4p1*ss[9];
  root_5 += 120*c0p3*c2p1*c3p1*c5p1*ss[10];
  root_5 += 120*c0p3*c2p1*c4p1*c5p1*ss[11];
  root_5 += 120*c0p3*c3p1*c4p1*c5p1*ss[12];
  root_5 += 120*c1p1*c2p1*c3p1*c4p3*ss[18];
  root_5 += 120*c1p1*c2p1*c3p1*c5p3*ss[21];
  root_5 += 180*c1p1*c2p1*c3p2*c4p2*ss[17];
  root_5 += 180*c1p1*c2p1*c3p2*c5p2*ss[19];
  root_5 += 120*c1p1*c2p1*c3p3*c4p1*ss[16];
  root_5 += 120*c1p1*c2p1*c3p3*c5p1*ss[17];
  root_5 += 120*c1p1*c2p1*c4p1*c5p3*ss[22];
  root_5 += 180*c1p1*c2p1*c4p2*c5p2*ss[21];
  root_5 += 120*c1p1*c2p1*c4p3*c5p1*ss[20];
  root_5 += 180*c1p1*c2p2*c3p1*c4p2*ss[16];
  root_5 += 180*c1p1*c2p2*c3p1*c5p2*ss[18];
  root_5 += 180*c1p1*c2p2*c3p2*c4p1*ss[15];
  root_5 += 180*c1p1*c2p2*c3p2*c5p1*ss[16];
  root_5 += 180*c1p1*c2p2*c4p1*c5p2*ss[19];
  root_5 += 180*c1p1*c2p2*c4p2*c5p1*ss[18];
  root_5 += 120*c1p1*c2p3*c3p1*c4p1*ss[14];
  root_5 += 120*c1p1*c2p3*c3p1*c5p1*ss[15];
  root_5 += 120*c1p1*c2p3*c4p1*c5p1*ss[16];
  root_5 += 120*c1p1*c3p1*c4p1*c5p3*ss[23];
  root_5 += 180*c1p1*c3p1*c4p2*c5p2*ss[22];
  root_5 += 120*c1p1*c3p1*c4p3*c5p1*ss[21];
  root_5 += 180*c1p1*c3p2*c4p1*c5p2*ss[21];
  root_5 += 180*c1p1*c3p2*c4p2*c5p1*ss[20];
  root_5 += 120*c1p1*c3p3*c4p1*c5p1*ss[19];
  root_5 += 180*c1p2*c2p1*c3p1*c4p2*ss[15];
  root_5 += 180*c1p2*c2p1*c3p1*c5p2*ss[17];
  root_5 += 180*c1p2*c2p1*c3p2*c4p1*ss[14];
  root_5 += 180*c1p2*c2p1*c3p2*c5p1*ss[15];
  root_5 += 180*c1p2*c2p1*c4p1*c5p2*ss[18];
  root_5 += 180*c1p2*c2p1*c4p2*c5p1*ss[17];
  root_5 += 180*c1p2*c2p2*c3p1*c4p1*ss[13];
  root_5 += 180*c1p2*c2p2*c3p1*c5p1*ss[14];
  root_5 += 180*c1p2*c2p2*c4p1*c5p1*ss[15];
  root_5 += 180*c1p2*c3p1*c4p1*c5p2*ss[19];
  root_5 += 180*c1p2*c3p1*c4p2*c5p1*ss[18];
  root_5 += 180*c1p2*c3p2*c4p1*c5p1*ss[17];
  root_5 += 120*c1p3*c2p1*c3p1*c4p1*ss[12];
  root_5 += 120*c1p3*c2p1*c3p1*c5p1*ss[13];
  root_5 += 120*c1p3*c2p1*c4p1*c5p1*ss[14];
  root_5 += 120*c1p3*c3p1*c4p1*c5p1*ss[15];
  root_5 += 120*c2p1*c3p1*c4p1*c5p3*ss[24];
  root_5 += 180*c2p1*c3p1*c4p2*c5p2*ss[23];
  root_5 += 120*c2p1*c3p1*c4p3*c5p1*ss[22];
  root_5 += 180*c2p1*c3p2*c4p1*c5p2*ss[22];
  root_5 += 180*c2p1*c3p2*c4p2*c5p1*ss[21];
  root_5 += 120*c2p1*c3p3*c4p1*c5p1*ss[20];
  root_5 += 180*c2p2*c3p1*c4p1*c5p2*ss[21];
  root_5 += 180*c2p2*c3p1*c4p2*c5p1*ss[20];
  root_5 += 180*c2p2*c3p2*c4p1*c5p1*ss[19];
  root_5 += 120*c2p3*c3p1*c4p1*c5p1*ss[18];
  root_5 += 30*c0p1*c1p1*c2p4*ss[9];
  root_5 += 30*c0p1*c1p1*c3p4*ss[13];
  root_5 += 30*c0p1*c1p1*c4p4*ss[17];
  root_5 += 30*c0p1*c1p1*c5p4*ss[21];
  root_5 += 60*c0p1*c1p2*c2p3*ss[8];
  root_5 += 60*c0p1*c1p2*c3p3*ss[11];
  root_5 += 60*c0p1*c1p2*c4p3*ss[14];
  root_5 += 60*c0p1*c1p2*c5p3*ss[17];
  root_5 += 60*c0p1*c1p3*c2p2*ss[7];
  root_5 += 60*c0p1*c1p3*c3p2*ss[9];
  root_5 += 60*c0p1*c1p3*c4p2*ss[11];
  root_5 += 60*c0p1*c1p3*c5p2*ss[13];
  root_5 += 30*c0p1*c1p4*c2p1*ss[6];
  root_5 += 30*c0p1*c1p4*c3p1*ss[7];
  root_5 += 30*c0p1*c1p4*c4p1*ss[8];
  root_5 += 30*c0p1*c1p4*c5p1*ss[9];
  root_5 += 30*c0p1*c2p1*c3p4*ss[14];
  root_5 += 30*c0p1*c2p1*c4p4*ss[18];
  root_5 += 30*c0p1*c2p1*c5p4*ss[22];
  root_5 += 60*c0p1*c2p2*c3p3*ss[13];
  root_5 += 60*c0p1*c2p2*c4p3*ss[16];
  root_5 += 60*c0p1*c2p2*c5p3*ss[19];
  root_5 += 60*c0p1*c2p3*c3p2*ss[12];
  root_5 += 60*c0p1*c2p3*c4p2*ss[14];
  root_5 += 60*c0p1*c2p3*c5p2*ss[16];
  root_5 += 30*c0p1*c2p4*c3p1*ss[11];
  root_5 += 30*c0p1*c2p4*c4p1*ss[12];
  root_5 += 30*c0p1*c2p4*c5p1*ss[13];
  root_5 += 30*c0p1*c3p1*c4p4*ss[19];
  root_5 += 30*c0p1*c3p1*c5p4*ss[23];
  root_5 += 60*c0p1*c3p2*c4p3*ss[18];
  root_5 += 60*c0p1*c3p2*c5p3*ss[21];
  root_5 += 60*c0p1*c3p3*c4p2*ss[17];
  root_5 += 60*c0p1*c3p3*c5p2*ss[19];
  root_5 += 30*c0p1*c3p4*c4p1*ss[16];
  root_5 += 30*c0p1*c3p4*c5p1*ss[17];
  root_5 += 30*c0p1*c4p1*c5p4*ss[24];
  root_5 += 60*c0p1*c4p2*c5p3*ss[23];
  root_5 += 60*c0p1*c4p3*c5p2*ss[22];
  root_5 += 30*c0p1*c4p4*c5p1*ss[21];
  root_5 += 60*c0p2*c1p1*c2p3*ss[7];
  root_5 += 60*c0p2*c1p1*c3p3*ss[10];
  root_5 += 60*c0p2*c1p1*c4p3*ss[13];
  root_5 += 60*c0p2*c1p1*c5p3*ss[16];
  root_5 += 90*c0p2*c1p2*c2p2*ss[6];
  root_5 += 90*c0p2*c1p2*c3p2*ss[8];
  root_5 += 90*c0p2*c1p2*c4p2*ss[10];
  root_5 += 90*c0p2*c1p2*c5p2*ss[12];
  root_5 += 60*c0p2*c1p3*c2p1*ss[5];
  root_5 += 60*c0p2*c1p3*c3p1*ss[6];
  root_5 += 60*c0p2*c1p3*c4p1*ss[7];
  root_5 += 60*c0p2*c1p3*c5p1*ss[8];
  root_5 += 60*c0p2*c2p1*c3p3*ss[11];
  root_5 += 60*c0p2*c2p1*c4p3*ss[14];
  root_5 += 60*c0p2*c2p1*c5p3*ss[17];
  root_5 += 90*c0p2*c2p2*c3p2*ss[10];
  root_5 += 90*c0p2*c2p2*c4p2*ss[12];
  root_5 += 90*c0p2*c2p2*c5p2*ss[14];
  root_5 += 60*c0p2*c2p3*c3p1*ss[9];
  root_5 += 60*c0p2*c2p3*c4p1*ss[10];
  root_5 += 60*c0p2*c2p3*c5p1*ss[11];
  root_5 += 60*c0p2*c3p1*c4p3*ss[15];
  root_5 += 60*c0p2*c3p1*c5p3*ss[18];
  root_5 += 90*c0p2*c3p2*c4p2*ss[14];
  root_5 += 90*c0p2*c3p2*c5p2*ss[16];
  root_5 += 60*c0p2*c3p3*c4p1*ss[13];
  root_5 += 60*c0p2*c3p3*c5p1*ss[14];
  root_5 += 60*c0p2*c4p1*c5p3*ss[19];
  root_5 += 90*c0p2*c4p2*c5p2*ss[18];
  root_5 += 60*c0p2*c4p3*c5p1*ss[17];
  root_5 += 60*c0p3*c1p1*c2p2*ss[5];
  root_5 += 60*c0p3*c1p1*c3p2*ss[7];
  root_5 += 60*c0p3*c1p1*c4p2*ss[9];
  root_5 += 60*c0p3*c1p1*c5p2*ss[11];
  root_5 += 60*c0p3*c1p2*c2p1*ss[4];
  root_5 += 60*c0p3*c1p2*c3p1*ss[5];
  root_5 += 60*c0p3*c1p2*c4p1*ss[6];
  root_5 += 60*c0p3*c1p2*c5p1*ss[7];
  root_5 += 60*c0p3*c2p1*c3p2*ss[8];
  root_5 += 60*c0p3*c2p1*c4p2*ss[10];
  root_5 += 60*c0p3*c2p1*c5p2*ss[12];
  root_5 += 60*c0p3*c2p2*c3p1*ss[7];
  root_5 += 60*c0p3*c2p2*c4p1*ss[8];
  root_5 += 60*c0p3*c2p2*c5p1*ss[9];
  root_5 += 60*c0p3*c3p1*c4p2*ss[11];
  root_5 += 60*c0p3*c3p1*c5p2*ss[13];
  root_5 += 60*c0p3*c3p2*c4p1*ss[10];
  root_5 += 60*c0p3*c3p2*c5p1*ss[11];
  root_5 += 60*c0p3*c4p1*c5p2*ss[14];
  root_5 += 60*c0p3*c4p2*c5p1*ss[13];
  root_5 += 30*c0p4*c1p1*c2p1*ss[3];
  root_5 += 30*c0p4*c1p1*c3p1*ss[4];
  root_5 += 30*c0p4*c1p1*c4p1*ss[5];
  root_5 += 30*c0p4*c1p1*c5p1*ss[6];
  root_5 += 30*c0p4*c2p1*c3p1*ss[5];
  root_5 += 30*c0p4*c2p1*c4p1*ss[6];
  root_5 += 30*c0p4*c2p1*c5p1*ss[7];
  root_5 += 30*c0p4*c3p1*c4p1*ss[7];
  root_5 += 30*c0p4*c3p1*c5p1*ss[8];
  root_5 += 30*c0p4*c4p1*c5p1*ss[9];
  root_5 += 30*c1p1*c2p1*c3p4*ss[15];
  root_5 += 30*c1p1*c2p1*c4p4*ss[19];
  root_5 += 30*c1p1*c2p1*c5p4*ss[23];
  root_5 += 60*c1p1*c2p2*c3p3*ss[14];
  root_5 += 60*c1p1*c2p2*c4p3*ss[17];
  root_5 += 60*c1p1*c2p2*c5p3*ss[20];
  root_5 += 60*c1p1*c2p3*c3p2*ss[13];
  root_5 += 60*c1p1*c2p3*c4p2*ss[15];
  root_5 += 60*c1p1*c2p3*c5p2*ss[17];
  root_5 += 30*c1p1*c2p4*c3p1*ss[12];
  root_5 += 30*c1p1*c2p4*c4p1*ss[13];
  root_5 += 30*c1p1*c2p4*c5p1*ss[14];
  root_5 += 30*c1p1*c3p1*c4p4*ss[20];
  root_5 += 30*c1p1*c3p1*c5p4*ss[24];
  root_5 += 60*c1p1*c3p2*c4p3*ss[19];
  root_5 += 60*c1p1*c3p2*c5p3*ss[22];
  root_5 += 60*c1p1*c3p3*c4p2*ss[18];
  root_5 += 60*c1p1*c3p3*c5p2*ss[20];
  root_5 += 30*c1p1*c3p4*c4p1*ss[17];
  root_5 += 30*c1p1*c3p4*c5p1*ss[18];
  root_5 += 30*c1p1*c4p1*c5p4*ss[25];
  root_5 += 60*c1p1*c4p2*c5p3*ss[24];
  root_5 += 60*c1p1*c4p3*c5p2*ss[23];
  root_5 += 30*c1p1*c4p4*c5p1*ss[22];
  root_5 += 60*c1p2*c2p1*c3p3*ss[13];
  root_5 += 60*c1p2*c2p1*c4p3*ss[16];
  root_5 += 60*c1p2*c2p1*c5p3*ss[19];
  root_5 += 90*c1p2*c2p2*c3p2*ss[12];
  root_5 += 90*c1p2*c2p2*c4p2*ss[14];
  root_5 += 90*c1p2*c2p2*c5p2*ss[16];
  root_5 += 60*c1p2*c2p3*c3p1*ss[11];
  root_5 += 60*c1p2*c2p3*c4p1*ss[12];
  root_5 += 60*c1p2*c2p3*c5p1*ss[13];
  root_5 += 60*c1p2*c3p1*c4p3*ss[17];
  root_5 += 60*c1p2*c3p1*c5p3*ss[20];
  root_5 += 90*c1p2*c3p2*c4p2*ss[16];
  root_5 += 90*c1p2*c3p2*c5p2*ss[18];
  root_5 += 60*c1p2*c3p3*c4p1*ss[15];
  root_5 += 60*c1p2*c3p3*c5p1*ss[16];
  root_5 += 60*c1p2*c4p1*c5p3*ss[21];
  root_5 += 90*c1p2*c4p2*c5p2*ss[20];
  root_5 += 60*c1p2*c4p3*c5p1*ss[19];
  root_5 += 60*c1p3*c2p1*c3p2*ss[11];
  root_5 += 60*c1p3*c2p1*c4p2*ss[13];
  root_5 += 60*c1p3*c2p1*c5p2*ss[15];
  root_5 += 60*c1p3*c2p2*c3p1*ss[10];
  root_5 += 60*c1p3*c2p2*c4p1*ss[11];
  root_5 += 60*c1p3*c2p2*c5p1*ss[12];
  root_5 += 60*c1p3*c3p1*c4p2*ss[14];
  root_5 += 60*c1p3*c3p1*c5p2*ss[16];
  root_5 += 60*c1p3*c3p2*c4p1*ss[13];
  root_5 += 60*c1p3*c3p2*c5p1*ss[14];
  root_5 += 60*c1p3*c4p1*c5p2*ss[17];
  root_5 += 60*c1p3*c4p2*c5p1*ss[16];
  root_5 += 30*c1p4*c2p1*c3p1*ss[9];
  root_5 += 30*c1p4*c2p1*c4p1*ss[10];
  root_5 += 30*c1p4*c2p1*c5p1*ss[11];
  root_5 += 30*c1p4*c3p1*c4p1*ss[11];
  root_5 += 30*c1p4*c3p1*c5p1*ss[12];
  root_5 += 30*c1p4*c4p1*c5p1*ss[13];
  root_5 += 30*c2p1*c3p1*c4p4*ss[21];
  root_5 += 30*c2p1*c3p1*c5p4*ss[25];
  root_5 += 60*c2p1*c3p2*c4p3*ss[20];
  root_5 += 60*c2p1*c3p2*c5p3*ss[23];
  root_5 += 60*c2p1*c3p3*c4p2*ss[19];
  root_5 += 60*c2p1*c3p3*c5p2*ss[21];
  root_5 += 30*c2p1*c3p4*c4p1*ss[18];
  root_5 += 30*c2p1*c3p4*c5p1*ss[19];
  root_5 += 30*c2p1*c4p1*c5p4*ss[26];
  root_5 += 60*c2p1*c4p2*c5p3*ss[25];
  root_5 += 60*c2p1*c4p3*c5p2*ss[24];
  root_5 += 30*c2p1*c4p4*c5p1*ss[23];
  root_5 += 60*c2p2*c3p1*c4p3*ss[19];
  root_5 += 60*c2p2*c3p1*c5p3*ss[22];
  root_5 += 90*c2p2*c3p2*c4p2*ss[18];
  root_5 += 90*c2p2*c3p2*c5p2*ss[20];
  root_5 += 60*c2p2*c3p3*c4p1*ss[17];
  root_5 += 60*c2p2*c3p3*c5p1*ss[18];
  root_5 += 60*c2p2*c4p1*c5p3*ss[23];
  root_5 += 90*c2p2*c4p2*c5p2*ss[22];
  root_5 += 60*c2p2*c4p3*c5p1*ss[21];
  root_5 += 60*c2p3*c3p1*c4p2*ss[17];
  root_5 += 60*c2p3*c3p1*c5p2*ss[19];
  root_5 += 60*c2p3*c3p2*c4p1*ss[16];
  root_5 += 60*c2p3*c3p2*c5p1*ss[17];
  root_5 += 60*c2p3*c4p1*c5p2*ss[20];
  root_5 += 60*c2p3*c4p2*c5p1*ss[19];
  root_5 += 30*c2p4*c3p1*c4p1*ss[15];
  root_5 += 30*c2p4*c3p1*c5p1*ss[16];
  root_5 += 30*c2p4*c4p1*c5p1*ss[17];
  root_5 += 30*c3p1*c4p1*c5p4*ss[27];
  root_5 += 60*c3p1*c4p2*c5p3*ss[26];
  root_5 += 60*c3p1*c4p3*c5p2*ss[25];
  root_5 += 30*c3p1*c4p4*c5p1*ss[24];
  root_5 += 60*c3p2*c4p1*c5p3*ss[25];
  root_5 += 90*c3p2*c4p2*c5p2*ss[24];
  root_5 += 60*c3p2*c4p3*c5p1*ss[23];
  root_5 += 60*c3p3*c4p1*c5p2*ss[23];
  root_5 += 60*c3p3*c4p2*c5p1*ss[22];
  root_5 += 30*c3p4*c4p1*c5p1*ss[21];
  root_5 += 6*c0p1*c1p5*ss[5];
  root_5 += 6*c0p1*c2p5*ss[10];
  root_5 += 6*c0p1*c3p5*ss[15];
  root_5 += 6*c0p1*c4p5*ss[20];
  root_5 += 6*c0p1*c5p5*ss[25];
  root_5 += 15*c0p2*c1p4*ss[4];
  root_5 += 15*c0p2*c2p4*ss[8];
  root_5 += 15*c0p2*c3p4*ss[12];
  root_5 += 15*c0p2*c4p4*ss[16];
  root_5 += 15*c0p2*c5p4*ss[20];
  root_5 += 20*c0p3*c1p3*ss[3];
  root_5 += 20*c0p3*c2p3*ss[6];
  root_5 += 20*c0p3*c3p3*ss[9];
  root_5 += 20*c0p3*c4p3*ss[12];
  root_5 += 20*c0p3*c5p3*ss[15];
  root_5 += 15*c0p4*c1p2*ss[2];
  root_5 += 15*c0p4*c2p2*ss[4];
  root_5 += 15*c0p4*c3p2*ss[6];
  root_5 += 15*c0p4*c4p2*ss[8];
  root_5 += 15*c0p4*c5p2*ss[10];
  root_5 += 6*c0p5*c1p1*ss[1];
  root_5 += 6*c0p5*c2p1*ss[2];
  root_5 += 6*c0p5*c3p1*ss[3];
  root_5 += 6*c0p5*c4p1*ss[4];
  root_5 += 6*c0p5*c5p1*ss[5];
  root_5 += 6*c1p1*c2p5*ss[11];
  root_5 += 6*c1p1*c3p5*ss[16];
  root_5 += 6*c1p1*c4p5*ss[21];
  root_5 += 6*c1p1*c5p5*ss[26];
  root_5 += 15*c1p2*c2p4*ss[10];
  root_5 += 15*c1p2*c3p4*ss[14];
  root_5 += 15*c1p2*c4p4*ss[18];
  root_5 += 15*c1p2*c5p4*ss[22];
  root_5 += 20*c1p3*c2p3*ss[9];
  root_5 += 20*c1p3*c3p3*ss[12];
  root_5 += 20*c1p3*c4p3*ss[15];
  root_5 += 20*c1p3*c5p3*ss[18];
  root_5 += 15*c1p4*c2p2*ss[8];
  root_5 += 15*c1p4*c3p2*ss[10];
  root_5 += 15*c1p4*c4p2*ss[12];
  root_5 += 15*c1p4*c5p2*ss[14];
  root_5 += 6*c1p5*c2p1*ss[7];
  root_5 += 6*c1p5*c3p1*ss[8];
  root_5 += 6*c1p5*c4p1*ss[9];
  root_5 += 6*c1p5*c5p1*ss[10];
  root_5 += 6*c2p1*c3p5*ss[17];
  root_5 += 6*c2p1*c4p5*ss[22];
  root_5 += 6*c2p1*c5p5*ss[27];
  root_5 += 15*c2p2*c3p4*ss[16];
  root_5 += 15*c2p2*c4p4*ss[20];
  root_5 += 15*c2p2*c5p4*ss[24];
  root_5 += 20*c2p3*c3p3*ss[15];
  root_5 += 20*c2p3*c4p3*ss[18];
  root_5 += 20*c2p3*c5p3*ss[21];
  root_5 += 15*c2p4*c3p2*ss[14];
  root_5 += 15*c2p4*c4p2*ss[16];
  root_5 += 15*c2p4*c5p2*ss[18];
  root_5 += 6*c2p5*c3p1*ss[13];
  root_5 += 6*c2p5*c4p1*ss[14];
  root_5 += 6*c2p5*c5p1*ss[15];
  root_5 += 6*c3p1*c4p5*ss[23];
  root_5 += 6*c3p1*c5p5*ss[28];
  root_5 += 15*c3p2*c4p4*ss[22];
  root_5 += 15*c3p2*c5p4*ss[26];
  root_5 += 20*c3p3*c4p3*ss[21];
  root_5 += 20*c3p3*c5p3*ss[24];
  root_5 += 15*c3p4*c4p2*ss[20];
  root_5 += 15*c3p4*c5p2*ss[22];
  root_5 += 6*c3p5*c4p1*ss[19];
  root_5 += 6*c3p5*c5p1*ss[20];
  root_5 += 6*c4p1*c5p5*ss[29];
  root_5 += 15*c4p2*c5p4*ss[28];
  root_5 += 20*c4p3*c5p3*ss[27];
  root_5 += 15*c4p4*c5p2*ss[26];
  root_5 += 6*c4p5*c5p1*ss[25];
  root_5 += c1p6*ss[6];
  root_5 += c2p6*ss[12];
  root_5 += c3p6*ss[18];
  root_5 += c4p6*ss[24];
  root_5 += c5p6*ss[30];
  root_5 += c0p6;
  root_5 += -tmom[6];
  roots[0]=convert<T>(root_0);
  roots[1]=convert<T>(root_1);
  roots[2]=convert<T>(root_2);
  roots[3]=convert<T>(root_3);
  roots[4]=convert<T>(root_4);
  roots[5]=convert<T>(root_5);
}
void roots_and_jacobian(const T *parameters, T *roots, T *jacobian) const {
  T c0p1 = parameters[0];
  T c1p1 = parameters[1];
  T c2p1 = parameters[2];
  T c3p1 = parameters[3];
  T c4p1 = parameters[4];
  T c5p1 = parameters[5];
  TACC root_0 = convert<TACC>(c1p1*ss[1]);
  root_0 += c2p1*ss[2];
  root_0 += c3p1*ss[3];
  root_0 += c4p1*ss[4];
  root_0 += c5p1*ss[5];
  root_0 += c0p1;
  root_0 += -tmom[1];
  T c0p2 = c0p1*c0p1;
  T c1p2 = c1p1*c1p1;
  T c2p2 = c2p1*c2p1;
  T c3p2 = c3p1*c3p1;
  T c4p2 = c4p1*c4p1;
  T c5p2 = c5p1*c5p1;
  TACC root_1 = convert<TACC>(2*c0p1*c1p1*ss[1]);
  root_1 += 2*c0p1*c2p1*ss[2];
  root_1 += 2*c0p1*c3p1*ss[3];
  root_1 += 2*c0p1*c4p1*ss[4];
  root_1 += 2*c0p1*c5p1*ss[5];
  root_1 += 2*c1p1*c2p1*ss[3];
  root_1 += 2*c1p1*c3p1*ss[4];
  root_1 += 2*c1p1*c4p1*ss[5];
  root_1 += 2*c1p1*c5p1*ss[6];
  root_1 += 2*c2p1*c3p1*ss[5];
  root_1 += 2*c2p1*c4p1*ss[6];
  root_1 += 2*c2p1*c5p1*ss[7];
  root_1 += 2*c3p1*c4p1*ss[7];
  root_1 += 2*c3p1*c5p1*ss[8];
  root_1 += 2*c4p1*c5p1*ss[9];
  root_1 += c1p2*ss[2];
  root_1 += c2p2*ss[4];
  root_1 += c3p2*ss[6];
  root_1 += c4p2*ss[8];
  root_1 += c5p2*ss[10];
  root_1 += c0p2;
  root_1 += -tmom[2];
  T c0p3 = c0p2*c0p1;
  T c1p3 = c1p2*c1p1;
  T c2p3 = c2p2*c2p1;
  T c3p3 = c3p2*c3p1;
  T c4p3 = c4p2*c4p1;
  T c5p3 = c5p2*c5p1;
  TACC root_2 = convert<TACC>(6*c0p1*c1p1*c2p1*ss[3]);
  root_2 += 6*c0p1*c1p1*c3p1*ss[4];
  root_2 += 6*c0p1*c1p1*c4p1*ss[5];
  root_2 += 6*c0p1*c1p1*c5p1*ss[6];
  root_2 += 6*c0p1*c2p1*c3p1*ss[5];
  root_2 += 6*c0p1*c2p1*c4p1*ss[6];
  root_2 += 6*c0p1*c2p1*c5p1*ss[7];
  root_2 += 6*c0p1*c3p1*c4p1*ss[7];
  root_2 += 6*c0p1*c3p1*c5p1*ss[8];
  root_2 += 6*c0p1*c4p1*c5p1*ss[9];
  root_2 += 6*c1p1*c2p1*c3p1*ss[6];
  root_2 += 6*c1p1*c2p1*c4p1*ss[7];
  root_2 += 6*c1p1*c2p1*c5p1*ss[8];
  root_2 += 6*c1p1*c3p1*c4p1*ss[8];
  root_2 += 6*c1p1*c3p1*c5p1*ss[9];
  root_2 += 6*c1p1*c4p1*c5p1*ss[10];
  root_2 += 6*c2p1*c3p1*c4p1*ss[9];
  root_2 += 6*c2p1*c3p1*c5p1*ss[10];
  root_2 += 6*c2p1*c4p1*c5p1*ss[11];
  root_2 += 6*c3p1*c4p1*c5p1*ss[12];
  root_2 += 3*c0p1*c1p2*ss[2];
  root_2 += 3*c0p1*c2p2*ss[4];
  root_2 += 3*c0p1*c3p2*ss[6];
  root_2 += 3*c0p1*c4p2*ss[8];
  root_2 += 3*c0p1*c5p2*ss[10];
  root_2 += 3*c0p2*c1p1*ss[1];
  root_2 += 3*c0p2*c2p1*ss[2];
  root_2 += 3*c0p2*c3p1*ss[3];
  root_2 += 3*c0p2*c4p1*ss[4];
  root_2 += 3*c0p2*c5p1*ss[5];
  root_2 += 3*c1p1*c2p2*ss[5];
  root_2 += 3*c1p1*c3p2*ss[7];
  root_2 += 3*c1p1*c4p2*ss[9];
  root_2 += 3*c1p1*c5p2*ss[11];
  root_2 += 3*c1p2*c2p1*ss[4];
  root_2 += 3*c1p2*c3p1*ss[5];
  root_2 += 3*c1p2*c4p1*ss[6];
  root_2 += 3*c1p2*c5p1*ss[7];
  root_2 += 3*c2p1*c3p2*ss[8];
  root_2 += 3*c2p1*c4p2*ss[10];
  root_2 += 3*c2p1*c5p2*ss[12];
  root_2 += 3*c2p2*c3p1*ss[7];
  root_2 += 3*c2p2*c4p1*ss[8];
  root_2 += 3*c2p2*c5p1*ss[9];
  root_2 += 3*c3p1*c4p2*ss[11];
  root_2 += 3*c3p1*c5p2*ss[13];
  root_2 += 3*c3p2*c4p1*ss[10];
  root_2 += 3*c3p2*c5p1*ss[11];
  root_2 += 3*c4p1*c5p2*ss[14];
  root_2 += 3*c4p2*c5p1*ss[13];
  root_2 += c1p3*ss[3];
  root_2 += c2p3*ss[6];
  root_2 += c3p3*ss[9];
  root_2 += c4p3*ss[12];
  root_2 += c5p3*ss[15];
  root_2 += c0p3;
  root_2 += -tmom[3];
  T c0p4 = c0p3*c0p1;
  T c1p4 = c1p3*c1p1;
  T c2p4 = c2p3*c2p1;
  T c3p4 = c3p3*c3p1;
  T c4p4 = c4p3*c4p1;
  T c5p4 = c5p3*c5p1;
  TACC root_3 = convert<TACC>(24*c0p1*c1p1*c2p1*c3p1*ss[6]);
  root_3 += 24*c0p1*c1p1*c2p1*c4p1*ss[7];
  root_3 += 24*c0p1*c1p1*c2p1*c5p1*ss[8];
  root_3 += 24*c0p1*c1p1*c3p1*c4p1*ss[8];
  root_3 += 24*c0p1*c1p1*c3p1*c5p1*ss[9];
  root_3 += 24*c0p1*c1p1*c4p1*c5p1*ss[10];
  root_3 += 24*c0p1*c2p1*c3p1*c4p1*ss[9];
  root_3 += 24*c0p1*c2p1*c3p1*c5p1*ss[10];
  root_3 += 24*c0p1*c2p1*c4p1*c5p1*ss[11];
  root_3 += 24*c0p1*c3p1*c4p1*c5p1*ss[12];
  root_3 += 24*c1p1*c2p1*c3p1*c4p1*ss[10];
  root_3 += 24*c1p1*c2p1*c3p1*c5p1*ss[11];
  root_3 += 24*c1p1*c2p1*c4p1*c5p1*ss[12];
  root_3 += 24*c1p1*c3p1*c4p1*c5p1*ss[13];
  root_3 += 24*c2p1*c3p1*c4p1*c5p1*ss[14];
  root_3 += 12*c0p1*c1p1*c2p2*ss[5];
  root_3 += 12*c0p1*c1p1*c3p2*ss[7];
  root_3 += 12*c0p1*c1p1*c4p2*ss[9];
  root_3 += 12*c0p1*c1p1*c5p2*ss[11];
  root_3 += 12*c0p1*c1p2*c2p1*ss[4];
  root_3 += 12*c0p1*c1p2*c3p1*ss[5];
  root_3 += 12*c0p1*c1p2*c4p1*ss[6];
  root_3 += 12*c0p1*c1p2*c5p1*ss[7];
  root_3 += 12*c0p1*c2p1*c3p2*ss[8];
  root_3 += 12*c0p1*c2p1*c4p2*ss[10];
  root_3 += 12*c0p1*c2p1*c5p2*ss[12];
  root_3 += 12*c0p1*c2p2*c3p1*ss[7];
  root_3 += 12*c0p1*c2p2*c4p1*ss[8];
  root_3 += 12*c0p1*c2p2*c5p1*ss[9];
  root_3 += 12*c0p1*c3p1*c4p2*ss[11];
  root_3 += 12*c0p1*c3p1*c5p2*ss[13];
  root_3 += 12*c0p1*c3p2*c4p1*ss[10];
  root_3 += 12*c0p1*c3p2*c5p1*ss[11];
  root_3 += 12*c0p1*c4p1*c5p2*ss[14];
  root_3 += 12*c0p1*c4p2*c5p1*ss[13];
  root_3 += 12*c0p2*c1p1*c2p1*ss[3];
  root_3 += 12*c0p2*c1p1*c3p1*ss[4];
  root_3 += 12*c0p2*c1p1*c4p1*ss[5];
  root_3 += 12*c0p2*c1p1*c5p1*ss[6];
  root_3 += 12*c0p2*c2p1*c3p1*ss[5];
  root_3 += 12*c0p2*c2p1*c4p1*ss[6];
  root_3 += 12*c0p2*c2p1*c5p1*ss[7];
  root_3 += 12*c0p2*c3p1*c4p1*ss[7];
  root_3 += 12*c0p2*c3p1*c5p1*ss[8];
  root_3 += 12*c0p2*c4p1*c5p1*ss[9];
  root_3 += 12*c1p1*c2p1*c3p2*ss[9];
  root_3 += 12*c1p1*c2p1*c4p2*ss[11];
  root_3 += 12*c1p1*c2p1*c5p2*ss[13];
  root_3 += 12*c1p1*c2p2*c3p1*ss[8];
  root_3 += 12*c1p1*c2p2*c4p1*ss[9];
  root_3 += 12*c1p1*c2p2*c5p1*ss[10];
  root_3 += 12*c1p1*c3p1*c4p2*ss[12];
  root_3 += 12*c1p1*c3p1*c5p2*ss[14];
  root_3 += 12*c1p1*c3p2*c4p1*ss[11];
  root_3 += 12*c1p1*c3p2*c5p1*ss[12];
  root_3 += 12*c1p1*c4p1*c5p2*ss[15];
  root_3 += 12*c1p1*c4p2*c5p1*ss[14];
  root_3 += 12*c1p2*c2p1*c3p1*ss[7];
  root_3 += 12*c1p2*c2p1*c4p1*ss[8];
  root_3 += 12*c1p2*c2p1*c5p1*ss[9];
  root_3 += 12*c1p2*c3p1*c4p1*ss[9];
  root_3 += 12*c1p2*c3p1*c5p1*ss[10];
  root_3 += 12*c1p2*c4p1*c5p1*ss[11];
  root_3 += 12*c2p1*c3p1*c4p2*ss[13];
  root_3 += 12*c2p1*c3p1*c5p2*ss[15];
  root_3 += 12*c2p1*c3p2*c4p1*ss[12];
  root_3 += 12*c2p1*c3p2*c5p1*ss[13];
  root_3 += 12*c2p1*c4p1*c5p2*ss[16];
  root_3 += 12*c2p1*c4p2*c5p1*ss[15];
  root_3 += 12*c2p2*c3p1*c4p1*ss[11];
  root_3 += 12*c2p2*c3p1*c5p1*ss[12];
  root_3 += 12*c2p2*c4p1*c5p1*ss[13];
  root_3 += 12*c3p1*c4p1*c5p2*ss[17];
  root_3 += 12*c3p1*c4p2*c5p1*ss[16];
  root_3 += 12*c3p2*c4p1*c5p1*ss[15];
  root_3 += 4*c0p1*c1p3*ss[3];
  root_3 += 4*c0p1*c2p3*ss[6];
  root_3 += 4*c0p1*c3p3*ss[9];
  root_3 += 4*c0p1*c4p3*ss[12];
  root_3 += 4*c0p1*c5p3*ss[15];
  root_3 += 6*c0p2*c1p2*ss[2];
  root_3 += 6*c0p2*c2p2*ss[4];
  root_3 += 6*c0p2*c3p2*ss[6];
  root_3 += 6*c0p2*c4p2*ss[8];
  root_3 += 6*c0p2*c5p2*ss[10];
  root_3 += 4*c0p3*c1p1*ss[1];
  root_3 += 4*c0p3*c2p1*ss[2];
  root_3 += 4*c0p3*c3p1*ss[3];
  root_3 += 4*c0p3*c4p1*ss[4];
  root_3 += 4*c0p3*c5p1*ss[5];
  root_3 += 4*c1p1*c2p3*ss[7];
  root_3 += 4*c1p1*c3p3*ss[10];
  root_3 += 4*c1p1*c4p3*ss[13];
  root_3 += 4*c1p1*c5p3*ss[16];
  root_3 += 6*c1p2*c2p2*ss[6];
  root_3 += 6*c1p2*c3p2*ss[8];
  root_3 += 6*c1p2*c4p2*ss[10];
  root_3 += 6*c1p2*c5p2*ss[12];
  root_3 += 4*c1p3*c2p1*ss[5];
  root_3 += 4*c1p3*c3p1*ss[6];
  root_3 += 4*c1p3*c4p1*ss[7];
  root_3 += 4*c1p3*c5p1*ss[8];
  root_3 += 4*c2p1*c3p3*ss[11];
  root_3 += 4*c2p1*c4p3*ss[14];
  root_3 += 4*c2p1*c5p3*ss[17];
  root_3 += 6*c2p2*c3p2*ss[10];
  root_3 += 6*c2p2*c4p2*ss[12];
  root_3 += 6*c2p2*c5p2*ss[14];
  root_3 += 4*c2p3*c3p1*ss[9];
  root_3 += 4*c2p3*c4p1*ss[10];
  root_3 += 4*c2p3*c5p1*ss[11];
  root_3 += 4*c3p1*c4p3*ss[15];
  root_3 += 4*c3p1*c5p3*ss[18];
  root_3 += 6*c3p2*c4p2*ss[14];
  root_3 += 6*c3p2*c5p2*ss[16];
  root_3 += 4*c3p3*c4p1*ss[13];
  root_3 += 4*c3p3*c5p1*ss[14];
  root_3 += 4*c4p1*c5p3*ss[19];
  root_3 += 6*c4p2*c5p2*ss[18];
  root_3 += 4*c4p3*c5p1*ss[17];
  root_3 += c1p4*ss[4];
  root_3 += c2p4*ss[8];
  root_3 += c3p4*ss[12];
  root_3 += c4p4*ss[16];
  root_3 += c5p4*ss[20];
  root_3 += c0p4;
  root_3 += -tmom[4];
  T c0p5 = c0p4*c0p1;
  T c1p5 = c1p4*c1p1;
  T c2p5 = c2p4*c2p1;
  T c3p5 = c3p4*c3p1;
  T c4p5 = c4p4*c4p1;
  T c5p5 = c5p4*c5p1;
  TACC root_4 = convert<TACC>(120*c0p1*c1p1*c2p1*c3p1*c4p1*ss[10]);
  root_4 += 120*c0p1*c1p1*c2p1*c3p1*c5p1*ss[11];
  root_4 += 120*c0p1*c1p1*c2p1*c4p1*c5p1*ss[12];
  root_4 += 120*c0p1*c1p1*c3p1*c4p1*c5p1*ss[13];
  root_4 += 120*c0p1*c2p1*c3p1*c4p1*c5p1*ss[14];
  root_4 += 120*c1p1*c2p1*c3p1*c4p1*c5p1*ss[15];
  root_4 += 60*c0p1*c1p1*c2p1*c3p2*ss[9];
  root_4 += 60*c0p1*c1p1*c2p1*c4p2*ss[11];
  root_4 += 60*c0p1*c1p1*c2p1*c5p2*ss[13];
  root_4 += 60*c0p1*c1p1*c2p2*c3p1*ss[8];
  root_4 += 60*c0p1*c1p1*c2p2*c4p1*ss[9];
  root_4 += 60*c0p1*c1p1*c2p2*c5p1*ss[10];
  root_4 += 60*c0p1*c1p1*c3p1*c4p2*ss[12];
  root_4 += 60*c0p1*c1p1*c3p1*c5p2*ss[14];
  root_4 += 60*c0p1*c1p1*c3p2*c4p1*ss[11];
  root_4 += 60*c0p1*c1p1*c3p2*c5p1*ss[12];
  root_4 += 60*c0p1*c1p1*c4p1*c5p2*ss[15];
  root_4 += 60*c0p1*c1p1*c4p2*c5p1*ss[14];
  root_4 += 60*c0p1*c1p2*c2p1*c3p1*ss[7];
  root_4 += 60*c0p1*c1p2*c2p1*c4p1*ss[8];
  root_4 += 60*c0p1*c1p2*c2p1*c5p1*ss[9];
  root_4 += 60*c0p1*c1p2*c3p1*c4p1*ss[9];
  root_4 += 60*c0p1*c1p2*c3p1*c5p1*ss[10];
  root_4 += 60*c0p1*c1p2*c4p1*c5p1*ss[11];
  root_4 += 60*c0p1*c2p1*c3p1*c4p2*ss[13];
  root_4 += 60*c0p1*c2p1*c3p1*c5p2*ss[15];
  root_4 += 60*c0p1*c2p1*c3p2*c4p1*ss[12];
  root_4 += 60*c0p1*c2p1*c3p2*c5p1*ss[13];
  root_4 += 60*c0p1*c2p1*c4p1*c5p2*ss[16];
  root_4 += 60*c0p1*c2p1*c4p2*c5p1*ss[15];
  root_4 += 60*c0p1*c2p2*c3p1*c4p1*ss[11];
  root_4 += 60*c0p1*c2p2*c3p1*c5p1*ss[12];
  root_4 += 60*c0p1*c2p2*c4p1*c5p1*ss[13];
  root_4 += 60*c0p1*c3p1*c4p1*c5p2*ss[17];
  root_4 += 60*c0p1*c3p1*c4p2*c5p1*ss[16];
  root_4 += 60*c0p1*c3p2*c4p1*c5p1*ss[15];
  root_4 += 60*c0p2*c1p1*c2p1*c3p1*ss[6];
  root_4 += 60*c0p2*c1p1*c2p1*c4p1*ss[7];
  root_4 += 60*c0p2*c1p1*c2p1*c5p1*ss[8];
  root_4 += 60*c0p2*c1p1*c3p1*c4p1*ss[8];
  root_4 += 60*c0p2*c1p1*c3p1*c5p1*ss[9];
  root_4 += 60*c0p2*c1p1*c4p1*c5p1*ss[10];
  root_4 += 60*c0p2*c2p1*c3p1*c4p1*ss[9];
  root_4 += 60*c0p2*c2p1*c3p1*c5p1*ss[10];
  root_4 += 60*c0p2*c2p1*c4p1*c5p1*ss[11];
  root_4 += 60*c0p2*c3p1*c4p1*c5p1*ss[12];
  root_4 += 60*c1p1*c2p1*c3p1*c4p2*ss[14];
  root_4 += 60*c1p1*c2p1*c3p1*c5p2*ss[16];
  root_4 += 60*c1p1*c2p1*c3p2*c4p1*ss[13];
  root_4 += 60*c1p1*c2p1*c3p2*c5p1*ss[14];
  root_4 += 60*c1p1*c2p1*c4p1*c5p2*ss[17];
  root_4 += 60*c1p1*c2p1*c4p2*c5p1*ss[16];
  root_4 += 60*c1p1*c2p2*c3p1*c4p1*ss[12];
  root_4 += 60*c1p1*c2p2*c3p1*c5p1*ss[13];
  root_4 += 60*c1p1*c2p2*c4p1*c5p1*ss[14];
  root_4 += 60*c1p1*c3p1*c4p1*c5p2*ss[18];
  root_4 += 60*c1p1*c3p1*c4p2*c5p1*ss[17];
  root_4 += 60*c1p1*c3p2*c4p1*c5p1*ss[16];
  root_4 += 60*c1p2*c2p1*c3p1*c4p1*ss[11];
  root_4 += 60*c1p2*c2p1*c3p1*c5p1*ss[12];
  root_4 += 60*c1p2*c2p1*c4p1*c5p1*ss[13];
  root_4 += 60*c1p2*c3p1*c4p1*c5p1*ss[14];
  root_4 += 60*c2p1*c3p1*c4p1*c5p2*ss[19];
  root_4 += 60*c2p1*c3p1*c4p2*c5p1*ss[18];
  root_4 += 60*c2p1*c3p2*c4p1*c5p1*ss[17];
  root_4 += 60*c2p2*c3p1*c4p1*c5p1*ss[16];
  root_4 += 20*c0p1*c1p1*c2p3*ss[7];
  root_4 += 20*c0p1*c1p1*c3p3*ss[10];
  root_4 += 20*c0p1*c1p1*c4p3*ss[13];
  root_4 += 20*c0p1*c1p1*c5p3*ss[16];
  root_4 += 30*c0p1*c1p2*c2p2*ss[6];
  root_4 += 30*c0p1*c1p2*c3p2*ss[8];
  root_4 += 30*c0p1*c1p2*c4p2*ss[10];
  root_4 += 30*c0p1*c1p2*c5p2*ss[12];
  root_4 += 20*c0p1*c1p3*c2p1*ss[5];
  root_4 += 20*c0p1*c1p3*c3p1*ss[6];
  root_4 += 20*c0p1*c1p3*c4p1*ss[7];
  root_4 += 20*c0p1*c1p3*c5p1*ss[8];
  root_4 += 20*c0p1*c2p1*c3p3*ss[11];
  root_4 += 20*c0p1*c2p1*c4p3*ss[14];
  root_4 += 20*c0p1*c2p1*c5p3*ss[17];
  root_4 += 30*c0p1*c2p2*c3p2*ss[10];
  root_4 += 30*c0p1*c2p2*c4p2*ss[12];
  root_4 += 30*c0p1*c2p2*c5p2*ss[14];
  root_4 += 20*c0p1*c2p3*c3p1*ss[9];
  root_4 += 20*c0p1*c2p3*c4p1*ss[10];
  root_4 += 20*c0p1*c2p3*c5p1*ss[11];
  root_4 += 20*c0p1*c3p1*c4p3*ss[15];
  root_4 += 20*c0p1*c3p1*c5p3*ss[18];
  root_4 += 30*c0p1*c3p2*c4p2*ss[14];
  root_4 += 30*c0p1*c3p2*c5p2*ss[16];
  root_4 += 20*c0p1*c3p3*c4p1*ss[13];
  root_4 += 20*c0p1*c3p3*c5p1*ss[14];
  root_4 += 20*c0p1*c4p1*c5p3*ss[19];
  root_4 += 30*c0p1*c4p2*c5p2*ss[18];
  root_4 += 20*c0p1*c4p3*c5p1*ss[17];
  root_4 += 30*c0p2*c1p1*c2p2*ss[5];
  root_4 += 30*c0p2*c1p1*c3p2*ss[7];
  root_4 += 30*c0p2*c1p1*c4p2*ss[9];
  root_4 += 30*c0p2*c1p1*c5p2*ss[11];
  root_4 += 30*c0p2*c1p2*c2p1*ss[4];
  root_4 += 30*c0p2*c1p2*c3p1*ss[5];
  root_4 += 30*c0p2*c1p2*c4p1*ss[6];
  root_4 += 30*c0p2*c1p2*c5p1*ss[7];
  root_4 += 30*c0p2*c2p1*c3p2*ss[8];
  root_4 += 30*c0p2*c2p1*c4p2*ss[10];
  root_4 += 30*c0p2*c2p1*c5p2*ss[12];
  root_4 += 30*c0p2*c2p2*c3p1*ss[7];
  root_4 += 30*c0p2*c2p2*c4p1*ss[8];
  root_4 += 30*c0p2*c2p2*c5p1*ss[9];
  root_4 += 30*c0p2*c3p1*c4p2*ss[11];
  root_4 += 30*c0p2*c3p1*c5p2*ss[13];
  root_4 += 30*c0p2*c3p2*c4p1*ss[10];
  root_4 += 30*c0p2*c3p2*c5p1*ss[11];
  root_4 += 30*c0p2*c4p1*c5p2*ss[14];
  root_4 += 30*c0p2*c4p2*c5p1*ss[13];
  root_4 += 20*c0p3*c1p1*c2p1*ss[3];
  root_4 += 20*c0p3*c1p1*c3p1*ss[4];
  root_4 += 20*c0p3*c1p1*c4p1*ss[5];
  root_4 += 20*c0p3*c1p1*c5p1*ss[6];
  root_4 += 20*c0p3*c2p1*c3p1*ss[5];
  root_4 += 20*c0p3*c2p1*c4p1*ss[6];
  root_4 += 20*c0p3*c2p1*c5p1*ss[7];
  root_4 += 20*c0p3*c3p1*c4p1*ss[7];
  root_4 += 20*c0p3*c3p1*c5p1*ss[8];
  root_4 += 20*c0p3*c4p1*c5p1*ss[9];
  root_4 += 20*c1p1*c2p1*c3p3*ss[12];
  root_4 += 20*c1p1*c2p1*c4p3*ss[15];
  root_4 += 20*c1p1*c2p1*c5p3*ss[18];
  root_4 += 30*c1p1*c2p2*c3p2*ss[11];
  root_4 += 30*c1p1*c2p2*c4p2*ss[13];
  root_4 += 30*c1p1*c2p2*c5p2*ss[15];
  root_4 += 20*c1p1*c2p3*c3p1*ss[10];
  root_4 += 20*c1p1*c2p3*c4p1*ss[11];
  root_4 += 20*c1p1*c2p3*c5p1*ss[12];
  root_4 += 20*c1p1*c3p1*c4p3*ss[16];
  root_4 += 20*c1p1*c3p1*c5p3*ss[19];
  root_4 += 30*c1p1*c3p2*c4p2*ss[15];
  root_4 += 30*c1p1*c3p2*c5p2*ss[17];
  root_4 += 20*c1p1*c3p3*c4p1*ss[14];
  root_4 += 20*c1p1*c3p3*c5p1*ss[15];
  root_4 += 20*c1p1*c4p1*c5p3*ss[20];
  root_4 += 30*c1p1*c4p2*c5p2*ss[19];
  root_4 += 20*c1p1*c4p3*c5p1*ss[18];
  root_4 += 30*c1p2*c2p1*c3p2*ss[10];
  root_4 += 30*c1p2*c2p1*c4p2*ss[12];
  root_4 += 30*c1p2*c2p1*c5p2*ss[14];
  root_4 += 30*c1p2*c2p2*c3p1*ss[9];
  root_4 += 30*c1p2*c2p2*c4p1*ss[10];
  root_4 += 30*c1p2*c2p2*c5p1*ss[11];
  root_4 += 30*c1p2*c3p1*c4p2*ss[13];
  root_4 += 30*c1p2*c3p1*c5p2*ss[15];
  root_4 += 30*c1p2*c3p2*c4p1*ss[12];
  root_4 += 30*c1p2*c3p2*c5p1*ss[13];
  root_4 += 30*c1p2*c4p1*c5p2*ss[16];
  root_4 += 30*c1p2*c4p2*c5p1*ss[15];
  root_4 += 20*c1p3*c2p1*c3p1*ss[8];
  root_4 += 20*c1p3*c2p1*c4p1*ss[9];
  root_4 += 20*c1p3*c2p1*c5p1*ss[10];
  root_4 += 20*c1p3*c3p1*c4p1*ss[10];
  root_4 += 20*c1p3*c3p1*c5p1*ss[11];
  root_4 += 20*c1p3*c4p1*c5p1*ss[12];
  root_4 += 20*c2p1*c3p1*c4p3*ss[17];
  root_4 += 20*c2p1*c3p1*c5p3*ss[20];
  root_4 += 30*c2p1*c3p2*c4p2*ss[16];
  root_4 += 30*c2p1*c3p2*c5p2*ss[18];
  root_4 += 20*c2p1*c3p3*c4p1*ss[15];
  root_4 += 20*c2p1*c3p3*c5p1*ss[16];
  root_4 += 20*c2p1*c4p1*c5p3*ss[21];
  root_4 += 30*c2p1*c4p2*c5p2*ss[20];
  root_4 += 20*c2p1*c4p3*c5p1*ss[19];
  root_4 += 30*c2p2*c3p1*c4p2*ss[15];
  root_4 += 30*c2p2*c3p1*c5p2*ss[17];
  root_4 += 30*c2p2*c3p2*c4p1*ss[14];
  root_4 += 30*c2p2*c3p2*c5p1*ss[15];
  root_4 += 30*c2p2*c4p1*c5p2*ss[18];
  root_4 += 30*c2p2*c4p2*c5p1*ss[17];
  root_4 += 20*c2p3*c3p1*c4p1*ss[13];
  root_4 += 20*c2p3*c3p1*c5p1*ss[14];
  root_4 += 20*c2p3*c4p1*c5p1*ss[15];
  root_4 += 20*c3p1*c4p1*c5p3*ss[22];
  root_4 += 30*c3p1*c4p2*c5p2*ss[21];
  root_4 += 20*c3p1*c4p3*c5p1*ss[20];
  root_4 += 30*c3p2*c4p1*c5p2*ss[20];
  root_4 += 30*c3p2*c4p2*c5p1*ss[19];
  root_4 += 20*c3p3*c4p1*c5p1*ss[18];
  root_4 += 5*c0p1*c1p4*ss[4];
  root_4 += 5*c0p1*c2p4*ss[8];
  root_4 += 5*c0p1*c3p4*ss[12];
  root_4 += 5*c0p1*c4p4*ss[16];
  root_4 += 5*c0p1*c5p4*ss[20];
  root_4 += 10*c0p2*c1p3*ss[3];
  root_4 += 10*c0p2*c2p3*ss[6];
  root_4 += 10*c0p2*c3p3*ss[9];
  root_4 += 10*c0p2*c4p3*ss[12];
  root_4 += 10*c0p2*c5p3*ss[15];
  root_4 += 10*c0p3*c1p2*ss[2];
  root_4 += 10*c0p3*c2p2*ss[4];
  root_4 += 10*c0p3*c3p2*ss[6];
  root_4 += 10*c0p3*c4p2*ss[8];
  root_4 += 10*c0p3*c5p2*ss[10];
  root_4 += 5*c0p4*c1p1*ss[1];
  root_4 += 5*c0p4*c2p1*ss[2];
  root_4 += 5*c0p4*c3p1*ss[3];
  root_4 += 5*c0p4*c4p1*ss[4];
  root_4 += 5*c0p4*c5p1*ss[5];
  root_4 += 5*c1p1*c2p4*ss[9];
  root_4 += 5*c1p1*c3p4*ss[13];
  root_4 += 5*c1p1*c4p4*ss[17];
  root_4 += 5*c1p1*c5p4*ss[21];
  root_4 += 10*c1p2*c2p3*ss[8];
  root_4 += 10*c1p2*c3p3*ss[11];
  root_4 += 10*c1p2*c4p3*ss[14];
  root_4 += 10*c1p2*c5p3*ss[17];
  root_4 += 10*c1p3*c2p2*ss[7];
  root_4 += 10*c1p3*c3p2*ss[9];
  root_4 += 10*c1p3*c4p2*ss[11];
  root_4 += 10*c1p3*c5p2*ss[13];
  root_4 += 5*c1p4*c2p1*ss[6];
  root_4 += 5*c1p4*c3p1*ss[7];
  root_4 += 5*c1p4*c4p1*ss[8];
  root_4 += 5*c1p4*c5p1*ss[9];
  root_4 += 5*c2p1*c3p4*ss[14];
  root_4 += 5*c2p1*c4p4*ss[18];
  root_4 += 5*c2p1*c5p4*ss[22];
  root_4 += 10*c2p2*c3p3*ss[13];
  root_4 += 10*c2p2*c4p3*ss[16];
  root_4 += 10*c2p2*c5p3*ss[19];
  root_4 += 10*c2p3*c3p2*ss[12];
  root_4 += 10*c2p3*c4p2*ss[14];
  root_4 += 10*c2p3*c5p2*ss[16];
  root_4 += 5*c2p4*c3p1*ss[11];
  root_4 += 5*c2p4*c4p1*ss[12];
  root_4 += 5*c2p4*c5p1*ss[13];
  root_4 += 5*c3p1*c4p4*ss[19];
  root_4 += 5*c3p1*c5p4*ss[23];
  root_4 += 10*c3p2*c4p3*ss[18];
  root_4 += 10*c3p2*c5p3*ss[21];
  root_4 += 10*c3p3*c4p2*ss[17];
  root_4 += 10*c3p3*c5p2*ss[19];
  root_4 += 5*c3p4*c4p1*ss[16];
  root_4 += 5*c3p4*c5p1*ss[17];
  root_4 += 5*c4p1*c5p4*ss[24];
  root_4 += 10*c4p2*c5p3*ss[23];
  root_4 += 10*c4p3*c5p2*ss[22];
  root_4 += 5*c4p4*c5p1*ss[21];
  root_4 += c1p5*ss[5];
  root_4 += c2p5*ss[10];
  root_4 += c3p5*ss[15];
  root_4 += c4p5*ss[20];
  root_4 += c5p5*ss[25];
  root_4 += c0p5;
  root_4 += -tmom[5];
  T c0p6 = c0p5*c0p1;
  T c1p6 = c1p5*c1p1;
  T c2p6 = c2p5*c2p1;
  T c3p6 = c3p5*c3p1;
  T c4p6 = c4p5*c4p1;
  T c5p6 = c5p5*c5p1;
  TACC root_5 = convert<TACC>(720*c0p1*c1p1*c2p1*c3p1*c4p1*c5p1*ss[15]);
  root_5 += 360*c0p1*c1p1*c2p1*c3p1*c4p2*ss[14];
  root_5 += 360*c0p1*c1p1*c2p1*c3p1*c5p2*ss[16];
  root_5 += 360*c0p1*c1p1*c2p1*c3p2*c4p1*ss[13];
  root_5 += 360*c0p1*c1p1*c2p1*c3p2*c5p1*ss[14];
  root_5 += 360*c0p1*c1p1*c2p1*c4p1*c5p2*ss[17];
  root_5 += 360*c0p1*c1p1*c2p1*c4p2*c5p1*ss[16];
  root_5 += 360*c0p1*c1p1*c2p2*c3p1*c4p1*ss[12];
  root_5 += 360*c0p1*c1p1*c2p2*c3p1*c5p1*ss[13];
  root_5 += 360*c0p1*c1p1*c2p2*c4p1*c5p1*ss[14];
  root_5 += 360*c0p1*c1p1*c3p1*c4p1*c5p2*ss[18];
  root_5 += 360*c0p1*c1p1*c3p1*c4p2*c5p1*ss[17];
  root_5 += 360*c0p1*c1p1*c3p2*c4p1*c5p1*ss[16];
  root_5 += 360*c0p1*c1p2*c2p1*c3p1*c4p1*ss[11];
  root_5 += 360*c0p1*c1p2*c2p1*c3p1*c5p1*ss[12];
  root_5 += 360*c0p1*c1p2*c2p1*c4p1*c5p1*ss[13];
  root_5 += 360*c0p1*c1p2*c3p1*c4p1*c5p1*ss[14];
  root_5 += 360*c0p1*c2p1*c3p1*c4p1*c5p2*ss[19];
  root_5 += 360*c0p1*c2p1*c3p1*c4p2*c5p1*ss[18];
  root_5 += 360*c0p1*c2p1*c3p2*c4p1*c5p1*ss[17];
  root_5 += 360*c0p1*c2p2*c3p1*c4p1*c5p1*ss[16];
  root_5 += 360*c0p2*c1p1*c2p1*c3p1*c4p1*ss[10];
  root_5 += 360*c0p2*c1p1*c2p1*c3p1*c5p1*ss[11];
  root_5 += 360*c0p2*c1p1*c2p1*c4p1*c5p1*ss[12];
  root_5 += 360*c0p2*c1p1*c3p1*c4p1*c5p1*ss[13];
  root_5 += 360*c0p2*c2p1*c3p1*c4p1*c5p1*ss[14];
  root_5 += 360*c1p1*c2p1*c3p1*c4p1*c5p2*ss[20];
  root_5 += 360*c1p1*c2p1*c3p1*c4p2*c5p1*ss[19];
  root_5 += 360*c1p1*c2p1*c3p2*c4p1*c5p1*ss[18];
  root_5 += 360*c1p1*c2p2*c3p1*c4p1*c5p1*ss[17];
  root_5 += 360*c1p2*c2p1*c3p1*c4p1*c5p1*ss[16];
  root_5 += 120*c0p1*c1p1*c2p1*c3p3*ss[12];
  root_5 += 120*c0p1*c1p1*c2p1*c4p3*ss[15];
  root_5 += 120*c0p1*c1p1*c2p1*c5p3*ss[18];
  root_5 += 180*c0p1*c1p1*c2p2*c3p2*ss[11];
  root_5 += 180*c0p1*c1p1*c2p2*c4p2*ss[13];
  root_5 += 180*c0p1*c1p1*c2p2*c5p2*ss[15];
  root_5 += 120*c0p1*c1p1*c2p3*c3p1*ss[10];
  root_5 += 120*c0p1*c1p1*c2p3*c4p1*ss[11];
  root_5 += 120*c0p1*c1p1*c2p3*c5p1*ss[12];
  root_5 += 120*c0p1*c1p1*c3p1*c4p3*ss[16];
  root_5 += 120*c0p1*c1p1*c3p1*c5p3*ss[19];
  root_5 += 180*c0p1*c1p1*c3p2*c4p2*ss[15];
  root_5 += 180*c0p1*c1p1*c3p2*c5p2*ss[17];
  root_5 += 120*c0p1*c1p1*c3p3*c4p1*ss[14];
  root_5 += 120*c0p1*c1p1*c3p3*c5p1*ss[15];
  root_5 += 120*c0p1*c1p1*c4p1*c5p3*ss[20];
  root_5 += 180*c0p1*c1p1*c4p2*c5p2*ss[19];
  root_5 += 120*c0p1*c1p1*c4p3*c5p1*ss[18];
  root_5 += 180*c0p1*c1p2*c2p1*c3p2*ss[10];
  root_5 += 180*c0p1*c1p2*c2p1*c4p2*ss[12];
  root_5 += 180*c0p1*c1p2*c2p1*c5p2*ss[14];
  root_5 += 180*c0p1*c1p2*c2p2*c3p1*ss[9];
  root_5 += 180*c0p1*c1p2*c2p2*c4p1*ss[10];
  root_5 += 180*c0p1*c1p2*c2p2*c5p1*ss[11];
  root_5 += 180*c0p1*c1p2*c3p1*c4p2*ss[13];
  root_5 += 180*c0p1*c1p2*c3p1*c5p2*ss[15];
  root_5 += 180*c0p1*c1p2*c3p2*c4p1*ss[12];
  root_5 += 180*c0p1*c1p2*c3p2*c5p1*ss[13];
  root_5 += 180*c0p1*c1p2*c4p1*c5p2*ss[16];
  root_5 += 180*c0p1*c1p2*c4p2*c5p1*ss[15];
  root_5 += 120*c0p1*c1p3*c2p1*c3p1*ss[8];
  root_5 += 120*c0p1*c1p3*c2p1*c4p1*ss[9];
  root_5 += 120*c0p1*c1p3*c2p1*c5p1*ss[10];
  root_5 += 120*c0p1*c1p3*c3p1*c4p1*ss[10];
  root_5 += 120*c0p1*c1p3*c3p1*c5p1*ss[11];
  root_5 += 120*c0p1*c1p3*c4p1*c5p1*ss[12];
  root_5 += 120*c0p1*c2p1*c3p1*c4p3*ss[17];
  root_5 += 120*c0p1*c2p1*c3p1*c5p3*ss[20];
  root_5 += 180*c0p1*c2p1*c3p2*c4p2*ss[16];
  root_5 += 180*c0p1*c2p1*c3p2*c5p2*ss[18];
  root_5 += 120*c0p1*c2p1*c3p3*c4p1*ss[15];
  root_5 += 120*c0p1*c2p1*c3p3*c5p1*ss[16];
  root_5 += 120*c0p1*c2p1*c4p1*c5p3*ss[21];
  root_5 += 180*c0p1*c2p1*c4p2*c5p2*ss[20];
  root_5 += 120*c0p1*c2p1*c4p3*c5p1*ss[19];
  root_5 += 180*c0p1*c2p2*c3p1*c4p2*ss[15];
  root_5 += 180*c0p1*c2p2*c3p1*c5p2*ss[17];
  root_5 += 180*c0p1*c2p2*c3p2*c4p1*ss[14];
  root_5 += 180*c0p1*c2p2*c3p2*c5p1*ss[15];
  root_5 += 180*c0p1*c2p2*c4p1*c5p2*ss[18];
  root_5 += 180*c0p1*c2p2*c4p2*c5p1*ss[17];
  root_5 += 120*c0p1*c2p3*c3p1*c4p1*ss[13];
  root_5 += 120*c0p1*c2p3*c3p1*c5p1*ss[14];
  root_5 += 120*c0p1*c2p3*c4p1*c5p1*ss[15];
  root_5 += 120*c0p1*c3p1*c4p1*c5p3*ss[22];
  root_5 += 180*c0p1*c3p1*c4p2*c5p2*ss[21];
  root_5 += 120*c0p1*c3p1*c4p3*c5p1*ss[20];
  root_5 += 180*c0p1*c3p2*c4p1*c5p2*ss[20];
  root_5 += 180*c0p1*c3p2*c4p2*c5p1*ss[19];
  root_5 += 120*c0p1*c3p3*c4p1*c5p1*ss[18];
  root_5 += 180*c0p2*c1p1*c2p1*c3p2*ss[9];
  root_5 += 180*c0p2*c1p1*c2p1*c4p2*ss[11];
  root_5 += 180*c0p2*c1p1*c2p1*c5p2*ss[13];
  root_5 += 180*c0p2*c1p1*c2p2*c3p1*ss[8];
  root_5 += 180*c0p2*c1p1*c2p2*c4p1*ss[9];
  root_5 += 180*c0p2*c1p1*c2p2*c5p1*ss[10];
  root_5 += 180*c0p2*c1p1*c3p1*c4p2*ss[12];
  root_5 += 180*c0p2*c1p1*c3p1*c5p2*ss[14];
  root_5 += 180*c0p2*c1p1*c3p2*c4p1*ss[11];
  root_5 += 180*c0p2*c1p1*c3p2*c5p1*ss[12];
  root_5 += 180*c0p2*c1p1*c4p1*c5p2*ss[15];
  root_5 += 180*c0p2*c1p1*c4p2*c5p1*ss[14];
  root_5 += 180*c0p2*c1p2*c2p1*c3p1*ss[7];
  root_5 += 180*c0p2*c1p2*c2p1*c4p1*ss[8];
  root_5 += 180*c0p2*c1p2*c2p1*c5p1*ss[9];
  root_5 += 180*c0p2*c1p2*c3p1*c4p1*ss[9];
  root_5 += 180*c0p2*c1p2*c3p1*c5p1*ss[10];
  root_5 += 180*c0p2*c1p2*c4p1*c5p1*ss[11];
  root_5 += 180*c0p2*c2p1*c3p1*c4p2*ss[13];
  root_5 += 180*c0p2*c2p1*c3p1*c5p2*ss[15];
  root_5 += 180*c0p2*c2p1*c3p2*c4p1*ss[12];
  root_5 += 180*c0p2*c2p1*c3p2*c5p1*ss[13];
  root_5 += 180*c0p2*c2p1*c4p1*c5p2*ss[16];
  root_5 += 180*c0p2*c2p1*c4p2*c5p1*ss[15];
  root_5 += 180*c0p2*c2p2*c3p1*c4p1*ss[11];
  root_5 += 180*c0p2*c2p2*c3p1*c5p1*ss[12];
  root_5 += 180*c0p2*c2p2*c4p1*c5p1*ss[13];
  root_5 += 180*c0p2*c3p1*c4p1*c5p2*ss[17];
  root_5 += 180*c0p2*c3p1*c4p2*c5p1*ss[16];
  root_5 += 180*c0p2*c3p2*c4p1*c5p1*ss[15];
  root_5 += 120*c0p3*c1p1*c2p1*c3p1*ss[6];
  root_5 += 120*c0p3*c1p1*c2p1*c4p1*ss[7];
  root_5 += 120*c0p3*c1p1*c2p1*c5p1*ss[8];
  root_5 += 120*c0p3*c1p1*c3p1*c4p1*ss[8];
  root_5 += 120*c0p3*c1p1*c3p1*c5p1*ss[9];
  root_5 += 120*c0p3*c1p1*c4p1*c5p1*ss[10];
  root_5 += 120*c0p3*c2p1*c3p1*c4p1*ss[9];
  root_5 += 120*c0p3*c2p1*c3p1*c5p1*ss[10];
  root_5 += 120*c0p3*c2p1*c4p1*c5p1*ss[11];
  root_5 += 120*c0p3*c3p1*c4p1*c5p1*ss[12];
  root_5 += 120*c1p1*c2p1*c3p1*c4p3*ss[18];
  root_5 += 120*c1p1*c2p1*c3p1*c5p3*ss[21];
  root_5 += 180*c1p1*c2p1*c3p2*c4p2*ss[17];
  root_5 += 180*c1p1*c2p1*c3p2*c5p2*ss[19];
  root_5 += 120*c1p1*c2p1*c3p3*c4p1*ss[16];
  root_5 += 120*c1p1*c2p1*c3p3*c5p1*ss[17];
  root_5 += 120*c1p1*c2p1*c4p1*c5p3*ss[22];
  root_5 += 180*c1p1*c2p1*c4p2*c5p2*ss[21];
  root_5 += 120*c1p1*c2p1*c4p3*c5p1*ss[20];
  root_5 += 180*c1p1*c2p2*c3p1*c4p2*ss[16];
  root_5 += 180*c1p1*c2p2*c3p1*c5p2*ss[18];
  root_5 += 180*c1p1*c2p2*c3p2*c4p1*ss[15];
  root_5 += 180*c1p1*c2p2*c3p2*c5p1*ss[16];
  root_5 += 180*c1p1*c2p2*c4p1*c5p2*ss[19];
  root_5 += 180*c1p1*c2p2*c4p2*c5p1*ss[18];
  root_5 += 120*c1p1*c2p3*c3p1*c4p1*ss[14];
  root_5 += 120*c1p1*c2p3*c3p1*c5p1*ss[15];
  root_5 += 120*c1p1*c2p3*c4p1*c5p1*ss[16];
  root_5 += 120*c1p1*c3p1*c4p1*c5p3*ss[23];
  root_5 += 180*c1p1*c3p1*c4p2*c5p2*ss[22];
  root_5 += 120*c1p1*c3p1*c4p3*c5p1*ss[21];
  root_5 += 180*c1p1*c3p2*c4p1*c5p2*ss[21];
  root_5 += 180*c1p1*c3p2*c4p2*c5p1*ss[20];
  root_5 += 120*c1p1*c3p3*c4p1*c5p1*ss[19];
  root_5 += 180*c1p2*c2p1*c3p1*c4p2*ss[15];
  root_5 += 180*c1p2*c2p1*c3p1*c5p2*ss[17];
  root_5 += 180*c1p2*c2p1*c3p2*c4p1*ss[14];
  root_5 += 180*c1p2*c2p1*c3p2*c5p1*ss[15];
  root_5 += 180*c1p2*c2p1*c4p1*c5p2*ss[18];
  root_5 += 180*c1p2*c2p1*c4p2*c5p1*ss[17];
  root_5 += 180*c1p2*c2p2*c3p1*c4p1*ss[13];
  root_5 += 180*c1p2*c2p2*c3p1*c5p1*ss[14];
  root_5 += 180*c1p2*c2p2*c4p1*c5p1*ss[15];
  root_5 += 180*c1p2*c3p1*c4p1*c5p2*ss[19];
  root_5 += 180*c1p2*c3p1*c4p2*c5p1*ss[18];
  root_5 += 180*c1p2*c3p2*c4p1*c5p1*ss[17];
  root_5 += 120*c1p3*c2p1*c3p1*c4p1*ss[12];
  root_5 += 120*c1p3*c2p1*c3p1*c5p1*ss[13];
  root_5 += 120*c1p3*c2p1*c4p1*c5p1*ss[14];
  root_5 += 120*c1p3*c3p1*c4p1*c5p1*ss[15];
  root_5 += 120*c2p1*c3p1*c4p1*c5p3*ss[24];
  root_5 += 180*c2p1*c3p1*c4p2*c5p2*ss[23];
  root_5 += 120*c2p1*c3p1*c4p3*c5p1*ss[22];
  root_5 += 180*c2p1*c3p2*c4p1*c5p2*ss[22];
  root_5 += 180*c2p1*c3p2*c4p2*c5p1*ss[21];
  root_5 += 120*c2p1*c3p3*c4p1*c5p1*ss[20];
  root_5 += 180*c2p2*c3p1*c4p1*c5p2*ss[21];
  root_5 += 180*c2p2*c3p1*c4p2*c5p1*ss[20];
  root_5 += 180*c2p2*c3p2*c4p1*c5p1*ss[19];
  root_5 += 120*c2p3*c3p1*c4p1*c5p1*ss[18];
  root_5 += 30*c0p1*c1p1*c2p4*ss[9];
  root_5 += 30*c0p1*c1p1*c3p4*ss[13];
  root_5 += 30*c0p1*c1p1*c4p4*ss[17];
  root_5 += 30*c0p1*c1p1*c5p4*ss[21];
  root_5 += 60*c0p1*c1p2*c2p3*ss[8];
  root_5 += 60*c0p1*c1p2*c3p3*ss[11];
  root_5 += 60*c0p1*c1p2*c4p3*ss[14];
  root_5 += 60*c0p1*c1p2*c5p3*ss[17];
  root_5 += 60*c0p1*c1p3*c2p2*ss[7];
  root_5 += 60*c0p1*c1p3*c3p2*ss[9];
  root_5 += 60*c0p1*c1p3*c4p2*ss[11];
  root_5 += 60*c0p1*c1p3*c5p2*ss[13];
  root_5 += 30*c0p1*c1p4*c2p1*ss[6];
  root_5 += 30*c0p1*c1p4*c3p1*ss[7];
  root_5 += 30*c0p1*c1p4*c4p1*ss[8];
  root_5 += 30*c0p1*c1p4*c5p1*ss[9];
  root_5 += 30*c0p1*c2p1*c3p4*ss[14];
  root_5 += 30*c0p1*c2p1*c4p4*ss[18];
  root_5 += 30*c0p1*c2p1*c5p4*ss[22];
  root_5 += 60*c0p1*c2p2*c3p3*ss[13];
  root_5 += 60*c0p1*c2p2*c4p3*ss[16];
  root_5 += 60*c0p1*c2p2*c5p3*ss[19];
  root_5 += 60*c0p1*c2p3*c3p2*ss[12];
  root_5 += 60*c0p1*c2p3*c4p2*ss[14];
  root_5 += 60*c0p1*c2p3*c5p2*ss[16];
  root_5 += 30*c0p1*c2p4*c3p1*ss[11];
  root_5 += 30*c0p1*c2p4*c4p1*ss[12];
  root_5 += 30*c0p1*c2p4*c5p1*ss[13];
  root_5 += 30*c0p1*c3p1*c4p4*ss[19];
  root_5 += 30*c0p1*c3p1*c5p4*ss[23];
  root_5 += 60*c0p1*c3p2*c4p3*ss[18];
  root_5 += 60*c0p1*c3p2*c5p3*ss[21];
  root_5 += 60*c0p1*c3p3*c4p2*ss[17];
  root_5 += 60*c0p1*c3p3*c5p2*ss[19];
  root_5 += 30*c0p1*c3p4*c4p1*ss[16];
  root_5 += 30*c0p1*c3p4*c5p1*ss[17];
  root_5 += 30*c0p1*c4p1*c5p4*ss[24];
  root_5 += 60*c0p1*c4p2*c5p3*ss[23];
  root_5 += 60*c0p1*c4p3*c5p2*ss[22];
  root_5 += 30*c0p1*c4p4*c5p1*ss[21];
  root_5 += 60*c0p2*c1p1*c2p3*ss[7];
  root_5 += 60*c0p2*c1p1*c3p3*ss[10];
  root_5 += 60*c0p2*c1p1*c4p3*ss[13];
  root_5 += 60*c0p2*c1p1*c5p3*ss[16];
  root_5 += 90*c0p2*c1p2*c2p2*ss[6];
  root_5 += 90*c0p2*c1p2*c3p2*ss[8];
  root_5 += 90*c0p2*c1p2*c4p2*ss[10];
  root_5 += 90*c0p2*c1p2*c5p2*ss[12];
  root_5 += 60*c0p2*c1p3*c2p1*ss[5];
  root_5 += 60*c0p2*c1p3*c3p1*ss[6];
  root_5 += 60*c0p2*c1p3*c4p1*ss[7];
  root_5 += 60*c0p2*c1p3*c5p1*ss[8];
  root_5 += 60*c0p2*c2p1*c3p3*ss[11];
  root_5 += 60*c0p2*c2p1*c4p3*ss[14];
  root_5 += 60*c0p2*c2p1*c5p3*ss[17];
  root_5 += 90*c0p2*c2p2*c3p2*ss[10];
  root_5 += 90*c0p2*c2p2*c4p2*ss[12];
  root_5 += 90*c0p2*c2p2*c5p2*ss[14];
  root_5 += 60*c0p2*c2p3*c3p1*ss[9];
  root_5 += 60*c0p2*c2p3*c4p1*ss[10];
  root_5 += 60*c0p2*c2p3*c5p1*ss[11];
  root_5 += 60*c0p2*c3p1*c4p3*ss[15];
  root_5 += 60*c0p2*c3p1*c5p3*ss[18];
  root_5 += 90*c0p2*c3p2*c4p2*ss[14];
  root_5 += 90*c0p2*c3p2*c5p2*ss[16];
  root_5 += 60*c0p2*c3p3*c4p1*ss[13];
  root_5 += 60*c0p2*c3p3*c5p1*ss[14];
  root_5 += 60*c0p2*c4p1*c5p3*ss[19];
  root_5 += 90*c0p2*c4p2*c5p2*ss[18];
  root_5 += 60*c0p2*c4p3*c5p1*ss[17];
  root_5 += 60*c0p3*c1p1*c2p2*ss[5];
  root_5 += 60*c0p3*c1p1*c3p2*ss[7];
  root_5 += 60*c0p3*c1p1*c4p2*ss[9];
  root_5 += 60*c0p3*c1p1*c5p2*ss[11];
  root_5 += 60*c0p3*c1p2*c2p1*ss[4];
  root_5 += 60*c0p3*c1p2*c3p1*ss[5];
  root_5 += 60*c0p3*c1p2*c4p1*ss[6];
  root_5 += 60*c0p3*c1p2*c5p1*ss[7];
  root_5 += 60*c0p3*c2p1*c3p2*ss[8];
  root_5 += 60*c0p3*c2p1*c4p2*ss[10];
  root_5 += 60*c0p3*c2p1*c5p2*ss[12];
  root_5 += 60*c0p3*c2p2*c3p1*ss[7];
  root_5 += 60*c0p3*c2p2*c4p1*ss[8];
  root_5 += 60*c0p3*c2p2*c5p1*ss[9];
  root_5 += 60*c0p3*c3p1*c4p2*ss[11];
  root_5 += 60*c0p3*c3p1*c5p2*ss[13];
  root_5 += 60*c0p3*c3p2*c4p1*ss[10];
  root_5 += 60*c0p3*c3p2*c5p1*ss[11];
  root_5 += 60*c0p3*c4p1*c5p2*ss[14];
  root_5 += 60*c0p3*c4p2*c5p1*ss[13];
  root_5 += 30*c0p4*c1p1*c2p1*ss[3];
  root_5 += 30*c0p4*c1p1*c3p1*ss[4];
  root_5 += 30*c0p4*c1p1*c4p1*ss[5];
  root_5 += 30*c0p4*c1p1*c5p1*ss[6];
  root_5 += 30*c0p4*c2p1*c3p1*ss[5];
  root_5 += 30*c0p4*c2p1*c4p1*ss[6];
  root_5 += 30*c0p4*c2p1*c5p1*ss[7];
  root_5 += 30*c0p4*c3p1*c4p1*ss[7];
  root_5 += 30*c0p4*c3p1*c5p1*ss[8];
  root_5 += 30*c0p4*c4p1*c5p1*ss[9];
  root_5 += 30*c1p1*c2p1*c3p4*ss[15];
  root_5 += 30*c1p1*c2p1*c4p4*ss[19];
  root_5 += 30*c1p1*c2p1*c5p4*ss[23];
  root_5 += 60*c1p1*c2p2*c3p3*ss[14];
  root_5 += 60*c1p1*c2p2*c4p3*ss[17];
  root_5 += 60*c1p1*c2p2*c5p3*ss[20];
  root_5 += 60*c1p1*c2p3*c3p2*ss[13];
  root_5 += 60*c1p1*c2p3*c4p2*ss[15];
  root_5 += 60*c1p1*c2p3*c5p2*ss[17];
  root_5 += 30*c1p1*c2p4*c3p1*ss[12];
  root_5 += 30*c1p1*c2p4*c4p1*ss[13];
  root_5 += 30*c1p1*c2p4*c5p1*ss[14];
  root_5 += 30*c1p1*c3p1*c4p4*ss[20];
  root_5 += 30*c1p1*c3p1*c5p4*ss[24];
  root_5 += 60*c1p1*c3p2*c4p3*ss[19];
  root_5 += 60*c1p1*c3p2*c5p3*ss[22];
  root_5 += 60*c1p1*c3p3*c4p2*ss[18];
  root_5 += 60*c1p1*c3p3*c5p2*ss[20];
  root_5 += 30*c1p1*c3p4*c4p1*ss[17];
  root_5 += 30*c1p1*c3p4*c5p1*ss[18];
  root_5 += 30*c1p1*c4p1*c5p4*ss[25];
  root_5 += 60*c1p1*c4p2*c5p3*ss[24];
  root_5 += 60*c1p1*c4p3*c5p2*ss[23];
  root_5 += 30*c1p1*c4p4*c5p1*ss[22];
  root_5 += 60*c1p2*c2p1*c3p3*ss[13];
  root_5 += 60*c1p2*c2p1*c4p3*ss[16];
  root_5 += 60*c1p2*c2p1*c5p3*ss[19];
  root_5 += 90*c1p2*c2p2*c3p2*ss[12];
  root_5 += 90*c1p2*c2p2*c4p2*ss[14];
  root_5 += 90*c1p2*c2p2*c5p2*ss[16];
  root_5 += 60*c1p2*c2p3*c3p1*ss[11];
  root_5 += 60*c1p2*c2p3*c4p1*ss[12];
  root_5 += 60*c1p2*c2p3*c5p1*ss[13];
  root_5 += 60*c1p2*c3p1*c4p3*ss[17];
  root_5 += 60*c1p2*c3p1*c5p3*ss[20];
  root_5 += 90*c1p2*c3p2*c4p2*ss[16];
  root_5 += 90*c1p2*c3p2*c5p2*ss[18];
  root_5 += 60*c1p2*c3p3*c4p1*ss[15];
  root_5 += 60*c1p2*c3p3*c5p1*ss[16];
  root_5 += 60*c1p2*c4p1*c5p3*ss[21];
  root_5 += 90*c1p2*c4p2*c5p2*ss[20];
  root_5 += 60*c1p2*c4p3*c5p1*ss[19];
  root_5 += 60*c1p3*c2p1*c3p2*ss[11];
  root_5 += 60*c1p3*c2p1*c4p2*ss[13];
  root_5 += 60*c1p3*c2p1*c5p2*ss[15];
  root_5 += 60*c1p3*c2p2*c3p1*ss[10];
  root_5 += 60*c1p3*c2p2*c4p1*ss[11];
  root_5 += 60*c1p3*c2p2*c5p1*ss[12];
  root_5 += 60*c1p3*c3p1*c4p2*ss[14];
  root_5 += 60*c1p3*c3p1*c5p2*ss[16];
  root_5 += 60*c1p3*c3p2*c4p1*ss[13];
  root_5 += 60*c1p3*c3p2*c5p1*ss[14];
  root_5 += 60*c1p3*c4p1*c5p2*ss[17];
  root_5 += 60*c1p3*c4p2*c5p1*ss[16];
  root_5 += 30*c1p4*c2p1*c3p1*ss[9];
  root_5 += 30*c1p4*c2p1*c4p1*ss[10];
  root_5 += 30*c1p4*c2p1*c5p1*ss[11];
  root_5 += 30*c1p4*c3p1*c4p1*ss[11];
  root_5 += 30*c1p4*c3p1*c5p1*ss[12];
  root_5 += 30*c1p4*c4p1*c5p1*ss[13];
  root_5 += 30*c2p1*c3p1*c4p4*ss[21];
  root_5 += 30*c2p1*c3p1*c5p4*ss[25];
  root_5 += 60*c2p1*c3p2*c4p3*ss[20];
  root_5 += 60*c2p1*c3p2*c5p3*ss[23];
  root_5 += 60*c2p1*c3p3*c4p2*ss[19];
  root_5 += 60*c2p1*c3p3*c5p2*ss[21];
  root_5 += 30*c2p1*c3p4*c4p1*ss[18];
  root_5 += 30*c2p1*c3p4*c5p1*ss[19];
  root_5 += 30*c2p1*c4p1*c5p4*ss[26];
  root_5 += 60*c2p1*c4p2*c5p3*ss[25];
  root_5 += 60*c2p1*c4p3*c5p2*ss[24];
  root_5 += 30*c2p1*c4p4*c5p1*ss[23];
  root_5 += 60*c2p2*c3p1*c4p3*ss[19];
  root_5 += 60*c2p2*c3p1*c5p3*ss[22];
  root_5 += 90*c2p2*c3p2*c4p2*ss[18];
  root_5 += 90*c2p2*c3p2*c5p2*ss[20];
  root_5 += 60*c2p2*c3p3*c4p1*ss[17];
  root_5 += 60*c2p2*c3p3*c5p1*ss[18];
  root_5 += 60*c2p2*c4p1*c5p3*ss[23];
  root_5 += 90*c2p2*c4p2*c5p2*ss[22];
  root_5 += 60*c2p2*c4p3*c5p1*ss[21];
  root_5 += 60*c2p3*c3p1*c4p2*ss[17];
  root_5 += 60*c2p3*c3p1*c5p2*ss[19];
  root_5 += 60*c2p3*c3p2*c4p1*ss[16];
  root_5 += 60*c2p3*c3p2*c5p1*ss[17];
  root_5 += 60*c2p3*c4p1*c5p2*ss[20];
  root_5 += 60*c2p3*c4p2*c5p1*ss[19];
  root_5 += 30*c2p4*c3p1*c4p1*ss[15];
  root_5 += 30*c2p4*c3p1*c5p1*ss[16];
  root_5 += 30*c2p4*c4p1*c5p1*ss[17];
  root_5 += 30*c3p1*c4p1*c5p4*ss[27];
  root_5 += 60*c3p1*c4p2*c5p3*ss[26];
  root_5 += 60*c3p1*c4p3*c5p2*ss[25];
  root_5 += 30*c3p1*c4p4*c5p1*ss[24];
  root_5 += 60*c3p2*c4p1*c5p3*ss[25];
  root_5 += 90*c3p2*c4p2*c5p2*ss[24];
  root_5 += 60*c3p2*c4p3*c5p1*ss[23];
  root_5 += 60*c3p3*c4p1*c5p2*ss[23];
  root_5 += 60*c3p3*c4p2*c5p1*ss[22];
  root_5 += 30*c3p4*c4p1*c5p1*ss[21];
  root_5 += 6*c0p1*c1p5*ss[5];
  root_5 += 6*c0p1*c2p5*ss[10];
  root_5 += 6*c0p1*c3p5*ss[15];
  root_5 += 6*c0p1*c4p5*ss[20];
  root_5 += 6*c0p1*c5p5*ss[25];
  root_5 += 15*c0p2*c1p4*ss[4];
  root_5 += 15*c0p2*c2p4*ss[8];
  root_5 += 15*c0p2*c3p4*ss[12];
  root_5 += 15*c0p2*c4p4*ss[16];
  root_5 += 15*c0p2*c5p4*ss[20];
  root_5 += 20*c0p3*c1p3*ss[3];
  root_5 += 20*c0p3*c2p3*ss[6];
  root_5 += 20*c0p3*c3p3*ss[9];
  root_5 += 20*c0p3*c4p3*ss[12];
  root_5 += 20*c0p3*c5p3*ss[15];
  root_5 += 15*c0p4*c1p2*ss[2];
  root_5 += 15*c0p4*c2p2*ss[4];
  root_5 += 15*c0p4*c3p2*ss[6];
  root_5 += 15*c0p4*c4p2*ss[8];
  root_5 += 15*c0p4*c5p2*ss[10];
  root_5 += 6*c0p5*c1p1*ss[1];
  root_5 += 6*c0p5*c2p1*ss[2];
  root_5 += 6*c0p5*c3p1*ss[3];
  root_5 += 6*c0p5*c4p1*ss[4];
  root_5 += 6*c0p5*c5p1*ss[5];
  root_5 += 6*c1p1*c2p5*ss[11];
  root_5 += 6*c1p1*c3p5*ss[16];
  root_5 += 6*c1p1*c4p5*ss[21];
  root_5 += 6*c1p1*c5p5*ss[26];
  root_5 += 15*c1p2*c2p4*ss[10];
  root_5 += 15*c1p2*c3p4*ss[14];
  root_5 += 15*c1p2*c4p4*ss[18];
  root_5 += 15*c1p2*c5p4*ss[22];
  root_5 += 20*c1p3*c2p3*ss[9];
  root_5 += 20*c1p3*c3p3*ss[12];
  root_5 += 20*c1p3*c4p3*ss[15];
  root_5 += 20*c1p3*c5p3*ss[18];
  root_5 += 15*c1p4*c2p2*ss[8];
  root_5 += 15*c1p4*c3p2*ss[10];
  root_5 += 15*c1p4*c4p2*ss[12];
  root_5 += 15*c1p4*c5p2*ss[14];
  root_5 += 6*c1p5*c2p1*ss[7];
  root_5 += 6*c1p5*c3p1*ss[8];
  root_5 += 6*c1p5*c4p1*ss[9];
  root_5 += 6*c1p5*c5p1*ss[10];
  root_5 += 6*c2p1*c3p5*ss[17];
  root_5 += 6*c2p1*c4p5*ss[22];
  root_5 += 6*c2p1*c5p5*ss[27];
  root_5 += 15*c2p2*c3p4*ss[16];
  root_5 += 15*c2p2*c4p4*ss[20];
  root_5 += 15*c2p2*c5p4*ss[24];
  root_5 += 20*c2p3*c3p3*ss[15];
  root_5 += 20*c2p3*c4p3*ss[18];
  root_5 += 20*c2p3*c5p3*ss[21];
  root_5 += 15*c2p4*c3p2*ss[14];
  root_5 += 15*c2p4*c4p2*ss[16];
  root_5 += 15*c2p4*c5p2*ss[18];
  root_5 += 6*c2p5*c3p1*ss[13];
  root_5 += 6*c2p5*c4p1*ss[14];
  root_5 += 6*c2p5*c5p1*ss[15];
  root_5 += 6*c3p1*c4p5*ss[23];
  root_5 += 6*c3p1*c5p5*ss[28];
  root_5 += 15*c3p2*c4p4*ss[22];
  root_5 += 15*c3p2*c5p4*ss[26];
  root_5 += 20*c3p3*c4p3*ss[21];
  root_5 += 20*c3p3*c5p3*ss[24];
  root_5 += 15*c3p4*c4p2*ss[20];
  root_5 += 15*c3p4*c5p2*ss[22];
  root_5 += 6*c3p5*c4p1*ss[19];
  root_5 += 6*c3p5*c5p1*ss[20];
  root_5 += 6*c4p1*c5p5*ss[29];
  root_5 += 15*c4p2*c5p4*ss[28];
  root_5 += 20*c4p3*c5p3*ss[27];
  root_5 += 15*c4p4*c5p2*ss[26];
  root_5 += 6*c4p5*c5p1*ss[25];
  root_5 += c1p6*ss[6];
  root_5 += c2p6*ss[12];
  root_5 += c3p6*ss[18];
  root_5 += c4p6*ss[24];
  root_5 += c5p6*ss[30];
  root_5 += c0p6;
  root_5 += -tmom[6];
  TACC jac_0_0 = convert<TACC>(1);
  TACC jac_0_1 = convert<TACC>(ss[1]);
  TACC jac_0_2 = convert<TACC>(ss[2]);
  TACC jac_0_3 = convert<TACC>(ss[3]);
  TACC jac_0_4 = convert<TACC>(ss[4]);
  TACC jac_0_5 = convert<TACC>(ss[5]);
  TACC jac_1_0 = convert<TACC>(2*c1p1*ss[1]);
  jac_1_0 += 2*c2p1*ss[2];
  jac_1_0 += 2*c3p1*ss[3];
  jac_1_0 += 2*c4p1*ss[4];
  jac_1_0 += 2*c5p1*ss[5];
  jac_1_0 += 2*c0p1;
  TACC jac_1_1 = convert<TACC>(2*c0p1*ss[1]);
  jac_1_1 += 2*c1p1*ss[2];
  jac_1_1 += 2*c2p1*ss[3];
  jac_1_1 += 2*c3p1*ss[4];
  jac_1_1 += 2*c4p1*ss[5];
  jac_1_1 += 2*c5p1*ss[6];
  TACC jac_1_2 = convert<TACC>(2*c0p1*ss[2]);
  jac_1_2 += 2*c1p1*ss[3];
  jac_1_2 += 2*c2p1*ss[4];
  jac_1_2 += 2*c3p1*ss[5];
  jac_1_2 += 2*c4p1*ss[6];
  jac_1_2 += 2*c5p1*ss[7];
  TACC jac_1_3 = convert<TACC>(2*c0p1*ss[3]);
  jac_1_3 += 2*c1p1*ss[4];
  jac_1_3 += 2*c2p1*ss[5];
  jac_1_3 += 2*c3p1*ss[6];
  jac_1_3 += 2*c4p1*ss[7];
  jac_1_3 += 2*c5p1*ss[8];
  TACC jac_1_4 = convert<TACC>(2*c0p1*ss[4]);
  jac_1_4 += 2*c1p1*ss[5];
  jac_1_4 += 2*c2p1*ss[6];
  jac_1_4 += 2*c3p1*ss[7];
  jac_1_4 += 2*c4p1*ss[8];
  jac_1_4 += 2*c5p1*ss[9];
  TACC jac_1_5 = convert<TACC>(2*c0p1*ss[5]);
  jac_1_5 += 2*c1p1*ss[6];
  jac_1_5 += 2*c2p1*ss[7];
  jac_1_5 += 2*c3p1*ss[8];
  jac_1_5 += 2*c4p1*ss[9];
  jac_1_5 += 2*c5p1*ss[10];
  TACC jac_2_0 = convert<TACC>(6*c0p1*c1p1*ss[1]);
  jac_2_0 += 6*c0p1*c2p1*ss[2];
  jac_2_0 += 6*c0p1*c3p1*ss[3];
  jac_2_0 += 6*c0p1*c4p1*ss[4];
  jac_2_0 += 6*c0p1*c5p1*ss[5];
  jac_2_0 += 6*c1p1*c2p1*ss[3];
  jac_2_0 += 6*c1p1*c3p1*ss[4];
  jac_2_0 += 6*c1p1*c4p1*ss[5];
  jac_2_0 += 6*c1p1*c5p1*ss[6];
  jac_2_0 += 6*c2p1*c3p1*ss[5];
  jac_2_0 += 6*c2p1*c4p1*ss[6];
  jac_2_0 += 6*c2p1*c5p1*ss[7];
  jac_2_0 += 6*c3p1*c4p1*ss[7];
  jac_2_0 += 6*c3p1*c5p1*ss[8];
  jac_2_0 += 6*c4p1*c5p1*ss[9];
  jac_2_0 += 3*c1p2*ss[2];
  jac_2_0 += 3*c2p2*ss[4];
  jac_2_0 += 3*c3p2*ss[6];
  jac_2_0 += 3*c4p2*ss[8];
  jac_2_0 += 3*c5p2*ss[10];
  jac_2_0 += 3*c0p2;
  TACC jac_2_1 = convert<TACC>(6*c0p1*c1p1*ss[2]);
  jac_2_1 += 6*c0p1*c2p1*ss[3];
  jac_2_1 += 6*c0p1*c3p1*ss[4];
  jac_2_1 += 6*c0p1*c4p1*ss[5];
  jac_2_1 += 6*c0p1*c5p1*ss[6];
  jac_2_1 += 6*c1p1*c2p1*ss[4];
  jac_2_1 += 6*c1p1*c3p1*ss[5];
  jac_2_1 += 6*c1p1*c4p1*ss[6];
  jac_2_1 += 6*c1p1*c5p1*ss[7];
  jac_2_1 += 6*c2p1*c3p1*ss[6];
  jac_2_1 += 6*c2p1*c4p1*ss[7];
  jac_2_1 += 6*c2p1*c5p1*ss[8];
  jac_2_1 += 6*c3p1*c4p1*ss[8];
  jac_2_1 += 6*c3p1*c5p1*ss[9];
  jac_2_1 += 6*c4p1*c5p1*ss[10];
  jac_2_1 += 3*c0p2*ss[1];
  jac_2_1 += 3*c1p2*ss[3];
  jac_2_1 += 3*c2p2*ss[5];
  jac_2_1 += 3*c3p2*ss[7];
  jac_2_1 += 3*c4p2*ss[9];
  jac_2_1 += 3*c5p2*ss[11];
  TACC jac_2_2 = convert<TACC>(6*c0p1*c1p1*ss[3]);
  jac_2_2 += 6*c0p1*c2p1*ss[4];
  jac_2_2 += 6*c0p1*c3p1*ss[5];
  jac_2_2 += 6*c0p1*c4p1*ss[6];
  jac_2_2 += 6*c0p1*c5p1*ss[7];
  jac_2_2 += 6*c1p1*c2p1*ss[5];
  jac_2_2 += 6*c1p1*c3p1*ss[6];
  jac_2_2 += 6*c1p1*c4p1*ss[7];
  jac_2_2 += 6*c1p1*c5p1*ss[8];
  jac_2_2 += 6*c2p1*c3p1*ss[7];
  jac_2_2 += 6*c2p1*c4p1*ss[8];
  jac_2_2 += 6*c2p1*c5p1*ss[9];
  jac_2_2 += 6*c3p1*c4p1*ss[9];
  jac_2_2 += 6*c3p1*c5p1*ss[10];
  jac_2_2 += 6*c4p1*c5p1*ss[11];
  jac_2_2 += 3*c0p2*ss[2];
  jac_2_2 += 3*c1p2*ss[4];
  jac_2_2 += 3*c2p2*ss[6];
  jac_2_2 += 3*c3p2*ss[8];
  jac_2_2 += 3*c4p2*ss[10];
  jac_2_2 += 3*c5p2*ss[12];
  TACC jac_2_3 = convert<TACC>(6*c0p1*c1p1*ss[4]);
  jac_2_3 += 6*c0p1*c2p1*ss[5];
  jac_2_3 += 6*c0p1*c3p1*ss[6];
  jac_2_3 += 6*c0p1*c4p1*ss[7];
  jac_2_3 += 6*c0p1*c5p1*ss[8];
  jac_2_3 += 6*c1p1*c2p1*ss[6];
  jac_2_3 += 6*c1p1*c3p1*ss[7];
  jac_2_3 += 6*c1p1*c4p1*ss[8];
  jac_2_3 += 6*c1p1*c5p1*ss[9];
  jac_2_3 += 6*c2p1*c3p1*ss[8];
  jac_2_3 += 6*c2p1*c4p1*ss[9];
  jac_2_3 += 6*c2p1*c5p1*ss[10];
  jac_2_3 += 6*c3p1*c4p1*ss[10];
  jac_2_3 += 6*c3p1*c5p1*ss[11];
  jac_2_3 += 6*c4p1*c5p1*ss[12];
  jac_2_3 += 3*c0p2*ss[3];
  jac_2_3 += 3*c1p2*ss[5];
  jac_2_3 += 3*c2p2*ss[7];
  jac_2_3 += 3*c3p2*ss[9];
  jac_2_3 += 3*c4p2*ss[11];
  jac_2_3 += 3*c5p2*ss[13];
  TACC jac_2_4 = convert<TACC>(6*c0p1*c1p1*ss[5]);
  jac_2_4 += 6*c0p1*c2p1*ss[6];
  jac_2_4 += 6*c0p1*c3p1*ss[7];
  jac_2_4 += 6*c0p1*c4p1*ss[8];
  jac_2_4 += 6*c0p1*c5p1*ss[9];
  jac_2_4 += 6*c1p1*c2p1*ss[7];
  jac_2_4 += 6*c1p1*c3p1*ss[8];
  jac_2_4 += 6*c1p1*c4p1*ss[9];
  jac_2_4 += 6*c1p1*c5p1*ss[10];
  jac_2_4 += 6*c2p1*c3p1*ss[9];
  jac_2_4 += 6*c2p1*c4p1*ss[10];
  jac_2_4 += 6*c2p1*c5p1*ss[11];
  jac_2_4 += 6*c3p1*c4p1*ss[11];
  jac_2_4 += 6*c3p1*c5p1*ss[12];
  jac_2_4 += 6*c4p1*c5p1*ss[13];
  jac_2_4 += 3*c0p2*ss[4];
  jac_2_4 += 3*c1p2*ss[6];
  jac_2_4 += 3*c2p2*ss[8];
  jac_2_4 += 3*c3p2*ss[10];
  jac_2_4 += 3*c4p2*ss[12];
  jac_2_4 += 3*c5p2*ss[14];
  TACC jac_2_5 = convert<TACC>(6*c0p1*c1p1*ss[6]);
  jac_2_5 += 6*c0p1*c2p1*ss[7];
  jac_2_5 += 6*c0p1*c3p1*ss[8];
  jac_2_5 += 6*c0p1*c4p1*ss[9];
  jac_2_5 += 6*c0p1*c5p1*ss[10];
  jac_2_5 += 6*c1p1*c2p1*ss[8];
  jac_2_5 += 6*c1p1*c3p1*ss[9];
  jac_2_5 += 6*c1p1*c4p1*ss[10];
  jac_2_5 += 6*c1p1*c5p1*ss[11];
  jac_2_5 += 6*c2p1*c3p1*ss[10];
  jac_2_5 += 6*c2p1*c4p1*ss[11];
  jac_2_5 += 6*c2p1*c5p1*ss[12];
  jac_2_5 += 6*c3p1*c4p1*ss[12];
  jac_2_5 += 6*c3p1*c5p1*ss[13];
  jac_2_5 += 6*c4p1*c5p1*ss[14];
  jac_2_5 += 3*c0p2*ss[5];
  jac_2_5 += 3*c1p2*ss[7];
  jac_2_5 += 3*c2p2*ss[9];
  jac_2_5 += 3*c3p2*ss[11];
  jac_2_5 += 3*c4p2*ss[13];
  jac_2_5 += 3*c5p2*ss[15];
  TACC jac_3_0 = convert<TACC>(24*c0p1*c1p1*c2p1*ss[3]);
  jac_3_0 += 24*c0p1*c1p1*c3p1*ss[4];
  jac_3_0 += 24*c0p1*c1p1*c4p1*ss[5];
  jac_3_0 += 24*c0p1*c1p1*c5p1*ss[6];
  jac_3_0 += 24*c0p1*c2p1*c3p1*ss[5];
  jac_3_0 += 24*c0p1*c2p1*c4p1*ss[6];
  jac_3_0 += 24*c0p1*c2p1*c5p1*ss[7];
  jac_3_0 += 24*c0p1*c3p1*c4p1*ss[7];
  jac_3_0 += 24*c0p1*c3p1*c5p1*ss[8];
  jac_3_0 += 24*c0p1*c4p1*c5p1*ss[9];
  jac_3_0 += 24*c1p1*c2p1*c3p1*ss[6];
  jac_3_0 += 24*c1p1*c2p1*c4p1*ss[7];
  jac_3_0 += 24*c1p1*c2p1*c5p1*ss[8];
  jac_3_0 += 24*c1p1*c3p1*c4p1*ss[8];
  jac_3_0 += 24*c1p1*c3p1*c5p1*ss[9];
  jac_3_0 += 24*c1p1*c4p1*c5p1*ss[10];
  jac_3_0 += 24*c2p1*c3p1*c4p1*ss[9];
  jac_3_0 += 24*c2p1*c3p1*c5p1*ss[10];
  jac_3_0 += 24*c2p1*c4p1*c5p1*ss[11];
  jac_3_0 += 24*c3p1*c4p1*c5p1*ss[12];
  jac_3_0 += 12*c0p1*c1p2*ss[2];
  jac_3_0 += 12*c0p1*c2p2*ss[4];
  jac_3_0 += 12*c0p1*c3p2*ss[6];
  jac_3_0 += 12*c0p1*c4p2*ss[8];
  jac_3_0 += 12*c0p1*c5p2*ss[10];
  jac_3_0 += 12*c0p2*c1p1*ss[1];
  jac_3_0 += 12*c0p2*c2p1*ss[2];
  jac_3_0 += 12*c0p2*c3p1*ss[3];
  jac_3_0 += 12*c0p2*c4p1*ss[4];
  jac_3_0 += 12*c0p2*c5p1*ss[5];
  jac_3_0 += 12*c1p1*c2p2*ss[5];
  jac_3_0 += 12*c1p1*c3p2*ss[7];
  jac_3_0 += 12*c1p1*c4p2*ss[9];
  jac_3_0 += 12*c1p1*c5p2*ss[11];
  jac_3_0 += 12*c1p2*c2p1*ss[4];
  jac_3_0 += 12*c1p2*c3p1*ss[5];
  jac_3_0 += 12*c1p2*c4p1*ss[6];
  jac_3_0 += 12*c1p2*c5p1*ss[7];
  jac_3_0 += 12*c2p1*c3p2*ss[8];
  jac_3_0 += 12*c2p1*c4p2*ss[10];
  jac_3_0 += 12*c2p1*c5p2*ss[12];
  jac_3_0 += 12*c2p2*c3p1*ss[7];
  jac_3_0 += 12*c2p2*c4p1*ss[8];
  jac_3_0 += 12*c2p2*c5p1*ss[9];
  jac_3_0 += 12*c3p1*c4p2*ss[11];
  jac_3_0 += 12*c3p1*c5p2*ss[13];
  jac_3_0 += 12*c3p2*c4p1*ss[10];
  jac_3_0 += 12*c3p2*c5p1*ss[11];
  jac_3_0 += 12*c4p1*c5p2*ss[14];
  jac_3_0 += 12*c4p2*c5p1*ss[13];
  jac_3_0 += 4*c1p3*ss[3];
  jac_3_0 += 4*c2p3*ss[6];
  jac_3_0 += 4*c3p3*ss[9];
  jac_3_0 += 4*c4p3*ss[12];
  jac_3_0 += 4*c5p3*ss[15];
  jac_3_0 += 4*c0p3;
  TACC jac_3_1 = convert<TACC>(24*c0p1*c1p1*c2p1*ss[4]);
  jac_3_1 += 24*c0p1*c1p1*c3p1*ss[5];
  jac_3_1 += 24*c0p1*c1p1*c4p1*ss[6];
  jac_3_1 += 24*c0p1*c1p1*c5p1*ss[7];
  jac_3_1 += 24*c0p1*c2p1*c3p1*ss[6];
  jac_3_1 += 24*c0p1*c2p1*c4p1*ss[7];
  jac_3_1 += 24*c0p1*c2p1*c5p1*ss[8];
  jac_3_1 += 24*c0p1*c3p1*c4p1*ss[8];
  jac_3_1 += 24*c0p1*c3p1*c5p1*ss[9];
  jac_3_1 += 24*c0p1*c4p1*c5p1*ss[10];
  jac_3_1 += 24*c1p1*c2p1*c3p1*ss[7];
  jac_3_1 += 24*c1p1*c2p1*c4p1*ss[8];
  jac_3_1 += 24*c1p1*c2p1*c5p1*ss[9];
  jac_3_1 += 24*c1p1*c3p1*c4p1*ss[9];
  jac_3_1 += 24*c1p1*c3p1*c5p1*ss[10];
  jac_3_1 += 24*c1p1*c4p1*c5p1*ss[11];
  jac_3_1 += 24*c2p1*c3p1*c4p1*ss[10];
  jac_3_1 += 24*c2p1*c3p1*c5p1*ss[11];
  jac_3_1 += 24*c2p1*c4p1*c5p1*ss[12];
  jac_3_1 += 24*c3p1*c4p1*c5p1*ss[13];
  jac_3_1 += 12*c0p1*c1p2*ss[3];
  jac_3_1 += 12*c0p1*c2p2*ss[5];
  jac_3_1 += 12*c0p1*c3p2*ss[7];
  jac_3_1 += 12*c0p1*c4p2*ss[9];
  jac_3_1 += 12*c0p1*c5p2*ss[11];
  jac_3_1 += 12*c0p2*c1p1*ss[2];
  jac_3_1 += 12*c0p2*c2p1*ss[3];
  jac_3_1 += 12*c0p2*c3p1*ss[4];
  jac_3_1 += 12*c0p2*c4p1*ss[5];
  jac_3_1 += 12*c0p2*c5p1*ss[6];
  jac_3_1 += 12*c1p1*c2p2*ss[6];
  jac_3_1 += 12*c1p1*c3p2*ss[8];
  jac_3_1 += 12*c1p1*c4p2*ss[10];
  jac_3_1 += 12*c1p1*c5p2*ss[12];
  jac_3_1 += 12*c1p2*c2p1*ss[5];
  jac_3_1 += 12*c1p2*c3p1*ss[6];
  jac_3_1 += 12*c1p2*c4p1*ss[7];
  jac_3_1 += 12*c1p2*c5p1*ss[8];
  jac_3_1 += 12*c2p1*c3p2*ss[9];
  jac_3_1 += 12*c2p1*c4p2*ss[11];
  jac_3_1 += 12*c2p1*c5p2*ss[13];
  jac_3_1 += 12*c2p2*c3p1*ss[8];
  jac_3_1 += 12*c2p2*c4p1*ss[9];
  jac_3_1 += 12*c2p2*c5p1*ss[10];
  jac_3_1 += 12*c3p1*c4p2*ss[12];
  jac_3_1 += 12*c3p1*c5p2*ss[14];
  jac_3_1 += 12*c3p2*c4p1*ss[11];
  jac_3_1 += 12*c3p2*c5p1*ss[12];
  jac_3_1 += 12*c4p1*c5p2*ss[15];
  jac_3_1 += 12*c4p2*c5p1*ss[14];
  jac_3_1 += 4*c0p3*ss[1];
  jac_3_1 += 4*c1p3*ss[4];
  jac_3_1 += 4*c2p3*ss[7];
  jac_3_1 += 4*c3p3*ss[10];
  jac_3_1 += 4*c4p3*ss[13];
  jac_3_1 += 4*c5p3*ss[16];
  TACC jac_3_2 = convert<TACC>(24*c0p1*c1p1*c2p1*ss[5]);
  jac_3_2 += 24*c0p1*c1p1*c3p1*ss[6];
  jac_3_2 += 24*c0p1*c1p1*c4p1*ss[7];
  jac_3_2 += 24*c0p1*c1p1*c5p1*ss[8];
  jac_3_2 += 24*c0p1*c2p1*c3p1*ss[7];
  jac_3_2 += 24*c0p1*c2p1*c4p1*ss[8];
  jac_3_2 += 24*c0p1*c2p1*c5p1*ss[9];
  jac_3_2 += 24*c0p1*c3p1*c4p1*ss[9];
  jac_3_2 += 24*c0p1*c3p1*c5p1*ss[10];
  jac_3_2 += 24*c0p1*c4p1*c5p1*ss[11];
  jac_3_2 += 24*c1p1*c2p1*c3p1*ss[8];
  jac_3_2 += 24*c1p1*c2p1*c4p1*ss[9];
  jac_3_2 += 24*c1p1*c2p1*c5p1*ss[10];
  jac_3_2 += 24*c1p1*c3p1*c4p1*ss[10];
  jac_3_2 += 24*c1p1*c3p1*c5p1*ss[11];
  jac_3_2 += 24*c1p1*c4p1*c5p1*ss[12];
  jac_3_2 += 24*c2p1*c3p1*c4p1*ss[11];
  jac_3_2 += 24*c2p1*c3p1*c5p1*ss[12];
  jac_3_2 += 24*c2p1*c4p1*c5p1*ss[13];
  jac_3_2 += 24*c3p1*c4p1*c5p1*ss[14];
  jac_3_2 += 12*c0p1*c1p2*ss[4];
  jac_3_2 += 12*c0p1*c2p2*ss[6];
  jac_3_2 += 12*c0p1*c3p2*ss[8];
  jac_3_2 += 12*c0p1*c4p2*ss[10];
  jac_3_2 += 12*c0p1*c5p2*ss[12];
  jac_3_2 += 12*c0p2*c1p1*ss[3];
  jac_3_2 += 12*c0p2*c2p1*ss[4];
  jac_3_2 += 12*c0p2*c3p1*ss[5];
  jac_3_2 += 12*c0p2*c4p1*ss[6];
  jac_3_2 += 12*c0p2*c5p1*ss[7];
  jac_3_2 += 12*c1p1*c2p2*ss[7];
  jac_3_2 += 12*c1p1*c3p2*ss[9];
  jac_3_2 += 12*c1p1*c4p2*ss[11];
  jac_3_2 += 12*c1p1*c5p2*ss[13];
  jac_3_2 += 12*c1p2*c2p1*ss[6];
  jac_3_2 += 12*c1p2*c3p1*ss[7];
  jac_3_2 += 12*c1p2*c4p1*ss[8];
  jac_3_2 += 12*c1p2*c5p1*ss[9];
  jac_3_2 += 12*c2p1*c3p2*ss[10];
  jac_3_2 += 12*c2p1*c4p2*ss[12];
  jac_3_2 += 12*c2p1*c5p2*ss[14];
  jac_3_2 += 12*c2p2*c3p1*ss[9];
  jac_3_2 += 12*c2p2*c4p1*ss[10];
  jac_3_2 += 12*c2p2*c5p1*ss[11];
  jac_3_2 += 12*c3p1*c4p2*ss[13];
  jac_3_2 += 12*c3p1*c5p2*ss[15];
  jac_3_2 += 12*c3p2*c4p1*ss[12];
  jac_3_2 += 12*c3p2*c5p1*ss[13];
  jac_3_2 += 12*c4p1*c5p2*ss[16];
  jac_3_2 += 12*c4p2*c5p1*ss[15];
  jac_3_2 += 4*c0p3*ss[2];
  jac_3_2 += 4*c1p3*ss[5];
  jac_3_2 += 4*c2p3*ss[8];
  jac_3_2 += 4*c3p3*ss[11];
  jac_3_2 += 4*c4p3*ss[14];
  jac_3_2 += 4*c5p3*ss[17];
  TACC jac_3_3 = convert<TACC>(24*c0p1*c1p1*c2p1*ss[6]);
  jac_3_3 += 24*c0p1*c1p1*c3p1*ss[7];
  jac_3_3 += 24*c0p1*c1p1*c4p1*ss[8];
  jac_3_3 += 24*c0p1*c1p1*c5p1*ss[9];
  jac_3_3 += 24*c0p1*c2p1*c3p1*ss[8];
  jac_3_3 += 24*c0p1*c2p1*c4p1*ss[9];
  jac_3_3 += 24*c0p1*c2p1*c5p1*ss[10];
  jac_3_3 += 24*c0p1*c3p1*c4p1*ss[10];
  jac_3_3 += 24*c0p1*c3p1*c5p1*ss[11];
  jac_3_3 += 24*c0p1*c4p1*c5p1*ss[12];
  jac_3_3 += 24*c1p1*c2p1*c3p1*ss[9];
  jac_3_3 += 24*c1p1*c2p1*c4p1*ss[10];
  jac_3_3 += 24*c1p1*c2p1*c5p1*ss[11];
  jac_3_3 += 24*c1p1*c3p1*c4p1*ss[11];
  jac_3_3 += 24*c1p1*c3p1*c5p1*ss[12];
  jac_3_3 += 24*c1p1*c4p1*c5p1*ss[13];
  jac_3_3 += 24*c2p1*c3p1*c4p1*ss[12];
  jac_3_3 += 24*c2p1*c3p1*c5p1*ss[13];
  jac_3_3 += 24*c2p1*c4p1*c5p1*ss[14];
  jac_3_3 += 24*c3p1*c4p1*c5p1*ss[15];
  jac_3_3 += 12*c0p1*c1p2*ss[5];
  jac_3_3 += 12*c0p1*c2p2*ss[7];
  jac_3_3 += 12*c0p1*c3p2*ss[9];
  jac_3_3 += 12*c0p1*c4p2*ss[11];
  jac_3_3 += 12*c0p1*c5p2*ss[13];
  jac_3_3 += 12*c0p2*c1p1*ss[4];
  jac_3_3 += 12*c0p2*c2p1*ss[5];
  jac_3_3 += 12*c0p2*c3p1*ss[6];
  jac_3_3 += 12*c0p2*c4p1*ss[7];
  jac_3_3 += 12*c0p2*c5p1*ss[8];
  jac_3_3 += 12*c1p1*c2p2*ss[8];
  jac_3_3 += 12*c1p1*c3p2*ss[10];
  jac_3_3 += 12*c1p1*c4p2*ss[12];
  jac_3_3 += 12*c1p1*c5p2*ss[14];
  jac_3_3 += 12*c1p2*c2p1*ss[7];
  jac_3_3 += 12*c1p2*c3p1*ss[8];
  jac_3_3 += 12*c1p2*c4p1*ss[9];
  jac_3_3 += 12*c1p2*c5p1*ss[10];
  jac_3_3 += 12*c2p1*c3p2*ss[11];
  jac_3_3 += 12*c2p1*c4p2*ss[13];
  jac_3_3 += 12*c2p1*c5p2*ss[15];
  jac_3_3 += 12*c2p2*c3p1*ss[10];
  jac_3_3 += 12*c2p2*c4p1*ss[11];
  jac_3_3 += 12*c2p2*c5p1*ss[12];
  jac_3_3 += 12*c3p1*c4p2*ss[14];
  jac_3_3 += 12*c3p1*c5p2*ss[16];
  jac_3_3 += 12*c3p2*c4p1*ss[13];
  jac_3_3 += 12*c3p2*c5p1*ss[14];
  jac_3_3 += 12*c4p1*c5p2*ss[17];
  jac_3_3 += 12*c4p2*c5p1*ss[16];
  jac_3_3 += 4*c0p3*ss[3];
  jac_3_3 += 4*c1p3*ss[6];
  jac_3_3 += 4*c2p3*ss[9];
  jac_3_3 += 4*c3p3*ss[12];
  jac_3_3 += 4*c4p3*ss[15];
  jac_3_3 += 4*c5p3*ss[18];
  TACC jac_3_4 = convert<TACC>(24*c0p1*c1p1*c2p1*ss[7]);
  jac_3_4 += 24*c0p1*c1p1*c3p1*ss[8];
  jac_3_4 += 24*c0p1*c1p1*c4p1*ss[9];
  jac_3_4 += 24*c0p1*c1p1*c5p1*ss[10];
  jac_3_4 += 24*c0p1*c2p1*c3p1*ss[9];
  jac_3_4 += 24*c0p1*c2p1*c4p1*ss[10];
  jac_3_4 += 24*c0p1*c2p1*c5p1*ss[11];
  jac_3_4 += 24*c0p1*c3p1*c4p1*ss[11];
  jac_3_4 += 24*c0p1*c3p1*c5p1*ss[12];
  jac_3_4 += 24*c0p1*c4p1*c5p1*ss[13];
  jac_3_4 += 24*c1p1*c2p1*c3p1*ss[10];
  jac_3_4 += 24*c1p1*c2p1*c4p1*ss[11];
  jac_3_4 += 24*c1p1*c2p1*c5p1*ss[12];
  jac_3_4 += 24*c1p1*c3p1*c4p1*ss[12];
  jac_3_4 += 24*c1p1*c3p1*c5p1*ss[13];
  jac_3_4 += 24*c1p1*c4p1*c5p1*ss[14];
  jac_3_4 += 24*c2p1*c3p1*c4p1*ss[13];
  jac_3_4 += 24*c2p1*c3p1*c5p1*ss[14];
  jac_3_4 += 24*c2p1*c4p1*c5p1*ss[15];
  jac_3_4 += 24*c3p1*c4p1*c5p1*ss[16];
  jac_3_4 += 12*c0p1*c1p2*ss[6];
  jac_3_4 += 12*c0p1*c2p2*ss[8];
  jac_3_4 += 12*c0p1*c3p2*ss[10];
  jac_3_4 += 12*c0p1*c4p2*ss[12];
  jac_3_4 += 12*c0p1*c5p2*ss[14];
  jac_3_4 += 12*c0p2*c1p1*ss[5];
  jac_3_4 += 12*c0p2*c2p1*ss[6];
  jac_3_4 += 12*c0p2*c3p1*ss[7];
  jac_3_4 += 12*c0p2*c4p1*ss[8];
  jac_3_4 += 12*c0p2*c5p1*ss[9];
  jac_3_4 += 12*c1p1*c2p2*ss[9];
  jac_3_4 += 12*c1p1*c3p2*ss[11];
  jac_3_4 += 12*c1p1*c4p2*ss[13];
  jac_3_4 += 12*c1p1*c5p2*ss[15];
  jac_3_4 += 12*c1p2*c2p1*ss[8];
  jac_3_4 += 12*c1p2*c3p1*ss[9];
  jac_3_4 += 12*c1p2*c4p1*ss[10];
  jac_3_4 += 12*c1p2*c5p1*ss[11];
  jac_3_4 += 12*c2p1*c3p2*ss[12];
  jac_3_4 += 12*c2p1*c4p2*ss[14];
  jac_3_4 += 12*c2p1*c5p2*ss[16];
  jac_3_4 += 12*c2p2*c3p1*ss[11];
  jac_3_4 += 12*c2p2*c4p1*ss[12];
  jac_3_4 += 12*c2p2*c5p1*ss[13];
  jac_3_4 += 12*c3p1*c4p2*ss[15];
  jac_3_4 += 12*c3p1*c5p2*ss[17];
  jac_3_4 += 12*c3p2*c4p1*ss[14];
  jac_3_4 += 12*c3p2*c5p1*ss[15];
  jac_3_4 += 12*c4p1*c5p2*ss[18];
  jac_3_4 += 12*c4p2*c5p1*ss[17];
  jac_3_4 += 4*c0p3*ss[4];
  jac_3_4 += 4*c1p3*ss[7];
  jac_3_4 += 4*c2p3*ss[10];
  jac_3_4 += 4*c3p3*ss[13];
  jac_3_4 += 4*c4p3*ss[16];
  jac_3_4 += 4*c5p3*ss[19];
  TACC jac_3_5 = convert<TACC>(24*c0p1*c1p1*c2p1*ss[8]);
  jac_3_5 += 24*c0p1*c1p1*c3p1*ss[9];
  jac_3_5 += 24*c0p1*c1p1*c4p1*ss[10];
  jac_3_5 += 24*c0p1*c1p1*c5p1*ss[11];
  jac_3_5 += 24*c0p1*c2p1*c3p1*ss[10];
  jac_3_5 += 24*c0p1*c2p1*c4p1*ss[11];
  jac_3_5 += 24*c0p1*c2p1*c5p1*ss[12];
  jac_3_5 += 24*c0p1*c3p1*c4p1*ss[12];
  jac_3_5 += 24*c0p1*c3p1*c5p1*ss[13];
  jac_3_5 += 24*c0p1*c4p1*c5p1*ss[14];
  jac_3_5 += 24*c1p1*c2p1*c3p1*ss[11];
  jac_3_5 += 24*c1p1*c2p1*c4p1*ss[12];
  jac_3_5 += 24*c1p1*c2p1*c5p1*ss[13];
  jac_3_5 += 24*c1p1*c3p1*c4p1*ss[13];
  jac_3_5 += 24*c1p1*c3p1*c5p1*ss[14];
  jac_3_5 += 24*c1p1*c4p1*c5p1*ss[15];
  jac_3_5 += 24*c2p1*c3p1*c4p1*ss[14];
  jac_3_5 += 24*c2p1*c3p1*c5p1*ss[15];
  jac_3_5 += 24*c2p1*c4p1*c5p1*ss[16];
  jac_3_5 += 24*c3p1*c4p1*c5p1*ss[17];
  jac_3_5 += 12*c0p1*c1p2*ss[7];
  jac_3_5 += 12*c0p1*c2p2*ss[9];
  jac_3_5 += 12*c0p1*c3p2*ss[11];
  jac_3_5 += 12*c0p1*c4p2*ss[13];
  jac_3_5 += 12*c0p1*c5p2*ss[15];
  jac_3_5 += 12*c0p2*c1p1*ss[6];
  jac_3_5 += 12*c0p2*c2p1*ss[7];
  jac_3_5 += 12*c0p2*c3p1*ss[8];
  jac_3_5 += 12*c0p2*c4p1*ss[9];
  jac_3_5 += 12*c0p2*c5p1*ss[10];
  jac_3_5 += 12*c1p1*c2p2*ss[10];
  jac_3_5 += 12*c1p1*c3p2*ss[12];
  jac_3_5 += 12*c1p1*c4p2*ss[14];
  jac_3_5 += 12*c1p1*c5p2*ss[16];
  jac_3_5 += 12*c1p2*c2p1*ss[9];
  jac_3_5 += 12*c1p2*c3p1*ss[10];
  jac_3_5 += 12*c1p2*c4p1*ss[11];
  jac_3_5 += 12*c1p2*c5p1*ss[12];
  jac_3_5 += 12*c2p1*c3p2*ss[13];
  jac_3_5 += 12*c2p1*c4p2*ss[15];
  jac_3_5 += 12*c2p1*c5p2*ss[17];
  jac_3_5 += 12*c2p2*c3p1*ss[12];
  jac_3_5 += 12*c2p2*c4p1*ss[13];
  jac_3_5 += 12*c2p2*c5p1*ss[14];
  jac_3_5 += 12*c3p1*c4p2*ss[16];
  jac_3_5 += 12*c3p1*c5p2*ss[18];
  jac_3_5 += 12*c3p2*c4p1*ss[15];
  jac_3_5 += 12*c3p2*c5p1*ss[16];
  jac_3_5 += 12*c4p1*c5p2*ss[19];
  jac_3_5 += 12*c4p2*c5p1*ss[18];
  jac_3_5 += 4*c0p3*ss[5];
  jac_3_5 += 4*c1p3*ss[8];
  jac_3_5 += 4*c2p3*ss[11];
  jac_3_5 += 4*c3p3*ss[14];
  jac_3_5 += 4*c4p3*ss[17];
  jac_3_5 += 4*c5p3*ss[20];
  TACC jac_4_0 = convert<TACC>(120*c0p1*c1p1*c2p1*c3p1*ss[6]);
  jac_4_0 += 120*c0p1*c1p1*c2p1*c4p1*ss[7];
  jac_4_0 += 120*c0p1*c1p1*c2p1*c5p1*ss[8];
  jac_4_0 += 120*c0p1*c1p1*c3p1*c4p1*ss[8];
  jac_4_0 += 120*c0p1*c1p1*c3p1*c5p1*ss[9];
  jac_4_0 += 120*c0p1*c1p1*c4p1*c5p1*ss[10];
  jac_4_0 += 120*c0p1*c2p1*c3p1*c4p1*ss[9];
  jac_4_0 += 120*c0p1*c2p1*c3p1*c5p1*ss[10];
  jac_4_0 += 120*c0p1*c2p1*c4p1*c5p1*ss[11];
  jac_4_0 += 120*c0p1*c3p1*c4p1*c5p1*ss[12];
  jac_4_0 += 120*c1p1*c2p1*c3p1*c4p1*ss[10];
  jac_4_0 += 120*c1p1*c2p1*c3p1*c5p1*ss[11];
  jac_4_0 += 120*c1p1*c2p1*c4p1*c5p1*ss[12];
  jac_4_0 += 120*c1p1*c3p1*c4p1*c5p1*ss[13];
  jac_4_0 += 120*c2p1*c3p1*c4p1*c5p1*ss[14];
  jac_4_0 += 60*c0p1*c1p1*c2p2*ss[5];
  jac_4_0 += 60*c0p1*c1p1*c3p2*ss[7];
  jac_4_0 += 60*c0p1*c1p1*c4p2*ss[9];
  jac_4_0 += 60*c0p1*c1p1*c5p2*ss[11];
  jac_4_0 += 60*c0p1*c1p2*c2p1*ss[4];
  jac_4_0 += 60*c0p1*c1p2*c3p1*ss[5];
  jac_4_0 += 60*c0p1*c1p2*c4p1*ss[6];
  jac_4_0 += 60*c0p1*c1p2*c5p1*ss[7];
  jac_4_0 += 60*c0p1*c2p1*c3p2*ss[8];
  jac_4_0 += 60*c0p1*c2p1*c4p2*ss[10];
  jac_4_0 += 60*c0p1*c2p1*c5p2*ss[12];
  jac_4_0 += 60*c0p1*c2p2*c3p1*ss[7];
  jac_4_0 += 60*c0p1*c2p2*c4p1*ss[8];
  jac_4_0 += 60*c0p1*c2p2*c5p1*ss[9];
  jac_4_0 += 60*c0p1*c3p1*c4p2*ss[11];
  jac_4_0 += 60*c0p1*c3p1*c5p2*ss[13];
  jac_4_0 += 60*c0p1*c3p2*c4p1*ss[10];
  jac_4_0 += 60*c0p1*c3p2*c5p1*ss[11];
  jac_4_0 += 60*c0p1*c4p1*c5p2*ss[14];
  jac_4_0 += 60*c0p1*c4p2*c5p1*ss[13];
  jac_4_0 += 60*c0p2*c1p1*c2p1*ss[3];
  jac_4_0 += 60*c0p2*c1p1*c3p1*ss[4];
  jac_4_0 += 60*c0p2*c1p1*c4p1*ss[5];
  jac_4_0 += 60*c0p2*c1p1*c5p1*ss[6];
  jac_4_0 += 60*c0p2*c2p1*c3p1*ss[5];
  jac_4_0 += 60*c0p2*c2p1*c4p1*ss[6];
  jac_4_0 += 60*c0p2*c2p1*c5p1*ss[7];
  jac_4_0 += 60*c0p2*c3p1*c4p1*ss[7];
  jac_4_0 += 60*c0p2*c3p1*c5p1*ss[8];
  jac_4_0 += 60*c0p2*c4p1*c5p1*ss[9];
  jac_4_0 += 60*c1p1*c2p1*c3p2*ss[9];
  jac_4_0 += 60*c1p1*c2p1*c4p2*ss[11];
  jac_4_0 += 60*c1p1*c2p1*c5p2*ss[13];
  jac_4_0 += 60*c1p1*c2p2*c3p1*ss[8];
  jac_4_0 += 60*c1p1*c2p2*c4p1*ss[9];
  jac_4_0 += 60*c1p1*c2p2*c5p1*ss[10];
  jac_4_0 += 60*c1p1*c3p1*c4p2*ss[12];
  jac_4_0 += 60*c1p1*c3p1*c5p2*ss[14];
  jac_4_0 += 60*c1p1*c3p2*c4p1*ss[11];
  jac_4_0 += 60*c1p1*c3p2*c5p1*ss[12];
  jac_4_0 += 60*c1p1*c4p1*c5p2*ss[15];
  jac_4_0 += 60*c1p1*c4p2*c5p1*ss[14];
  jac_4_0 += 60*c1p2*c2p1*c3p1*ss[7];
  jac_4_0 += 60*c1p2*c2p1*c4p1*ss[8];
  jac_4_0 += 60*c1p2*c2p1*c5p1*ss[9];
  jac_4_0 += 60*c1p2*c3p1*c4p1*ss[9];
  jac_4_0 += 60*c1p2*c3p1*c5p1*ss[10];
  jac_4_0 += 60*c1p2*c4p1*c5p1*ss[11];
  jac_4_0 += 60*c2p1*c3p1*c4p2*ss[13];
  jac_4_0 += 60*c2p1*c3p1*c5p2*ss[15];
  jac_4_0 += 60*c2p1*c3p2*c4p1*ss[12];
  jac_4_0 += 60*c2p1*c3p2*c5p1*ss[13];
  jac_4_0 += 60*c2p1*c4p1*c5p2*ss[16];
  jac_4_0 += 60*c2p1*c4p2*c5p1*ss[15];
  jac_4_0 += 60*c2p2*c3p1*c4p1*ss[11];
  jac_4_0 += 60*c2p2*c3p1*c5p1*ss[12];
  jac_4_0 += 60*c2p2*c4p1*c5p1*ss[13];
  jac_4_0 += 60*c3p1*c4p1*c5p2*ss[17];
  jac_4_0 += 60*c3p1*c4p2*c5p1*ss[16];
  jac_4_0 += 60*c3p2*c4p1*c5p1*ss[15];
  jac_4_0 += 20*c0p1*c1p3*ss[3];
  jac_4_0 += 20*c0p1*c2p3*ss[6];
  jac_4_0 += 20*c0p1*c3p3*ss[9];
  jac_4_0 += 20*c0p1*c4p3*ss[12];
  jac_4_0 += 20*c0p1*c5p3*ss[15];
  jac_4_0 += 30*c0p2*c1p2*ss[2];
  jac_4_0 += 30*c0p2*c2p2*ss[4];
  jac_4_0 += 30*c0p2*c3p2*ss[6];
  jac_4_0 += 30*c0p2*c4p2*ss[8];
  jac_4_0 += 30*c0p2*c5p2*ss[10];
  jac_4_0 += 20*c0p3*c1p1*ss[1];
  jac_4_0 += 20*c0p3*c2p1*ss[2];
  jac_4_0 += 20*c0p3*c3p1*ss[3];
  jac_4_0 += 20*c0p3*c4p1*ss[4];
  jac_4_0 += 20*c0p3*c5p1*ss[5];
  jac_4_0 += 20*c1p1*c2p3*ss[7];
  jac_4_0 += 20*c1p1*c3p3*ss[10];
  jac_4_0 += 20*c1p1*c4p3*ss[13];
  jac_4_0 += 20*c1p1*c5p3*ss[16];
  jac_4_0 += 30*c1p2*c2p2*ss[6];
  jac_4_0 += 30*c1p2*c3p2*ss[8];
  jac_4_0 += 30*c1p2*c4p2*ss[10];
  jac_4_0 += 30*c1p2*c5p2*ss[12];
  jac_4_0 += 20*c1p3*c2p1*ss[5];
  jac_4_0 += 20*c1p3*c3p1*ss[6];
  jac_4_0 += 20*c1p3*c4p1*ss[7];
  jac_4_0 += 20*c1p3*c5p1*ss[8];
  jac_4_0 += 20*c2p1*c3p3*ss[11];
  jac_4_0 += 20*c2p1*c4p3*ss[14];
  jac_4_0 += 20*c2p1*c5p3*ss[17];
  jac_4_0 += 30*c2p2*c3p2*ss[10];
  jac_4_0 += 30*c2p2*c4p2*ss[12];
  jac_4_0 += 30*c2p2*c5p2*ss[14];
  jac_4_0 += 20*c2p3*c3p1*ss[9];
  jac_4_0 += 20*c2p3*c4p1*ss[10];
  jac_4_0 += 20*c2p3*c5p1*ss[11];
  jac_4_0 += 20*c3p1*c4p3*ss[15];
  jac_4_0 += 20*c3p1*c5p3*ss[18];
  jac_4_0 += 30*c3p2*c4p2*ss[14];
  jac_4_0 += 30*c3p2*c5p2*ss[16];
  jac_4_0 += 20*c3p3*c4p1*ss[13];
  jac_4_0 += 20*c3p3*c5p1*ss[14];
  jac_4_0 += 20*c4p1*c5p3*ss[19];
  jac_4_0 += 30*c4p2*c5p2*ss[18];
  jac_4_0 += 20*c4p3*c5p1*ss[17];
  jac_4_0 += 5*c1p4*ss[4];
  jac_4_0 += 5*c2p4*ss[8];
  jac_4_0 += 5*c3p4*ss[12];
  jac_4_0 += 5*c4p4*ss[16];
  jac_4_0 += 5*c5p4*ss[20];
  jac_4_0 += 5*c0p4;
  TACC jac_4_1 = convert<TACC>(120*c0p1*c1p1*c2p1*c3p1*ss[7]);
  jac_4_1 += 120*c0p1*c1p1*c2p1*c4p1*ss[8];
  jac_4_1 += 120*c0p1*c1p1*c2p1*c5p1*ss[9];
  jac_4_1 += 120*c0p1*c1p1*c3p1*c4p1*ss[9];
  jac_4_1 += 120*c0p1*c1p1*c3p1*c5p1*ss[10];
  jac_4_1 += 120*c0p1*c1p1*c4p1*c5p1*ss[11];
  jac_4_1 += 120*c0p1*c2p1*c3p1*c4p1*ss[10];
  jac_4_1 += 120*c0p1*c2p1*c3p1*c5p1*ss[11];
  jac_4_1 += 120*c0p1*c2p1*c4p1*c5p1*ss[12];
  jac_4_1 += 120*c0p1*c3p1*c4p1*c5p1*ss[13];
  jac_4_1 += 120*c1p1*c2p1*c3p1*c4p1*ss[11];
  jac_4_1 += 120*c1p1*c2p1*c3p1*c5p1*ss[12];
  jac_4_1 += 120*c1p1*c2p1*c4p1*c5p1*ss[13];
  jac_4_1 += 120*c1p1*c3p1*c4p1*c5p1*ss[14];
  jac_4_1 += 120*c2p1*c3p1*c4p1*c5p1*ss[15];
  jac_4_1 += 60*c0p1*c1p1*c2p2*ss[6];
  jac_4_1 += 60*c0p1*c1p1*c3p2*ss[8];
  jac_4_1 += 60*c0p1*c1p1*c4p2*ss[10];
  jac_4_1 += 60*c0p1*c1p1*c5p2*ss[12];
  jac_4_1 += 60*c0p1*c1p2*c2p1*ss[5];
  jac_4_1 += 60*c0p1*c1p2*c3p1*ss[6];
  jac_4_1 += 60*c0p1*c1p2*c4p1*ss[7];
  jac_4_1 += 60*c0p1*c1p2*c5p1*ss[8];
  jac_4_1 += 60*c0p1*c2p1*c3p2*ss[9];
  jac_4_1 += 60*c0p1*c2p1*c4p2*ss[11];
  jac_4_1 += 60*c0p1*c2p1*c5p2*ss[13];
  jac_4_1 += 60*c0p1*c2p2*c3p1*ss[8];
  jac_4_1 += 60*c0p1*c2p2*c4p1*ss[9];
  jac_4_1 += 60*c0p1*c2p2*c5p1*ss[10];
  jac_4_1 += 60*c0p1*c3p1*c4p2*ss[12];
  jac_4_1 += 60*c0p1*c3p1*c5p2*ss[14];
  jac_4_1 += 60*c0p1*c3p2*c4p1*ss[11];
  jac_4_1 += 60*c0p1*c3p2*c5p1*ss[12];
  jac_4_1 += 60*c0p1*c4p1*c5p2*ss[15];
  jac_4_1 += 60*c0p1*c4p2*c5p1*ss[14];
  jac_4_1 += 60*c0p2*c1p1*c2p1*ss[4];
  jac_4_1 += 60*c0p2*c1p1*c3p1*ss[5];
  jac_4_1 += 60*c0p2*c1p1*c4p1*ss[6];
  jac_4_1 += 60*c0p2*c1p1*c5p1*ss[7];
  jac_4_1 += 60*c0p2*c2p1*c3p1*ss[6];
  jac_4_1 += 60*c0p2*c2p1*c4p1*ss[7];
  jac_4_1 += 60*c0p2*c2p1*c5p1*ss[8];
  jac_4_1 += 60*c0p2*c3p1*c4p1*ss[8];
  jac_4_1 += 60*c0p2*c3p1*c5p1*ss[9];
  jac_4_1 += 60*c0p2*c4p1*c5p1*ss[10];
  jac_4_1 += 60*c1p1*c2p1*c3p2*ss[10];
  jac_4_1 += 60*c1p1*c2p1*c4p2*ss[12];
  jac_4_1 += 60*c1p1*c2p1*c5p2*ss[14];
  jac_4_1 += 60*c1p1*c2p2*c3p1*ss[9];
  jac_4_1 += 60*c1p1*c2p2*c4p1*ss[10];
  jac_4_1 += 60*c1p1*c2p2*c5p1*ss[11];
  jac_4_1 += 60*c1p1*c3p1*c4p2*ss[13];
  jac_4_1 += 60*c1p1*c3p1*c5p2*ss[15];
  jac_4_1 += 60*c1p1*c3p2*c4p1*ss[12];
  jac_4_1 += 60*c1p1*c3p2*c5p1*ss[13];
  jac_4_1 += 60*c1p1*c4p1*c5p2*ss[16];
  jac_4_1 += 60*c1p1*c4p2*c5p1*ss[15];
  jac_4_1 += 60*c1p2*c2p1*c3p1*ss[8];
  jac_4_1 += 60*c1p2*c2p1*c4p1*ss[9];
  jac_4_1 += 60*c1p2*c2p1*c5p1*ss[10];
  jac_4_1 += 60*c1p2*c3p1*c4p1*ss[10];
  jac_4_1 += 60*c1p2*c3p1*c5p1*ss[11];
  jac_4_1 += 60*c1p2*c4p1*c5p1*ss[12];
  jac_4_1 += 60*c2p1*c3p1*c4p2*ss[14];
  jac_4_1 += 60*c2p1*c3p1*c5p2*ss[16];
  jac_4_1 += 60*c2p1*c3p2*c4p1*ss[13];
  jac_4_1 += 60*c2p1*c3p2*c5p1*ss[14];
  jac_4_1 += 60*c2p1*c4p1*c5p2*ss[17];
  jac_4_1 += 60*c2p1*c4p2*c5p1*ss[16];
  jac_4_1 += 60*c2p2*c3p1*c4p1*ss[12];
  jac_4_1 += 60*c2p2*c3p1*c5p1*ss[13];
  jac_4_1 += 60*c2p2*c4p1*c5p1*ss[14];
  jac_4_1 += 60*c3p1*c4p1*c5p2*ss[18];
  jac_4_1 += 60*c3p1*c4p2*c5p1*ss[17];
  jac_4_1 += 60*c3p2*c4p1*c5p1*ss[16];
  jac_4_1 += 20*c0p1*c1p3*ss[4];
  jac_4_1 += 20*c0p1*c2p3*ss[7];
  jac_4_1 += 20*c0p1*c3p3*ss[10];
  jac_4_1 += 20*c0p1*c4p3*ss[13];
  jac_4_1 += 20*c0p1*c5p3*ss[16];
  jac_4_1 += 30*c0p2*c1p2*ss[3];
  jac_4_1 += 30*c0p2*c2p2*ss[5];
  jac_4_1 += 30*c0p2*c3p2*ss[7];
  jac_4_1 += 30*c0p2*c4p2*ss[9];
  jac_4_1 += 30*c0p2*c5p2*ss[11];
  jac_4_1 += 20*c0p3*c1p1*ss[2];
  jac_4_1 += 20*c0p3*c2p1*ss[3];
  jac_4_1 += 20*c0p3*c3p1*ss[4];
  jac_4_1 += 20*c0p3*c4p1*ss[5];
  jac_4_1 += 20*c0p3*c5p1*ss[6];
  jac_4_1 += 20*c1p1*c2p3*ss[8];
  jac_4_1 += 20*c1p1*c3p3*ss[11];
  jac_4_1 += 20*c1p1*c4p3*ss[14];
  jac_4_1 += 20*c1p1*c5p3*ss[17];
  jac_4_1 += 30*c1p2*c2p2*ss[7];
  jac_4_1 += 30*c1p2*c3p2*ss[9];
  jac_4_1 += 30*c1p2*c4p2*ss[11];
  jac_4_1 += 30*c1p2*c5p2*ss[13];
  jac_4_1 += 20*c1p3*c2p1*ss[6];
  jac_4_1 += 20*c1p3*c3p1*ss[7];
  jac_4_1 += 20*c1p3*c4p1*ss[8];
  jac_4_1 += 20*c1p3*c5p1*ss[9];
  jac_4_1 += 20*c2p1*c3p3*ss[12];
  jac_4_1 += 20*c2p1*c4p3*ss[15];
  jac_4_1 += 20*c2p1*c5p3*ss[18];
  jac_4_1 += 30*c2p2*c3p2*ss[11];
  jac_4_1 += 30*c2p2*c4p2*ss[13];
  jac_4_1 += 30*c2p2*c5p2*ss[15];
  jac_4_1 += 20*c2p3*c3p1*ss[10];
  jac_4_1 += 20*c2p3*c4p1*ss[11];
  jac_4_1 += 20*c2p3*c5p1*ss[12];
  jac_4_1 += 20*c3p1*c4p3*ss[16];
  jac_4_1 += 20*c3p1*c5p3*ss[19];
  jac_4_1 += 30*c3p2*c4p2*ss[15];
  jac_4_1 += 30*c3p2*c5p2*ss[17];
  jac_4_1 += 20*c3p3*c4p1*ss[14];
  jac_4_1 += 20*c3p3*c5p1*ss[15];
  jac_4_1 += 20*c4p1*c5p3*ss[20];
  jac_4_1 += 30*c4p2*c5p2*ss[19];
  jac_4_1 += 20*c4p3*c5p1*ss[18];
  jac_4_1 += 5*c0p4*ss[1];
  jac_4_1 += 5*c1p4*ss[5];
  jac_4_1 += 5*c2p4*ss[9];
  jac_4_1 += 5*c3p4*ss[13];
  jac_4_1 += 5*c4p4*ss[17];
  jac_4_1 += 5*c5p4*ss[21];
  TACC jac_4_2 = convert<TACC>(120*c0p1*c1p1*c2p1*c3p1*ss[8]);
  jac_4_2 += 120*c0p1*c1p1*c2p1*c4p1*ss[9];
  jac_4_2 += 120*c0p1*c1p1*c2p1*c5p1*ss[10];
  jac_4_2 += 120*c0p1*c1p1*c3p1*c4p1*ss[10];
  jac_4_2 += 120*c0p1*c1p1*c3p1*c5p1*ss[11];
  jac_4_2 += 120*c0p1*c1p1*c4p1*c5p1*ss[12];
  jac_4_2 += 120*c0p1*c2p1*c3p1*c4p1*ss[11];
  jac_4_2 += 120*c0p1*c2p1*c3p1*c5p1*ss[12];
  jac_4_2 += 120*c0p1*c2p1*c4p1*c5p1*ss[13];
  jac_4_2 += 120*c0p1*c3p1*c4p1*c5p1*ss[14];
  jac_4_2 += 120*c1p1*c2p1*c3p1*c4p1*ss[12];
  jac_4_2 += 120*c1p1*c2p1*c3p1*c5p1*ss[13];
  jac_4_2 += 120*c1p1*c2p1*c4p1*c5p1*ss[14];
  jac_4_2 += 120*c1p1*c3p1*c4p1*c5p1*ss[15];
  jac_4_2 += 120*c2p1*c3p1*c4p1*c5p1*ss[16];
  jac_4_2 += 60*c0p1*c1p1*c2p2*ss[7];
  jac_4_2 += 60*c0p1*c1p1*c3p2*ss[9];
  jac_4_2 += 60*c0p1*c1p1*c4p2*ss[11];
  jac_4_2 += 60*c0p1*c1p1*c5p2*ss[13];
  jac_4_2 += 60*c0p1*c1p2*c2p1*ss[6];
  jac_4_2 += 60*c0p1*c1p2*c3p1*ss[7];
  jac_4_2 += 60*c0p1*c1p2*c4p1*ss[8];
  jac_4_2 += 60*c0p1*c1p2*c5p1*ss[9];
  jac_4_2 += 60*c0p1*c2p1*c3p2*ss[10];
  jac_4_2 += 60*c0p1*c2p1*c4p2*ss[12];
  jac_4_2 += 60*c0p1*c2p1*c5p2*ss[14];
  jac_4_2 += 60*c0p1*c2p2*c3p1*ss[9];
  jac_4_2 += 60*c0p1*c2p2*c4p1*ss[10];
  jac_4_2 += 60*c0p1*c2p2*c5p1*ss[11];
  jac_4_2 += 60*c0p1*c3p1*c4p2*ss[13];
  jac_4_2 += 60*c0p1*c3p1*c5p2*ss[15];
  jac_4_2 += 60*c0p1*c3p2*c4p1*ss[12];
  jac_4_2 += 60*c0p1*c3p2*c5p1*ss[13];
  jac_4_2 += 60*c0p1*c4p1*c5p2*ss[16];
  jac_4_2 += 60*c0p1*c4p2*c5p1*ss[15];
  jac_4_2 += 60*c0p2*c1p1*c2p1*ss[5];
  jac_4_2 += 60*c0p2*c1p1*c3p1*ss[6];
  jac_4_2 += 60*c0p2*c1p1*c4p1*ss[7];
  jac_4_2 += 60*c0p2*c1p1*c5p1*ss[8];
  jac_4_2 += 60*c0p2*c2p1*c3p1*ss[7];
  jac_4_2 += 60*c0p2*c2p1*c4p1*ss[8];
  jac_4_2 += 60*c0p2*c2p1*c5p1*ss[9];
  jac_4_2 += 60*c0p2*c3p1*c4p1*ss[9];
  jac_4_2 += 60*c0p2*c3p1*c5p1*ss[10];
  jac_4_2 += 60*c0p2*c4p1*c5p1*ss[11];
  jac_4_2 += 60*c1p1*c2p1*c3p2*ss[11];
  jac_4_2 += 60*c1p1*c2p1*c4p2*ss[13];
  jac_4_2 += 60*c1p1*c2p1*c5p2*ss[15];
  jac_4_2 += 60*c1p1*c2p2*c3p1*ss[10];
  jac_4_2 += 60*c1p1*c2p2*c4p1*ss[11];
  jac_4_2 += 60*c1p1*c2p2*c5p1*ss[12];
  jac_4_2 += 60*c1p1*c3p1*c4p2*ss[14];
  jac_4_2 += 60*c1p1*c3p1*c5p2*ss[16];
  jac_4_2 += 60*c1p1*c3p2*c4p1*ss[13];
  jac_4_2 += 60*c1p1*c3p2*c5p1*ss[14];
  jac_4_2 += 60*c1p1*c4p1*c5p2*ss[17];
  jac_4_2 += 60*c1p1*c4p2*c5p1*ss[16];
  jac_4_2 += 60*c1p2*c2p1*c3p1*ss[9];
  jac_4_2 += 60*c1p2*c2p1*c4p1*ss[10];
  jac_4_2 += 60*c1p2*c2p1*c5p1*ss[11];
  jac_4_2 += 60*c1p2*c3p1*c4p1*ss[11];
  jac_4_2 += 60*c1p2*c3p1*c5p1*ss[12];
  jac_4_2 += 60*c1p2*c4p1*c5p1*ss[13];
  jac_4_2 += 60*c2p1*c3p1*c4p2*ss[15];
  jac_4_2 += 60*c2p1*c3p1*c5p2*ss[17];
  jac_4_2 += 60*c2p1*c3p2*c4p1*ss[14];
  jac_4_2 += 60*c2p1*c3p2*c5p1*ss[15];
  jac_4_2 += 60*c2p1*c4p1*c5p2*ss[18];
  jac_4_2 += 60*c2p1*c4p2*c5p1*ss[17];
  jac_4_2 += 60*c2p2*c3p1*c4p1*ss[13];
  jac_4_2 += 60*c2p2*c3p1*c5p1*ss[14];
  jac_4_2 += 60*c2p2*c4p1*c5p1*ss[15];
  jac_4_2 += 60*c3p1*c4p1*c5p2*ss[19];
  jac_4_2 += 60*c3p1*c4p2*c5p1*ss[18];
  jac_4_2 += 60*c3p2*c4p1*c5p1*ss[17];
  jac_4_2 += 20*c0p1*c1p3*ss[5];
  jac_4_2 += 20*c0p1*c2p3*ss[8];
  jac_4_2 += 20*c0p1*c3p3*ss[11];
  jac_4_2 += 20*c0p1*c4p3*ss[14];
  jac_4_2 += 20*c0p1*c5p3*ss[17];
  jac_4_2 += 30*c0p2*c1p2*ss[4];
  jac_4_2 += 30*c0p2*c2p2*ss[6];
  jac_4_2 += 30*c0p2*c3p2*ss[8];
  jac_4_2 += 30*c0p2*c4p2*ss[10];
  jac_4_2 += 30*c0p2*c5p2*ss[12];
  jac_4_2 += 20*c0p3*c1p1*ss[3];
  jac_4_2 += 20*c0p3*c2p1*ss[4];
  jac_4_2 += 20*c0p3*c3p1*ss[5];
  jac_4_2 += 20*c0p3*c4p1*ss[6];
  jac_4_2 += 20*c0p3*c5p1*ss[7];
  jac_4_2 += 20*c1p1*c2p3*ss[9];
  jac_4_2 += 20*c1p1*c3p3*ss[12];
  jac_4_2 += 20*c1p1*c4p3*ss[15];
  jac_4_2 += 20*c1p1*c5p3*ss[18];
  jac_4_2 += 30*c1p2*c2p2*ss[8];
  jac_4_2 += 30*c1p2*c3p2*ss[10];
  jac_4_2 += 30*c1p2*c4p2*ss[12];
  jac_4_2 += 30*c1p2*c5p2*ss[14];
  jac_4_2 += 20*c1p3*c2p1*ss[7];
  jac_4_2 += 20*c1p3*c3p1*ss[8];
  jac_4_2 += 20*c1p3*c4p1*ss[9];
  jac_4_2 += 20*c1p3*c5p1*ss[10];
  jac_4_2 += 20*c2p1*c3p3*ss[13];
  jac_4_2 += 20*c2p1*c4p3*ss[16];
  jac_4_2 += 20*c2p1*c5p3*ss[19];
  jac_4_2 += 30*c2p2*c3p2*ss[12];
  jac_4_2 += 30*c2p2*c4p2*ss[14];
  jac_4_2 += 30*c2p2*c5p2*ss[16];
  jac_4_2 += 20*c2p3*c3p1*ss[11];
  jac_4_2 += 20*c2p3*c4p1*ss[12];
  jac_4_2 += 20*c2p3*c5p1*ss[13];
  jac_4_2 += 20*c3p1*c4p3*ss[17];
  jac_4_2 += 20*c3p1*c5p3*ss[20];
  jac_4_2 += 30*c3p2*c4p2*ss[16];
  jac_4_2 += 30*c3p2*c5p2*ss[18];
  jac_4_2 += 20*c3p3*c4p1*ss[15];
  jac_4_2 += 20*c3p3*c5p1*ss[16];
  jac_4_2 += 20*c4p1*c5p3*ss[21];
  jac_4_2 += 30*c4p2*c5p2*ss[20];
  jac_4_2 += 20*c4p3*c5p1*ss[19];
  jac_4_2 += 5*c0p4*ss[2];
  jac_4_2 += 5*c1p4*ss[6];
  jac_4_2 += 5*c2p4*ss[10];
  jac_4_2 += 5*c3p4*ss[14];
  jac_4_2 += 5*c4p4*ss[18];
  jac_4_2 += 5*c5p4*ss[22];
  TACC jac_4_3 = convert<TACC>(120*c0p1*c1p1*c2p1*c3p1*ss[9]);
  jac_4_3 += 120*c0p1*c1p1*c2p1*c4p1*ss[10];
  jac_4_3 += 120*c0p1*c1p1*c2p1*c5p1*ss[11];
  jac_4_3 += 120*c0p1*c1p1*c3p1*c4p1*ss[11];
  jac_4_3 += 120*c0p1*c1p1*c3p1*c5p1*ss[12];
  jac_4_3 += 120*c0p1*c1p1*c4p1*c5p1*ss[13];
  jac_4_3 += 120*c0p1*c2p1*c3p1*c4p1*ss[12];
  jac_4_3 += 120*c0p1*c2p1*c3p1*c5p1*ss[13];
  jac_4_3 += 120*c0p1*c2p1*c4p1*c5p1*ss[14];
  jac_4_3 += 120*c0p1*c3p1*c4p1*c5p1*ss[15];
  jac_4_3 += 120*c1p1*c2p1*c3p1*c4p1*ss[13];
  jac_4_3 += 120*c1p1*c2p1*c3p1*c5p1*ss[14];
  jac_4_3 += 120*c1p1*c2p1*c4p1*c5p1*ss[15];
  jac_4_3 += 120*c1p1*c3p1*c4p1*c5p1*ss[16];
  jac_4_3 += 120*c2p1*c3p1*c4p1*c5p1*ss[17];
  jac_4_3 += 60*c0p1*c1p1*c2p2*ss[8];
  jac_4_3 += 60*c0p1*c1p1*c3p2*ss[10];
  jac_4_3 += 60*c0p1*c1p1*c4p2*ss[12];
  jac_4_3 += 60*c0p1*c1p1*c5p2*ss[14];
  jac_4_3 += 60*c0p1*c1p2*c2p1*ss[7];
  jac_4_3 += 60*c0p1*c1p2*c3p1*ss[8];
  jac_4_3 += 60*c0p1*c1p2*c4p1*ss[9];
  jac_4_3 += 60*c0p1*c1p2*c5p1*ss[10];
  jac_4_3 += 60*c0p1*c2p1*c3p2*ss[11];
  jac_4_3 += 60*c0p1*c2p1*c4p2*ss[13];
  jac_4_3 += 60*c0p1*c2p1*c5p2*ss[15];
  jac_4_3 += 60*c0p1*c2p2*c3p1*ss[10];
  jac_4_3 += 60*c0p1*c2p2*c4p1*ss[11];
  jac_4_3 += 60*c0p1*c2p2*c5p1*ss[12];
  jac_4_3 += 60*c0p1*c3p1*c4p2*ss[14];
  jac_4_3 += 60*c0p1*c3p1*c5p2*ss[16];
  jac_4_3 += 60*c0p1*c3p2*c4p1*ss[13];
  jac_4_3 += 60*c0p1*c3p2*c5p1*ss[14];
  jac_4_3 += 60*c0p1*c4p1*c5p2*ss[17];
  jac_4_3 += 60*c0p1*c4p2*c5p1*ss[16];
  jac_4_3 += 60*c0p2*c1p1*c2p1*ss[6];
  jac_4_3 += 60*c0p2*c1p1*c3p1*ss[7];
  jac_4_3 += 60*c0p2*c1p1*c4p1*ss[8];
  jac_4_3 += 60*c0p2*c1p1*c5p1*ss[9];
  jac_4_3 += 60*c0p2*c2p1*c3p1*ss[8];
  jac_4_3 += 60*c0p2*c2p1*c4p1*ss[9];
  jac_4_3 += 60*c0p2*c2p1*c5p1*ss[10];
  jac_4_3 += 60*c0p2*c3p1*c4p1*ss[10];
  jac_4_3 += 60*c0p2*c3p1*c5p1*ss[11];
  jac_4_3 += 60*c0p2*c4p1*c5p1*ss[12];
  jac_4_3 += 60*c1p1*c2p1*c3p2*ss[12];
  jac_4_3 += 60*c1p1*c2p1*c4p2*ss[14];
  jac_4_3 += 60*c1p1*c2p1*c5p2*ss[16];
  jac_4_3 += 60*c1p1*c2p2*c3p1*ss[11];
  jac_4_3 += 60*c1p1*c2p2*c4p1*ss[12];
  jac_4_3 += 60*c1p1*c2p2*c5p1*ss[13];
  jac_4_3 += 60*c1p1*c3p1*c4p2*ss[15];
  jac_4_3 += 60*c1p1*c3p1*c5p2*ss[17];
  jac_4_3 += 60*c1p1*c3p2*c4p1*ss[14];
  jac_4_3 += 60*c1p1*c3p2*c5p1*ss[15];
  jac_4_3 += 60*c1p1*c4p1*c5p2*ss[18];
  jac_4_3 += 60*c1p1*c4p2*c5p1*ss[17];
  jac_4_3 += 60*c1p2*c2p1*c3p1*ss[10];
  jac_4_3 += 60*c1p2*c2p1*c4p1*ss[11];
  jac_4_3 += 60*c1p2*c2p1*c5p1*ss[12];
  jac_4_3 += 60*c1p2*c3p1*c4p1*ss[12];
  jac_4_3 += 60*c1p2*c3p1*c5p1*ss[13];
  jac_4_3 += 60*c1p2*c4p1*c5p1*ss[14];
  jac_4_3 += 60*c2p1*c3p1*c4p2*ss[16];
  jac_4_3 += 60*c2p1*c3p1*c5p2*ss[18];
  jac_4_3 += 60*c2p1*c3p2*c4p1*ss[15];
  jac_4_3 += 60*c2p1*c3p2*c5p1*ss[16];
  jac_4_3 += 60*c2p1*c4p1*c5p2*ss[19];
  jac_4_3 += 60*c2p1*c4p2*c5p1*ss[18];
  jac_4_3 += 60*c2p2*c3p1*c4p1*ss[14];
  jac_4_3 += 60*c2p2*c3p1*c5p1*ss[15];
  jac_4_3 += 60*c2p2*c4p1*c5p1*ss[16];
  jac_4_3 += 60*c3p1*c4p1*c5p2*ss[20];
  jac_4_3 += 60*c3p1*c4p2*c5p1*ss[19];
  jac_4_3 += 60*c3p2*c4p1*c5p1*ss[18];
  jac_4_3 += 20*c0p1*c1p3*ss[6];
  jac_4_3 += 20*c0p1*c2p3*ss[9];
  jac_4_3 += 20*c0p1*c3p3*ss[12];
  jac_4_3 += 20*c0p1*c4p3*ss[15];
  jac_4_3 += 20*c0p1*c5p3*ss[18];
  jac_4_3 += 30*c0p2*c1p2*ss[5];
  jac_4_3 += 30*c0p2*c2p2*ss[7];
  jac_4_3 += 30*c0p2*c3p2*ss[9];
  jac_4_3 += 30*c0p2*c4p2*ss[11];
  jac_4_3 += 30*c0p2*c5p2*ss[13];
  jac_4_3 += 20*c0p3*c1p1*ss[4];
  jac_4_3 += 20*c0p3*c2p1*ss[5];
  jac_4_3 += 20*c0p3*c3p1*ss[6];
  jac_4_3 += 20*c0p3*c4p1*ss[7];
  jac_4_3 += 20*c0p3*c5p1*ss[8];
  jac_4_3 += 20*c1p1*c2p3*ss[10];
  jac_4_3 += 20*c1p1*c3p3*ss[13];
  jac_4_3 += 20*c1p1*c4p3*ss[16];
  jac_4_3 += 20*c1p1*c5p3*ss[19];
  jac_4_3 += 30*c1p2*c2p2*ss[9];
  jac_4_3 += 30*c1p2*c3p2*ss[11];
  jac_4_3 += 30*c1p2*c4p2*ss[13];
  jac_4_3 += 30*c1p2*c5p2*ss[15];
  jac_4_3 += 20*c1p3*c2p1*ss[8];
  jac_4_3 += 20*c1p3*c3p1*ss[9];
  jac_4_3 += 20*c1p3*c4p1*ss[10];
  jac_4_3 += 20*c1p3*c5p1*ss[11];
  jac_4_3 += 20*c2p1*c3p3*ss[14];
  jac_4_3 += 20*c2p1*c4p3*ss[17];
  jac_4_3 += 20*c2p1*c5p3*ss[20];
  jac_4_3 += 30*c2p2*c3p2*ss[13];
  jac_4_3 += 30*c2p2*c4p2*ss[15];
  jac_4_3 += 30*c2p2*c5p2*ss[17];
  jac_4_3 += 20*c2p3*c3p1*ss[12];
  jac_4_3 += 20*c2p3*c4p1*ss[13];
  jac_4_3 += 20*c2p3*c5p1*ss[14];
  jac_4_3 += 20*c3p1*c4p3*ss[18];
  jac_4_3 += 20*c3p1*c5p3*ss[21];
  jac_4_3 += 30*c3p2*c4p2*ss[17];
  jac_4_3 += 30*c3p2*c5p2*ss[19];
  jac_4_3 += 20*c3p3*c4p1*ss[16];
  jac_4_3 += 20*c3p3*c5p1*ss[17];
  jac_4_3 += 20*c4p1*c5p3*ss[22];
  jac_4_3 += 30*c4p2*c5p2*ss[21];
  jac_4_3 += 20*c4p3*c5p1*ss[20];
  jac_4_3 += 5*c0p4*ss[3];
  jac_4_3 += 5*c1p4*ss[7];
  jac_4_3 += 5*c2p4*ss[11];
  jac_4_3 += 5*c3p4*ss[15];
  jac_4_3 += 5*c4p4*ss[19];
  jac_4_3 += 5*c5p4*ss[23];
  TACC jac_4_4 = convert<TACC>(120*c0p1*c1p1*c2p1*c3p1*ss[10]);
  jac_4_4 += 120*c0p1*c1p1*c2p1*c4p1*ss[11];
  jac_4_4 += 120*c0p1*c1p1*c2p1*c5p1*ss[12];
  jac_4_4 += 120*c0p1*c1p1*c3p1*c4p1*ss[12];
  jac_4_4 += 120*c0p1*c1p1*c3p1*c5p1*ss[13];
  jac_4_4 += 120*c0p1*c1p1*c4p1*c5p1*ss[14];
  jac_4_4 += 120*c0p1*c2p1*c3p1*c4p1*ss[13];
  jac_4_4 += 120*c0p1*c2p1*c3p1*c5p1*ss[14];
  jac_4_4 += 120*c0p1*c2p1*c4p1*c5p1*ss[15];
  jac_4_4 += 120*c0p1*c3p1*c4p1*c5p1*ss[16];
  jac_4_4 += 120*c1p1*c2p1*c3p1*c4p1*ss[14];
  jac_4_4 += 120*c1p1*c2p1*c3p1*c5p1*ss[15];
  jac_4_4 += 120*c1p1*c2p1*c4p1*c5p1*ss[16];
  jac_4_4 += 120*c1p1*c3p1*c4p1*c5p1*ss[17];
  jac_4_4 += 120*c2p1*c3p1*c4p1*c5p1*ss[18];
  jac_4_4 += 60*c0p1*c1p1*c2p2*ss[9];
  jac_4_4 += 60*c0p1*c1p1*c3p2*ss[11];
  jac_4_4 += 60*c0p1*c1p1*c4p2*ss[13];
  jac_4_4 += 60*c0p1*c1p1*c5p2*ss[15];
  jac_4_4 += 60*c0p1*c1p2*c2p1*ss[8];
  jac_4_4 += 60*c0p1*c1p2*c3p1*ss[9];
  jac_4_4 += 60*c0p1*c1p2*c4p1*ss[10];
  jac_4_4 += 60*c0p1*c1p2*c5p1*ss[11];
  jac_4_4 += 60*c0p1*c2p1*c3p2*ss[12];
  jac_4_4 += 60*c0p1*c2p1*c4p2*ss[14];
  jac_4_4 += 60*c0p1*c2p1*c5p2*ss[16];
  jac_4_4 += 60*c0p1*c2p2*c3p1*ss[11];
  jac_4_4 += 60*c0p1*c2p2*c4p1*ss[12];
  jac_4_4 += 60*c0p1*c2p2*c5p1*ss[13];
  jac_4_4 += 60*c0p1*c3p1*c4p2*ss[15];
  jac_4_4 += 60*c0p1*c3p1*c5p2*ss[17];
  jac_4_4 += 60*c0p1*c3p2*c4p1*ss[14];
  jac_4_4 += 60*c0p1*c3p2*c5p1*ss[15];
  jac_4_4 += 60*c0p1*c4p1*c5p2*ss[18];
  jac_4_4 += 60*c0p1*c4p2*c5p1*ss[17];
  jac_4_4 += 60*c0p2*c1p1*c2p1*ss[7];
  jac_4_4 += 60*c0p2*c1p1*c3p1*ss[8];
  jac_4_4 += 60*c0p2*c1p1*c4p1*ss[9];
  jac_4_4 += 60*c0p2*c1p1*c5p1*ss[10];
  jac_4_4 += 60*c0p2*c2p1*c3p1*ss[9];
  jac_4_4 += 60*c0p2*c2p1*c4p1*ss[10];
  jac_4_4 += 60*c0p2*c2p1*c5p1*ss[11];
  jac_4_4 += 60*c0p2*c3p1*c4p1*ss[11];
  jac_4_4 += 60*c0p2*c3p1*c5p1*ss[12];
  jac_4_4 += 60*c0p2*c4p1*c5p1*ss[13];
  jac_4_4 += 60*c1p1*c2p1*c3p2*ss[13];
  jac_4_4 += 60*c1p1*c2p1*c4p2*ss[15];
  jac_4_4 += 60*c1p1*c2p1*c5p2*ss[17];
  jac_4_4 += 60*c1p1*c2p2*c3p1*ss[12];
  jac_4_4 += 60*c1p1*c2p2*c4p1*ss[13];
  jac_4_4 += 60*c1p1*c2p2*c5p1*ss[14];
  jac_4_4 += 60*c1p1*c3p1*c4p2*ss[16];
  jac_4_4 += 60*c1p1*c3p1*c5p2*ss[18];
  jac_4_4 += 60*c1p1*c3p2*c4p1*ss[15];
  jac_4_4 += 60*c1p1*c3p2*c5p1*ss[16];
  jac_4_4 += 60*c1p1*c4p1*c5p2*ss[19];
  jac_4_4 += 60*c1p1*c4p2*c5p1*ss[18];
  jac_4_4 += 60*c1p2*c2p1*c3p1*ss[11];
  jac_4_4 += 60*c1p2*c2p1*c4p1*ss[12];
  jac_4_4 += 60*c1p2*c2p1*c5p1*ss[13];
  jac_4_4 += 60*c1p2*c3p1*c4p1*ss[13];
  jac_4_4 += 60*c1p2*c3p1*c5p1*ss[14];
  jac_4_4 += 60*c1p2*c4p1*c5p1*ss[15];
  jac_4_4 += 60*c2p1*c3p1*c4p2*ss[17];
  jac_4_4 += 60*c2p1*c3p1*c5p2*ss[19];
  jac_4_4 += 60*c2p1*c3p2*c4p1*ss[16];
  jac_4_4 += 60*c2p1*c3p2*c5p1*ss[17];
  jac_4_4 += 60*c2p1*c4p1*c5p2*ss[20];
  jac_4_4 += 60*c2p1*c4p2*c5p1*ss[19];
  jac_4_4 += 60*c2p2*c3p1*c4p1*ss[15];
  jac_4_4 += 60*c2p2*c3p1*c5p1*ss[16];
  jac_4_4 += 60*c2p2*c4p1*c5p1*ss[17];
  jac_4_4 += 60*c3p1*c4p1*c5p2*ss[21];
  jac_4_4 += 60*c3p1*c4p2*c5p1*ss[20];
  jac_4_4 += 60*c3p2*c4p1*c5p1*ss[19];
  jac_4_4 += 20*c0p1*c1p3*ss[7];
  jac_4_4 += 20*c0p1*c2p3*ss[10];
  jac_4_4 += 20*c0p1*c3p3*ss[13];
  jac_4_4 += 20*c0p1*c4p3*ss[16];
  jac_4_4 += 20*c0p1*c5p3*ss[19];
  jac_4_4 += 30*c0p2*c1p2*ss[6];
  jac_4_4 += 30*c0p2*c2p2*ss[8];
  jac_4_4 += 30*c0p2*c3p2*ss[10];
  jac_4_4 += 30*c0p2*c4p2*ss[12];
  jac_4_4 += 30*c0p2*c5p2*ss[14];
  jac_4_4 += 20*c0p3*c1p1*ss[5];
  jac_4_4 += 20*c0p3*c2p1*ss[6];
  jac_4_4 += 20*c0p3*c3p1*ss[7];
  jac_4_4 += 20*c0p3*c4p1*ss[8];
  jac_4_4 += 20*c0p3*c5p1*ss[9];
  jac_4_4 += 20*c1p1*c2p3*ss[11];
  jac_4_4 += 20*c1p1*c3p3*ss[14];
  jac_4_4 += 20*c1p1*c4p3*ss[17];
  jac_4_4 += 20*c1p1*c5p3*ss[20];
  jac_4_4 += 30*c1p2*c2p2*ss[10];
  jac_4_4 += 30*c1p2*c3p2*ss[12];
  jac_4_4 += 30*c1p2*c4p2*ss[14];
  jac_4_4 += 30*c1p2*c5p2*ss[16];
  jac_4_4 += 20*c1p3*c2p1*ss[9];
  jac_4_4 += 20*c1p3*c3p1*ss[10];
  jac_4_4 += 20*c1p3*c4p1*ss[11];
  jac_4_4 += 20*c1p3*c5p1*ss[12];
  jac_4_4 += 20*c2p1*c3p3*ss[15];
  jac_4_4 += 20*c2p1*c4p3*ss[18];
  jac_4_4 += 20*c2p1*c5p3*ss[21];
  jac_4_4 += 30*c2p2*c3p2*ss[14];
  jac_4_4 += 30*c2p2*c4p2*ss[16];
  jac_4_4 += 30*c2p2*c5p2*ss[18];
  jac_4_4 += 20*c2p3*c3p1*ss[13];
  jac_4_4 += 20*c2p3*c4p1*ss[14];
  jac_4_4 += 20*c2p3*c5p1*ss[15];
  jac_4_4 += 20*c3p1*c4p3*ss[19];
  jac_4_4 += 20*c3p1*c5p3*ss[22];
  jac_4_4 += 30*c3p2*c4p2*ss[18];
  jac_4_4 += 30*c3p2*c5p2*ss[20];
  jac_4_4 += 20*c3p3*c4p1*ss[17];
  jac_4_4 += 20*c3p3*c5p1*ss[18];
  jac_4_4 += 20*c4p1*c5p3*ss[23];
  jac_4_4 += 30*c4p2*c5p2*ss[22];
  jac_4_4 += 20*c4p3*c5p1*ss[21];
  jac_4_4 += 5*c0p4*ss[4];
  jac_4_4 += 5*c1p4*ss[8];
  jac_4_4 += 5*c2p4*ss[12];
  jac_4_4 += 5*c3p4*ss[16];
  jac_4_4 += 5*c4p4*ss[20];
  jac_4_4 += 5*c5p4*ss[24];
  TACC jac_4_5 = convert<TACC>(120*c0p1*c1p1*c2p1*c3p1*ss[11]);
  jac_4_5 += 120*c0p1*c1p1*c2p1*c4p1*ss[12];
  jac_4_5 += 120*c0p1*c1p1*c2p1*c5p1*ss[13];
  jac_4_5 += 120*c0p1*c1p1*c3p1*c4p1*ss[13];
  jac_4_5 += 120*c0p1*c1p1*c3p1*c5p1*ss[14];
  jac_4_5 += 120*c0p1*c1p1*c4p1*c5p1*ss[15];
  jac_4_5 += 120*c0p1*c2p1*c3p1*c4p1*ss[14];
  jac_4_5 += 120*c0p1*c2p1*c3p1*c5p1*ss[15];
  jac_4_5 += 120*c0p1*c2p1*c4p1*c5p1*ss[16];
  jac_4_5 += 120*c0p1*c3p1*c4p1*c5p1*ss[17];
  jac_4_5 += 120*c1p1*c2p1*c3p1*c4p1*ss[15];
  jac_4_5 += 120*c1p1*c2p1*c3p1*c5p1*ss[16];
  jac_4_5 += 120*c1p1*c2p1*c4p1*c5p1*ss[17];
  jac_4_5 += 120*c1p1*c3p1*c4p1*c5p1*ss[18];
  jac_4_5 += 120*c2p1*c3p1*c4p1*c5p1*ss[19];
  jac_4_5 += 60*c0p1*c1p1*c2p2*ss[10];
  jac_4_5 += 60*c0p1*c1p1*c3p2*ss[12];
  jac_4_5 += 60*c0p1*c1p1*c4p2*ss[14];
  jac_4_5 += 60*c0p1*c1p1*c5p2*ss[16];
  jac_4_5 += 60*c0p1*c1p2*c2p1*ss[9];
  jac_4_5 += 60*c0p1*c1p2*c3p1*ss[10];
  jac_4_5 += 60*c0p1*c1p2*c4p1*ss[11];
  jac_4_5 += 60*c0p1*c1p2*c5p1*ss[12];
  jac_4_5 += 60*c0p1*c2p1*c3p2*ss[13];
  jac_4_5 += 60*c0p1*c2p1*c4p2*ss[15];
  jac_4_5 += 60*c0p1*c2p1*c5p2*ss[17];
  jac_4_5 += 60*c0p1*c2p2*c3p1*ss[12];
  jac_4_5 += 60*c0p1*c2p2*c4p1*ss[13];
  jac_4_5 += 60*c0p1*c2p2*c5p1*ss[14];
  jac_4_5 += 60*c0p1*c3p1*c4p2*ss[16];
  jac_4_5 += 60*c0p1*c3p1*c5p2*ss[18];
  jac_4_5 += 60*c0p1*c3p2*c4p1*ss[15];
  jac_4_5 += 60*c0p1*c3p2*c5p1*ss[16];
  jac_4_5 += 60*c0p1*c4p1*c5p2*ss[19];
  jac_4_5 += 60*c0p1*c4p2*c5p1*ss[18];
  jac_4_5 += 60*c0p2*c1p1*c2p1*ss[8];
  jac_4_5 += 60*c0p2*c1p1*c3p1*ss[9];
  jac_4_5 += 60*c0p2*c1p1*c4p1*ss[10];
  jac_4_5 += 60*c0p2*c1p1*c5p1*ss[11];
  jac_4_5 += 60*c0p2*c2p1*c3p1*ss[10];
  jac_4_5 += 60*c0p2*c2p1*c4p1*ss[11];
  jac_4_5 += 60*c0p2*c2p1*c5p1*ss[12];
  jac_4_5 += 60*c0p2*c3p1*c4p1*ss[12];
  jac_4_5 += 60*c0p2*c3p1*c5p1*ss[13];
  jac_4_5 += 60*c0p2*c4p1*c5p1*ss[14];
  jac_4_5 += 60*c1p1*c2p1*c3p2*ss[14];
  jac_4_5 += 60*c1p1*c2p1*c4p2*ss[16];
  jac_4_5 += 60*c1p1*c2p1*c5p2*ss[18];
  jac_4_5 += 60*c1p1*c2p2*c3p1*ss[13];
  jac_4_5 += 60*c1p1*c2p2*c4p1*ss[14];
  jac_4_5 += 60*c1p1*c2p2*c5p1*ss[15];
  jac_4_5 += 60*c1p1*c3p1*c4p2*ss[17];
  jac_4_5 += 60*c1p1*c3p1*c5p2*ss[19];
  jac_4_5 += 60*c1p1*c3p2*c4p1*ss[16];
  jac_4_5 += 60*c1p1*c3p2*c5p1*ss[17];
  jac_4_5 += 60*c1p1*c4p1*c5p2*ss[20];
  jac_4_5 += 60*c1p1*c4p2*c5p1*ss[19];
  jac_4_5 += 60*c1p2*c2p1*c3p1*ss[12];
  jac_4_5 += 60*c1p2*c2p1*c4p1*ss[13];
  jac_4_5 += 60*c1p2*c2p1*c5p1*ss[14];
  jac_4_5 += 60*c1p2*c3p1*c4p1*ss[14];
  jac_4_5 += 60*c1p2*c3p1*c5p1*ss[15];
  jac_4_5 += 60*c1p2*c4p1*c5p1*ss[16];
  jac_4_5 += 60*c2p1*c3p1*c4p2*ss[18];
  jac_4_5 += 60*c2p1*c3p1*c5p2*ss[20];
  jac_4_5 += 60*c2p1*c3p2*c4p1*ss[17];
  jac_4_5 += 60*c2p1*c3p2*c5p1*ss[18];
  jac_4_5 += 60*c2p1*c4p1*c5p2*ss[21];
  jac_4_5 += 60*c2p1*c4p2*c5p1*ss[20];
  jac_4_5 += 60*c2p2*c3p1*c4p1*ss[16];
  jac_4_5 += 60*c2p2*c3p1*c5p1*ss[17];
  jac_4_5 += 60*c2p2*c4p1*c5p1*ss[18];
  jac_4_5 += 60*c3p1*c4p1*c5p2*ss[22];
  jac_4_5 += 60*c3p1*c4p2*c5p1*ss[21];
  jac_4_5 += 60*c3p2*c4p1*c5p1*ss[20];
  jac_4_5 += 20*c0p1*c1p3*ss[8];
  jac_4_5 += 20*c0p1*c2p3*ss[11];
  jac_4_5 += 20*c0p1*c3p3*ss[14];
  jac_4_5 += 20*c0p1*c4p3*ss[17];
  jac_4_5 += 20*c0p1*c5p3*ss[20];
  jac_4_5 += 30*c0p2*c1p2*ss[7];
  jac_4_5 += 30*c0p2*c2p2*ss[9];
  jac_4_5 += 30*c0p2*c3p2*ss[11];
  jac_4_5 += 30*c0p2*c4p2*ss[13];
  jac_4_5 += 30*c0p2*c5p2*ss[15];
  jac_4_5 += 20*c0p3*c1p1*ss[6];
  jac_4_5 += 20*c0p3*c2p1*ss[7];
  jac_4_5 += 20*c0p3*c3p1*ss[8];
  jac_4_5 += 20*c0p3*c4p1*ss[9];
  jac_4_5 += 20*c0p3*c5p1*ss[10];
  jac_4_5 += 20*c1p1*c2p3*ss[12];
  jac_4_5 += 20*c1p1*c3p3*ss[15];
  jac_4_5 += 20*c1p1*c4p3*ss[18];
  jac_4_5 += 20*c1p1*c5p3*ss[21];
  jac_4_5 += 30*c1p2*c2p2*ss[11];
  jac_4_5 += 30*c1p2*c3p2*ss[13];
  jac_4_5 += 30*c1p2*c4p2*ss[15];
  jac_4_5 += 30*c1p2*c5p2*ss[17];
  jac_4_5 += 20*c1p3*c2p1*ss[10];
  jac_4_5 += 20*c1p3*c3p1*ss[11];
  jac_4_5 += 20*c1p3*c4p1*ss[12];
  jac_4_5 += 20*c1p3*c5p1*ss[13];
  jac_4_5 += 20*c2p1*c3p3*ss[16];
  jac_4_5 += 20*c2p1*c4p3*ss[19];
  jac_4_5 += 20*c2p1*c5p3*ss[22];
  jac_4_5 += 30*c2p2*c3p2*ss[15];
  jac_4_5 += 30*c2p2*c4p2*ss[17];
  jac_4_5 += 30*c2p2*c5p2*ss[19];
  jac_4_5 += 20*c2p3*c3p1*ss[14];
  jac_4_5 += 20*c2p3*c4p1*ss[15];
  jac_4_5 += 20*c2p3*c5p1*ss[16];
  jac_4_5 += 20*c3p1*c4p3*ss[20];
  jac_4_5 += 20*c3p1*c5p3*ss[23];
  jac_4_5 += 30*c3p2*c4p2*ss[19];
  jac_4_5 += 30*c3p2*c5p2*ss[21];
  jac_4_5 += 20*c3p3*c4p1*ss[18];
  jac_4_5 += 20*c3p3*c5p1*ss[19];
  jac_4_5 += 20*c4p1*c5p3*ss[24];
  jac_4_5 += 30*c4p2*c5p2*ss[23];
  jac_4_5 += 20*c4p3*c5p1*ss[22];
  jac_4_5 += 5*c0p4*ss[5];
  jac_4_5 += 5*c1p4*ss[9];
  jac_4_5 += 5*c2p4*ss[13];
  jac_4_5 += 5*c3p4*ss[17];
  jac_4_5 += 5*c4p4*ss[21];
  jac_4_5 += 5*c5p4*ss[25];
  TACC jac_5_0 = convert<TACC>(720*c0p1*c1p1*c2p1*c3p1*c4p1*ss[10]);
  jac_5_0 += 720*c0p1*c1p1*c2p1*c3p1*c5p1*ss[11];
  jac_5_0 += 720*c0p1*c1p1*c2p1*c4p1*c5p1*ss[12];
  jac_5_0 += 720*c0p1*c1p1*c3p1*c4p1*c5p1*ss[13];
  jac_5_0 += 720*c0p1*c2p1*c3p1*c4p1*c5p1*ss[14];
  jac_5_0 += 720*c1p1*c2p1*c3p1*c4p1*c5p1*ss[15];
  jac_5_0 += 360*c0p1*c1p1*c2p1*c3p2*ss[9];
  jac_5_0 += 360*c0p1*c1p1*c2p1*c4p2*ss[11];
  jac_5_0 += 360*c0p1*c1p1*c2p1*c5p2*ss[13];
  jac_5_0 += 360*c0p1*c1p1*c2p2*c3p1*ss[8];
  jac_5_0 += 360*c0p1*c1p1*c2p2*c4p1*ss[9];
  jac_5_0 += 360*c0p1*c1p1*c2p2*c5p1*ss[10];
  jac_5_0 += 360*c0p1*c1p1*c3p1*c4p2*ss[12];
  jac_5_0 += 360*c0p1*c1p1*c3p1*c5p2*ss[14];
  jac_5_0 += 360*c0p1*c1p1*c3p2*c4p1*ss[11];
  jac_5_0 += 360*c0p1*c1p1*c3p2*c5p1*ss[12];
  jac_5_0 += 360*c0p1*c1p1*c4p1*c5p2*ss[15];
  jac_5_0 += 360*c0p1*c1p1*c4p2*c5p1*ss[14];
  jac_5_0 += 360*c0p1*c1p2*c2p1*c3p1*ss[7];
  jac_5_0 += 360*c0p1*c1p2*c2p1*c4p1*ss[8];
  jac_5_0 += 360*c0p1*c1p2*c2p1*c5p1*ss[9];
  jac_5_0 += 360*c0p1*c1p2*c3p1*c4p1*ss[9];
  jac_5_0 += 360*c0p1*c1p2*c3p1*c5p1*ss[10];
  jac_5_0 += 360*c0p1*c1p2*c4p1*c5p1*ss[11];
  jac_5_0 += 360*c0p1*c2p1*c3p1*c4p2*ss[13];
  jac_5_0 += 360*c0p1*c2p1*c3p1*c5p2*ss[15];
  jac_5_0 += 360*c0p1*c2p1*c3p2*c4p1*ss[12];
  jac_5_0 += 360*c0p1*c2p1*c3p2*c5p1*ss[13];
  jac_5_0 += 360*c0p1*c2p1*c4p1*c5p2*ss[16];
  jac_5_0 += 360*c0p1*c2p1*c4p2*c5p1*ss[15];
  jac_5_0 += 360*c0p1*c2p2*c3p1*c4p1*ss[11];
  jac_5_0 += 360*c0p1*c2p2*c3p1*c5p1*ss[12];
  jac_5_0 += 360*c0p1*c2p2*c4p1*c5p1*ss[13];
  jac_5_0 += 360*c0p1*c3p1*c4p1*c5p2*ss[17];
  jac_5_0 += 360*c0p1*c3p1*c4p2*c5p1*ss[16];
  jac_5_0 += 360*c0p1*c3p2*c4p1*c5p1*ss[15];
  jac_5_0 += 360*c0p2*c1p1*c2p1*c3p1*ss[6];
  jac_5_0 += 360*c0p2*c1p1*c2p1*c4p1*ss[7];
  jac_5_0 += 360*c0p2*c1p1*c2p1*c5p1*ss[8];
  jac_5_0 += 360*c0p2*c1p1*c3p1*c4p1*ss[8];
  jac_5_0 += 360*c0p2*c1p1*c3p1*c5p1*ss[9];
  jac_5_0 += 360*c0p2*c1p1*c4p1*c5p1*ss[10];
  jac_5_0 += 360*c0p2*c2p1*c3p1*c4p1*ss[9];
  jac_5_0 += 360*c0p2*c2p1*c3p1*c5p1*ss[10];
  jac_5_0 += 360*c0p2*c2p1*c4p1*c5p1*ss[11];
  jac_5_0 += 360*c0p2*c3p1*c4p1*c5p1*ss[12];
  jac_5_0 += 360*c1p1*c2p1*c3p1*c4p2*ss[14];
  jac_5_0 += 360*c1p1*c2p1*c3p1*c5p2*ss[16];
  jac_5_0 += 360*c1p1*c2p1*c3p2*c4p1*ss[13];
  jac_5_0 += 360*c1p1*c2p1*c3p2*c5p1*ss[14];
  jac_5_0 += 360*c1p1*c2p1*c4p1*c5p2*ss[17];
  jac_5_0 += 360*c1p1*c2p1*c4p2*c5p1*ss[16];
  jac_5_0 += 360*c1p1*c2p2*c3p1*c4p1*ss[12];
  jac_5_0 += 360*c1p1*c2p2*c3p1*c5p1*ss[13];
  jac_5_0 += 360*c1p1*c2p2*c4p1*c5p1*ss[14];
  jac_5_0 += 360*c1p1*c3p1*c4p1*c5p2*ss[18];
  jac_5_0 += 360*c1p1*c3p1*c4p2*c5p1*ss[17];
  jac_5_0 += 360*c1p1*c3p2*c4p1*c5p1*ss[16];
  jac_5_0 += 360*c1p2*c2p1*c3p1*c4p1*ss[11];
  jac_5_0 += 360*c1p2*c2p1*c3p1*c5p1*ss[12];
  jac_5_0 += 360*c1p2*c2p1*c4p1*c5p1*ss[13];
  jac_5_0 += 360*c1p2*c3p1*c4p1*c5p1*ss[14];
  jac_5_0 += 360*c2p1*c3p1*c4p1*c5p2*ss[19];
  jac_5_0 += 360*c2p1*c3p1*c4p2*c5p1*ss[18];
  jac_5_0 += 360*c2p1*c3p2*c4p1*c5p1*ss[17];
  jac_5_0 += 360*c2p2*c3p1*c4p1*c5p1*ss[16];
  jac_5_0 += 120*c0p1*c1p1*c2p3*ss[7];
  jac_5_0 += 120*c0p1*c1p1*c3p3*ss[10];
  jac_5_0 += 120*c0p1*c1p1*c4p3*ss[13];
  jac_5_0 += 120*c0p1*c1p1*c5p3*ss[16];
  jac_5_0 += 180*c0p1*c1p2*c2p2*ss[6];
  jac_5_0 += 180*c0p1*c1p2*c3p2*ss[8];
  jac_5_0 += 180*c0p1*c1p2*c4p2*ss[10];
  jac_5_0 += 180*c0p1*c1p2*c5p2*ss[12];
  jac_5_0 += 120*c0p1*c1p3*c2p1*ss[5];
  jac_5_0 += 120*c0p1*c1p3*c3p1*ss[6];
  jac_5_0 += 120*c0p1*c1p3*c4p1*ss[7];
  jac_5_0 += 120*c0p1*c1p3*c5p1*ss[8];
  jac_5_0 += 120*c0p1*c2p1*c3p3*ss[11];
  jac_5_0 += 120*c0p1*c2p1*c4p3*ss[14];
  jac_5_0 += 120*c0p1*c2p1*c5p3*ss[17];
  jac_5_0 += 180*c0p1*c2p2*c3p2*ss[10];
  jac_5_0 += 180*c0p1*c2p2*c4p2*ss[12];
  jac_5_0 += 180*c0p1*c2p2*c5p2*ss[14];
  jac_5_0 += 120*c0p1*c2p3*c3p1*ss[9];
  jac_5_0 += 120*c0p1*c2p3*c4p1*ss[10];
  jac_5_0 += 120*c0p1*c2p3*c5p1*ss[11];
  jac_5_0 += 120*c0p1*c3p1*c4p3*ss[15];
  jac_5_0 += 120*c0p1*c3p1*c5p3*ss[18];
  jac_5_0 += 180*c0p1*c3p2*c4p2*ss[14];
  jac_5_0 += 180*c0p1*c3p2*c5p2*ss[16];
  jac_5_0 += 120*c0p1*c3p3*c4p1*ss[13];
  jac_5_0 += 120*c0p1*c3p3*c5p1*ss[14];
  jac_5_0 += 120*c0p1*c4p1*c5p3*ss[19];
  jac_5_0 += 180*c0p1*c4p2*c5p2*ss[18];
  jac_5_0 += 120*c0p1*c4p3*c5p1*ss[17];
  jac_5_0 += 180*c0p2*c1p1*c2p2*ss[5];
  jac_5_0 += 180*c0p2*c1p1*c3p2*ss[7];
  jac_5_0 += 180*c0p2*c1p1*c4p2*ss[9];
  jac_5_0 += 180*c0p2*c1p1*c5p2*ss[11];
  jac_5_0 += 180*c0p2*c1p2*c2p1*ss[4];
  jac_5_0 += 180*c0p2*c1p2*c3p1*ss[5];
  jac_5_0 += 180*c0p2*c1p2*c4p1*ss[6];
  jac_5_0 += 180*c0p2*c1p2*c5p1*ss[7];
  jac_5_0 += 180*c0p2*c2p1*c3p2*ss[8];
  jac_5_0 += 180*c0p2*c2p1*c4p2*ss[10];
  jac_5_0 += 180*c0p2*c2p1*c5p2*ss[12];
  jac_5_0 += 180*c0p2*c2p2*c3p1*ss[7];
  jac_5_0 += 180*c0p2*c2p2*c4p1*ss[8];
  jac_5_0 += 180*c0p2*c2p2*c5p1*ss[9];
  jac_5_0 += 180*c0p2*c3p1*c4p2*ss[11];
  jac_5_0 += 180*c0p2*c3p1*c5p2*ss[13];
  jac_5_0 += 180*c0p2*c3p2*c4p1*ss[10];
  jac_5_0 += 180*c0p2*c3p2*c5p1*ss[11];
  jac_5_0 += 180*c0p2*c4p1*c5p2*ss[14];
  jac_5_0 += 180*c0p2*c4p2*c5p1*ss[13];
  jac_5_0 += 120*c0p3*c1p1*c2p1*ss[3];
  jac_5_0 += 120*c0p3*c1p1*c3p1*ss[4];
  jac_5_0 += 120*c0p3*c1p1*c4p1*ss[5];
  jac_5_0 += 120*c0p3*c1p1*c5p1*ss[6];
  jac_5_0 += 120*c0p3*c2p1*c3p1*ss[5];
  jac_5_0 += 120*c0p3*c2p1*c4p1*ss[6];
  jac_5_0 += 120*c0p3*c2p1*c5p1*ss[7];
  jac_5_0 += 120*c0p3*c3p1*c4p1*ss[7];
  jac_5_0 += 120*c0p3*c3p1*c5p1*ss[8];
  jac_5_0 += 120*c0p3*c4p1*c5p1*ss[9];
  jac_5_0 += 120*c1p1*c2p1*c3p3*ss[12];
  jac_5_0 += 120*c1p1*c2p1*c4p3*ss[15];
  jac_5_0 += 120*c1p1*c2p1*c5p3*ss[18];
  jac_5_0 += 180*c1p1*c2p2*c3p2*ss[11];
  jac_5_0 += 180*c1p1*c2p2*c4p2*ss[13];
  jac_5_0 += 180*c1p1*c2p2*c5p2*ss[15];
  jac_5_0 += 120*c1p1*c2p3*c3p1*ss[10];
  jac_5_0 += 120*c1p1*c2p3*c4p1*ss[11];
  jac_5_0 += 120*c1p1*c2p3*c5p1*ss[12];
  jac_5_0 += 120*c1p1*c3p1*c4p3*ss[16];
  jac_5_0 += 120*c1p1*c3p1*c5p3*ss[19];
  jac_5_0 += 180*c1p1*c3p2*c4p2*ss[15];
  jac_5_0 += 180*c1p1*c3p2*c5p2*ss[17];
  jac_5_0 += 120*c1p1*c3p3*c4p1*ss[14];
  jac_5_0 += 120*c1p1*c3p3*c5p1*ss[15];
  jac_5_0 += 120*c1p1*c4p1*c5p3*ss[20];
  jac_5_0 += 180*c1p1*c4p2*c5p2*ss[19];
  jac_5_0 += 120*c1p1*c4p3*c5p1*ss[18];
  jac_5_0 += 180*c1p2*c2p1*c3p2*ss[10];
  jac_5_0 += 180*c1p2*c2p1*c4p2*ss[12];
  jac_5_0 += 180*c1p2*c2p1*c5p2*ss[14];
  jac_5_0 += 180*c1p2*c2p2*c3p1*ss[9];
  jac_5_0 += 180*c1p2*c2p2*c4p1*ss[10];
  jac_5_0 += 180*c1p2*c2p2*c5p1*ss[11];
  jac_5_0 += 180*c1p2*c3p1*c4p2*ss[13];
  jac_5_0 += 180*c1p2*c3p1*c5p2*ss[15];
  jac_5_0 += 180*c1p2*c3p2*c4p1*ss[12];
  jac_5_0 += 180*c1p2*c3p2*c5p1*ss[13];
  jac_5_0 += 180*c1p2*c4p1*c5p2*ss[16];
  jac_5_0 += 180*c1p2*c4p2*c5p1*ss[15];
  jac_5_0 += 120*c1p3*c2p1*c3p1*ss[8];
  jac_5_0 += 120*c1p3*c2p1*c4p1*ss[9];
  jac_5_0 += 120*c1p3*c2p1*c5p1*ss[10];
  jac_5_0 += 120*c1p3*c3p1*c4p1*ss[10];
  jac_5_0 += 120*c1p3*c3p1*c5p1*ss[11];
  jac_5_0 += 120*c1p3*c4p1*c5p1*ss[12];
  jac_5_0 += 120*c2p1*c3p1*c4p3*ss[17];
  jac_5_0 += 120*c2p1*c3p1*c5p3*ss[20];
  jac_5_0 += 180*c2p1*c3p2*c4p2*ss[16];
  jac_5_0 += 180*c2p1*c3p2*c5p2*ss[18];
  jac_5_0 += 120*c2p1*c3p3*c4p1*ss[15];
  jac_5_0 += 120*c2p1*c3p3*c5p1*ss[16];
  jac_5_0 += 120*c2p1*c4p1*c5p3*ss[21];
  jac_5_0 += 180*c2p1*c4p2*c5p2*ss[20];
  jac_5_0 += 120*c2p1*c4p3*c5p1*ss[19];
  jac_5_0 += 180*c2p2*c3p1*c4p2*ss[15];
  jac_5_0 += 180*c2p2*c3p1*c5p2*ss[17];
  jac_5_0 += 180*c2p2*c3p2*c4p1*ss[14];
  jac_5_0 += 180*c2p2*c3p2*c5p1*ss[15];
  jac_5_0 += 180*c2p2*c4p1*c5p2*ss[18];
  jac_5_0 += 180*c2p2*c4p2*c5p1*ss[17];
  jac_5_0 += 120*c2p3*c3p1*c4p1*ss[13];
  jac_5_0 += 120*c2p3*c3p1*c5p1*ss[14];
  jac_5_0 += 120*c2p3*c4p1*c5p1*ss[15];
  jac_5_0 += 120*c3p1*c4p1*c5p3*ss[22];
  jac_5_0 += 180*c3p1*c4p2*c5p2*ss[21];
  jac_5_0 += 120*c3p1*c4p3*c5p1*ss[20];
  jac_5_0 += 180*c3p2*c4p1*c5p2*ss[20];
  jac_5_0 += 180*c3p2*c4p2*c5p1*ss[19];
  jac_5_0 += 120*c3p3*c4p1*c5p1*ss[18];
  jac_5_0 += 30*c0p1*c1p4*ss[4];
  jac_5_0 += 30*c0p1*c2p4*ss[8];
  jac_5_0 += 30*c0p1*c3p4*ss[12];
  jac_5_0 += 30*c0p1*c4p4*ss[16];
  jac_5_0 += 30*c0p1*c5p4*ss[20];
  jac_5_0 += 60*c0p2*c1p3*ss[3];
  jac_5_0 += 60*c0p2*c2p3*ss[6];
  jac_5_0 += 60*c0p2*c3p3*ss[9];
  jac_5_0 += 60*c0p2*c4p3*ss[12];
  jac_5_0 += 60*c0p2*c5p3*ss[15];
  jac_5_0 += 60*c0p3*c1p2*ss[2];
  jac_5_0 += 60*c0p3*c2p2*ss[4];
  jac_5_0 += 60*c0p3*c3p2*ss[6];
  jac_5_0 += 60*c0p3*c4p2*ss[8];
  jac_5_0 += 60*c0p3*c5p2*ss[10];
  jac_5_0 += 30*c0p4*c1p1*ss[1];
  jac_5_0 += 30*c0p4*c2p1*ss[2];
  jac_5_0 += 30*c0p4*c3p1*ss[3];
  jac_5_0 += 30*c0p4*c4p1*ss[4];
  jac_5_0 += 30*c0p4*c5p1*ss[5];
  jac_5_0 += 30*c1p1*c2p4*ss[9];
  jac_5_0 += 30*c1p1*c3p4*ss[13];
  jac_5_0 += 30*c1p1*c4p4*ss[17];
  jac_5_0 += 30*c1p1*c5p4*ss[21];
  jac_5_0 += 60*c1p2*c2p3*ss[8];
  jac_5_0 += 60*c1p2*c3p3*ss[11];
  jac_5_0 += 60*c1p2*c4p3*ss[14];
  jac_5_0 += 60*c1p2*c5p3*ss[17];
  jac_5_0 += 60*c1p3*c2p2*ss[7];
  jac_5_0 += 60*c1p3*c3p2*ss[9];
  jac_5_0 += 60*c1p3*c4p2*ss[11];
  jac_5_0 += 60*c1p3*c5p2*ss[13];
  jac_5_0 += 30*c1p4*c2p1*ss[6];
  jac_5_0 += 30*c1p4*c3p1*ss[7];
  jac_5_0 += 30*c1p4*c4p1*ss[8];
  jac_5_0 += 30*c1p4*c5p1*ss[9];
  jac_5_0 += 30*c2p1*c3p4*ss[14];
  jac_5_0 += 30*c2p1*c4p4*ss[18];
  jac_5_0 += 30*c2p1*c5p4*ss[22];
  jac_5_0 += 60*c2p2*c3p3*ss[13];
  jac_5_0 += 60*c2p2*c4p3*ss[16];
  jac_5_0 += 60*c2p2*c5p3*ss[19];
  jac_5_0 += 60*c2p3*c3p2*ss[12];
  jac_5_0 += 60*c2p3*c4p2*ss[14];
  jac_5_0 += 60*c2p3*c5p2*ss[16];
  jac_5_0 += 30*c2p4*c3p1*ss[11];
  jac_5_0 += 30*c2p4*c4p1*ss[12];
  jac_5_0 += 30*c2p4*c5p1*ss[13];
  jac_5_0 += 30*c3p1*c4p4*ss[19];
  jac_5_0 += 30*c3p1*c5p4*ss[23];
  jac_5_0 += 60*c3p2*c4p3*ss[18];
  jac_5_0 += 60*c3p2*c5p3*ss[21];
  jac_5_0 += 60*c3p3*c4p2*ss[17];
  jac_5_0 += 60*c3p3*c5p2*ss[19];
  jac_5_0 += 30*c3p4*c4p1*ss[16];
  jac_5_0 += 30*c3p4*c5p1*ss[17];
  jac_5_0 += 30*c4p1*c5p4*ss[24];
  jac_5_0 += 60*c4p2*c5p3*ss[23];
  jac_5_0 += 60*c4p3*c5p2*ss[22];
  jac_5_0 += 30*c4p4*c5p1*ss[21];
  jac_5_0 += 6*c1p5*ss[5];
  jac_5_0 += 6*c2p5*ss[10];
  jac_5_0 += 6*c3p5*ss[15];
  jac_5_0 += 6*c4p5*ss[20];
  jac_5_0 += 6*c5p5*ss[25];
  jac_5_0 += 6*c0p5;
  TACC jac_5_1 = convert<TACC>(720*c0p1*c1p1*c2p1*c3p1*c4p1*ss[11]);
  jac_5_1 += 720*c0p1*c1p1*c2p1*c3p1*c5p1*ss[12];
  jac_5_1 += 720*c0p1*c1p1*c2p1*c4p1*c5p1*ss[13];
  jac_5_1 += 720*c0p1*c1p1*c3p1*c4p1*c5p1*ss[14];
  jac_5_1 += 720*c0p1*c2p1*c3p1*c4p1*c5p1*ss[15];
  jac_5_1 += 720*c1p1*c2p1*c3p1*c4p1*c5p1*ss[16];
  jac_5_1 += 360*c0p1*c1p1*c2p1*c3p2*ss[10];
  jac_5_1 += 360*c0p1*c1p1*c2p1*c4p2*ss[12];
  jac_5_1 += 360*c0p1*c1p1*c2p1*c5p2*ss[14];
  jac_5_1 += 360*c0p1*c1p1*c2p2*c3p1*ss[9];
  jac_5_1 += 360*c0p1*c1p1*c2p2*c4p1*ss[10];
  jac_5_1 += 360*c0p1*c1p1*c2p2*c5p1*ss[11];
  jac_5_1 += 360*c0p1*c1p1*c3p1*c4p2*ss[13];
  jac_5_1 += 360*c0p1*c1p1*c3p1*c5p2*ss[15];
  jac_5_1 += 360*c0p1*c1p1*c3p2*c4p1*ss[12];
  jac_5_1 += 360*c0p1*c1p1*c3p2*c5p1*ss[13];
  jac_5_1 += 360*c0p1*c1p1*c4p1*c5p2*ss[16];
  jac_5_1 += 360*c0p1*c1p1*c4p2*c5p1*ss[15];
  jac_5_1 += 360*c0p1*c1p2*c2p1*c3p1*ss[8];
  jac_5_1 += 360*c0p1*c1p2*c2p1*c4p1*ss[9];
  jac_5_1 += 360*c0p1*c1p2*c2p1*c5p1*ss[10];
  jac_5_1 += 360*c0p1*c1p2*c3p1*c4p1*ss[10];
  jac_5_1 += 360*c0p1*c1p2*c3p1*c5p1*ss[11];
  jac_5_1 += 360*c0p1*c1p2*c4p1*c5p1*ss[12];
  jac_5_1 += 360*c0p1*c2p1*c3p1*c4p2*ss[14];
  jac_5_1 += 360*c0p1*c2p1*c3p1*c5p2*ss[16];
  jac_5_1 += 360*c0p1*c2p1*c3p2*c4p1*ss[13];
  jac_5_1 += 360*c0p1*c2p1*c3p2*c5p1*ss[14];
  jac_5_1 += 360*c0p1*c2p1*c4p1*c5p2*ss[17];
  jac_5_1 += 360*c0p1*c2p1*c4p2*c5p1*ss[16];
  jac_5_1 += 360*c0p1*c2p2*c3p1*c4p1*ss[12];
  jac_5_1 += 360*c0p1*c2p2*c3p1*c5p1*ss[13];
  jac_5_1 += 360*c0p1*c2p2*c4p1*c5p1*ss[14];
  jac_5_1 += 360*c0p1*c3p1*c4p1*c5p2*ss[18];
  jac_5_1 += 360*c0p1*c3p1*c4p2*c5p1*ss[17];
  jac_5_1 += 360*c0p1*c3p2*c4p1*c5p1*ss[16];
  jac_5_1 += 360*c0p2*c1p1*c2p1*c3p1*ss[7];
  jac_5_1 += 360*c0p2*c1p1*c2p1*c4p1*ss[8];
  jac_5_1 += 360*c0p2*c1p1*c2p1*c5p1*ss[9];
  jac_5_1 += 360*c0p2*c1p1*c3p1*c4p1*ss[9];
  jac_5_1 += 360*c0p2*c1p1*c3p1*c5p1*ss[10];
  jac_5_1 += 360*c0p2*c1p1*c4p1*c5p1*ss[11];
  jac_5_1 += 360*c0p2*c2p1*c3p1*c4p1*ss[10];
  jac_5_1 += 360*c0p2*c2p1*c3p1*c5p1*ss[11];
  jac_5_1 += 360*c0p2*c2p1*c4p1*c5p1*ss[12];
  jac_5_1 += 360*c0p2*c3p1*c4p1*c5p1*ss[13];
  jac_5_1 += 360*c1p1*c2p1*c3p1*c4p2*ss[15];
  jac_5_1 += 360*c1p1*c2p1*c3p1*c5p2*ss[17];
  jac_5_1 += 360*c1p1*c2p1*c3p2*c4p1*ss[14];
  jac_5_1 += 360*c1p1*c2p1*c3p2*c5p1*ss[15];
  jac_5_1 += 360*c1p1*c2p1*c4p1*c5p2*ss[18];
  jac_5_1 += 360*c1p1*c2p1*c4p2*c5p1*ss[17];
  jac_5_1 += 360*c1p1*c2p2*c3p1*c4p1*ss[13];
  jac_5_1 += 360*c1p1*c2p2*c3p1*c5p1*ss[14];
  jac_5_1 += 360*c1p1*c2p2*c4p1*c5p1*ss[15];
  jac_5_1 += 360*c1p1*c3p1*c4p1*c5p2*ss[19];
  jac_5_1 += 360*c1p1*c3p1*c4p2*c5p1*ss[18];
  jac_5_1 += 360*c1p1*c3p2*c4p1*c5p1*ss[17];
  jac_5_1 += 360*c1p2*c2p1*c3p1*c4p1*ss[12];
  jac_5_1 += 360*c1p2*c2p1*c3p1*c5p1*ss[13];
  jac_5_1 += 360*c1p2*c2p1*c4p1*c5p1*ss[14];
  jac_5_1 += 360*c1p2*c3p1*c4p1*c5p1*ss[15];
  jac_5_1 += 360*c2p1*c3p1*c4p1*c5p2*ss[20];
  jac_5_1 += 360*c2p1*c3p1*c4p2*c5p1*ss[19];
  jac_5_1 += 360*c2p1*c3p2*c4p1*c5p1*ss[18];
  jac_5_1 += 360*c2p2*c3p1*c4p1*c5p1*ss[17];
  jac_5_1 += 120*c0p1*c1p1*c2p3*ss[8];
  jac_5_1 += 120*c0p1*c1p1*c3p3*ss[11];
  jac_5_1 += 120*c0p1*c1p1*c4p3*ss[14];
  jac_5_1 += 120*c0p1*c1p1*c5p3*ss[17];
  jac_5_1 += 180*c0p1*c1p2*c2p2*ss[7];
  jac_5_1 += 180*c0p1*c1p2*c3p2*ss[9];
  jac_5_1 += 180*c0p1*c1p2*c4p2*ss[11];
  jac_5_1 += 180*c0p1*c1p2*c5p2*ss[13];
  jac_5_1 += 120*c0p1*c1p3*c2p1*ss[6];
  jac_5_1 += 120*c0p1*c1p3*c3p1*ss[7];
  jac_5_1 += 120*c0p1*c1p3*c4p1*ss[8];
  jac_5_1 += 120*c0p1*c1p3*c5p1*ss[9];
  jac_5_1 += 120*c0p1*c2p1*c3p3*ss[12];
  jac_5_1 += 120*c0p1*c2p1*c4p3*ss[15];
  jac_5_1 += 120*c0p1*c2p1*c5p3*ss[18];
  jac_5_1 += 180*c0p1*c2p2*c3p2*ss[11];
  jac_5_1 += 180*c0p1*c2p2*c4p2*ss[13];
  jac_5_1 += 180*c0p1*c2p2*c5p2*ss[15];
  jac_5_1 += 120*c0p1*c2p3*c3p1*ss[10];
  jac_5_1 += 120*c0p1*c2p3*c4p1*ss[11];
  jac_5_1 += 120*c0p1*c2p3*c5p1*ss[12];
  jac_5_1 += 120*c0p1*c3p1*c4p3*ss[16];
  jac_5_1 += 120*c0p1*c3p1*c5p3*ss[19];
  jac_5_1 += 180*c0p1*c3p2*c4p2*ss[15];
  jac_5_1 += 180*c0p1*c3p2*c5p2*ss[17];
  jac_5_1 += 120*c0p1*c3p3*c4p1*ss[14];
  jac_5_1 += 120*c0p1*c3p3*c5p1*ss[15];
  jac_5_1 += 120*c0p1*c4p1*c5p3*ss[20];
  jac_5_1 += 180*c0p1*c4p2*c5p2*ss[19];
  jac_5_1 += 120*c0p1*c4p3*c5p1*ss[18];
  jac_5_1 += 180*c0p2*c1p1*c2p2*ss[6];
  jac_5_1 += 180*c0p2*c1p1*c3p2*ss[8];
  jac_5_1 += 180*c0p2*c1p1*c4p2*ss[10];
  jac_5_1 += 180*c0p2*c1p1*c5p2*ss[12];
  jac_5_1 += 180*c0p2*c1p2*c2p1*ss[5];
  jac_5_1 += 180*c0p2*c1p2*c3p1*ss[6];
  jac_5_1 += 180*c0p2*c1p2*c4p1*ss[7];
  jac_5_1 += 180*c0p2*c1p2*c5p1*ss[8];
  jac_5_1 += 180*c0p2*c2p1*c3p2*ss[9];
  jac_5_1 += 180*c0p2*c2p1*c4p2*ss[11];
  jac_5_1 += 180*c0p2*c2p1*c5p2*ss[13];
  jac_5_1 += 180*c0p2*c2p2*c3p1*ss[8];
  jac_5_1 += 180*c0p2*c2p2*c4p1*ss[9];
  jac_5_1 += 180*c0p2*c2p2*c5p1*ss[10];
  jac_5_1 += 180*c0p2*c3p1*c4p2*ss[12];
  jac_5_1 += 180*c0p2*c3p1*c5p2*ss[14];
  jac_5_1 += 180*c0p2*c3p2*c4p1*ss[11];
  jac_5_1 += 180*c0p2*c3p2*c5p1*ss[12];
  jac_5_1 += 180*c0p2*c4p1*c5p2*ss[15];
  jac_5_1 += 180*c0p2*c4p2*c5p1*ss[14];
  jac_5_1 += 120*c0p3*c1p1*c2p1*ss[4];
  jac_5_1 += 120*c0p3*c1p1*c3p1*ss[5];
  jac_5_1 += 120*c0p3*c1p1*c4p1*ss[6];
  jac_5_1 += 120*c0p3*c1p1*c5p1*ss[7];
  jac_5_1 += 120*c0p3*c2p1*c3p1*ss[6];
  jac_5_1 += 120*c0p3*c2p1*c4p1*ss[7];
  jac_5_1 += 120*c0p3*c2p1*c5p1*ss[8];
  jac_5_1 += 120*c0p3*c3p1*c4p1*ss[8];
  jac_5_1 += 120*c0p3*c3p1*c5p1*ss[9];
  jac_5_1 += 120*c0p3*c4p1*c5p1*ss[10];
  jac_5_1 += 120*c1p1*c2p1*c3p3*ss[13];
  jac_5_1 += 120*c1p1*c2p1*c4p3*ss[16];
  jac_5_1 += 120*c1p1*c2p1*c5p3*ss[19];
  jac_5_1 += 180*c1p1*c2p2*c3p2*ss[12];
  jac_5_1 += 180*c1p1*c2p2*c4p2*ss[14];
  jac_5_1 += 180*c1p1*c2p2*c5p2*ss[16];
  jac_5_1 += 120*c1p1*c2p3*c3p1*ss[11];
  jac_5_1 += 120*c1p1*c2p3*c4p1*ss[12];
  jac_5_1 += 120*c1p1*c2p3*c5p1*ss[13];
  jac_5_1 += 120*c1p1*c3p1*c4p3*ss[17];
  jac_5_1 += 120*c1p1*c3p1*c5p3*ss[20];
  jac_5_1 += 180*c1p1*c3p2*c4p2*ss[16];
  jac_5_1 += 180*c1p1*c3p2*c5p2*ss[18];
  jac_5_1 += 120*c1p1*c3p3*c4p1*ss[15];
  jac_5_1 += 120*c1p1*c3p3*c5p1*ss[16];
  jac_5_1 += 120*c1p1*c4p1*c5p3*ss[21];
  jac_5_1 += 180*c1p1*c4p2*c5p2*ss[20];
  jac_5_1 += 120*c1p1*c4p3*c5p1*ss[19];
  jac_5_1 += 180*c1p2*c2p1*c3p2*ss[11];
  jac_5_1 += 180*c1p2*c2p1*c4p2*ss[13];
  jac_5_1 += 180*c1p2*c2p1*c5p2*ss[15];
  jac_5_1 += 180*c1p2*c2p2*c3p1*ss[10];
  jac_5_1 += 180*c1p2*c2p2*c4p1*ss[11];
  jac_5_1 += 180*c1p2*c2p2*c5p1*ss[12];
  jac_5_1 += 180*c1p2*c3p1*c4p2*ss[14];
  jac_5_1 += 180*c1p2*c3p1*c5p2*ss[16];
  jac_5_1 += 180*c1p2*c3p2*c4p1*ss[13];
  jac_5_1 += 180*c1p2*c3p2*c5p1*ss[14];
  jac_5_1 += 180*c1p2*c4p1*c5p2*ss[17];
  jac_5_1 += 180*c1p2*c4p2*c5p1*ss[16];
  jac_5_1 += 120*c1p3*c2p1*c3p1*ss[9];
  jac_5_1 += 120*c1p3*c2p1*c4p1*ss[10];
  jac_5_1 += 120*c1p3*c2p1*c5p1*ss[11];
  jac_5_1 += 120*c1p3*c3p1*c4p1*ss[11];
  jac_5_1 += 120*c1p3*c3p1*c5p1*ss[12];
  jac_5_1 += 120*c1p3*c4p1*c5p1*ss[13];
  jac_5_1 += 120*c2p1*c3p1*c4p3*ss[18];
  jac_5_1 += 120*c2p1*c3p1*c5p3*ss[21];
  jac_5_1 += 180*c2p1*c3p2*c4p2*ss[17];
  jac_5_1 += 180*c2p1*c3p2*c5p2*ss[19];
  jac_5_1 += 120*c2p1*c3p3*c4p1*ss[16];
  jac_5_1 += 120*c2p1*c3p3*c5p1*ss[17];
  jac_5_1 += 120*c2p1*c4p1*c5p3*ss[22];
  jac_5_1 += 180*c2p1*c4p2*c5p2*ss[21];
  jac_5_1 += 120*c2p1*c4p3*c5p1*ss[20];
  jac_5_1 += 180*c2p2*c3p1*c4p2*ss[16];
  jac_5_1 += 180*c2p2*c3p1*c5p2*ss[18];
  jac_5_1 += 180*c2p2*c3p2*c4p1*ss[15];
  jac_5_1 += 180*c2p2*c3p2*c5p1*ss[16];
  jac_5_1 += 180*c2p2*c4p1*c5p2*ss[19];
  jac_5_1 += 180*c2p2*c4p2*c5p1*ss[18];
  jac_5_1 += 120*c2p3*c3p1*c4p1*ss[14];
  jac_5_1 += 120*c2p3*c3p1*c5p1*ss[15];
  jac_5_1 += 120*c2p3*c4p1*c5p1*ss[16];
  jac_5_1 += 120*c3p1*c4p1*c5p3*ss[23];
  jac_5_1 += 180*c3p1*c4p2*c5p2*ss[22];
  jac_5_1 += 120*c3p1*c4p3*c5p1*ss[21];
  jac_5_1 += 180*c3p2*c4p1*c5p2*ss[21];
  jac_5_1 += 180*c3p2*c4p2*c5p1*ss[20];
  jac_5_1 += 120*c3p3*c4p1*c5p1*ss[19];
  jac_5_1 += 30*c0p1*c1p4*ss[5];
  jac_5_1 += 30*c0p1*c2p4*ss[9];
  jac_5_1 += 30*c0p1*c3p4*ss[13];
  jac_5_1 += 30*c0p1*c4p4*ss[17];
  jac_5_1 += 30*c0p1*c5p4*ss[21];
  jac_5_1 += 60*c0p2*c1p3*ss[4];
  jac_5_1 += 60*c0p2*c2p3*ss[7];
  jac_5_1 += 60*c0p2*c3p3*ss[10];
  jac_5_1 += 60*c0p2*c4p3*ss[13];
  jac_5_1 += 60*c0p2*c5p3*ss[16];
  jac_5_1 += 60*c0p3*c1p2*ss[3];
  jac_5_1 += 60*c0p3*c2p2*ss[5];
  jac_5_1 += 60*c0p3*c3p2*ss[7];
  jac_5_1 += 60*c0p3*c4p2*ss[9];
  jac_5_1 += 60*c0p3*c5p2*ss[11];
  jac_5_1 += 30*c0p4*c1p1*ss[2];
  jac_5_1 += 30*c0p4*c2p1*ss[3];
  jac_5_1 += 30*c0p4*c3p1*ss[4];
  jac_5_1 += 30*c0p4*c4p1*ss[5];
  jac_5_1 += 30*c0p4*c5p1*ss[6];
  jac_5_1 += 30*c1p1*c2p4*ss[10];
  jac_5_1 += 30*c1p1*c3p4*ss[14];
  jac_5_1 += 30*c1p1*c4p4*ss[18];
  jac_5_1 += 30*c1p1*c5p4*ss[22];
  jac_5_1 += 60*c1p2*c2p3*ss[9];
  jac_5_1 += 60*c1p2*c3p3*ss[12];
  jac_5_1 += 60*c1p2*c4p3*ss[15];
  jac_5_1 += 60*c1p2*c5p3*ss[18];
  jac_5_1 += 60*c1p3*c2p2*ss[8];
  jac_5_1 += 60*c1p3*c3p2*ss[10];
  jac_5_1 += 60*c1p3*c4p2*ss[12];
  jac_5_1 += 60*c1p3*c5p2*ss[14];
  jac_5_1 += 30*c1p4*c2p1*ss[7];
  jac_5_1 += 30*c1p4*c3p1*ss[8];
  jac_5_1 += 30*c1p4*c4p1*ss[9];
  jac_5_1 += 30*c1p4*c5p1*ss[10];
  jac_5_1 += 30*c2p1*c3p4*ss[15];
  jac_5_1 += 30*c2p1*c4p4*ss[19];
  jac_5_1 += 30*c2p1*c5p4*ss[23];
  jac_5_1 += 60*c2p2*c3p3*ss[14];
  jac_5_1 += 60*c2p2*c4p3*ss[17];
  jac_5_1 += 60*c2p2*c5p3*ss[20];
  jac_5_1 += 60*c2p3*c3p2*ss[13];
  jac_5_1 += 60*c2p3*c4p2*ss[15];
  jac_5_1 += 60*c2p3*c5p2*ss[17];
  jac_5_1 += 30*c2p4*c3p1*ss[12];
  jac_5_1 += 30*c2p4*c4p1*ss[13];
  jac_5_1 += 30*c2p4*c5p1*ss[14];
  jac_5_1 += 30*c3p1*c4p4*ss[20];
  jac_5_1 += 30*c3p1*c5p4*ss[24];
  jac_5_1 += 60*c3p2*c4p3*ss[19];
  jac_5_1 += 60*c3p2*c5p3*ss[22];
  jac_5_1 += 60*c3p3*c4p2*ss[18];
  jac_5_1 += 60*c3p3*c5p2*ss[20];
  jac_5_1 += 30*c3p4*c4p1*ss[17];
  jac_5_1 += 30*c3p4*c5p1*ss[18];
  jac_5_1 += 30*c4p1*c5p4*ss[25];
  jac_5_1 += 60*c4p2*c5p3*ss[24];
  jac_5_1 += 60*c4p3*c5p2*ss[23];
  jac_5_1 += 30*c4p4*c5p1*ss[22];
  jac_5_1 += 6*c0p5*ss[1];
  jac_5_1 += 6*c1p5*ss[6];
  jac_5_1 += 6*c2p5*ss[11];
  jac_5_1 += 6*c3p5*ss[16];
  jac_5_1 += 6*c4p5*ss[21];
  jac_5_1 += 6*c5p5*ss[26];
  TACC jac_5_2 = convert<TACC>(720*c0p1*c1p1*c2p1*c3p1*c4p1*ss[12]);
  jac_5_2 += 720*c0p1*c1p1*c2p1*c3p1*c5p1*ss[13];
  jac_5_2 += 720*c0p1*c1p1*c2p1*c4p1*c5p1*ss[14];
  jac_5_2 += 720*c0p1*c1p1*c3p1*c4p1*c5p1*ss[15];
  jac_5_2 += 720*c0p1*c2p1*c3p1*c4p1*c5p1*ss[16];
  jac_5_2 += 720*c1p1*c2p1*c3p1*c4p1*c5p1*ss[17];
  jac_5_2 += 360*c0p1*c1p1*c2p1*c3p2*ss[11];
  jac_5_2 += 360*c0p1*c1p1*c2p1*c4p2*ss[13];
  jac_5_2 += 360*c0p1*c1p1*c2p1*c5p2*ss[15];
  jac_5_2 += 360*c0p1*c1p1*c2p2*c3p1*ss[10];
  jac_5_2 += 360*c0p1*c1p1*c2p2*c4p1*ss[11];
  jac_5_2 += 360*c0p1*c1p1*c2p2*c5p1*ss[12];
  jac_5_2 += 360*c0p1*c1p1*c3p1*c4p2*ss[14];
  jac_5_2 += 360*c0p1*c1p1*c3p1*c5p2*ss[16];
  jac_5_2 += 360*c0p1*c1p1*c3p2*c4p1*ss[13];
  jac_5_2 += 360*c0p1*c1p1*c3p2*c5p1*ss[14];
  jac_5_2 += 360*c0p1*c1p1*c4p1*c5p2*ss[17];
  jac_5_2 += 360*c0p1*c1p1*c4p2*c5p1*ss[16];
  jac_5_2 += 360*c0p1*c1p2*c2p1*c3p1*ss[9];
  jac_5_2 += 360*c0p1*c1p2*c2p1*c4p1*ss[10];
  jac_5_2 += 360*c0p1*c1p2*c2p1*c5p1*ss[11];
  jac_5_2 += 360*c0p1*c1p2*c3p1*c4p1*ss[11];
  jac_5_2 += 360*c0p1*c1p2*c3p1*c5p1*ss[12];
  jac_5_2 += 360*c0p1*c1p2*c4p1*c5p1*ss[13];
  jac_5_2 += 360*c0p1*c2p1*c3p1*c4p2*ss[15];
  jac_5_2 += 360*c0p1*c2p1*c3p1*c5p2*ss[17];
  jac_5_2 += 360*c0p1*c2p1*c3p2*c4p1*ss[14];
  jac_5_2 += 360*c0p1*c2p1*c3p2*c5p1*ss[15];
  jac_5_2 += 360*c0p1*c2p1*c4p1*c5p2*ss[18];
  jac_5_2 += 360*c0p1*c2p1*c4p2*c5p1*ss[17];
  jac_5_2 += 360*c0p1*c2p2*c3p1*c4p1*ss[13];
  jac_5_2 += 360*c0p1*c2p2*c3p1*c5p1*ss[14];
  jac_5_2 += 360*c0p1*c2p2*c4p1*c5p1*ss[15];
  jac_5_2 += 360*c0p1*c3p1*c4p1*c5p2*ss[19];
  jac_5_2 += 360*c0p1*c3p1*c4p2*c5p1*ss[18];
  jac_5_2 += 360*c0p1*c3p2*c4p1*c5p1*ss[17];
  jac_5_2 += 360*c0p2*c1p1*c2p1*c3p1*ss[8];
  jac_5_2 += 360*c0p2*c1p1*c2p1*c4p1*ss[9];
  jac_5_2 += 360*c0p2*c1p1*c2p1*c5p1*ss[10];
  jac_5_2 += 360*c0p2*c1p1*c3p1*c4p1*ss[10];
  jac_5_2 += 360*c0p2*c1p1*c3p1*c5p1*ss[11];
  jac_5_2 += 360*c0p2*c1p1*c4p1*c5p1*ss[12];
  jac_5_2 += 360*c0p2*c2p1*c3p1*c4p1*ss[11];
  jac_5_2 += 360*c0p2*c2p1*c3p1*c5p1*ss[12];
  jac_5_2 += 360*c0p2*c2p1*c4p1*c5p1*ss[13];
  jac_5_2 += 360*c0p2*c3p1*c4p1*c5p1*ss[14];
  jac_5_2 += 360*c1p1*c2p1*c3p1*c4p2*ss[16];
  jac_5_2 += 360*c1p1*c2p1*c3p1*c5p2*ss[18];
  jac_5_2 += 360*c1p1*c2p1*c3p2*c4p1*ss[15];
  jac_5_2 += 360*c1p1*c2p1*c3p2*c5p1*ss[16];
  jac_5_2 += 360*c1p1*c2p1*c4p1*c5p2*ss[19];
  jac_5_2 += 360*c1p1*c2p1*c4p2*c5p1*ss[18];
  jac_5_2 += 360*c1p1*c2p2*c3p1*c4p1*ss[14];
  jac_5_2 += 360*c1p1*c2p2*c3p1*c5p1*ss[15];
  jac_5_2 += 360*c1p1*c2p2*c4p1*c5p1*ss[16];
  jac_5_2 += 360*c1p1*c3p1*c4p1*c5p2*ss[20];
  jac_5_2 += 360*c1p1*c3p1*c4p2*c5p1*ss[19];
  jac_5_2 += 360*c1p1*c3p2*c4p1*c5p1*ss[18];
  jac_5_2 += 360*c1p2*c2p1*c3p1*c4p1*ss[13];
  jac_5_2 += 360*c1p2*c2p1*c3p1*c5p1*ss[14];
  jac_5_2 += 360*c1p2*c2p1*c4p1*c5p1*ss[15];
  jac_5_2 += 360*c1p2*c3p1*c4p1*c5p1*ss[16];
  jac_5_2 += 360*c2p1*c3p1*c4p1*c5p2*ss[21];
  jac_5_2 += 360*c2p1*c3p1*c4p2*c5p1*ss[20];
  jac_5_2 += 360*c2p1*c3p2*c4p1*c5p1*ss[19];
  jac_5_2 += 360*c2p2*c3p1*c4p1*c5p1*ss[18];
  jac_5_2 += 120*c0p1*c1p1*c2p3*ss[9];
  jac_5_2 += 120*c0p1*c1p1*c3p3*ss[12];
  jac_5_2 += 120*c0p1*c1p1*c4p3*ss[15];
  jac_5_2 += 120*c0p1*c1p1*c5p3*ss[18];
  jac_5_2 += 180*c0p1*c1p2*c2p2*ss[8];
  jac_5_2 += 180*c0p1*c1p2*c3p2*ss[10];
  jac_5_2 += 180*c0p1*c1p2*c4p2*ss[12];
  jac_5_2 += 180*c0p1*c1p2*c5p2*ss[14];
  jac_5_2 += 120*c0p1*c1p3*c2p1*ss[7];
  jac_5_2 += 120*c0p1*c1p3*c3p1*ss[8];
  jac_5_2 += 120*c0p1*c1p3*c4p1*ss[9];
  jac_5_2 += 120*c0p1*c1p3*c5p1*ss[10];
  jac_5_2 += 120*c0p1*c2p1*c3p3*ss[13];
  jac_5_2 += 120*c0p1*c2p1*c4p3*ss[16];
  jac_5_2 += 120*c0p1*c2p1*c5p3*ss[19];
  jac_5_2 += 180*c0p1*c2p2*c3p2*ss[12];
  jac_5_2 += 180*c0p1*c2p2*c4p2*ss[14];
  jac_5_2 += 180*c0p1*c2p2*c5p2*ss[16];
  jac_5_2 += 120*c0p1*c2p3*c3p1*ss[11];
  jac_5_2 += 120*c0p1*c2p3*c4p1*ss[12];
  jac_5_2 += 120*c0p1*c2p3*c5p1*ss[13];
  jac_5_2 += 120*c0p1*c3p1*c4p3*ss[17];
  jac_5_2 += 120*c0p1*c3p1*c5p3*ss[20];
  jac_5_2 += 180*c0p1*c3p2*c4p2*ss[16];
  jac_5_2 += 180*c0p1*c3p2*c5p2*ss[18];
  jac_5_2 += 120*c0p1*c3p3*c4p1*ss[15];
  jac_5_2 += 120*c0p1*c3p3*c5p1*ss[16];
  jac_5_2 += 120*c0p1*c4p1*c5p3*ss[21];
  jac_5_2 += 180*c0p1*c4p2*c5p2*ss[20];
  jac_5_2 += 120*c0p1*c4p3*c5p1*ss[19];
  jac_5_2 += 180*c0p2*c1p1*c2p2*ss[7];
  jac_5_2 += 180*c0p2*c1p1*c3p2*ss[9];
  jac_5_2 += 180*c0p2*c1p1*c4p2*ss[11];
  jac_5_2 += 180*c0p2*c1p1*c5p2*ss[13];
  jac_5_2 += 180*c0p2*c1p2*c2p1*ss[6];
  jac_5_2 += 180*c0p2*c1p2*c3p1*ss[7];
  jac_5_2 += 180*c0p2*c1p2*c4p1*ss[8];
  jac_5_2 += 180*c0p2*c1p2*c5p1*ss[9];
  jac_5_2 += 180*c0p2*c2p1*c3p2*ss[10];
  jac_5_2 += 180*c0p2*c2p1*c4p2*ss[12];
  jac_5_2 += 180*c0p2*c2p1*c5p2*ss[14];
  jac_5_2 += 180*c0p2*c2p2*c3p1*ss[9];
  jac_5_2 += 180*c0p2*c2p2*c4p1*ss[10];
  jac_5_2 += 180*c0p2*c2p2*c5p1*ss[11];
  jac_5_2 += 180*c0p2*c3p1*c4p2*ss[13];
  jac_5_2 += 180*c0p2*c3p1*c5p2*ss[15];
  jac_5_2 += 180*c0p2*c3p2*c4p1*ss[12];
  jac_5_2 += 180*c0p2*c3p2*c5p1*ss[13];
  jac_5_2 += 180*c0p2*c4p1*c5p2*ss[16];
  jac_5_2 += 180*c0p2*c4p2*c5p1*ss[15];
  jac_5_2 += 120*c0p3*c1p1*c2p1*ss[5];
  jac_5_2 += 120*c0p3*c1p1*c3p1*ss[6];
  jac_5_2 += 120*c0p3*c1p1*c4p1*ss[7];
  jac_5_2 += 120*c0p3*c1p1*c5p1*ss[8];
  jac_5_2 += 120*c0p3*c2p1*c3p1*ss[7];
  jac_5_2 += 120*c0p3*c2p1*c4p1*ss[8];
  jac_5_2 += 120*c0p3*c2p1*c5p1*ss[9];
  jac_5_2 += 120*c0p3*c3p1*c4p1*ss[9];
  jac_5_2 += 120*c0p3*c3p1*c5p1*ss[10];
  jac_5_2 += 120*c0p3*c4p1*c5p1*ss[11];
  jac_5_2 += 120*c1p1*c2p1*c3p3*ss[14];
  jac_5_2 += 120*c1p1*c2p1*c4p3*ss[17];
  jac_5_2 += 120*c1p1*c2p1*c5p3*ss[20];
  jac_5_2 += 180*c1p1*c2p2*c3p2*ss[13];
  jac_5_2 += 180*c1p1*c2p2*c4p2*ss[15];
  jac_5_2 += 180*c1p1*c2p2*c5p2*ss[17];
  jac_5_2 += 120*c1p1*c2p3*c3p1*ss[12];
  jac_5_2 += 120*c1p1*c2p3*c4p1*ss[13];
  jac_5_2 += 120*c1p1*c2p3*c5p1*ss[14];
  jac_5_2 += 120*c1p1*c3p1*c4p3*ss[18];
  jac_5_2 += 120*c1p1*c3p1*c5p3*ss[21];
  jac_5_2 += 180*c1p1*c3p2*c4p2*ss[17];
  jac_5_2 += 180*c1p1*c3p2*c5p2*ss[19];
  jac_5_2 += 120*c1p1*c3p3*c4p1*ss[16];
  jac_5_2 += 120*c1p1*c3p3*c5p1*ss[17];
  jac_5_2 += 120*c1p1*c4p1*c5p3*ss[22];
  jac_5_2 += 180*c1p1*c4p2*c5p2*ss[21];
  jac_5_2 += 120*c1p1*c4p3*c5p1*ss[20];
  jac_5_2 += 180*c1p2*c2p1*c3p2*ss[12];
  jac_5_2 += 180*c1p2*c2p1*c4p2*ss[14];
  jac_5_2 += 180*c1p2*c2p1*c5p2*ss[16];
  jac_5_2 += 180*c1p2*c2p2*c3p1*ss[11];
  jac_5_2 += 180*c1p2*c2p2*c4p1*ss[12];
  jac_5_2 += 180*c1p2*c2p2*c5p1*ss[13];
  jac_5_2 += 180*c1p2*c3p1*c4p2*ss[15];
  jac_5_2 += 180*c1p2*c3p1*c5p2*ss[17];
  jac_5_2 += 180*c1p2*c3p2*c4p1*ss[14];
  jac_5_2 += 180*c1p2*c3p2*c5p1*ss[15];
  jac_5_2 += 180*c1p2*c4p1*c5p2*ss[18];
  jac_5_2 += 180*c1p2*c4p2*c5p1*ss[17];
  jac_5_2 += 120*c1p3*c2p1*c3p1*ss[10];
  jac_5_2 += 120*c1p3*c2p1*c4p1*ss[11];
  jac_5_2 += 120*c1p3*c2p1*c5p1*ss[12];
  jac_5_2 += 120*c1p3*c3p1*c4p1*ss[12];
  jac_5_2 += 120*c1p3*c3p1*c5p1*ss[13];
  jac_5_2 += 120*c1p3*c4p1*c5p1*ss[14];
  jac_5_2 += 120*c2p1*c3p1*c4p3*ss[19];
  jac_5_2 += 120*c2p1*c3p1*c5p3*ss[22];
  jac_5_2 += 180*c2p1*c3p2*c4p2*ss[18];
  jac_5_2 += 180*c2p1*c3p2*c5p2*ss[20];
  jac_5_2 += 120*c2p1*c3p3*c4p1*ss[17];
  jac_5_2 += 120*c2p1*c3p3*c5p1*ss[18];
  jac_5_2 += 120*c2p1*c4p1*c5p3*ss[23];
  jac_5_2 += 180*c2p1*c4p2*c5p2*ss[22];
  jac_5_2 += 120*c2p1*c4p3*c5p1*ss[21];
  jac_5_2 += 180*c2p2*c3p1*c4p2*ss[17];
  jac_5_2 += 180*c2p2*c3p1*c5p2*ss[19];
  jac_5_2 += 180*c2p2*c3p2*c4p1*ss[16];
  jac_5_2 += 180*c2p2*c3p2*c5p1*ss[17];
  jac_5_2 += 180*c2p2*c4p1*c5p2*ss[20];
  jac_5_2 += 180*c2p2*c4p2*c5p1*ss[19];
  jac_5_2 += 120*c2p3*c3p1*c4p1*ss[15];
  jac_5_2 += 120*c2p3*c3p1*c5p1*ss[16];
  jac_5_2 += 120*c2p3*c4p1*c5p1*ss[17];
  jac_5_2 += 120*c3p1*c4p1*c5p3*ss[24];
  jac_5_2 += 180*c3p1*c4p2*c5p2*ss[23];
  jac_5_2 += 120*c3p1*c4p3*c5p1*ss[22];
  jac_5_2 += 180*c3p2*c4p1*c5p2*ss[22];
  jac_5_2 += 180*c3p2*c4p2*c5p1*ss[21];
  jac_5_2 += 120*c3p3*c4p1*c5p1*ss[20];
  jac_5_2 += 30*c0p1*c1p4*ss[6];
  jac_5_2 += 30*c0p1*c2p4*ss[10];
  jac_5_2 += 30*c0p1*c3p4*ss[14];
  jac_5_2 += 30*c0p1*c4p4*ss[18];
  jac_5_2 += 30*c0p1*c5p4*ss[22];
  jac_5_2 += 60*c0p2*c1p3*ss[5];
  jac_5_2 += 60*c0p2*c2p3*ss[8];
  jac_5_2 += 60*c0p2*c3p3*ss[11];
  jac_5_2 += 60*c0p2*c4p3*ss[14];
  jac_5_2 += 60*c0p2*c5p3*ss[17];
  jac_5_2 += 60*c0p3*c1p2*ss[4];
  jac_5_2 += 60*c0p3*c2p2*ss[6];
  jac_5_2 += 60*c0p3*c3p2*ss[8];
  jac_5_2 += 60*c0p3*c4p2*ss[10];
  jac_5_2 += 60*c0p3*c5p2*ss[12];
  jac_5_2 += 30*c0p4*c1p1*ss[3];
  jac_5_2 += 30*c0p4*c2p1*ss[4];
  jac_5_2 += 30*c0p4*c3p1*ss[5];
  jac_5_2 += 30*c0p4*c4p1*ss[6];
  jac_5_2 += 30*c0p4*c5p1*ss[7];
  jac_5_2 += 30*c1p1*c2p4*ss[11];
  jac_5_2 += 30*c1p1*c3p4*ss[15];
  jac_5_2 += 30*c1p1*c4p4*ss[19];
  jac_5_2 += 30*c1p1*c5p4*ss[23];
  jac_5_2 += 60*c1p2*c2p3*ss[10];
  jac_5_2 += 60*c1p2*c3p3*ss[13];
  jac_5_2 += 60*c1p2*c4p3*ss[16];
  jac_5_2 += 60*c1p2*c5p3*ss[19];
  jac_5_2 += 60*c1p3*c2p2*ss[9];
  jac_5_2 += 60*c1p3*c3p2*ss[11];
  jac_5_2 += 60*c1p3*c4p2*ss[13];
  jac_5_2 += 60*c1p3*c5p2*ss[15];
  jac_5_2 += 30*c1p4*c2p1*ss[8];
  jac_5_2 += 30*c1p4*c3p1*ss[9];
  jac_5_2 += 30*c1p4*c4p1*ss[10];
  jac_5_2 += 30*c1p4*c5p1*ss[11];
  jac_5_2 += 30*c2p1*c3p4*ss[16];
  jac_5_2 += 30*c2p1*c4p4*ss[20];
  jac_5_2 += 30*c2p1*c5p4*ss[24];
  jac_5_2 += 60*c2p2*c3p3*ss[15];
  jac_5_2 += 60*c2p2*c4p3*ss[18];
  jac_5_2 += 60*c2p2*c5p3*ss[21];
  jac_5_2 += 60*c2p3*c3p2*ss[14];
  jac_5_2 += 60*c2p3*c4p2*ss[16];
  jac_5_2 += 60*c2p3*c5p2*ss[18];
  jac_5_2 += 30*c2p4*c3p1*ss[13];
  jac_5_2 += 30*c2p4*c4p1*ss[14];
  jac_5_2 += 30*c2p4*c5p1*ss[15];
  jac_5_2 += 30*c3p1*c4p4*ss[21];
  jac_5_2 += 30*c3p1*c5p4*ss[25];
  jac_5_2 += 60*c3p2*c4p3*ss[20];
  jac_5_2 += 60*c3p2*c5p3*ss[23];
  jac_5_2 += 60*c3p3*c4p2*ss[19];
  jac_5_2 += 60*c3p3*c5p2*ss[21];
  jac_5_2 += 30*c3p4*c4p1*ss[18];
  jac_5_2 += 30*c3p4*c5p1*ss[19];
  jac_5_2 += 30*c4p1*c5p4*ss[26];
  jac_5_2 += 60*c4p2*c5p3*ss[25];
  jac_5_2 += 60*c4p3*c5p2*ss[24];
  jac_5_2 += 30*c4p4*c5p1*ss[23];
  jac_5_2 += 6*c0p5*ss[2];
  jac_5_2 += 6*c1p5*ss[7];
  jac_5_2 += 6*c2p5*ss[12];
  jac_5_2 += 6*c3p5*ss[17];
  jac_5_2 += 6*c4p5*ss[22];
  jac_5_2 += 6*c5p5*ss[27];
  TACC jac_5_3 = convert<TACC>(720*c0p1*c1p1*c2p1*c3p1*c4p1*ss[13]);
  jac_5_3 += 720*c0p1*c1p1*c2p1*c3p1*c5p1*ss[14];
  jac_5_3 += 720*c0p1*c1p1*c2p1*c4p1*c5p1*ss[15];
  jac_5_3 += 720*c0p1*c1p1*c3p1*c4p1*c5p1*ss[16];
  jac_5_3 += 720*c0p1*c2p1*c3p1*c4p1*c5p1*ss[17];
  jac_5_3 += 720*c1p1*c2p1*c3p1*c4p1*c5p1*ss[18];
  jac_5_3 += 360*c0p1*c1p1*c2p1*c3p2*ss[12];
  jac_5_3 += 360*c0p1*c1p1*c2p1*c4p2*ss[14];
  jac_5_3 += 360*c0p1*c1p1*c2p1*c5p2*ss[16];
  jac_5_3 += 360*c0p1*c1p1*c2p2*c3p1*ss[11];
  jac_5_3 += 360*c0p1*c1p1*c2p2*c4p1*ss[12];
  jac_5_3 += 360*c0p1*c1p1*c2p2*c5p1*ss[13];
  jac_5_3 += 360*c0p1*c1p1*c3p1*c4p2*ss[15];
  jac_5_3 += 360*c0p1*c1p1*c3p1*c5p2*ss[17];
  jac_5_3 += 360*c0p1*c1p1*c3p2*c4p1*ss[14];
  jac_5_3 += 360*c0p1*c1p1*c3p2*c5p1*ss[15];
  jac_5_3 += 360*c0p1*c1p1*c4p1*c5p2*ss[18];
  jac_5_3 += 360*c0p1*c1p1*c4p2*c5p1*ss[17];
  jac_5_3 += 360*c0p1*c1p2*c2p1*c3p1*ss[10];
  jac_5_3 += 360*c0p1*c1p2*c2p1*c4p1*ss[11];
  jac_5_3 += 360*c0p1*c1p2*c2p1*c5p1*ss[12];
  jac_5_3 += 360*c0p1*c1p2*c3p1*c4p1*ss[12];
  jac_5_3 += 360*c0p1*c1p2*c3p1*c5p1*ss[13];
  jac_5_3 += 360*c0p1*c1p2*c4p1*c5p1*ss[14];
  jac_5_3 += 360*c0p1*c2p1*c3p1*c4p2*ss[16];
  jac_5_3 += 360*c0p1*c2p1*c3p1*c5p2*ss[18];
  jac_5_3 += 360*c0p1*c2p1*c3p2*c4p1*ss[15];
  jac_5_3 += 360*c0p1*c2p1*c3p2*c5p1*ss[16];
  jac_5_3 += 360*c0p1*c2p1*c4p1*c5p2*ss[19];
  jac_5_3 += 360*c0p1*c2p1*c4p2*c5p1*ss[18];
  jac_5_3 += 360*c0p1*c2p2*c3p1*c4p1*ss[14];
  jac_5_3 += 360*c0p1*c2p2*c3p1*c5p1*ss[15];
  jac_5_3 += 360*c0p1*c2p2*c4p1*c5p1*ss[16];
  jac_5_3 += 360*c0p1*c3p1*c4p1*c5p2*ss[20];
  jac_5_3 += 360*c0p1*c3p1*c4p2*c5p1*ss[19];
  jac_5_3 += 360*c0p1*c3p2*c4p1*c5p1*ss[18];
  jac_5_3 += 360*c0p2*c1p1*c2p1*c3p1*ss[9];
  jac_5_3 += 360*c0p2*c1p1*c2p1*c4p1*ss[10];
  jac_5_3 += 360*c0p2*c1p1*c2p1*c5p1*ss[11];
  jac_5_3 += 360*c0p2*c1p1*c3p1*c4p1*ss[11];
  jac_5_3 += 360*c0p2*c1p1*c3p1*c5p1*ss[12];
  jac_5_3 += 360*c0p2*c1p1*c4p1*c5p1*ss[13];
  jac_5_3 += 360*c0p2*c2p1*c3p1*c4p1*ss[12];
  jac_5_3 += 360*c0p2*c2p1*c3p1*c5p1*ss[13];
  jac_5_3 += 360*c0p2*c2p1*c4p1*c5p1*ss[14];
  jac_5_3 += 360*c0p2*c3p1*c4p1*c5p1*ss[15];
  jac_5_3 += 360*c1p1*c2p1*c3p1*c4p2*ss[17];
  jac_5_3 += 360*c1p1*c2p1*c3p1*c5p2*ss[19];
  jac_5_3 += 360*c1p1*c2p1*c3p2*c4p1*ss[16];
  jac_5_3 += 360*c1p1*c2p1*c3p2*c5p1*ss[17];
  jac_5_3 += 360*c1p1*c2p1*c4p1*c5p2*ss[20];
  jac_5_3 += 360*c1p1*c2p1*c4p2*c5p1*ss[19];
  jac_5_3 += 360*c1p1*c2p2*c3p1*c4p1*ss[15];
  jac_5_3 += 360*c1p1*c2p2*c3p1*c5p1*ss[16];
  jac_5_3 += 360*c1p1*c2p2*c4p1*c5p1*ss[17];
  jac_5_3 += 360*c1p1*c3p1*c4p1*c5p2*ss[21];
  jac_5_3 += 360*c1p1*c3p1*c4p2*c5p1*ss[20];
  jac_5_3 += 360*c1p1*c3p2*c4p1*c5p1*ss[19];
  jac_5_3 += 360*c1p2*c2p1*c3p1*c4p1*ss[14];
  jac_5_3 += 360*c1p2*c2p1*c3p1*c5p1*ss[15];
  jac_5_3 += 360*c1p2*c2p1*c4p1*c5p1*ss[16];
  jac_5_3 += 360*c1p2*c3p1*c4p1*c5p1*ss[17];
  jac_5_3 += 360*c2p1*c3p1*c4p1*c5p2*ss[22];
  jac_5_3 += 360*c2p1*c3p1*c4p2*c5p1*ss[21];
  jac_5_3 += 360*c2p1*c3p2*c4p1*c5p1*ss[20];
  jac_5_3 += 360*c2p2*c3p1*c4p1*c5p1*ss[19];
  jac_5_3 += 120*c0p1*c1p1*c2p3*ss[10];
  jac_5_3 += 120*c0p1*c1p1*c3p3*ss[13];
  jac_5_3 += 120*c0p1*c1p1*c4p3*ss[16];
  jac_5_3 += 120*c0p1*c1p1*c5p3*ss[19];
  jac_5_3 += 180*c0p1*c1p2*c2p2*ss[9];
  jac_5_3 += 180*c0p1*c1p2*c3p2*ss[11];
  jac_5_3 += 180*c0p1*c1p2*c4p2*ss[13];
  jac_5_3 += 180*c0p1*c1p2*c5p2*ss[15];
  jac_5_3 += 120*c0p1*c1p3*c2p1*ss[8];
  jac_5_3 += 120*c0p1*c1p3*c3p1*ss[9];
  jac_5_3 += 120*c0p1*c1p3*c4p1*ss[10];
  jac_5_3 += 120*c0p1*c1p3*c5p1*ss[11];
  jac_5_3 += 120*c0p1*c2p1*c3p3*ss[14];
  jac_5_3 += 120*c0p1*c2p1*c4p3*ss[17];
  jac_5_3 += 120*c0p1*c2p1*c5p3*ss[20];
  jac_5_3 += 180*c0p1*c2p2*c3p2*ss[13];
  jac_5_3 += 180*c0p1*c2p2*c4p2*ss[15];
  jac_5_3 += 180*c0p1*c2p2*c5p2*ss[17];
  jac_5_3 += 120*c0p1*c2p3*c3p1*ss[12];
  jac_5_3 += 120*c0p1*c2p3*c4p1*ss[13];
  jac_5_3 += 120*c0p1*c2p3*c5p1*ss[14];
  jac_5_3 += 120*c0p1*c3p1*c4p3*ss[18];
  jac_5_3 += 120*c0p1*c3p1*c5p3*ss[21];
  jac_5_3 += 180*c0p1*c3p2*c4p2*ss[17];
  jac_5_3 += 180*c0p1*c3p2*c5p2*ss[19];
  jac_5_3 += 120*c0p1*c3p3*c4p1*ss[16];
  jac_5_3 += 120*c0p1*c3p3*c5p1*ss[17];
  jac_5_3 += 120*c0p1*c4p1*c5p3*ss[22];
  jac_5_3 += 180*c0p1*c4p2*c5p2*ss[21];
  jac_5_3 += 120*c0p1*c4p3*c5p1*ss[20];
  jac_5_3 += 180*c0p2*c1p1*c2p2*ss[8];
  jac_5_3 += 180*c0p2*c1p1*c3p2*ss[10];
  jac_5_3 += 180*c0p2*c1p1*c4p2*ss[12];
  jac_5_3 += 180*c0p2*c1p1*c5p2*ss[14];
  jac_5_3 += 180*c0p2*c1p2*c2p1*ss[7];
  jac_5_3 += 180*c0p2*c1p2*c3p1*ss[8];
  jac_5_3 += 180*c0p2*c1p2*c4p1*ss[9];
  jac_5_3 += 180*c0p2*c1p2*c5p1*ss[10];
  jac_5_3 += 180*c0p2*c2p1*c3p2*ss[11];
  jac_5_3 += 180*c0p2*c2p1*c4p2*ss[13];
  jac_5_3 += 180*c0p2*c2p1*c5p2*ss[15];
  jac_5_3 += 180*c0p2*c2p2*c3p1*ss[10];
  jac_5_3 += 180*c0p2*c2p2*c4p1*ss[11];
  jac_5_3 += 180*c0p2*c2p2*c5p1*ss[12];
  jac_5_3 += 180*c0p2*c3p1*c4p2*ss[14];
  jac_5_3 += 180*c0p2*c3p1*c5p2*ss[16];
  jac_5_3 += 180*c0p2*c3p2*c4p1*ss[13];
  jac_5_3 += 180*c0p2*c3p2*c5p1*ss[14];
  jac_5_3 += 180*c0p2*c4p1*c5p2*ss[17];
  jac_5_3 += 180*c0p2*c4p2*c5p1*ss[16];
  jac_5_3 += 120*c0p3*c1p1*c2p1*ss[6];
  jac_5_3 += 120*c0p3*c1p1*c3p1*ss[7];
  jac_5_3 += 120*c0p3*c1p1*c4p1*ss[8];
  jac_5_3 += 120*c0p3*c1p1*c5p1*ss[9];
  jac_5_3 += 120*c0p3*c2p1*c3p1*ss[8];
  jac_5_3 += 120*c0p3*c2p1*c4p1*ss[9];
  jac_5_3 += 120*c0p3*c2p1*c5p1*ss[10];
  jac_5_3 += 120*c0p3*c3p1*c4p1*ss[10];
  jac_5_3 += 120*c0p3*c3p1*c5p1*ss[11];
  jac_5_3 += 120*c0p3*c4p1*c5p1*ss[12];
  jac_5_3 += 120*c1p1*c2p1*c3p3*ss[15];
  jac_5_3 += 120*c1p1*c2p1*c4p3*ss[18];
  jac_5_3 += 120*c1p1*c2p1*c5p3*ss[21];
  jac_5_3 += 180*c1p1*c2p2*c3p2*ss[14];
  jac_5_3 += 180*c1p1*c2p2*c4p2*ss[16];
  jac_5_3 += 180*c1p1*c2p2*c5p2*ss[18];
  jac_5_3 += 120*c1p1*c2p3*c3p1*ss[13];
  jac_5_3 += 120*c1p1*c2p3*c4p1*ss[14];
  jac_5_3 += 120*c1p1*c2p3*c5p1*ss[15];
  jac_5_3 += 120*c1p1*c3p1*c4p3*ss[19];
  jac_5_3 += 120*c1p1*c3p1*c5p3*ss[22];
  jac_5_3 += 180*c1p1*c3p2*c4p2*ss[18];
  jac_5_3 += 180*c1p1*c3p2*c5p2*ss[20];
  jac_5_3 += 120*c1p1*c3p3*c4p1*ss[17];
  jac_5_3 += 120*c1p1*c3p3*c5p1*ss[18];
  jac_5_3 += 120*c1p1*c4p1*c5p3*ss[23];
  jac_5_3 += 180*c1p1*c4p2*c5p2*ss[22];
  jac_5_3 += 120*c1p1*c4p3*c5p1*ss[21];
  jac_5_3 += 180*c1p2*c2p1*c3p2*ss[13];
  jac_5_3 += 180*c1p2*c2p1*c4p2*ss[15];
  jac_5_3 += 180*c1p2*c2p1*c5p2*ss[17];
  jac_5_3 += 180*c1p2*c2p2*c3p1*ss[12];
  jac_5_3 += 180*c1p2*c2p2*c4p1*ss[13];
  jac_5_3 += 180*c1p2*c2p2*c5p1*ss[14];
  jac_5_3 += 180*c1p2*c3p1*c4p2*ss[16];
  jac_5_3 += 180*c1p2*c3p1*c5p2*ss[18];
  jac_5_3 += 180*c1p2*c3p2*c4p1*ss[15];
  jac_5_3 += 180*c1p2*c3p2*c5p1*ss[16];
  jac_5_3 += 180*c1p2*c4p1*c5p2*ss[19];
  jac_5_3 += 180*c1p2*c4p2*c5p1*ss[18];
  jac_5_3 += 120*c1p3*c2p1*c3p1*ss[11];
  jac_5_3 += 120*c1p3*c2p1*c4p1*ss[12];
  jac_5_3 += 120*c1p3*c2p1*c5p1*ss[13];
  jac_5_3 += 120*c1p3*c3p1*c4p1*ss[13];
  jac_5_3 += 120*c1p3*c3p1*c5p1*ss[14];
  jac_5_3 += 120*c1p3*c4p1*c5p1*ss[15];
  jac_5_3 += 120*c2p1*c3p1*c4p3*ss[20];
  jac_5_3 += 120*c2p1*c3p1*c5p3*ss[23];
  jac_5_3 += 180*c2p1*c3p2*c4p2*ss[19];
  jac_5_3 += 180*c2p1*c3p2*c5p2*ss[21];
  jac_5_3 += 120*c2p1*c3p3*c4p1*ss[18];
  jac_5_3 += 120*c2p1*c3p3*c5p1*ss[19];
  jac_5_3 += 120*c2p1*c4p1*c5p3*ss[24];
  jac_5_3 += 180*c2p1*c4p2*c5p2*ss[23];
  jac_5_3 += 120*c2p1*c4p3*c5p1*ss[22];
  jac_5_3 += 180*c2p2*c3p1*c4p2*ss[18];
  jac_5_3 += 180*c2p2*c3p1*c5p2*ss[20];
  jac_5_3 += 180*c2p2*c3p2*c4p1*ss[17];
  jac_5_3 += 180*c2p2*c3p2*c5p1*ss[18];
  jac_5_3 += 180*c2p2*c4p1*c5p2*ss[21];
  jac_5_3 += 180*c2p2*c4p2*c5p1*ss[20];
  jac_5_3 += 120*c2p3*c3p1*c4p1*ss[16];
  jac_5_3 += 120*c2p3*c3p1*c5p1*ss[17];
  jac_5_3 += 120*c2p3*c4p1*c5p1*ss[18];
  jac_5_3 += 120*c3p1*c4p1*c5p3*ss[25];
  jac_5_3 += 180*c3p1*c4p2*c5p2*ss[24];
  jac_5_3 += 120*c3p1*c4p3*c5p1*ss[23];
  jac_5_3 += 180*c3p2*c4p1*c5p2*ss[23];
  jac_5_3 += 180*c3p2*c4p2*c5p1*ss[22];
  jac_5_3 += 120*c3p3*c4p1*c5p1*ss[21];
  jac_5_3 += 30*c0p1*c1p4*ss[7];
  jac_5_3 += 30*c0p1*c2p4*ss[11];
  jac_5_3 += 30*c0p1*c3p4*ss[15];
  jac_5_3 += 30*c0p1*c4p4*ss[19];
  jac_5_3 += 30*c0p1*c5p4*ss[23];
  jac_5_3 += 60*c0p2*c1p3*ss[6];
  jac_5_3 += 60*c0p2*c2p3*ss[9];
  jac_5_3 += 60*c0p2*c3p3*ss[12];
  jac_5_3 += 60*c0p2*c4p3*ss[15];
  jac_5_3 += 60*c0p2*c5p3*ss[18];
  jac_5_3 += 60*c0p3*c1p2*ss[5];
  jac_5_3 += 60*c0p3*c2p2*ss[7];
  jac_5_3 += 60*c0p3*c3p2*ss[9];
  jac_5_3 += 60*c0p3*c4p2*ss[11];
  jac_5_3 += 60*c0p3*c5p2*ss[13];
  jac_5_3 += 30*c0p4*c1p1*ss[4];
  jac_5_3 += 30*c0p4*c2p1*ss[5];
  jac_5_3 += 30*c0p4*c3p1*ss[6];
  jac_5_3 += 30*c0p4*c4p1*ss[7];
  jac_5_3 += 30*c0p4*c5p1*ss[8];
  jac_5_3 += 30*c1p1*c2p4*ss[12];
  jac_5_3 += 30*c1p1*c3p4*ss[16];
  jac_5_3 += 30*c1p1*c4p4*ss[20];
  jac_5_3 += 30*c1p1*c5p4*ss[24];
  jac_5_3 += 60*c1p2*c2p3*ss[11];
  jac_5_3 += 60*c1p2*c3p3*ss[14];
  jac_5_3 += 60*c1p2*c4p3*ss[17];
  jac_5_3 += 60*c1p2*c5p3*ss[20];
  jac_5_3 += 60*c1p3*c2p2*ss[10];
  jac_5_3 += 60*c1p3*c3p2*ss[12];
  jac_5_3 += 60*c1p3*c4p2*ss[14];
  jac_5_3 += 60*c1p3*c5p2*ss[16];
  jac_5_3 += 30*c1p4*c2p1*ss[9];
  jac_5_3 += 30*c1p4*c3p1*ss[10];
  jac_5_3 += 30*c1p4*c4p1*ss[11];
  jac_5_3 += 30*c1p4*c5p1*ss[12];
  jac_5_3 += 30*c2p1*c3p4*ss[17];
  jac_5_3 += 30*c2p1*c4p4*ss[21];
  jac_5_3 += 30*c2p1*c5p4*ss[25];
  jac_5_3 += 60*c2p2*c3p3*ss[16];
  jac_5_3 += 60*c2p2*c4p3*ss[19];
  jac_5_3 += 60*c2p2*c5p3*ss[22];
  jac_5_3 += 60*c2p3*c3p2*ss[15];
  jac_5_3 += 60*c2p3*c4p2*ss[17];
  jac_5_3 += 60*c2p3*c5p2*ss[19];
  jac_5_3 += 30*c2p4*c3p1*ss[14];
  jac_5_3 += 30*c2p4*c4p1*ss[15];
  jac_5_3 += 30*c2p4*c5p1*ss[16];
  jac_5_3 += 30*c3p1*c4p4*ss[22];
  jac_5_3 += 30*c3p1*c5p4*ss[26];
  jac_5_3 += 60*c3p2*c4p3*ss[21];
  jac_5_3 += 60*c3p2*c5p3*ss[24];
  jac_5_3 += 60*c3p3*c4p2*ss[20];
  jac_5_3 += 60*c3p3*c5p2*ss[22];
  jac_5_3 += 30*c3p4*c4p1*ss[19];
  jac_5_3 += 30*c3p4*c5p1*ss[20];
  jac_5_3 += 30*c4p1*c5p4*ss[27];
  jac_5_3 += 60*c4p2*c5p3*ss[26];
  jac_5_3 += 60*c4p3*c5p2*ss[25];
  jac_5_3 += 30*c4p4*c5p1*ss[24];
  jac_5_3 += 6*c0p5*ss[3];
  jac_5_3 += 6*c1p5*ss[8];
  jac_5_3 += 6*c2p5*ss[13];
  jac_5_3 += 6*c3p5*ss[18];
  jac_5_3 += 6*c4p5*ss[23];
  jac_5_3 += 6*c5p5*ss[28];
  TACC jac_5_4 = convert<TACC>(720*c0p1*c1p1*c2p1*c3p1*c4p1*ss[14]);
  jac_5_4 += 720*c0p1*c1p1*c2p1*c3p1*c5p1*ss[15];
  jac_5_4 += 720*c0p1*c1p1*c2p1*c4p1*c5p1*ss[16];
  jac_5_4 += 720*c0p1*c1p1*c3p1*c4p1*c5p1*ss[17];
  jac_5_4 += 720*c0p1*c2p1*c3p1*c4p1*c5p1*ss[18];
  jac_5_4 += 720*c1p1*c2p1*c3p1*c4p1*c5p1*ss[19];
  jac_5_4 += 360*c0p1*c1p1*c2p1*c3p2*ss[13];
  jac_5_4 += 360*c0p1*c1p1*c2p1*c4p2*ss[15];
  jac_5_4 += 360*c0p1*c1p1*c2p1*c5p2*ss[17];
  jac_5_4 += 360*c0p1*c1p1*c2p2*c3p1*ss[12];
  jac_5_4 += 360*c0p1*c1p1*c2p2*c4p1*ss[13];
  jac_5_4 += 360*c0p1*c1p1*c2p2*c5p1*ss[14];
  jac_5_4 += 360*c0p1*c1p1*c3p1*c4p2*ss[16];
  jac_5_4 += 360*c0p1*c1p1*c3p1*c5p2*ss[18];
  jac_5_4 += 360*c0p1*c1p1*c3p2*c4p1*ss[15];
  jac_5_4 += 360*c0p1*c1p1*c3p2*c5p1*ss[16];
  jac_5_4 += 360*c0p1*c1p1*c4p1*c5p2*ss[19];
  jac_5_4 += 360*c0p1*c1p1*c4p2*c5p1*ss[18];
  jac_5_4 += 360*c0p1*c1p2*c2p1*c3p1*ss[11];
  jac_5_4 += 360*c0p1*c1p2*c2p1*c4p1*ss[12];
  jac_5_4 += 360*c0p1*c1p2*c2p1*c5p1*ss[13];
  jac_5_4 += 360*c0p1*c1p2*c3p1*c4p1*ss[13];
  jac_5_4 += 360*c0p1*c1p2*c3p1*c5p1*ss[14];
  jac_5_4 += 360*c0p1*c1p2*c4p1*c5p1*ss[15];
  jac_5_4 += 360*c0p1*c2p1*c3p1*c4p2*ss[17];
  jac_5_4 += 360*c0p1*c2p1*c3p1*c5p2*ss[19];
  jac_5_4 += 360*c0p1*c2p1*c3p2*c4p1*ss[16];
  jac_5_4 += 360*c0p1*c2p1*c3p2*c5p1*ss[17];
  jac_5_4 += 360*c0p1*c2p1*c4p1*c5p2*ss[20];
  jac_5_4 += 360*c0p1*c2p1*c4p2*c5p1*ss[19];
  jac_5_4 += 360*c0p1*c2p2*c3p1*c4p1*ss[15];
  jac_5_4 += 360*c0p1*c2p2*c3p1*c5p1*ss[16];
  jac_5_4 += 360*c0p1*c2p2*c4p1*c5p1*ss[17];
  jac_5_4 += 360*c0p1*c3p1*c4p1*c5p2*ss[21];
  jac_5_4 += 360*c0p1*c3p1*c4p2*c5p1*ss[20];
  jac_5_4 += 360*c0p1*c3p2*c4p1*c5p1*ss[19];
  jac_5_4 += 360*c0p2*c1p1*c2p1*c3p1*ss[10];
  jac_5_4 += 360*c0p2*c1p1*c2p1*c4p1*ss[11];
  jac_5_4 += 360*c0p2*c1p1*c2p1*c5p1*ss[12];
  jac_5_4 += 360*c0p2*c1p1*c3p1*c4p1*ss[12];
  jac_5_4 += 360*c0p2*c1p1*c3p1*c5p1*ss[13];
  jac_5_4 += 360*c0p2*c1p1*c4p1*c5p1*ss[14];
  jac_5_4 += 360*c0p2*c2p1*c3p1*c4p1*ss[13];
  jac_5_4 += 360*c0p2*c2p1*c3p1*c5p1*ss[14];
  jac_5_4 += 360*c0p2*c2p1*c4p1*c5p1*ss[15];
  jac_5_4 += 360*c0p2*c3p1*c4p1*c5p1*ss[16];
  jac_5_4 += 360*c1p1*c2p1*c3p1*c4p2*ss[18];
  jac_5_4 += 360*c1p1*c2p1*c3p1*c5p2*ss[20];
  jac_5_4 += 360*c1p1*c2p1*c3p2*c4p1*ss[17];
  jac_5_4 += 360*c1p1*c2p1*c3p2*c5p1*ss[18];
  jac_5_4 += 360*c1p1*c2p1*c4p1*c5p2*ss[21];
  jac_5_4 += 360*c1p1*c2p1*c4p2*c5p1*ss[20];
  jac_5_4 += 360*c1p1*c2p2*c3p1*c4p1*ss[16];
  jac_5_4 += 360*c1p1*c2p2*c3p1*c5p1*ss[17];
  jac_5_4 += 360*c1p1*c2p2*c4p1*c5p1*ss[18];
  jac_5_4 += 360*c1p1*c3p1*c4p1*c5p2*ss[22];
  jac_5_4 += 360*c1p1*c3p1*c4p2*c5p1*ss[21];
  jac_5_4 += 360*c1p1*c3p2*c4p1*c5p1*ss[20];
  jac_5_4 += 360*c1p2*c2p1*c3p1*c4p1*ss[15];
  jac_5_4 += 360*c1p2*c2p1*c3p1*c5p1*ss[16];
  jac_5_4 += 360*c1p2*c2p1*c4p1*c5p1*ss[17];
  jac_5_4 += 360*c1p2*c3p1*c4p1*c5p1*ss[18];
  jac_5_4 += 360*c2p1*c3p1*c4p1*c5p2*ss[23];
  jac_5_4 += 360*c2p1*c3p1*c4p2*c5p1*ss[22];
  jac_5_4 += 360*c2p1*c3p2*c4p1*c5p1*ss[21];
  jac_5_4 += 360*c2p2*c3p1*c4p1*c5p1*ss[20];
  jac_5_4 += 120*c0p1*c1p1*c2p3*ss[11];
  jac_5_4 += 120*c0p1*c1p1*c3p3*ss[14];
  jac_5_4 += 120*c0p1*c1p1*c4p3*ss[17];
  jac_5_4 += 120*c0p1*c1p1*c5p3*ss[20];
  jac_5_4 += 180*c0p1*c1p2*c2p2*ss[10];
  jac_5_4 += 180*c0p1*c1p2*c3p2*ss[12];
  jac_5_4 += 180*c0p1*c1p2*c4p2*ss[14];
  jac_5_4 += 180*c0p1*c1p2*c5p2*ss[16];
  jac_5_4 += 120*c0p1*c1p3*c2p1*ss[9];
  jac_5_4 += 120*c0p1*c1p3*c3p1*ss[10];
  jac_5_4 += 120*c0p1*c1p3*c4p1*ss[11];
  jac_5_4 += 120*c0p1*c1p3*c5p1*ss[12];
  jac_5_4 += 120*c0p1*c2p1*c3p3*ss[15];
  jac_5_4 += 120*c0p1*c2p1*c4p3*ss[18];
  jac_5_4 += 120*c0p1*c2p1*c5p3*ss[21];
  jac_5_4 += 180*c0p1*c2p2*c3p2*ss[14];
  jac_5_4 += 180*c0p1*c2p2*c4p2*ss[16];
  jac_5_4 += 180*c0p1*c2p2*c5p2*ss[18];
  jac_5_4 += 120*c0p1*c2p3*c3p1*ss[13];
  jac_5_4 += 120*c0p1*c2p3*c4p1*ss[14];
  jac_5_4 += 120*c0p1*c2p3*c5p1*ss[15];
  jac_5_4 += 120*c0p1*c3p1*c4p3*ss[19];
  jac_5_4 += 120*c0p1*c3p1*c5p3*ss[22];
  jac_5_4 += 180*c0p1*c3p2*c4p2*ss[18];
  jac_5_4 += 180*c0p1*c3p2*c5p2*ss[20];
  jac_5_4 += 120*c0p1*c3p3*c4p1*ss[17];
  jac_5_4 += 120*c0p1*c3p3*c5p1*ss[18];
  jac_5_4 += 120*c0p1*c4p1*c5p3*ss[23];
  jac_5_4 += 180*c0p1*c4p2*c5p2*ss[22];
  jac_5_4 += 120*c0p1*c4p3*c5p1*ss[21];
  jac_5_4 += 180*c0p2*c1p1*c2p2*ss[9];
  jac_5_4 += 180*c0p2*c1p1*c3p2*ss[11];
  jac_5_4 += 180*c0p2*c1p1*c4p2*ss[13];
  jac_5_4 += 180*c0p2*c1p1*c5p2*ss[15];
  jac_5_4 += 180*c0p2*c1p2*c2p1*ss[8];
  jac_5_4 += 180*c0p2*c1p2*c3p1*ss[9];
  jac_5_4 += 180*c0p2*c1p2*c4p1*ss[10];
  jac_5_4 += 180*c0p2*c1p2*c5p1*ss[11];
  jac_5_4 += 180*c0p2*c2p1*c3p2*ss[12];
  jac_5_4 += 180*c0p2*c2p1*c4p2*ss[14];
  jac_5_4 += 180*c0p2*c2p1*c5p2*ss[16];
  jac_5_4 += 180*c0p2*c2p2*c3p1*ss[11];
  jac_5_4 += 180*c0p2*c2p2*c4p1*ss[12];
  jac_5_4 += 180*c0p2*c2p2*c5p1*ss[13];
  jac_5_4 += 180*c0p2*c3p1*c4p2*ss[15];
  jac_5_4 += 180*c0p2*c3p1*c5p2*ss[17];
  jac_5_4 += 180*c0p2*c3p2*c4p1*ss[14];
  jac_5_4 += 180*c0p2*c3p2*c5p1*ss[15];
  jac_5_4 += 180*c0p2*c4p1*c5p2*ss[18];
  jac_5_4 += 180*c0p2*c4p2*c5p1*ss[17];
  jac_5_4 += 120*c0p3*c1p1*c2p1*ss[7];
  jac_5_4 += 120*c0p3*c1p1*c3p1*ss[8];
  jac_5_4 += 120*c0p3*c1p1*c4p1*ss[9];
  jac_5_4 += 120*c0p3*c1p1*c5p1*ss[10];
  jac_5_4 += 120*c0p3*c2p1*c3p1*ss[9];
  jac_5_4 += 120*c0p3*c2p1*c4p1*ss[10];
  jac_5_4 += 120*c0p3*c2p1*c5p1*ss[11];
  jac_5_4 += 120*c0p3*c3p1*c4p1*ss[11];
  jac_5_4 += 120*c0p3*c3p1*c5p1*ss[12];
  jac_5_4 += 120*c0p3*c4p1*c5p1*ss[13];
  jac_5_4 += 120*c1p1*c2p1*c3p3*ss[16];
  jac_5_4 += 120*c1p1*c2p1*c4p3*ss[19];
  jac_5_4 += 120*c1p1*c2p1*c5p3*ss[22];
  jac_5_4 += 180*c1p1*c2p2*c3p2*ss[15];
  jac_5_4 += 180*c1p1*c2p2*c4p2*ss[17];
  jac_5_4 += 180*c1p1*c2p2*c5p2*ss[19];
  jac_5_4 += 120*c1p1*c2p3*c3p1*ss[14];
  jac_5_4 += 120*c1p1*c2p3*c4p1*ss[15];
  jac_5_4 += 120*c1p1*c2p3*c5p1*ss[16];
  jac_5_4 += 120*c1p1*c3p1*c4p3*ss[20];
  jac_5_4 += 120*c1p1*c3p1*c5p3*ss[23];
  jac_5_4 += 180*c1p1*c3p2*c4p2*ss[19];
  jac_5_4 += 180*c1p1*c3p2*c5p2*ss[21];
  jac_5_4 += 120*c1p1*c3p3*c4p1*ss[18];
  jac_5_4 += 120*c1p1*c3p3*c5p1*ss[19];
  jac_5_4 += 120*c1p1*c4p1*c5p3*ss[24];
  jac_5_4 += 180*c1p1*c4p2*c5p2*ss[23];
  jac_5_4 += 120*c1p1*c4p3*c5p1*ss[22];
  jac_5_4 += 180*c1p2*c2p1*c3p2*ss[14];
  jac_5_4 += 180*c1p2*c2p1*c4p2*ss[16];
  jac_5_4 += 180*c1p2*c2p1*c5p2*ss[18];
  jac_5_4 += 180*c1p2*c2p2*c3p1*ss[13];
  jac_5_4 += 180*c1p2*c2p2*c4p1*ss[14];
  jac_5_4 += 180*c1p2*c2p2*c5p1*ss[15];
  jac_5_4 += 180*c1p2*c3p1*c4p2*ss[17];
  jac_5_4 += 180*c1p2*c3p1*c5p2*ss[19];
  jac_5_4 += 180*c1p2*c3p2*c4p1*ss[16];
  jac_5_4 += 180*c1p2*c3p2*c5p1*ss[17];
  jac_5_4 += 180*c1p2*c4p1*c5p2*ss[20];
  jac_5_4 += 180*c1p2*c4p2*c5p1*ss[19];
  jac_5_4 += 120*c1p3*c2p1*c3p1*ss[12];
  jac_5_4 += 120*c1p3*c2p1*c4p1*ss[13];
  jac_5_4 += 120*c1p3*c2p1*c5p1*ss[14];
  jac_5_4 += 120*c1p3*c3p1*c4p1*ss[14];
  jac_5_4 += 120*c1p3*c3p1*c5p1*ss[15];
  jac_5_4 += 120*c1p3*c4p1*c5p1*ss[16];
  jac_5_4 += 120*c2p1*c3p1*c4p3*ss[21];
  jac_5_4 += 120*c2p1*c3p1*c5p3*ss[24];
  jac_5_4 += 180*c2p1*c3p2*c4p2*ss[20];
  jac_5_4 += 180*c2p1*c3p2*c5p2*ss[22];
  jac_5_4 += 120*c2p1*c3p3*c4p1*ss[19];
  jac_5_4 += 120*c2p1*c3p3*c5p1*ss[20];
  jac_5_4 += 120*c2p1*c4p1*c5p3*ss[25];
  jac_5_4 += 180*c2p1*c4p2*c5p2*ss[24];
  jac_5_4 += 120*c2p1*c4p3*c5p1*ss[23];
  jac_5_4 += 180*c2p2*c3p1*c4p2*ss[19];
  jac_5_4 += 180*c2p2*c3p1*c5p2*ss[21];
  jac_5_4 += 180*c2p2*c3p2*c4p1*ss[18];
  jac_5_4 += 180*c2p2*c3p2*c5p1*ss[19];
  jac_5_4 += 180*c2p2*c4p1*c5p2*ss[22];
  jac_5_4 += 180*c2p2*c4p2*c5p1*ss[21];
  jac_5_4 += 120*c2p3*c3p1*c4p1*ss[17];
  jac_5_4 += 120*c2p3*c3p1*c5p1*ss[18];
  jac_5_4 += 120*c2p3*c4p1*c5p1*ss[19];
  jac_5_4 += 120*c3p1*c4p1*c5p3*ss[26];
  jac_5_4 += 180*c3p1*c4p2*c5p2*ss[25];
  jac_5_4 += 120*c3p1*c4p3*c5p1*ss[24];
  jac_5_4 += 180*c3p2*c4p1*c5p2*ss[24];
  jac_5_4 += 180*c3p2*c4p2*c5p1*ss[23];
  jac_5_4 += 120*c3p3*c4p1*c5p1*ss[22];
  jac_5_4 += 30*c0p1*c1p4*ss[8];
  jac_5_4 += 30*c0p1*c2p4*ss[12];
  jac_5_4 += 30*c0p1*c3p4*ss[16];
  jac_5_4 += 30*c0p1*c4p4*ss[20];
  jac_5_4 += 30*c0p1*c5p4*ss[24];
  jac_5_4 += 60*c0p2*c1p3*ss[7];
  jac_5_4 += 60*c0p2*c2p3*ss[10];
  jac_5_4 += 60*c0p2*c3p3*ss[13];
  jac_5_4 += 60*c0p2*c4p3*ss[16];
  jac_5_4 += 60*c0p2*c5p3*ss[19];
  jac_5_4 += 60*c0p3*c1p2*ss[6];
  jac_5_4 += 60*c0p3*c2p2*ss[8];
  jac_5_4 += 60*c0p3*c3p2*ss[10];
  jac_5_4 += 60*c0p3*c4p2*ss[12];
  jac_5_4 += 60*c0p3*c5p2*ss[14];
  jac_5_4 += 30*c0p4*c1p1*ss[5];
  jac_5_4 += 30*c0p4*c2p1*ss[6];
  jac_5_4 += 30*c0p4*c3p1*ss[7];
  jac_5_4 += 30*c0p4*c4p1*ss[8];
  jac_5_4 += 30*c0p4*c5p1*ss[9];
  jac_5_4 += 30*c1p1*c2p4*ss[13];
  jac_5_4 += 30*c1p1*c3p4*ss[17];
  jac_5_4 += 30*c1p1*c4p4*ss[21];
  jac_5_4 += 30*c1p1*c5p4*ss[25];
  jac_5_4 += 60*c1p2*c2p3*ss[12];
  jac_5_4 += 60*c1p2*c3p3*ss[15];
  jac_5_4 += 60*c1p2*c4p3*ss[18];
  jac_5_4 += 60*c1p2*c5p3*ss[21];
  jac_5_4 += 60*c1p3*c2p2*ss[11];
  jac_5_4 += 60*c1p3*c3p2*ss[13];
  jac_5_4 += 60*c1p3*c4p2*ss[15];
  jac_5_4 += 60*c1p3*c5p2*ss[17];
  jac_5_4 += 30*c1p4*c2p1*ss[10];
  jac_5_4 += 30*c1p4*c3p1*ss[11];
  jac_5_4 += 30*c1p4*c4p1*ss[12];
  jac_5_4 += 30*c1p4*c5p1*ss[13];
  jac_5_4 += 30*c2p1*c3p4*ss[18];
  jac_5_4 += 30*c2p1*c4p4*ss[22];
  jac_5_4 += 30*c2p1*c5p4*ss[26];
  jac_5_4 += 60*c2p2*c3p3*ss[17];
  jac_5_4 += 60*c2p2*c4p3*ss[20];
  jac_5_4 += 60*c2p2*c5p3*ss[23];
  jac_5_4 += 60*c2p3*c3p2*ss[16];
  jac_5_4 += 60*c2p3*c4p2*ss[18];
  jac_5_4 += 60*c2p3*c5p2*ss[20];
  jac_5_4 += 30*c2p4*c3p1*ss[15];
  jac_5_4 += 30*c2p4*c4p1*ss[16];
  jac_5_4 += 30*c2p4*c5p1*ss[17];
  jac_5_4 += 30*c3p1*c4p4*ss[23];
  jac_5_4 += 30*c3p1*c5p4*ss[27];
  jac_5_4 += 60*c3p2*c4p3*ss[22];
  jac_5_4 += 60*c3p2*c5p3*ss[25];
  jac_5_4 += 60*c3p3*c4p2*ss[21];
  jac_5_4 += 60*c3p3*c5p2*ss[23];
  jac_5_4 += 30*c3p4*c4p1*ss[20];
  jac_5_4 += 30*c3p4*c5p1*ss[21];
  jac_5_4 += 30*c4p1*c5p4*ss[28];
  jac_5_4 += 60*c4p2*c5p3*ss[27];
  jac_5_4 += 60*c4p3*c5p2*ss[26];
  jac_5_4 += 30*c4p4*c5p1*ss[25];
  jac_5_4 += 6*c0p5*ss[4];
  jac_5_4 += 6*c1p5*ss[9];
  jac_5_4 += 6*c2p5*ss[14];
  jac_5_4 += 6*c3p5*ss[19];
  jac_5_4 += 6*c4p5*ss[24];
  jac_5_4 += 6*c5p5*ss[29];
  TACC jac_5_5 = convert<TACC>(720*c0p1*c1p1*c2p1*c3p1*c4p1*ss[15]);
  jac_5_5 += 720*c0p1*c1p1*c2p1*c3p1*c5p1*ss[16];
  jac_5_5 += 720*c0p1*c1p1*c2p1*c4p1*c5p1*ss[17];
  jac_5_5 += 720*c0p1*c1p1*c3p1*c4p1*c5p1*ss[18];
  jac_5_5 += 720*c0p1*c2p1*c3p1*c4p1*c5p1*ss[19];
  jac_5_5 += 720*c1p1*c2p1*c3p1*c4p1*c5p1*ss[20];
  jac_5_5 += 360*c0p1*c1p1*c2p1*c3p2*ss[14];
  jac_5_5 += 360*c0p1*c1p1*c2p1*c4p2*ss[16];
  jac_5_5 += 360*c0p1*c1p1*c2p1*c5p2*ss[18];
  jac_5_5 += 360*c0p1*c1p1*c2p2*c3p1*ss[13];
  jac_5_5 += 360*c0p1*c1p1*c2p2*c4p1*ss[14];
  jac_5_5 += 360*c0p1*c1p1*c2p2*c5p1*ss[15];
  jac_5_5 += 360*c0p1*c1p1*c3p1*c4p2*ss[17];
  jac_5_5 += 360*c0p1*c1p1*c3p1*c5p2*ss[19];
  jac_5_5 += 360*c0p1*c1p1*c3p2*c4p1*ss[16];
  jac_5_5 += 360*c0p1*c1p1*c3p2*c5p1*ss[17];
  jac_5_5 += 360*c0p1*c1p1*c4p1*c5p2*ss[20];
  jac_5_5 += 360*c0p1*c1p1*c4p2*c5p1*ss[19];
  jac_5_5 += 360*c0p1*c1p2*c2p1*c3p1*ss[12];
  jac_5_5 += 360*c0p1*c1p2*c2p1*c4p1*ss[13];
  jac_5_5 += 360*c0p1*c1p2*c2p1*c5p1*ss[14];
  jac_5_5 += 360*c0p1*c1p2*c3p1*c4p1*ss[14];
  jac_5_5 += 360*c0p1*c1p2*c3p1*c5p1*ss[15];
  jac_5_5 += 360*c0p1*c1p2*c4p1*c5p1*ss[16];
  jac_5_5 += 360*c0p1*c2p1*c3p1*c4p2*ss[18];
  jac_5_5 += 360*c0p1*c2p1*c3p1*c5p2*ss[20];
  jac_5_5 += 360*c0p1*c2p1*c3p2*c4p1*ss[17];
  jac_5_5 += 360*c0p1*c2p1*c3p2*c5p1*ss[18];
  jac_5_5 += 360*c0p1*c2p1*c4p1*c5p2*ss[21];
  jac_5_5 += 360*c0p1*c2p1*c4p2*c5p1*ss[20];
  jac_5_5 += 360*c0p1*c2p2*c3p1*c4p1*ss[16];
  jac_5_5 += 360*c0p1*c2p2*c3p1*c5p1*ss[17];
  jac_5_5 += 360*c0p1*c2p2*c4p1*c5p1*ss[18];
  jac_5_5 += 360*c0p1*c3p1*c4p1*c5p2*ss[22];
  jac_5_5 += 360*c0p1*c3p1*c4p2*c5p1*ss[21];
  jac_5_5 += 360*c0p1*c3p2*c4p1*c5p1*ss[20];
  jac_5_5 += 360*c0p2*c1p1*c2p1*c3p1*ss[11];
  jac_5_5 += 360*c0p2*c1p1*c2p1*c4p1*ss[12];
  jac_5_5 += 360*c0p2*c1p1*c2p1*c5p1*ss[13];
  jac_5_5 += 360*c0p2*c1p1*c3p1*c4p1*ss[13];
  jac_5_5 += 360*c0p2*c1p1*c3p1*c5p1*ss[14];
  jac_5_5 += 360*c0p2*c1p1*c4p1*c5p1*ss[15];
  jac_5_5 += 360*c0p2*c2p1*c3p1*c4p1*ss[14];
  jac_5_5 += 360*c0p2*c2p1*c3p1*c5p1*ss[15];
  jac_5_5 += 360*c0p2*c2p1*c4p1*c5p1*ss[16];
  jac_5_5 += 360*c0p2*c3p1*c4p1*c5p1*ss[17];
  jac_5_5 += 360*c1p1*c2p1*c3p1*c4p2*ss[19];
  jac_5_5 += 360*c1p1*c2p1*c3p1*c5p2*ss[21];
  jac_5_5 += 360*c1p1*c2p1*c3p2*c4p1*ss[18];
  jac_5_5 += 360*c1p1*c2p1*c3p2*c5p1*ss[19];
  jac_5_5 += 360*c1p1*c2p1*c4p1*c5p2*ss[22];
  jac_5_5 += 360*c1p1*c2p1*c4p2*c5p1*ss[21];
  jac_5_5 += 360*c1p1*c2p2*c3p1*c4p1*ss[17];
  jac_5_5 += 360*c1p1*c2p2*c3p1*c5p1*ss[18];
  jac_5_5 += 360*c1p1*c2p2*c4p1*c5p1*ss[19];
  jac_5_5 += 360*c1p1*c3p1*c4p1*c5p2*ss[23];
  jac_5_5 += 360*c1p1*c3p1*c4p2*c5p1*ss[22];
  jac_5_5 += 360*c1p1*c3p2*c4p1*c5p1*ss[21];
  jac_5_5 += 360*c1p2*c2p1*c3p1*c4p1*ss[16];
  jac_5_5 += 360*c1p2*c2p1*c3p1*c5p1*ss[17];
  jac_5_5 += 360*c1p2*c2p1*c4p1*c5p1*ss[18];
  jac_5_5 += 360*c1p2*c3p1*c4p1*c5p1*ss[19];
  jac_5_5 += 360*c2p1*c3p1*c4p1*c5p2*ss[24];
  jac_5_5 += 360*c2p1*c3p1*c4p2*c5p1*ss[23];
  jac_5_5 += 360*c2p1*c3p2*c4p1*c5p1*ss[22];
  jac_5_5 += 360*c2p2*c3p1*c4p1*c5p1*ss[21];
  jac_5_5 += 120*c0p1*c1p1*c2p3*ss[12];
  jac_5_5 += 120*c0p1*c1p1*c3p3*ss[15];
  jac_5_5 += 120*c0p1*c1p1*c4p3*ss[18];
  jac_5_5 += 120*c0p1*c1p1*c5p3*ss[21];
  jac_5_5 += 180*c0p1*c1p2*c2p2*ss[11];
  jac_5_5 += 180*c0p1*c1p2*c3p2*ss[13];
  jac_5_5 += 180*c0p1*c1p2*c4p2*ss[15];
  jac_5_5 += 180*c0p1*c1p2*c5p2*ss[17];
  jac_5_5 += 120*c0p1*c1p3*c2p1*ss[10];
  jac_5_5 += 120*c0p1*c1p3*c3p1*ss[11];
  jac_5_5 += 120*c0p1*c1p3*c4p1*ss[12];
  jac_5_5 += 120*c0p1*c1p3*c5p1*ss[13];
  jac_5_5 += 120*c0p1*c2p1*c3p3*ss[16];
  jac_5_5 += 120*c0p1*c2p1*c4p3*ss[19];
  jac_5_5 += 120*c0p1*c2p1*c5p3*ss[22];
  jac_5_5 += 180*c0p1*c2p2*c3p2*ss[15];
  jac_5_5 += 180*c0p1*c2p2*c4p2*ss[17];
  jac_5_5 += 180*c0p1*c2p2*c5p2*ss[19];
  jac_5_5 += 120*c0p1*c2p3*c3p1*ss[14];
  jac_5_5 += 120*c0p1*c2p3*c4p1*ss[15];
  jac_5_5 += 120*c0p1*c2p3*c5p1*ss[16];
  jac_5_5 += 120*c0p1*c3p1*c4p3*ss[20];
  jac_5_5 += 120*c0p1*c3p1*c5p3*ss[23];
  jac_5_5 += 180*c0p1*c3p2*c4p2*ss[19];
  jac_5_5 += 180*c0p1*c3p2*c5p2*ss[21];
  jac_5_5 += 120*c0p1*c3p3*c4p1*ss[18];
  jac_5_5 += 120*c0p1*c3p3*c5p1*ss[19];
  jac_5_5 += 120*c0p1*c4p1*c5p3*ss[24];
  jac_5_5 += 180*c0p1*c4p2*c5p2*ss[23];
  jac_5_5 += 120*c0p1*c4p3*c5p1*ss[22];
  jac_5_5 += 180*c0p2*c1p1*c2p2*ss[10];
  jac_5_5 += 180*c0p2*c1p1*c3p2*ss[12];
  jac_5_5 += 180*c0p2*c1p1*c4p2*ss[14];
  jac_5_5 += 180*c0p2*c1p1*c5p2*ss[16];
  jac_5_5 += 180*c0p2*c1p2*c2p1*ss[9];
  jac_5_5 += 180*c0p2*c1p2*c3p1*ss[10];
  jac_5_5 += 180*c0p2*c1p2*c4p1*ss[11];
  jac_5_5 += 180*c0p2*c1p2*c5p1*ss[12];
  jac_5_5 += 180*c0p2*c2p1*c3p2*ss[13];
  jac_5_5 += 180*c0p2*c2p1*c4p2*ss[15];
  jac_5_5 += 180*c0p2*c2p1*c5p2*ss[17];
  jac_5_5 += 180*c0p2*c2p2*c3p1*ss[12];
  jac_5_5 += 180*c0p2*c2p2*c4p1*ss[13];
  jac_5_5 += 180*c0p2*c2p2*c5p1*ss[14];
  jac_5_5 += 180*c0p2*c3p1*c4p2*ss[16];
  jac_5_5 += 180*c0p2*c3p1*c5p2*ss[18];
  jac_5_5 += 180*c0p2*c3p2*c4p1*ss[15];
  jac_5_5 += 180*c0p2*c3p2*c5p1*ss[16];
  jac_5_5 += 180*c0p2*c4p1*c5p2*ss[19];
  jac_5_5 += 180*c0p2*c4p2*c5p1*ss[18];
  jac_5_5 += 120*c0p3*c1p1*c2p1*ss[8];
  jac_5_5 += 120*c0p3*c1p1*c3p1*ss[9];
  jac_5_5 += 120*c0p3*c1p1*c4p1*ss[10];
  jac_5_5 += 120*c0p3*c1p1*c5p1*ss[11];
  jac_5_5 += 120*c0p3*c2p1*c3p1*ss[10];
  jac_5_5 += 120*c0p3*c2p1*c4p1*ss[11];
  jac_5_5 += 120*c0p3*c2p1*c5p1*ss[12];
  jac_5_5 += 120*c0p3*c3p1*c4p1*ss[12];
  jac_5_5 += 120*c0p3*c3p1*c5p1*ss[13];
  jac_5_5 += 120*c0p3*c4p1*c5p1*ss[14];
  jac_5_5 += 120*c1p1*c2p1*c3p3*ss[17];
  jac_5_5 += 120*c1p1*c2p1*c4p3*ss[20];
  jac_5_5 += 120*c1p1*c2p1*c5p3*ss[23];
  jac_5_5 += 180*c1p1*c2p2*c3p2*ss[16];
  jac_5_5 += 180*c1p1*c2p2*c4p2*ss[18];
  jac_5_5 += 180*c1p1*c2p2*c5p2*ss[20];
  jac_5_5 += 120*c1p1*c2p3*c3p1*ss[15];
  jac_5_5 += 120*c1p1*c2p3*c4p1*ss[16];
  jac_5_5 += 120*c1p1*c2p3*c5p1*ss[17];
  jac_5_5 += 120*c1p1*c3p1*c4p3*ss[21];
  jac_5_5 += 120*c1p1*c3p1*c5p3*ss[24];
  jac_5_5 += 180*c1p1*c3p2*c4p2*ss[20];
  jac_5_5 += 180*c1p1*c3p2*c5p2*ss[22];
  jac_5_5 += 120*c1p1*c3p3*c4p1*ss[19];
  jac_5_5 += 120*c1p1*c3p3*c5p1*ss[20];
  jac_5_5 += 120*c1p1*c4p1*c5p3*ss[25];
  jac_5_5 += 180*c1p1*c4p2*c5p2*ss[24];
  jac_5_5 += 120*c1p1*c4p3*c5p1*ss[23];
  jac_5_5 += 180*c1p2*c2p1*c3p2*ss[15];
  jac_5_5 += 180*c1p2*c2p1*c4p2*ss[17];
  jac_5_5 += 180*c1p2*c2p1*c5p2*ss[19];
  jac_5_5 += 180*c1p2*c2p2*c3p1*ss[14];
  jac_5_5 += 180*c1p2*c2p2*c4p1*ss[15];
  jac_5_5 += 180*c1p2*c2p2*c5p1*ss[16];
  jac_5_5 += 180*c1p2*c3p1*c4p2*ss[18];
  jac_5_5 += 180*c1p2*c3p1*c5p2*ss[20];
  jac_5_5 += 180*c1p2*c3p2*c4p1*ss[17];
  jac_5_5 += 180*c1p2*c3p2*c5p1*ss[18];
  jac_5_5 += 180*c1p2*c4p1*c5p2*ss[21];
  jac_5_5 += 180*c1p2*c4p2*c5p1*ss[20];
  jac_5_5 += 120*c1p3*c2p1*c3p1*ss[13];
  jac_5_5 += 120*c1p3*c2p1*c4p1*ss[14];
  jac_5_5 += 120*c1p3*c2p1*c5p1*ss[15];
  jac_5_5 += 120*c1p3*c3p1*c4p1*ss[15];
  jac_5_5 += 120*c1p3*c3p1*c5p1*ss[16];
  jac_5_5 += 120*c1p3*c4p1*c5p1*ss[17];
  jac_5_5 += 120*c2p1*c3p1*c4p3*ss[22];
  jac_5_5 += 120*c2p1*c3p1*c5p3*ss[25];
  jac_5_5 += 180*c2p1*c3p2*c4p2*ss[21];
  jac_5_5 += 180*c2p1*c3p2*c5p2*ss[23];
  jac_5_5 += 120*c2p1*c3p3*c4p1*ss[20];
  jac_5_5 += 120*c2p1*c3p3*c5p1*ss[21];
  jac_5_5 += 120*c2p1*c4p1*c5p3*ss[26];
  jac_5_5 += 180*c2p1*c4p2*c5p2*ss[25];
  jac_5_5 += 120*c2p1*c4p3*c5p1*ss[24];
  jac_5_5 += 180*c2p2*c3p1*c4p2*ss[20];
  jac_5_5 += 180*c2p2*c3p1*c5p2*ss[22];
  jac_5_5 += 180*c2p2*c3p2*c4p1*ss[19];
  jac_5_5 += 180*c2p2*c3p2*c5p1*ss[20];
  jac_5_5 += 180*c2p2*c4p1*c5p2*ss[23];
  jac_5_5 += 180*c2p2*c4p2*c5p1*ss[22];
  jac_5_5 += 120*c2p3*c3p1*c4p1*ss[18];
  jac_5_5 += 120*c2p3*c3p1*c5p1*ss[19];
  jac_5_5 += 120*c2p3*c4p1*c5p1*ss[20];
  jac_5_5 += 120*c3p1*c4p1*c5p3*ss[27];
  jac_5_5 += 180*c3p1*c4p2*c5p2*ss[26];
  jac_5_5 += 120*c3p1*c4p3*c5p1*ss[25];
  jac_5_5 += 180*c3p2*c4p1*c5p2*ss[25];
  jac_5_5 += 180*c3p2*c4p2*c5p1*ss[24];
  jac_5_5 += 120*c3p3*c4p1*c5p1*ss[23];
  jac_5_5 += 30*c0p1*c1p4*ss[9];
  jac_5_5 += 30*c0p1*c2p4*ss[13];
  jac_5_5 += 30*c0p1*c3p4*ss[17];
  jac_5_5 += 30*c0p1*c4p4*ss[21];
  jac_5_5 += 30*c0p1*c5p4*ss[25];
  jac_5_5 += 60*c0p2*c1p3*ss[8];
  jac_5_5 += 60*c0p2*c2p3*ss[11];
  jac_5_5 += 60*c0p2*c3p3*ss[14];
  jac_5_5 += 60*c0p2*c4p3*ss[17];
  jac_5_5 += 60*c0p2*c5p3*ss[20];
  jac_5_5 += 60*c0p3*c1p2*ss[7];
  jac_5_5 += 60*c0p3*c2p2*ss[9];
  jac_5_5 += 60*c0p3*c3p2*ss[11];
  jac_5_5 += 60*c0p3*c4p2*ss[13];
  jac_5_5 += 60*c0p3*c5p2*ss[15];
  jac_5_5 += 30*c0p4*c1p1*ss[6];
  jac_5_5 += 30*c0p4*c2p1*ss[7];
  jac_5_5 += 30*c0p4*c3p1*ss[8];
  jac_5_5 += 30*c0p4*c4p1*ss[9];
  jac_5_5 += 30*c0p4*c5p1*ss[10];
  jac_5_5 += 30*c1p1*c2p4*ss[14];
  jac_5_5 += 30*c1p1*c3p4*ss[18];
  jac_5_5 += 30*c1p1*c4p4*ss[22];
  jac_5_5 += 30*c1p1*c5p4*ss[26];
  jac_5_5 += 60*c1p2*c2p3*ss[13];
  jac_5_5 += 60*c1p2*c3p3*ss[16];
  jac_5_5 += 60*c1p2*c4p3*ss[19];
  jac_5_5 += 60*c1p2*c5p3*ss[22];
  jac_5_5 += 60*c1p3*c2p2*ss[12];
  jac_5_5 += 60*c1p3*c3p2*ss[14];
  jac_5_5 += 60*c1p3*c4p2*ss[16];
  jac_5_5 += 60*c1p3*c5p2*ss[18];
  jac_5_5 += 30*c1p4*c2p1*ss[11];
  jac_5_5 += 30*c1p4*c3p1*ss[12];
  jac_5_5 += 30*c1p4*c4p1*ss[13];
  jac_5_5 += 30*c1p4*c5p1*ss[14];
  jac_5_5 += 30*c2p1*c3p4*ss[19];
  jac_5_5 += 30*c2p1*c4p4*ss[23];
  jac_5_5 += 30*c2p1*c5p4*ss[27];
  jac_5_5 += 60*c2p2*c3p3*ss[18];
  jac_5_5 += 60*c2p2*c4p3*ss[21];
  jac_5_5 += 60*c2p2*c5p3*ss[24];
  jac_5_5 += 60*c2p3*c3p2*ss[17];
  jac_5_5 += 60*c2p3*c4p2*ss[19];
  jac_5_5 += 60*c2p3*c5p2*ss[21];
  jac_5_5 += 30*c2p4*c3p1*ss[16];
  jac_5_5 += 30*c2p4*c4p1*ss[17];
  jac_5_5 += 30*c2p4*c5p1*ss[18];
  jac_5_5 += 30*c3p1*c4p4*ss[24];
  jac_5_5 += 30*c3p1*c5p4*ss[28];
  jac_5_5 += 60*c3p2*c4p3*ss[23];
  jac_5_5 += 60*c3p2*c5p3*ss[26];
  jac_5_5 += 60*c3p3*c4p2*ss[22];
  jac_5_5 += 60*c3p3*c5p2*ss[24];
  jac_5_5 += 30*c3p4*c4p1*ss[21];
  jac_5_5 += 30*c3p4*c5p1*ss[22];
  jac_5_5 += 30*c4p1*c5p4*ss[29];
  jac_5_5 += 60*c4p2*c5p3*ss[28];
  jac_5_5 += 60*c4p3*c5p2*ss[27];
  jac_5_5 += 30*c4p4*c5p1*ss[26];
  jac_5_5 += 6*c0p5*ss[5];
  jac_5_5 += 6*c1p5*ss[10];
  jac_5_5 += 6*c2p5*ss[15];
  jac_5_5 += 6*c3p5*ss[20];
  jac_5_5 += 6*c4p5*ss[25];
  jac_5_5 += 6*c5p5*ss[30];
  roots[0]=convert<T>(root_0);
  roots[1]=convert<T>(root_1);
  roots[2]=convert<T>(root_2);
  roots[3]=convert<T>(root_3);
  roots[4]=convert<T>(root_4);
  roots[5]=convert<T>(root_5);
  jacobian[0] = convert<T>(jac_0_0);
  jacobian[1] = convert<T>(jac_0_1);
  jacobian[2] = convert<T>(jac_0_2);
  jacobian[3] = convert<T>(jac_0_3);
  jacobian[4] = convert<T>(jac_0_4);
  jacobian[5] = convert<T>(jac_0_5);
  jacobian[6] = convert<T>(jac_1_0);
  jacobian[7] = convert<T>(jac_1_1);
  jacobian[8] = convert<T>(jac_1_2);
  jacobian[9] = convert<T>(jac_1_3);
  jacobian[10] = convert<T>(jac_1_4);
  jacobian[11] = convert<T>(jac_1_5);
  jacobian[12] = convert<T>(jac_2_0);
  jacobian[13] = convert<T>(jac_2_1);
  jacobian[14] = convert<T>(jac_2_2);
  jacobian[15] = convert<T>(jac_2_3);
  jacobian[16] = convert<T>(jac_2_4);
  jacobian[17] = convert<T>(jac_2_5);
  jacobian[18] = convert<T>(jac_3_0);
  jacobian[19] = convert<T>(jac_3_1);
  jacobian[20] = convert<T>(jac_3_2);
  jacobian[21] = convert<T>(jac_3_3);
  jacobian[22] = convert<T>(jac_3_4);
  jacobian[23] = convert<T>(jac_3_5);
  jacobian[24] = convert<T>(jac_4_0);
  jacobian[25] = convert<T>(jac_4_1);
  jacobian[26] = convert<T>(jac_4_2);
  jacobian[27] = convert<T>(jac_4_3);
  jacobian[28] = convert<T>(jac_4_4);
  jacobian[29] = convert<T>(jac_4_5);
  jacobian[30] = convert<T>(jac_5_0);
  jacobian[31] = convert<T>(jac_5_1);
  jacobian[32] = convert<T>(jac_5_2);
  jacobian[33] = convert<T>(jac_5_3);
  jacobian[34] = convert<T>(jac_5_4);
  jacobian[35] = convert<T>(jac_5_5);
}
}; // QuinticSystem

}; // systems
}; // random
}; // flopoco

#endif
